/**
 * @file    buf.h
 * @brief   Buffer module
 *
 *          The buffer is an ordered collection of data values. Before a new data value is stored at the head of the buffer all
 *          data elements are moved by one position. The shift operation deletes the oldest value at the tail of the buffer. 
 *
 * @par     Copyright:
 *
 *          Thermo Electron (Karlsruhe) GmbH, Dieselstrasse 4, 76227 Karlsruhe, Germany
 *
 *          (c) Copyright 2009 - 2010, Thermo Fisher Scientific
 *
 *          All rights reserved. Thermo Fisher Scientific source code is an unpublished work and the use of a copyright notice does
 *          not imply otherwise. This source code contains confidential, trade secret material of Thermo Fisher Scientific. Any
 *          attempt or participation in deciphering, decoding, reverse engineering or in any way altering the source code is
 *          strictly prohibited, unless the prior written consent of Thermo Fisher Scientific is obtained.
 *
 * @par     Creation Date:
 *
 *          05-Dec-2009
 *
 * @par     Last Modification:
 *
 *          27-Jan-2010
 *
 * @par     Project:
 *
 *          MARS III
 *
 * @par     Editor:
 *
 *          Microsoft Visual C++ 6.0
 *
 * @par     Compiler:
 *
 *          Microsoft Visual C++ 6.0
 *
 * @author  Uwe Bauer
 *
 * @par     Revision History:
 */

#ifndef BUF_H
#define BUF_H

/*
 **********************************************************************************************************************************
 *                                                           INCLUDES                                                             *
 **********************************************************************************************************************************
 */

//#include "typedefs.h"

/*
 **********************************************************************************************************************************
 *                                                       GLOBAL DATA TYPES                                                        *
 **********************************************************************************************************************************
 */

/**
 * Buffer object
 */
typedef struct buf_queue {                                          
    Uint32  size;                                                   /**< Buffer size                                              */
    float32   *tail;                                                   /**< Data start adress                                        */
    Uint32  cnt;                                                    /**< Number of samples in the buffer                          */
    Uint16   full;                                                   /**< Buffer load status                                       */
    Uint16   rdy;                                                    /**< Data ready flag                                          */
    void (*cpyFun)(float32 *src, float32 *dst, Uint32 n);                 /**< Buffer data copy function                                */
} BUF_QUEUE;

/*
 **********************************************************************************************************************************
 *                                                   GLOBAL FUNCTION PROTOTYPES                                                   *
 **********************************************************************************************************************************
 */

/**
 * Initialize an empty buffer
 *
 * BufInit() is the initialization code for the BUF module. The function must be called before any other BUF module function can
 * used. BufInit() is responsible for initializing the internal variables used by the module and specify the copy function to be
 * executed when the BufCpy() function is called.
 *
 * @param   p       Pointer to the used buffer instance.
 * @param   data    Pointer to the used data memory.
 * @param   n       Number of buffer data elements.
 * @param   cpyFun  Pointer to the data copy function when the BufCpy() function is called.
 */
void BufInit(BUF_QUEUE *p, float32 *data, const Uint32 n, void (*cpyFun)(float32 *, float32 *, Uint32));

/**
 * Reset buffer
 *
 * BufReset() initialize the buffer data memory with zero values, resets the buffer element count and full load indicator.
 *
 * @param   p       Pointer to the used buffer instance.
 */
void BufReset(BUF_QUEUE *p);

/**
 * Insert a new value at the head of the buffer
 *
 * BufInsNxt()inserts a value to the head of the buffer. In the case were the buffer is full all stored data elements moved by one
 * position before the new value can be stored to the head. In this situation the first element which was inserted into the buffer
 * is removed. The buffer follows the first in first out (FIFO) principle.
 *
 * @param   p       Pointer to the used buffer instance.
 * @param   val     New value which has to be inserted into the buffer.
 */
void BufInsNxt(BUF_QUEUE *p, const float32 val);

/**
 * Read buffer load status
 *
 * The buffer load status can be obtained by calling BufIsFull(). The function returns 1 if the buffer is totally filled otherwise 0.
 *
 * @param   p       Pointer to the used buffer instance.
 * @returns         1 if the buffer is full otherwise 0.
 */
Uint16 BufIsFull(const BUF_QUEUE *p);

/**
 * Read buffer count
 *
 * The number of stored buffer data elements can be obtained by calling BufGetCnt()
 *
 * @param   p       Pointer to the used buffer instance.
 * @returns         the number of stored data elements in the buffer.
 */
Uint32 BufGetCnt(const BUF_QUEUE *p);

/**
 * Copy buffer data
 *
 * BufCpy() copy the buffer data to the specified destination memory region.
 *
 * @param   p       Pointer to the used buffer instance, were the data has to copy from.
 * @param   dst     Pointer to the new memory region, were the data has to copy.
 */
void BufCpy(BUF_QUEUE *p, float32 *dst);

/**
 * Buffer data ready
 *
 * The buffer data ready status can be obtained by calling BufIsDataRdy(). The function returns 1 when valid buffer data is
 * available otherwise 0. The data ready flag is set after the BufInsNext() function is called and a new value is inserted into
 * the buffer. The flag is rested after the BufCpy() function is called and the whole buffer content is copied.
 *
 * @param   p       Pointer to the used buffer instance.
 * @returns         1 when valid buffer data is available otherwise 0.
 */
Uint16 BufIsDataRdy(const BUF_QUEUE *p);

#endif  /* EOF BUF_H    */
