/*
************************************************************************************************************************************
*                                              READY-TO-USE EMBEDDED BUILDING BLOCKS
*
*                             (c) Copyright 2015-2016; Thermo Electron (Karlsruhe) GmbH; Karlsruhe-Germany
*
*       All rights reserved. Protected by international copyright laws. 
*
*       TMO source code is an unpublished work and the use of a copyright notice does not imply otherwise. This source code
*       contains confidential, trade secret material of TMO. Any attempt or participation in deciphering, decoding, reverse
*       engineering or in any way altering the source code is strictly prohibited, unless the prior written consent of TMO
*       is obtained.
*
************************************************************************************************************************************
*/

/**
************************************************************************************************************************************
*
*                                                         FILE DESCRIPTION
*
* @file         cbuf.h
* @brief        Circular Buffers
* @details      At circular buffers the new input values are continuously written into the memory. A pointer points to the memory
*               location in which the newest input value is written. When the last element of the memory is reached, the write
*               operation is proceeding with the first memory element. The new value x(n) overwrites the oldest value x(n-N) in the
*               memory and the shifting operation is not necessary.
* @version      V1.00.00
* @author       UwBa
*
************************************************************************************************************************************
* Notes         (1) The note(s) section should be omitted if no notes are given.
************************************************************************************************************************************
*/

/*
************************************************************************************************************************************
*                                                           MODULE START
************************************************************************************************************************************
*/

#ifndef CBUF_H
#define CBUF_H                                                  /**< CBUF.H file encapsulation to avoid multiple parsing.         */

/*
************************************************************************************************************************************
*                                                          INCLUDE FILES
************************************************************************************************************************************
*/

//#include "typedef.h"

/*
************************************************************************************************************************************
*                                                   EXTERNAL C LANGUAGE LINKAGE
*
* Note(s) : (1) C++ compilers must 'extern'ally declare all C function prototypes & variable/object declarations for correct C
*               language linkage.
************************************************************************************************************************************
*/

#ifdef __cplusplus
extern "C" {                                                    /* See Note #1.                                                   */
#endif

/*
************************************************************************************************************************************
*                                                            EXTERNALS
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                             DEFINES
************************************************************************************************************************************
*/

/** Minimum circular buffer length
 *
 *  A circular buffer needs at least 2 data elements.
 */
#if !defined(CBUF_MIN_LEN)
#define CBUF_MIN_LEN    ((INT16U) 2)
#endif

/*
************************************************************************************************************************************
*                                                            DATA TYPES
************************************************************************************************************************************
*/

/** Circular buffer object with 64-bit precision.
 */
typedef struct cbuf {
    INT16U  len;                                                /**< Buffer length.                                               */
    INT16U  idx;                                                /**< Pointer to the next free data memory location.               */
    FP64   *elmt;                                               /**< Pointer to 64-bit floating point data elements.              */
} CBUF;

/** Circular buffer object with 32-bit precision.
 */
typedef struct cbuf_fp32 {
    INT16U  len;                                                /**< Buffer length.                                               */
    INT16U  idx;                                                /**< Pointer to the next free data memory location.               */
    INT16U  uFull;                                              /**< state of buffer, 1 - full, 0 - not yet full                  */
    FP32   *elmt;                                               /**< Pointer to 32-bit floating point data elements.              */
} CBUF_FP32;

/*
************************************************************************************************************************************
*                                                         GLOBAL VARIABLES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                              MACROS
************************************************************************************************************************************
*/

/** Macro that returns a specific data element from a circular buffer
 *
 *  The memory pointer is decremented by the given element number. An element number of 0 returns the newest value and N-1 returns
 *  the oldest value. Similar with the write function the pointer is decremented modulo N.
 *
 *  @param      bp      Pointer to a circular buffer instance.
 *  @param      n       n-th unit time delayed data element.
 *  @param      y       The content of the specified data element.
 *  24.03.16 type cast wichtig!!!
 */
#if !defined(CBUF_RD)
#define CBUF_RD(bp, n, y) \
    do { \
        INT16S dly; \
        dly = ((INT16S)(bp)->idx) - 1 - n; \
        if (0 > dly) \
        { \
            dly += (bp)->len; \
        } \
        y = *((bp)->elmt + dly); \
    } while (0)
#endif
//        dly = ((INT32S)(bp)->idx) - 1 - n; \


/*
************************************************************************************************************************************
*                                       64-BIT PRECISION CIRCULAR BUFFER FUNCTION PROTOTYPES
************************************************************************************************************************************
*/

/** Configure a 64-bit precision circular buffer 
 *
 *  The function dynamically allocates the required data memory according to the given buffer length and initialize it with zeros.
 *  The pointer is initialized in that way that it points to the first data memory location. In case that the dynamic memory
 *  allocation failed or the given memory size is out of range the corresponding error code is returned. For a successful
 *  implementation of circular buffers at least 2 data memory elements are needed.
 *
 *  @param      p       Pointer to a circular buffer instance.
 *  @param      len     Circular buffer length.
 *  @returns            ERR_EBADLEN if length < 2, ERR_ENOMEM if dynamic memory allocation failed and 0 if successful.
 */
INT16U CBuf_Cfg(CBUF *p, const INT16U len);

/** Memory de-allocation of a 64-bit precision circular buffer
 *
 *  The function frees the dynamically allocated memory of the data elements in a circular buffer. This means primarily that all
 *  data elements are removed from the circular buffer object.
 *
 *  @param      p       Pointer to a circular buffer instance.
 */
void CBuf_Destroy(CBUF *p);

/** Initializing a 64-bit precision circular buffer with a given start value
 *
 *  The function initializes the complete circular buffer with a given start value. In case that the start value is not-a-number or
 *  infinite, the circular buffer will not be initialized and returned the corresponding error number to the calling program.
 *
 *  @param      p       Pointer to a circular buffer instance.
 *  @param      val     64-bit floating point start value.
 *  @returns            ERR_ENAN if start value is NaN, ERR_EOVRFLW if start value is infinite and 0 if successful.
 */
INT16U CBuf_Init(CBUF *p, const FP64 val);

/** Write value into a 64-bit precision circular buffer
 *
 *  In the corresponding C code, the pointer is defined as an integer variable. The variable is incremented modulo N, the variable
 *  successively has the values \f$ 0, 1, \ldots , (N-1), 0, 1, \ldots , (N-1) \f$. A new value is written to the location of the
 *  circular buffer at which the pointer is currently standing. Afterwards the pointer is incremented, it now points to the location
 *  of the circular buffer, were the oldest input value \f$ x_{n-N} \f$ is stored.
 *
 *  @param      p       Pointer to a circular buffer instance.
 *  @param      val     64-bit floating point input value.
 */
void CBuf_Wrt(CBUF *p, const FP64 val);

/** Read a specific data element from a 64-bit precision circular buffer
 *
 *  The memory pointer is decremented by the given element number. An element number of 0 returns the newest value and N-1 returns
 *  the oldest value. Element numbers outside the specified range of \f$ 0 \leq n \leq N - 1 \f$ are automatically aligned to the
 *  limit. Similar with the write function the pointer is decremented modulo N.
 *
 *  @param      p       Pointer to a circular buffer instance.
 *  @param      n       n-th unit time delayed data element.
 *  @returns            the content of the specified 64-bit floating point data element.
 */
FP64 CBuf_Rd(CBUF *p, const INT16U n);

/*
************************************************************************************************************************************
*                                       32-BIT PRECISION CIRCULAR BUFFER FUNCTION PROTOTYPES
************************************************************************************************************************************
*/

/** Configure a 32-bit precision circular buffer
 *
 *  The function dynamically allocates the required data memory according to the given buffer length and initialize it with zeros.
 *  The pointer is initialized in that way that it points to the first data memory location. In case that the dynamic memory
 *  allocation failed or the given memory size is out of range the corresponding error code is returned. For a successful
 *  implementation of circular buffers at least 2 data memory elements are needed.
 *
 *  @param      p       Pointer to a 32-bit floating point circular buffer instance.
 *  @param      len     Circular buffer length.
 *  @returns            ERR_EBADLEN if length < 2, ERR_ENOMEM if dynamic memory allocation failed and 0 if successful.
 */
INT16U CBuf_CfgFP32(CBUF_FP32 *p, const INT16U len);

/** Memory de-allocation of a 32-bit precision circular buffer
 *
 *  The function frees the dynamically allocated memory of the data elements in a circular buffer. This means primarily that all
 *  data elements are removed from the circular buffer object.
 *
 *  @param      p       Pointer to a 32-bit floating point circular buffer instance.
 */
void CBuf_DestroyFP32(CBUF_FP32 *p);

/** Initializing a 32-bit precision circular buffer with a given start value
 *
 *  @param      p       Pointer to a 32-bit floating point circular buffer instance.
 *  @param      val     Start value.
 *  @returns            ERR_ENAN if start value is NaN, ERR_EOVRFLW if start value is infinite and 0 if successful.
 */
INT16U CBuf_InitFP32(CBUF_FP32 *p, const FP32 val);

/** Write value into a 32-bit precision circular buffer
 *
 *  In the corresponding C code, the pointer is defined as an integer variable. The variable is incremented modulo N, the variable
 *  successively has the values \f$ 0, 1, \ldots , (N-1), 0, 1, \ldots , (N-1) \f$. A new value is written to the location of the
 *  circular buffer at which the pointer is currently standing. Afterwards the pointer is incremented, it now points to the location
 *  of the circular buffer, were the oldest input value \f$ x_{n-N} \f$ is stored.
 *
 *  @param      p       Pointer to a 32-bit floating point circular buffer instance.
 *  @param      val     32-bit floating point input value.
 */
void CBuf_WrtFP32(CBUF_FP32 *p, const FP32 val);

/** Read a specific data element from a 32-bit precision circular buffer
 *
 *  The memory pointer is decremented by the given element number. An element number of 0 returns the newest value and N-1 returns
 *  the oldest value. Element numbers outside the specified range of \f$ 0 \leq n \leq N - 1 \f$ are automatically aligned to the
 *  limit. Similar with the write function the pointer is decremented modulo N.
 *
 *  @param      p       Pointer to a 32-bit floating point circular buffer instance.
 *  @param      n       n-th unit time delayed data element.
 *  @returns            the content of the specified 32-bit floating point data element.
 */
FP32 CBuf_RdFP32(CBUF_FP32 *p, const INT16U n);


Uint16 CBufIsFull (const CBUF_FP32 *p);

/*
************************************************************************************************************************************
*                                                 EXTERNAL C LANGUAGE LINKAGE END
************************************************************************************************************************************
*/

#ifdef __cplusplus
}                                                               /* End of 'extern'al C language linkage.                          */
#endif

/*
************************************************************************************************************************************
*                                                            MODULE END
************************************************************************************************************************************
*/

#endif                                                          /* End of CBUF.H header file.                                     */
