/*
************************************************************************************************************************************
*                                              READY-TO-USE EMBEDDED BUILDING BLOCKS
*
*                             (c) Copyright 2015-2016; Thermo Electron (Karlsruhe) GmbH; Karlsruhe-Germany
*
*       All rights reserved. Protected by international copyright laws. 
*
*       TMO source code is an unpublished work and the use of a copyright notice does not imply otherwise. This source code
*       contains confidential, trade secret material of TMO. Any attempt or participation in deciphering, decoding, reverse
*       engineering or in any way altering the source code is strictly prohibited, unless the prior written consent of TMO
*       is obtained.
*
************************************************************************************************************************************
*/

/**
************************************************************************************************************************************
*
*                                                         FILE DESCRIPTION
*
* @file         cmplx.h
* @brief        Complex Math
* @details      The functions in this module provide support for complex numbers and associated math. The implemented algorithms
*               take care to avoid unnecessary intermediate underflows and overflows, allowing the functions to be evaluated over
*               as much of the complex plane as possible.
* @version      V1.00.00
* @author       UwBa
*
************************************************************************************************************************************
* Notes         (1) The note(s) section should be omitted if no notes are given.
************************************************************************************************************************************
*/

/*
************************************************************************************************************************************
*                                                           MODULE START
************************************************************************************************************************************
*/

#ifndef CMPLX_H
#define CMPLX_H                                                     /**< CMPLX.H file encapsulation to avoid multiple parsing.    */

/*
************************************************************************************************************************************
*                                                          INCLUDE FILES
************************************************************************************************************************************
*/

//#include "typedef.h"

/*
************************************************************************************************************************************
*                                                   EXTERNAL C LANGUAGE LINKAGE
*
* Note(s) : (1) C++ compilers must 'extern'ally declare all C function prototypes & variable/object declarations for correct C
*               language linkage.
************************************************************************************************************************************
*/

#ifdef __cplusplus
extern "C" {                                                                        /* See Note #1.                               */
#endif

/*
************************************************************************************************************************************
*                                                            EXTERNALS
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                             DEFINES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                            DATA TYPES
************************************************************************************************************************************
*/

                                                                            /* Two consecutive built-in types as a complex number:*/
typedef       FP32 *CMPLX_PAK_FP32;                                         /**< 32-bit complex floating point number.            */
typedef       FP64 *CMPLX_PAK_FP64;                                         /**< 64-bit complex floating point number.            */
typedef const FP32 *CMPLX_PAK_FP32_CONST;                                   /**< 32-bit complex floating point constant.          */
typedef const FP64 *CMPLX_PAK_FP64_CONST;                                   /**< 64-bit complex floating point constant.          */

                                                                            /* 2N consecutive built-in types as N complex numbers:*/
typedef       FP32 *CMPLX_PAK_FP32_ARRAY;                                   /**< 32-bit complex floating point array.             */
typedef       FP64 *CMPLX_PAK_FP64_ARRAY;                                   /**< 64-bit complex floating point array.             */
typedef const FP32 *CMPLX_PAK_FP32_ARRAY_CONST;                             /**< 32-bit complex floating point array of constants.*/
typedef const FP64 *CMPLX_PAK_FP64_ARRAY_CONST;                             /**< 64-bit complex floating point array of constants.*/

                                                                            /* Pointer to packed complex numbers:                 */
typedef       FP32 *CMPLX_PAK_FP32_PTR;                                     /**< Pointer to 32-bit complex floating point number. */
typedef       FP64 *CMPLX_PAK_FP64_PTR;                                     /**< Pointer to 64-bit complex floating point number. */
typedef const FP32 *CMPLX_PAK_FP32_PTR_CONST;                               /**< Pointer to 32-bit floating point constant.       */
typedef const FP64 *CMPLX_PAK_FP64_PTR_CONST;                               /**< Pointer to 64-bit floating point constant.       */

typedef struct cmplx_fp32 {                                                 /**  32-bit packed complex floating point number.     */
    FP32    dat[2];
} CMPLX_FP32;

typedef struct cmplx_fp64 {                                                 /**  64-bit packed complex floating point number.     */
    FP64    dat[2];
} CMPLX_FP64;

/*
************************************************************************************************************************************
*                                                         GLOBAL VARIABLES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                              MACROS
************************************************************************************************************************************
*/

/** Macro that returns the real part of a complex number z.
 */
#define CMPLX_REAL(z) ((z).dat[0])

/** Macro that returns the imaginary part of a complex number z.
 */
#define CMPLX_IMAG(z) ((z).dat[1])

/** Macro that returns the pointer to a complex number.
 */
#define CMPLX_PTR(zp) ((zp)->dat)

/** Macro that returns the pointer to the real part of a complex number.
 */
#define CMPLX_REAL_PTR(zp) ((zp)->dat[0])

/** Macro that returns the pointer to the imaginary part of a complex number.
 */
#define CMPLX_IMAG_PTR(zp) ((zp)->dat[1])

/** Macro returns 1 if z1 and z2 are equal and 0 otherwise.
 */
#define CMPLX_IS_EQUAL(z1, z2) (((z1).dat[0] == (z2).dat[0]) && ((z1).dat[1] == (z2).dat[1]))

/** Macro uses Cartesian components (x, y) to set the real and imaginary part of the complex number pointed to by zp.
 */
#define CMPLX_SET(zp, x, y) do {(zp)->dat[0] = (x); (zp)->dat[1] = (y);} while(0)

/** Macro that allow to set the real part of the complex number, pointed to by zp, independently.
 */
#define CMPLX_REAL_SET(zp, x) do {(zp)->dat[0] = (x);} while(0)

/** Macro that allow to set the imaginary part of the complex number, pointed to by zp, independently.
 */
#define CMPLX_IMAG_SET(zp, y) do {(zp)->dat[1] = (y);} while(0)

/** Macro that uses Cartesian components (x, y) to set the real and imaginary part of the n-th complex number in a packed complex
 *  array.
 */
#define CMPLX_PAK_SET(zp, n, x, y) do {*((zp) + 2*(n)) = (x); *((zp) + (2*(n) + 1)) = (y);} while(0)

/*
************************************************************************************************************************************
*                                          FUNCTION PROTOTYPES CONSTRUCT COMPLEX NUMBERS
************************************************************************************************************************************
*/

/** Construct complex data from Cartesian coordinates
 *
 *  This function combines two numeric inputs into a complex output, making the first input real and the second imaginary. The
 *  function using the Cartesian components (x, y) to set the real and imaginary part of the complex number \f$ z = x + j\cdot y\f$
 *  with \f$ x, y \in \mathbb{R} \f$.
 *
 *  @param      x   Real part.
 *  @param      y   Imaginary part.
 *  @returns        the constructed complex number.
 */
CMPLX_FP64 Cmplx_Rect(const FP64 x, const FP64 y);

/** Construct complex data from polar coordinates
 *
 *  The function returns the complex number \f$ z = r \cdot e^{(j \cdot \theta)} = r \cdot (cos(\theta) + j \cdot sin(\theta)) \f$
 *  from the polar representation.
 *
 *  @param      r       Radial coordinate.
 *  @param      theta   Angular coordinate.
 *  @returns            the constructed complex number.
 */
CMPLX_FP64 Cmplx_Polar(const FP64 r, const FP64 theta);

/*
************************************************************************************************************************************
*                                        FUNCTION PROTOTYPES PROPERTIES OF COMPLEX NUMBERS
************************************************************************************************************************************
*/

/** Complex phase angle
 *
 *  The function returns the angle \f$  \angle z \f$ of a complex number z in radian. The angle is between \f$ \pm \pi \f$.
 *
 *  @param      z   Complex number.
 *  @returns        the angle in radian.
 */
FP64 Cmplx_Angle(const CMPLX_FP64 z);

/** Complex magnitude
 *
 *  The function calculates the magnitude \f$ |z| = \sqrt {x^2 + y^2} \f$ of a complex number \f$ z = x + j\cdot y\f$ with 
 *  \f$ x, y \in \mathbb{R} \f$. The magnitude is defined as a non negative real number.
 *
 *  @param      z   Complex number.
 *  @returns        the complex modulus (magnitude).
 */
FP64 Cmplx_Abs(const CMPLX_FP64 z);

/** Complex square magnitude
 *
 *  The function calculates the square magnitude \f$ |z|^2 = {x^2 + y^2} \f$ of a complex number \f$ z = x + j\cdot y\f$ with 
 *  \f$ x, y \in \mathbb{R} \f$. The square magnitude is defined as a non negative real number.
 *
 *  @param      z   Complex number.
 *  @returns        the square magnitude.
 */
FP64 Cmplx_AbsSqr(const CMPLX_FP64 z);

/** Natural logarithm of the magnitude of the complex number
 *
 *  The function returns the natural logarithm of the magnitude of the complex number z, log|z|.
 *
 *  @param      z   Complex number.
 *  @returns        the natural logarithm.
 */
FP64 Cmplx_LogAbs(const CMPLX_FP64 z);

/*
************************************************************************************************************************************
*                                         FUNCTION PROTOTYPES COMPLEX ARITHMETIC OPERATORS
************************************************************************************************************************************
*/

/** Negative complex number
 *
 *  The function return the negative of the complex number z, \f$ -z = -(x) + j \cdot (-y) \f$.
 *
 *  @param      z   Complex number.
 *  @returns        the negative complex number -z.
 */
CMPLX_FP64 Cmplx_Neg(const CMPLX_FP64 z);

/** Complex conjugate
 *
 *  The function calculates the complex conjugate \f$ z* = x - j\cdot y\f$ of the complex number z, \f$ z = x + j\cdot y\f$
 *  with \f$ x, y \in \mathbb{R} \f$.
 *
 *  @param      z   Complex number.
 *  @returns        the complex conjugate.
 */
CMPLX_FP64 Cmplx_Conj(const CMPLX_FP64 z);

/** Complex inverse
 *
 *  The function returns the inverse of the complex number z, \f$ z^{-1} = {(x - j\cdot y) \over (x^2 + y^2)} \f$
 *
 *  @param      z   Complex number.
 *  @returns        the inverse of the complex number z.
 */
CMPLX_FP64 Cmplx_Inv(const CMPLX_FP64 z);

/** Additon of two complex numbers
 *
 *  To add complex numbers the function add each element separately:\n\n
 *  \f$ z = z_1 + z_2 = (x_1 + j \cdot y_1) + (x_2 + j \cdot y_2) = (x_1 + x_2) + j \cdot (y_1 + y_2)\f$
 *
 *  @param      z1  First complex summand.
 *  @param      z2  Second complex summand.
 *  @returns        the complex sum.
 */
CMPLX_FP64 Cmplx_Add(const CMPLX_FP64 z1, const CMPLX_FP64 z2);

/** Addition of a complex and a real number
 *
 *  The function returns the sum of the complex number z and the real number x, \f$ z_1 = z + x\f$.
 *
 *  @param      z   Complex summand.
 *  @param      x   Real summand.
 *  @returns        the complex sum.
 */
CMPLX_FP64 Cmplx_AddReal(const CMPLX_FP64 z, const FP64 x);

/** Addition of a complex and a imaginary number
 *
 *  The function returns the sum of the complex number z and the imaginary number y, \f$ z_1 = z + j \cdot y\f$.
 *
 *  @param      z   Complex summand.
 *  @param      y   Imaginary summand.
 *  @returns        the complex sum.
 */
CMPLX_FP64 Cmplx_AddImag(const CMPLX_FP64 z, const FP64 y);

/** Subtraction of two complex numbers
 *
 *  To subtract complex numbers the function subtract each element separately:\n\n
 *  \f$ z = z_1 - z_2 = (x_1 + j \cdot y_1) - (x_2 + j \cdot y_2) = (x_1 - x_2) + j \cdot (y_1 - y_2)\f$
 *
 *  @param      z1  Complex minuend.
 *  @param      z2  Complex subtrahend.
 *  @returns        the complex difference.
 */
CMPLX_FP64 Cmplx_Sub(const CMPLX_FP64 z1, const CMPLX_FP64 z2);

/** Subtraction of a complex and a real number
 *
 *  The function returns the difference of the complex number z and the real number x, \f$ z_1 = z - x\f$.
 *
 *  @param      z   Complex minuend.
 *  @param      x   Real subtrahend.
 *  @returns        the complex difference.
 */
CMPLX_FP64 Cmplx_SubReal(const CMPLX_FP64 z, const FP64 x);

/** Subtraction of a complex and a imaginary number
 *
 *  The function returns the diference of the complex number z and the imaginary number \f$ j \cdot y\f$, 
 *  \f$ z_1 = z - j \cdot y\f$.
 *
 *  @param      z   Complex minuend.
 *  @param      y   Imaginary subtrahend.
 *  @returns        the complex difference.
 */
CMPLX_FP64 Cmplx_SubImag(const CMPLX_FP64 z, const FP64 y);

/** Multiply complex numbers
 *
 *  Each part of the first complex number gets multiplied by each part of the second complex number. With \f$ j^2 = -1\f$ we get:\n\n
 *
 *  \f[ z = a \cdot b = (x_a + j \cdot y_a) \cdot (x_b + j \cdot y_b) = 
 *  (x_a \cdot x_b - y_a \cdot y_b) + j \cdot  (x_a \cdot y_b + y_a \cdot x_b)\f]
 *
 *  @param      a   First complex factor.
 *  @param      b   Second complex factor.
 *  @returns        the complex product.
 */
CMPLX_FP64 Cmplx_Mul(const CMPLX_FP64 a, const CMPLX_FP64 b);

/** Multiply a complex number with a real factor
 *
 *  This function returns the product of the complex number a and the real number x, \f$ z = a \cdot x \f$.
 *
 *  @param      a   Complex number.
 *  @param      x   Real factor.
 *  @returns        the complex product.
 */
CMPLX_FP64 Cmplx_MulReal(const CMPLX_FP64 a, const FP64 x);

/** Multiply a complex number with a imaginary factor
 *
 *  This function returns the product of the complex number a and the imaginary number y, \f$ z = a \cdot (j \cdot y) \f$.
 *
 *  @param      a   Complex number.
 *  @param      y   Imaginary factor.
 *  @returns        the complex product.
 */
CMPLX_FP64 Cmplx_MulImag(const CMPLX_FP64 a, const FP64 y);

/** Divide complex numbers
 *
 *  The conjugate is used to simplify the division operation: \n\n
 *  \f[ z = {a \over b} = {x_a + j \cdot y_a \over x_b + j \cdot y_b} \cdot 
 *  {x_b - j \cdot y_b \over x_b - j \cdot y_b} = 
 *  {(x_a + j \cdot y_a) \cdot (x_b - j \cdot y_b) \over x_b^2 + y_b^2} =
 *  {x_a \cdot x_b + y_a \cdot y_b \over x_b^2 + y_b^2} + j \cdot {y_a \cdot x_b - x_a \cdot y_b \over x_b^2 + y_b^2}\f]
 *
 *  @param      a   Complex numerator.
 *  @param      b   Complex denominator.
 *  @returns        the complex quotient.
 */
CMPLX_FP64 Cmplx_Div(const CMPLX_FP64 a, const CMPLX_FP64 b);

/** Divide a complex number with a real number
 *
 *  The function returns the quotient of the complex number a and the real number x, \f$ z = {a \over x} = 
 *  {x_a \over x} + j \cdot {y_a \over x} \f$.
 *
 *  @param      a   Complex numerator.
 *  @param      x   Real denominator.
 *  @returns        the complex quotient.
 */
CMPLX_FP64 Cmplx_DivReal(const CMPLX_FP64 a, const FP64 x);

/** Divide a complex number with an imaginary number
 *
 *  The function returns the quotient of the complex number a and the imaginary number y, \f$ z = {a \over (j \cdot y)} \f$.
 *
 *  @param      a   Complex numerator.
 *  @param      y   Imaginary denominator.
 *  @returns        the complex quotient.
 */
CMPLX_FP64 Cmplx_DivImag(const CMPLX_FP64 a, const FP64 y);

/*
************************************************************************************************************************************
*                                                   ELEMENTARY COMPLEX FUNCTIONS
************************************************************************************************************************************
*/

/** Square root of a complex number
 *
 *  The function returns the square root of the complex number a, \f$ z = \sqrt {a} \f$. The branch cut is the negative real axis.
 *  The result always lies in the right half of the complex plane. 
 *
 *  @param      a   Complex number.
 *  @returns        the complex square root.
 */
CMPLX_FP64 Cmplx_Sqrt(const CMPLX_FP64 a);

/** Complex square root of a real number
 *
 *  This function returns the complex square root of the real number x, where x may be negative.
 *
 *  @param      x   Real number.
 *  @returns        the complex square root.
 */
CMPLX_FP64 Cmplx_SqrtReal(const FP64 x);

/** Complex exponent
 *
 *  This function returns the complex exponential of the complex number a, \f$ z = e^a \f$. 
 *
 *  @param      a   Complex number.
 *  @returns        the complex exponential.
 */
CMPLX_FP64 Cmplx_Exp(const CMPLX_FP64 a);

/** Complex power
 *
 *  The function returns the complex number z raised to the complex power \f$ z = a^b \f$.
 *
 *  @param      a   Complex number.
 *  @param      b   Complex exponent.
 *  @returns        the complex power.
 */
CMPLX_FP64 Cmplx_Pow(const CMPLX_FP64 a, const CMPLX_FP64 b);

/** Real power
 *
 *  The function returns the complex number z raised to the real power x of a complex number a, \f$ z = a^x \f$.
 *
 *  @param      a   Complex number.
 *  @param      x   Real exponent.
 *  @returns        the complex number.
 */
CMPLX_FP64 Cmplx_PowReal(const CMPLX_FP64 a, const FP64 x);

/** Complex natural logarithm
 *
 *  The function returns the complex natural logarithm (base e) of the complex number a, \f$ z = log(a) \f$. The branch cut is
 *  the negative real axis. 
 *
 *  @param      a   Complex number.
 *  @returns        the natural logarithm.
 */
CMPLX_FP64 Cmplx_Log(const CMPLX_FP64 a);

/** Complex base-10 logarithm
 *
 *  The function returns the complex base-10 logarithm of the complex number a, \f$ z = log10(a) \f$.
 *
 *  @param      a   Complex number.
 *  @returns        the complex base-10 logarithm.
 */
CMPLX_FP64 Cmplx_Log10(const CMPLX_FP64 a);

/** Complex base-b logarithm 
 *
 *  The function returns the complex base-b logarithm of the complex number a. This quantity is computed as the ratio 
 *  \f$ z = {log(a) \over log(b)} \f$.
 *
 *  @param      a   Complex number.
 *  @param      b   Complex base.
 *  @returns        the complex base-b logarithm.
 */
CMPLX_FP64 Cmplx_LogBase(const CMPLX_FP64 a, const CMPLX_FP64 b);

/*
************************************************************************************************************************************
*                                                 COMPLEX TRIGONOMETRIC FUNCTIONS
************************************************************************************************************************************
*/

/** Complex sine
 *
 *  The function returns the complex sine of the complex number a.
 *
 *  @param      a   Complex number.
 *  @returns        the complex sine.
 */
CMPLX_FP64 Cmplx_Sin(const CMPLX_FP64 a);

/** Complex cosine
 *
 *  The function returns the complex cosine of the complex number a.
 *
 *  @param      a   Complex number.
 *  @returns        the complex cosine.
 */
CMPLX_FP64 Cmplx_Cos(const CMPLX_FP64 a);

/** Complex tangent
 *
 *  The function returns the complex tangent of the complex number a.
 *
 *  @param      a   Complex number.
 *  @returns        the complex tangent.
 */
CMPLX_FP64 Cmplx_Tan(const CMPLX_FP64 a);

/** Complex secant
 *
 *  The function returns the complex secant of the complex number a.
 *
 *  @param      a   Complex number.
 *  @returns        the complex secant.
 */
CMPLX_FP64 Cmplx_Sec(const CMPLX_FP64 a);

/** Complex cosecant
 *
 *  The function returns the complex cosecant of the complex number a.
 *
 *  @param      a   Complex number.
 *  @returns        the complex cosecant.
 */
CMPLX_FP64 Cmplx_Csc(const CMPLX_FP64 a);

/** Complex cotangent
 *
 *  The function returns the complex cotangent of the complex number a.
 *
 *  @param      a   Complex number.
 *  @returns        the complex cotangent.
 */
CMPLX_FP64 Cmplx_Cot(const CMPLX_FP64 a);

/*
************************************************************************************************************************************
*                                                 INVERSE COMPLEX TRIGONOMETRIC FUNCTIONS
************************************************************************************************************************************
*/

/** Complex arcsine
 *
 *  The function returns the complex arcsine of the complex number z. The branch cuts are on the real axis, less than -1 and
 *  greater than 1. 
 *
 *  @param      z   Complex number.
 *  @returns        the complex arcsine.
 */
CMPLX_FP64 Cmplx_Asin(const CMPLX_FP64 z);

/** Complex arcsine of a real number
 *
 *  The function returns the complex arcsine of the real number z. For z between -1 and 1, the function returns a real value in the
 *  range \f$ [-{\pi \over 2},{\pi \over 2}] \f$. For z less than -1 the result has a real part of \f$ -{\pi \over 2} \f$ and a
 *  positive imaginary part. For z greater than 1 the result has a real part of \f$ {\pi \over 2} \f$ and a negative imaginary
 *  part.
 *
 *  @param      z   Complex number.
 *  @returns        the complex arcsine.
 */
CMPLX_FP64 Cmplx_AsinReal(const FP64 z);

/** Complex arccosine
 *
 *  The function returns the complex arccosine of the complex number z. The branch cuts are on the real axis, less than -1 and
 *  greater than 1.
 *
 *  @param      z   Complex number.
 *  @returns        the complex arcsine.
 */
CMPLX_FP64 Cmplx_Acos(const CMPLX_FP64 z);

/** Complex arccosine of a real number
 *
 *  The function returns the complex arccosine of the real number z. For z between -1 and 1, the function returns a real value in
 *  the range \f$ [0,\pi] \f$. For z less than -1 the result has a real part of \f$ \pi \f$ and a negative imaginary part. For z
 *  greater than 1 the result is purely imaginary and positive. 
 *
 *  @param      z   Complex number.
 *  @returns        the complex arccosine.
 */
CMPLX_FP64 Cmplx_AcosReal(const FP64 z);

/** Complex arctangent
 *
 *  The function returns the complex arctangent of the complex number z. The branch cuts are on the imaginary axis, below -j and
 *  above j.
 *
 *  @param      z   Complex number.
 *  @returns        the complex arctangent.
 */
CMPLX_FP64 Cmplx_Atan(const CMPLX_FP64 z);

/** Complex arcsecant
 *
 *  The function returns the complex arcsecant of the complex number z, \f$ asec(z) = acos({1 \over z}) \f$. 
 *
 *  @param      z   Complex number.
 *  @returns        the complex arcsecant.
 */
CMPLX_FP64 Cmplx_Asec(const CMPLX_FP64 z);

/** Complex arcsecant of a real number
 *
 *  The function returns the complex arcsecant of the real number z, \f$ asec(z) = acos({1 \over z}) \f$. 
 *
 *  @param      z   Real number.
 *  @returns        the complex arcsecant.
 */
CMPLX_FP64 Cmplx_AsecReal(const FP64 z);

/** Complex arccosecant
 *
 *  The function returns the complex arccosecant of the complex number z, \f$ acsc(z) = asin({1 \over z}) \f$.
 *
 *  @param      z   Complex number.
 *  @returns        the complex arccosecant.
 */
CMPLX_FP64 Cmplx_Acsc(const CMPLX_FP64 z);

/** Complex arccosecant of a real number
 *
 *  The function returns the complex arccosecant of the real number z, \f$ acsc(z) = asin({1 \over z}) \f$.
 *
 *  @param      z   Complex number.
 *  @returns        the complex arccosecant.
 */
CMPLX_FP64 Cmplx_AcscReal(const FP64 z);

/** Complex arccotangent 
 *
 *  The function returns the complex arccotangent of the complex number z, \f$ acot(z) = atan({1 \over z}) \f$.
 *
 *  @param      z   Complex number.
 *  @returns        the complex arccotangent.
 */
CMPLX_FP64 Cmplx_Acot(CMPLX_FP64 z);

/*
************************************************************************************************************************************
*                                                   COMPLEX HYPERBOLIC FUNCTIONS
************************************************************************************************************************************
*/

/** Complex hyperbolic sine
 *
 *  The function returns the complex hyperbolic sine of the complex number z. 
 *
 *  @param      z   Complex number.
 *  @returns        the complex hyperbolic sine.
 */
CMPLX_FP64 Cmplx_Sinh(CMPLX_FP64 z);

/** Complex hyperbolic cosine
 *
 *  The function returns the complex hyperbolic cosine of the complex number z. 
 *
 *  @param      z   Complex number.
 *  @returns        the complex hyperbolic cosine.
 */
CMPLX_FP64 Cmplx_Cosh(CMPLX_FP64 z);

/** Complex hyperbolic tangent
 *
 *  The function returns the complex hyperbolic tangent of the complex number z, \f$ tanh(z) = {sinh(z) \over cosh(z)} \f$. 
 *
 *  @param      z   Complex number.
 *  @returns        the complex hyperbolic tangent.
 */
CMPLX_FP64 Cmplx_Tanh(CMPLX_FP64 z);

/** Complex hyperbolic secant
 *
 *  The function returns the complex hyperbolic secant of the complex number z, \f$ sech(z) = {1 \over cosh(z)} \f$.
 *
 *  @param      z   Complex number.
 *  @returns        the complex hyperbolic secant.
 */
CMPLX_FP64 Cmplx_Sech(CMPLX_FP64 z);

/** Complex hyperbolic cosecant
 *
 *  The function returns the complex hyperbolic cosecant of the complex number z, \f$ csch(z) = {1 \over sinh(z)} \f$.
 *
 *  @param      z   Complex number.
 *  @returns        the complex hyperbolic cosecant.
 */
CMPLX_FP64 Cmplx_Csch(CMPLX_FP64 z);

/** Complex hyperbolic cotangent
 *
 *  The function returns the complex hyperbolic cotangent of the complex number z, \f$ coth(z) = {1 \over tanh(z)} \f$.
 *
 *  @param      z   Complex number.
 *  @returns        the complex hyperbolic cotangent.
 */
CMPLX_FP64 Cmplx_Coth(CMPLX_FP64 z);

/*
************************************************************************************************************************************
*                                               INVERSE COMPLEX HYPERBOLIC FUNCTIONS
************************************************************************************************************************************
*/

/** Complex hyperbolic arcsine
 *
 *  The function returns the complex hyperbolic arcsine of the complex number z. The branch cuts are on the imaginary axis, below 
 *  -j and above j. 
 *
 *  @param      z   Complex number.
 *  @returns        the complex hyperbolic arcsine.
 */
CMPLX_FP64 Cmplx_Asinh(const CMPLX_FP64 z);

/** Complex hyperbolic arccosine
 *
 *  The function returns the complex hyperbolic arccosine of the complex number z. The branch cut is on the real axis, less than 1.
 *  Note that in this case the negative square root \f$ acosh(z)= log(z- \sqrt{z^2-1}) \f$. 
 *
 *  @param      z   Complex number.
 *  @returns        the complex hyperbolic arccosine.
 */
CMPLX_FP64 Cmplx_Acosh(const CMPLX_FP64 z);

/** Complex hyperbolic arccosine of the real number
 *
 *  The function returns the complex hyperbolic arccosine of the real number z.
 *
 *  @param      z   Real number.
 *  @returns        the complex hyperbolic arccosine.
 */
CMPLX_FP64 Cmplx_AcoshReal(const FP64 z);

/** Complex hyperbolic arctangent
 *
 *  The function returns the complex hyperbolic arctangent of the complex number z. The branch cuts are on the real axis, less
 *  than -1 and greater than 1.
 *
 *  @param      z   Complex number.
 *  @returns        the complex hyperbolic arctangent.
 */
CMPLX_FP64 Cmplx_Atanh(const CMPLX_FP64 z);

/** Complex hyperbolic arctangent of the real number
 *
 *  The function returns the complex hyperbolic arctangent of the real number z. 
 *
 *  @param      z   Real number.
 *  @returns        the complex hyperbolic arctangent.
 */
CMPLX_FP64 Cmplx_AtanhReal(const FP64 z);

/** Complex hyperbolic arcsecant
 *
 *  The function returns the complex hyperbolic arcsecant of the complex number z, \f$ asech(z) = acosh({1 \over z}) \f$. 
 *
 *  @param      z   Complex number.
 *  @returns        the complex hyperbolic arcsecant.
 */
CMPLX_FP64 Cmplx_Asech(const CMPLX_FP64 z);

/** Complex hyperbolic arccosecant
 *
 *  The function returns the complex hyperbolic arccosecant of the complex number z, \f$ acsch(z) = asin({1 \over z}) \f$.
 *
 *  @param      z   Complex number.
 *  @returns        the complex hyperbolic arccosecant.
 */
CMPLX_FP64 Cmplx_Acsch(const CMPLX_FP64 z);

/** Complex hyperbolic arccotangent
 * 
 *  The function returns the complex hyperbolic arccotangent of the complex number z, \f$ acoth(z) = atanh({1 \over z}) \f$.  
 *
 *  @param      z   Complex number.
 *  @returns        the complex hyperbolic arccotangent.
 */
CMPLX_FP64 Cmplx_Acoth(const CMPLX_FP64 z);

/*
************************************************************************************************************************************
*                                                 EXTERNAL C LANGUAGE LINKAGE END
************************************************************************************************************************************
*/

#ifdef __cplusplus
}                                                                                   /* End of 'extern'al C language linkage.      */
#endif

/*
************************************************************************************************************************************
*                                                            MODULE END
************************************************************************************************************************************
*/

#endif                                                                              /* End of complex math header file.           */
