/*
************************************************************************************************************************************
*                                              READY-TO-USE EMBEDDED BUILDING BLOCKS
*
*                             (c) Copyright 2015-2016; Thermo Electron (Karlsruhe) GmbH; Karlsruhe-Germany
*
*       All rights reserved. Protected by international copyright laws. 
*
*       TMO source code is an unpublished work and the use of a copyright notice does not imply otherwise. This source code
*       contains confidential, trade secret material of TMO. Any attempt or participation in deciphering, decoding, reverse
*       engineering or in any way altering the source code is strictly prohibited, unless the prior written consent of TMO
*       is obtained.
*
************************************************************************************************************************************
*/

/**
************************************************************************************************************************************
*
*                                                         FILE DESCRIPTION
*
* @file         dft.h
* @brief        Discrete Fourier Transform
* @details      @todo
* @version      V1.00.00
* @author       UwBa
*
************************************************************************************************************************************
* Notes         (1) The note(s) section should be omitted if no notes are given.
************************************************************************************************************************************
*/

/*
************************************************************************************************************************************
*                                                           MODULE START
************************************************************************************************************************************
*/

#ifndef DFT_H
#define DFT_H                                                   /**< DFT.H file encapsulation to avoid multiple parsing.          */

/*
************************************************************************************************************************************
*                                                          INCLUDE FILES
************************************************************************************************************************************
*/

//#include "typedef.h"
#include "cbuf.h"

/*
************************************************************************************************************************************
*                                                   EXTERNAL C LANGUAGE LINKAGE
*
* Note(s) : (1) C++ compilers must 'extern'ally declare all C function prototypes & variable/object declarations for correct C
*               language linkage.
************************************************************************************************************************************
*/

#ifdef __cplusplus
extern "C" {                                                    /* See Note #1.                                                   */
#endif

/*
************************************************************************************************************************************
*                                                            EXTERNALS
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                             DEFINES
************************************************************************************************************************************
*/

#if !defined(DFT_GOERTZEL_MIN_NBR_SMP)
#define DFT_GOERTZEL_MIN_NBR_SMP    ((INT16U)3)                 /**< Minimum number of samples for Goertzel DFT computation       */
#endif

/*
************************************************************************************************************************************
*                                                            DATA TYPES
************************************************************************************************************************************
*/

/** Goertzel 64-bit precision single point DFT calculation
 */
typedef struct dft_goertzel_sng_pnt {
    FP64        CycleTmISR;                                     /**< Cycle time interrupt service routine [s].                    */
    FP64        SmpTm;                                          /**< Sampling time [s]                                            */
    FP64        TrgFrq;                                         /**< Target frequency [Hz]                                        */
    INT16U      NbrSmp;                                         /**< Number of samples [1]                                        */
    INT16U      DecFactor;                                      /**< Decimation factor [1]                                        */
    INT16U      SngFrqPnt;                                      /**< Single frequency point [1]                                   */
    FP64        Coeff[3];                                       /**< Goertzel coefficients [1]                                    */
    FP64        ScFactor;                                       /**< Scaling factor [1]                                           */
    FP64        SigMag;                                         /**< Signal magnitude (unit dpends on application)                */
    FP64        SigAngle;                                       /**< Signal angle [rad]                                           */
    CBUF       *PtrCBuf;                                        /**< Pointer to circular buffer.                                  */
} DFT_GOERTZEL_SNG_PNT;

/** Goertzel 32-bit precision single point DFT calculation
 */
typedef struct dft_goertzel_sng_pnt_fp32 {
    FP32        CycleTmISR;                                     /**< Cycle time interrupt service routine [s].                    */
    FP32        SmpTm;                                          /**< Sampling time [s]                                            */
    FP32        TrgFrq;                                         /**< Target frequency [Hz]                                        */
    INT16U      NbrSmp;                                         /**< Number of samples [1]                                        */
    INT16U      DecFactor;                                      /**< Decimation factor [1]                                        */
    INT16U      SngFrqPnt;                                      /**< Single frequency point [1]                                   */
    FP32        Coeff[3];                                       /**< Goertzel coefficients [1]                                    */
    FP32        ScFactor;                                       /**< Scaling factor [1]   */
    FP32        ScFac_1;
    FP32        ScKfrq;                                        
    FP32        SigMag;                                         /**< Signal magnitude (unit dpends on application)                */
    FP32        SigAngle;                                       /**< Signal angle [rad]                                           */
    CBUF_FP32  *PtrCBuf;                                        /**< Pointer to circular buffer.                                  */
} DFT_GOERTZEL_SNG_PNT_FP32;

/*
************************************************************************************************************************************
*                                                         GLOBAL VARIABLES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                              MACROS
************************************************************************************************************************************
*/

/** Macro that returns the magnitude of the single frequency point Goertzel DFT calculation.
 */
#if !defined(DFT_GOERTZEL_GET_MAG)
#define DFT_GOERTZEL_GET_MAG(sp) ((sp)->SigMag)
#endif

/** Macro that returns the phase angle, in radians, of the single frequency point Goertzel DFT calculation.
 */
#if !defined(DFT_GOERTZEL_GET_ANGLE)
#define DFT_GOERTZEL_GET_ANGLE(sp) ((sp)->SigAngle)
#endif

/** Macro that returns the required sample time, in seconds, of the single frequency point Goertzel DFT calculation.
 */
#if !defined(DFT_GOERTZEL_GET_SMP_TM)
#define DFT_GOERTZEL_GET_SMP_TM(sp) ((sp)->SmpTm)
#endif

/** Macro that returns the number of samples of the single frequency point Goertzel DFT calculation.
 */
#if !defined(DFT_GOERTZEL_GET_NBR_SMP)
#define DFT_GOERTZEL_GET_NBR_SMP(sp) ((sp)->NbrSmp)
#endif

/** Macro that returns the required decimation factor of the single frequency point Goertzel DFT calculation.
 */
#if !defined(DFT_GOERTZEL_GET_DEC_FACTOR)
#define DFT_GOERTZEL_GET_DEC_FACTOR(sp) ((sp)->DecFactor)
#endif

#if !defined(DFT_GOERTZEL_GET_ISR_CYCLE_TM)
#define DFT_GOERTZEL_GET_ISR_CYCLE_TM(sp) ((sp)->CycleTmISR)
#endif

/*
************************************************************************************************************************************
*                                          64-BIT PRECISION GOERTZEL FUNCTION PROTOTYPES
************************************************************************************************************************************
*/

/** Setup 64-bit precision single point Goertzel DFT algorithm
 *
 *  The function presets the circular buffer including the buffer read function. Normally, the setup function is called only once
 *  at startup. In case of an invalid circular buffer instance or read function, the function returns the corresponding error code.
 *
 *  @param      p               Pointer to a 64-bit precision single point Goertzel instance
 *  @param      fnct            Pointer to circular buffer read function
 *  @param      ptr_cbuf        Pointer to circular buffer instance
 *  @returns                    ERR_EINVAL for invalid buffer read functions, ERR_EFAULT for invalid pointer to buffer instances
 *                              otherwise 0.
 */
INT32U DFT_GoertzelSngPntSetup(DFT_GOERTZEL_SNG_PNT *p, CBUF *ptr_cbuf);

/** Configurate 64-bit precision single point Goertzel algorithm
 *
 *  Based on the given target frequency, minimum possible sample time and maximum number of samples the function calculates the
 *  resulting number of samples, decimation factor, sample time, coefficients of the second order Goertzel algorithm and the scaling
 *  factor.
 *
 *  @param      p               Pointer to a 64-bit precision single point Goertzel instance
 *  @param      trg_frq         Single point target frequency [Hz]
 *  @param      min_smp_tm      Smallest permissible sampling time [s]
 *  @param      max_nbr_smp     Maximum number of samples [1]
 *  @returns                    ERR_FAILURE in case that the ratio of sample rate and frequency of intrerest does not met the
 *                              required minimum number of samples otherwise 0.
 */
INT32U DFT_GoertzelSngPntCfg(DFT_GOERTZEL_SNG_PNT *p, const FP64 trg_frq, const FP64 min_smp_tm, const INT32U max_nbr_smp);

/** 64-bit precision Goertzel algorithm which computes a single point in the discrete Fourier transform
 *
 *  The function computes the single point discrete Fourier transform (DFT) of a real input sequence using the second-order Goertzel
 *  algorithm.
 *
 *  @param      p           Pointer to a 64-bit precision single point Goertzel instance
 */
void DFT_GoertzelSngPnt(DFT_GOERTZEL_SNG_PNT *p);

/*
************************************************************************************************************************************
*                                          32-BIT PRECISION GOERTZEL FUNCTION PROTOTYPES
************************************************************************************************************************************
*/

/** Setup 32-bit precision single point Goertzel DFT algorithm
 *
 *  The function presets the circular buffer including the buffer read function. Normally, the setup function is called only once
 *  at startup. In case of an invalid circular buffer instance or read function, the function returns the corresponding error code.
 *
 *  @param      p               Pointer to a FP32 single point Goertzel instance
 *  @param      ptr_cbuf        Pointer to circular buffer instance
 *  @returns                    ERR_EINVAL for invalid buffer read functions, ERR_EFAULT for invalid pointer to buffer instances
 *                              otherwise 0.
 */
INT16U DFT_GoertzelSngPntSetupFP32(DFT_GOERTZEL_SNG_PNT_FP32 *p, CBUF_FP32 *ptr_cbuf);

/** Configurate 32-bit precision single point Goertzel algorithm
 *
 *  Based on the given target frequency, minimum possible sample time and maximum number of samples the function calculates the
 *  resulting number of samples, decimation factor, sample time, coefficients of the second order Goertzel algorithm and the scaling
 *  factor.
 *
 *  @param      p               Pointer to a FP32 single point Goertzel instance
 *  @param      trg_frq         Single point target frequency [Hz]
 *  @param      min_smp_tm      Smallest permissible sampling time [s]
 *  @param      max_nbr_smp     Maximum number of samples [1]
 *  @returns                    ERR_FAILURE in case that the ratio of sample rate and frequency of intrerest does not met the
 *                              required minimum number of samples otherwise 0.
 */
INT16U DFT_GoertzelSngPntCfgFP32(DFT_GOERTZEL_SNG_PNT_FP32 *p, const FP32 trg_frq, const FP32 min_smp_tm, const INT16U max_nbr_smp);

/** 32-bit precision Goertzel algorithm which computes a single point in the discrete Fourier transform
 *
 *  The function computes the single point discrete Fourier transform (DFT) of a real input sequence using the second-order Goertzel
 *  algorithm.
 *
 *  @param      p           Pointer to a FP32 single point Goertzel instance
 */
void DFT_GoertzelSngPntFP32(DFT_GOERTZEL_SNG_PNT_FP32 *p);

/*
************************************************************************************************************************************
*                                                 EXTERNAL C LANGUAGE LINKAGE END
************************************************************************************************************************************
*/

#ifdef __cplusplus
}                                                               /* End of 'extern'al C language linkage.                          */
#endif

/*
************************************************************************************************************************************
*                                                            MODULE END
************************************************************************************************************************************
*/

#endif                                                          /* End of DFT.H header file.                                      */
