/*
************************************************************************************************************************************
*                                              READY-TO-USE EMBEDDED BUILDING BLOCKS
*
*                             (c) Copyright 2015-2016; Thermo Electron (Karlsruhe) GmbH; Karlsruhe-Germany
*
*       All rights reserved. Protected by international copyright laws. 
*
*       TMO source code is an unpublished work and the use of a copyright notice does not imply otherwise. This source code
*       contains confidential, trade secret material of TMO. Any attempt or participation in deciphering, decoding, reverse
*       engineering or in any way altering the source code is strictly prohibited, unless the prior written consent of TMO
*       is obtained.
*
************************************************************************************************************************************
*/

/**
************************************************************************************************************************************
*
*                                                         FILE DESCRIPTION
*
* @file         err.h
* @brief        Exception / Error Handler
* @details      This module contains functions to report and handle errors. By examining the status information returned by every
*               function of the ready-to-use embedded building blocks the user can determine whether it succeeded or failed, and if
*               it failed the user can find out what the precise cause of failure was.
* @version      V1.00.00
* @author       UwBa
*
************************************************************************************************************************************
* Notes         (1) The note(s) section should be omitted if no notes are given.
************************************************************************************************************************************
*/

/*
************************************************************************************************************************************
*                                                           MODULE START
************************************************************************************************************************************
*/

#ifndef ERR_H
#define ERR_H                                                   /**< ERR.H file encapsulation to avoid multiple parsing.          */

/*
************************************************************************************************************************************
*                                                          INCLUDE FILES
************************************************************************************************************************************
*/

//#include <stdio.h>                                              /* Because of macros __FILE__ and __LINE__                        */
//#include <stdlib.h>                                             /* Because of memory deallocation free()                          */

//#include "typedef.h"                                            /* Because of target platform independend data types              */
#include "queue.h"                                              /* Because of queue based error reporting                         */

/*
************************************************************************************************************************************
*                                                   EXTERNAL C LANGUAGE LINKAGE
*
* Note(s) : (1) C++ compilers must 'extern'ally declare all C function prototypes & variable/object declarations for correct C
*               language linkage.
************************************************************************************************************************************
*/

#ifdef __cplusplus
extern "C" {                                                    /* See Note #1.                                                   */
#endif

/*
************************************************************************************************************************************
*                                                            EXTERNALS
************************************************************************************************************************************
*/

#ifdef  ERR_MODULE
    QUEUE  Err_Hdlr_Queue = {0, NULL, NULL, free};
#else
    extern QUEUE  Err_Hdlr_Queue;                               /**< Error handler queue                                          */
#endif

/*
************************************************************************************************************************************
*                                                              ERRNOs
************************************************************************************************************************************
*/

enum {
    ERR_SUCCESS  =  0,                                          /**< Successful                                                   */
    ERR_FAILURE  =  1,                                          /**< Common failure                                               */
    ERR_CONTINUE =  2,                                          /**< Iteration has not converged                                  */
    ERR_EDOM     =  3,                                          /**< Input domain error, e.g sqrt(-1)                             */
    ERR_ERANGE   =  4,                                          /**< Output range error, e.g. exp(1e100)                          */
    ERR_EFAULT   =  5,                                          /**< Invalid pointer                                              */
    ERR_EINVAL   =  6,                                          /**< Invalid argument supplied by user                            */
    ERR_EFAILED  =  7,                                          /**< Generic failure                                              */
    ERR_EFACTOR  =  8,                                          /**< Factorization failed                                         */
    ERR_ESANITY  =  9,                                          /**< Sanity check failed - shouldn't happen                       */
    ERR_ENOMEM   = 10,                                          /**< Dynamic memory allocation malloc failed                      */
    ERR_EBADFUNC = 11,                                          /**< Problem with user-supplied function                          */
    ERR_ERUNAWAY = 12,                                          /**< Iterative process is out of control                          */
    ERR_EMAXITER = 13,                                          /**< Exceeded max number of iterations                            */
    ERR_EZERODIV = 14,                                          /**< Tried to divide by zero                                      */
    ERR_EBADTOL  = 15,                                          /**< User specified an invalid tolerance                          */
    ERR_ETOL     = 16,                                          /**< Failed to reach the specified tolerance                      */
    ERR_EUNDRFLW = 17,                                          /**< Underflow                                                    */
    ERR_EOVRFLW  = 18,                                          /**< Overflow                                                     */
    ERR_ELOSS    = 19,                                          /**< Loss of accuracy                                             */
    ERR_EROUND   = 20,                                          /**< Failed because of roundoff error                             */
    ERR_EBADLEN  = 21,                                          /**< Matrix, vector lengths are not conformant                    */
    ERR_ENOTSQR  = 22,                                          /**< Matrix not square                                            */
    ERR_ESING    = 23,                                          /**< Apparent singularity detected                                */
    ERR_EDIVERGE = 24,                                          /**< Integral or series is divergent                              */
    ERR_EUNSUP   = 25,                                          /**< Requested feature is not supported by the hardware           */
    ERR_EUNIMPL  = 26,                                          /**< Requested feature not (yet) implemented                      */
    ERR_ECACHE   = 27,                                          /**< Cache limit exceeded                                         */
    ERR_ETABLE   = 28,                                          /**< Table limit exceeded                                         */
    ERR_ENOPROG  = 29,                                          /**< Iteration is not making progress towards solution            */
    ERR_ENOPROGJ = 30,                                          /**< Jacobian evaluations are not improving the solution          */
    ERR_ETOLF    = 31,                                          /**< Cannot reach the specified tolerance in function value       */
    ERR_ETOLX    = 32,                                          /**< Cannot reach the specified tolerance in control variable X   */
    ERR_ETOLG    = 33,                                          /**< Cannot reach the specified tolerance in gradient             */
    ERR_EOF      = 34,                                          /**< End of file                                                  */
    ERR_ENAN     = 35                                           /**< Not a number                                                 */
} ;

/*
************************************************************************************************************************************
*                                                            CONSTANTS
************************************************************************************************************************************
*/

#if !defined(ERR_HDLR_QUEUE_SIZE_MAX)
#define ERR_HDLR_QUEUE_SIZE_MAX 10                              /**< Maximum size (number of error records) of error handler queue*/
#endif

/*
************************************************************************************************************************************
*                                                            DATA TYPES
************************************************************************************************************************************
*/

/** Error handler record
 */
typedef struct err_hdlr_rec {
    const CHAR     *reason;                                     /**< Reason for the error.                                        */
    const CHAR     *file;                                       /**< File in which the error occure.                              */
          INT32U    line;                                       /**< Line number in the file in which the error occure.           */
          INT32U    no;                                         /**< Error number.                                                */
} ERR_HDLR_REC;

/*
************************************************************************************************************************************
*                                                         GLOBAL VARIABLES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                              MACROS
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                ERROR HANDLER FUNCTION PROTOTYPES
************************************************************************************************************************************
*/

/** Error handler
 *
 *  The default behavior of the error handler is to record a short error message and log it in the error handler queue. It is the
 *  responsibility of the user to check the return values of routines, handle the return values and the logged error messages in
 *  the error handler queue.
 *
 *  The error handler will be passed four arguments which specify the reason for the error (a string), the name of the source file
 *  in which it occurred (also a string), the line number in that file (an integer) and the error number (an integer). The source
 *  file and line number are set at compile time using the __FILE__ and __LINE__ directives in the preprocessor.
 *
 *  @param      reason      Reason for the error
 *  @param      file        File in which the error occure
 *  @param      line        Line number in the file in which the error occure
 *  @param      no          Error number
 *  @returns                ERR_SUCCESS if error record is successfully logged and ERR_ENOMEM if dynamic memory allocation failed
 */
//INT32S Err_Hdlr(const CHAR *reason, const CHAR *file, INT32U line, INT32U no);

/*
************************************************************************************************************************************
*                                         INFINITIES AND NOT-A-NUMBER FUNCTION PROTOTYPES
************************************************************************************************************************************
*/

/** Positive infinity
 *
 *  The function returns the IEEE representation of positive infinity, \f$ +\infty \f$.
 *
 *  @returns    the positive infinity.
 */
FP64 Err_PosInf(void);

/** Negative infinity
 *
 *  The function returns the IEEE representation of negative infinity, \f$ -\infty \f$.
 *
 *  @returns    the negative infinity.
 */
FP64 Err_NegInf(void);

/** Not-A-Number
 *
 *  The function returns the IEEE representation of the Not-a-Number symbol, NaN.
 *
 *  @returns    NaN.
 */
FP64 Err_NaN(void);

/** True for Not-a-Number
 *
 *  The function returns 1 if x is not-a-number otherwise 0.
 *
 *  @param      x   Real number
 *  @returns        1 if x is not-a-number otherwise 0.
 */
INT32S Err_IsNaN(const FP64 x);

/** True for infinite numbers
 *
 *  The function returns 1 if x \f$ +\infty \f$, -1 for \f$ -\infty \f$ otherwise 0.
 *
 *  @param      x   Real number
 *  @returns        1 if x \f$ +\infty \f$, -1 for \f$ -\infty \f$ otherwise 0.
 */
INT32S Err_IsInf(const FP64 x);

/** True for finite numbers
 *
 *  The function returns 1 if x is finite otherwise 0.
 *
 *  @param      x   Real number
 *  @returns        1 if x is finite otherwise 0.
 */
INT32S Err_Finite(const FP64 x);

/*
************************************************************************************************************************************
*                                                       ERROR HANDLER ERRORS
************************************************************************************************************************************
*/

/** Macro ERR_HDLR call the error handler, record the error source and return the error code to the calling program
 */
#ifdef RAUS

    #if !defined(ERR_HDLR)
    #define ERR_HDLR(reason, no) \
           do { \
           Err_Hdlr(reason, __FILE__, __LINE__, no) ; \
           return no ; \
           } while (0)
    #endif

    /** Macro ERR_HDLR_VAL call the error handler, record the error source and return the given value to the calling program
     */
    #if !defined(ERR_HDLR_VAL)
    #define ERR_HDLR_VAL(reason, no, value) \
           do { \
           Err_Hdlr(reason, __FILE__, __LINE__, no) ; \
           return value ; \
           } while (0)
    #endif

    /** Macro ERR_HDLR_VOID call the error handler, recording the error source and then return to calling program (for void functions
     *  which still need to generate an error and terminate program execution)
     */
    #if !defined(ERR_HDLR_VOID)
    #define ERR_HDLR_VOID(reason, no) \
           do { \
           Err_Hdlr(reason, __FILE__, __LINE__, no) ; \
           return ; \
           } while (0)
    #endif

    /** Macro ERR_HDLR_REC_ONLY call the error handler and record the error source
     */
    #if !defined(ERR_HDLR_REC_ONLY)
    #define ERR_HDLR_REC_ONLY(reason, no) \
           do { \
           Err_Hdlr(reason, __FILE__, __LINE__, no) ; \
           } while (0)
    #endif

    /** Macro ERR_HDLR_CLEAN_UP() executes error handler memory deallocation including queue elements and error data records and 
     *  initializes the error handler queue for the next use
     */
    #if !defined(ERR_HDLR_CLEAN_UP)
    #define ERR_HDLR_CLEAN_UP() \
        do { \
        Queue_Destroy(&Err_Hdlr_Queue); \
        Queue_Init(&Err_Hdlr_Queue, (void *)free); \
        } while (0)
    #endif

#endif

/*
************************************************************************************************************************************
*                                                 INFINITIES AND NOT-A-NUMBER ERRORS
************************************************************************************************************************************
*/

/** Positive infinity
 *
 *  This macro contains the IEEE representation of positive infinity, \f$ +\infty \f$. It is computed from the expression 
 *  \f$ {+1.0 \over 0.0} \f$. 
 */
#if !defined(ERR_POS_INF)
#define ERR_POS_INF (Err_PosInf())
#endif

/** Negative infinity
 *
 *  This macro contains the IEEE representation of negative infinity, \f$ -\infty \f$. It is computed from the expression 
 *  \f$ {-1.0 \over 0.0} \f$. 
 */
#if !defined(ERR_NEG_INF)
#define ERR_NEG_INF (Err_NegInf())
#endif

/** Not-A-Number representation
 *
 *  This macro contains the IEEE representation of the Not-a-Number symbol, NaN. It is computed from the ratio 
 *  \f$ {0.0 \over 0.0} \f$.
 */
#if !defined(ERR_NAN)
#define ERR_NAN     (Err_NaN())
#endif

/*
************************************************************************************************************************************
*                                                 EXTERNAL C LANGUAGE LINKAGE END
************************************************************************************************************************************
*/

#ifdef __cplusplus
}                                                               /* End of 'extern'al C language linkage.                          */
#endif

/*
************************************************************************************************************************************
*                                                            MODULE END
************************************************************************************************************************************
*/

#endif                                                          /* End of ERR.H header file.                                      */
