/**
 * @file    fft.h
 * @brief   Fast Fourier transformation module
 *
 *          The module contains C-functions for general purpose signal-processing in the frequency domain. These routines are
 *          typically used in computationally intensive real-time applications where optimal execution speed is critical. The
 *          module contains the following routines:
 *
 *              - Radix 2 forward FFT with complex input
 *              - Bit-reverse the FFT output
 *              - Generate the FFT coefficient table
 *
 * @par     Copyright:
 *
 *          Thermo Electron (Karlsruhe) GmbH, Dieselstrasse 4, 76227 Karlsruhe, Germany
 *
 *          (c) Copyright 2009 - 2010, Thermo Fisher Scientific
 *
 *          All rights reserved. Thermo Fisher Scientific source code is an unpublished work and the use of a copyright notice does
 *          not imply otherwise. This source code contains confidential, trade secret material of Thermo Fisher Scientific. Any
 *          attempt or participation in deciphering, decoding, reverse engineering or in any way altering the source code is
 *          strictly prohibited, unless the prior written consent of Thermo Fisher Scientific is obtained.
 *
 * @par     Creation Date:
 *
 *          05-Dec-2009
 *
 * @par     Last Modification:
 *
 *          18-Jul-2012
 *
 * @par     Project:
 *
 *          K1
 *
 * @par     Editor:
 *
 *          Microsoft Visual C++ 8.0
 *
 * @par     Compiler:
 *
 *          Microsoft Visual C++ 8.0
 *
 * @author  Uwe Bauer
 *
 * @par     Revision History:
 */

#ifndef FFT_H
#define FFT_H

/*
 **********************************************************************************************************************************
 *                                                           INCLUDES                                                             *
 **********************************************************************************************************************************
 */
//#include <math.h>
//#include "typedefs.h"
//#include "F28335Inc.h"
//#include "C28x_FPU_FastRTS.h"

/**********************************************************************************************************************************
 *                                                   GLOBAL FUNCTION PROTOTYPES                                                   *
 **********************************************************************************************************************************/

/**
 * Generate the FFT coefficient table
 *
 * Generate real and imaginary twiddle table of size n complex numbers.
 *
 * @param   w   Output sequence of FFT coefficients w has n complex numbers. The real and imaginary values are interleaved in
 *              memory.
 * @param   n   FFT size which is a power of 2 and > 4
 */
void FftGenTwfRx2(FP32 *w, INT32U n);

/**
 * Bit-reverse the FFT output
 *
 * @param   x   FFT output sequence x has n complex numbers in bit-reversed order.
 * @param   n   FFT size which is a power of 2 and > 4.
 */
void FftBitRev(FP32 *x, INT32U n);

/*
 * The follwoing C code is used to perform the final scaling of the IFFT
 *
 * Divide each element of x by n
 *
 * @param   x   IFFT output sequence x has n complex numbers in bit-reversed order.
 * @param   n   IFFT size which is a power of 2 and > 4.
 */
void FftInvCmplxRx2Scale(FP32 *x, INT32U n);

/**
 * Radix 2 forward FFT with complex input
 *
 * This routine is used to compute the complex, radix-2, fast fourier transform of a complex sequence of size n, and a power of 2.
 * The routine requires normal order input and normal order coefficents (twiddle factors) in a special sequence and produces
 * results that are in bit-reversed order.
 *
 * @param   n       FFT size which is a power of 2 and > 4
 * @param   x       Input and output sequences x has n complex numbers. The real and imaginary values are interleaved in memory.
 *                  The input is in normal order and output is in bit-reversed order.
 * @param   w       FFT coefficients w has n complex numbers. The real and imaginary values are interleaved in memory.
 * @param   nMin    FFT of size n can be divided into several steps. The number of steps must be a power of 2 and allow as much
 *                  data reuse as possible.
 */
void FftCmplxRx2(INT32U n, FP32 *x, FP32 *w, INT32U nMin);

/**
 * Inverse complex radix 2 decimation in frequency FFT
 *
 * @param   n       FFT size which is a power of 2 and > 4
 * @param   x       Input and output sequences x has n complex numbers. The real and imaginary values are interleaved in memory.
 *                  The input is in normal order and output is in bit-reversed order.
 * @param   w       FFT coefficients w has n complex numbers. The real and imaginary values are interleaved in memory.
 * @param   nMin    FFT of size n can be divided into several steps. The number of steps must be a power of 2 and allow as much
 *                  data reuse as possible.
 */
void FftInvCmplxRx2(INT32U n, FP32 * x, FP32 * w, INT32U nMin);

/**
 * Covert real input sequences into complex sequences
 *
 * The function converts an input sequence of n real numbers into a complex sequence with n complex numbers. The imaginary part of
 * the complex sequence is set to zero.
 *
 * @param   r       Sequence r has n real numbers.
 * @param   c       Sequence c has n complex numbers.
 * @param   n       Sequence size.
 */
void FftReal2Cmplx(FP32 *r, FP32 *c, INT32U n);

/**
 * Converter between rectangular and polar coordinates
 *
 * The function converts the complex FFT output sequence into polar coordinates. The magnitude and angle are interleaved in memory.
 * The polar coordinates pairs start with the magnitude value followed by the angle. The angle is calculated in radiant. The first
 * pair of values corresponds with the DC-offset, followed by the first, second and so on higher harmonics. To reduce the
 * calculation time the number of frequency components who should convert into polar coordinates can be limited. The number of
 * components should be smaller n/2-1 and is automatically limited. Because of the symmetry of the FFT output the second half does
 * not bring further more information.
 *
 * @param   x       Complex FFT output sequence and converted rectangular sequence.
 * @param   n       FFT size which is a power of 2 and > 4.
 * @param   n1      Number of frequency components who should convert into polar coordinates.
 */
void FftRect2Pol(FP32 *x, INT32U n, INT32U n1);

/**
 * Get magnitude of a given harmonic
 *
 * @param   x       Fourier series with DC-offset, magintude and phase coefficients
 * @param   n       Harmonic of interest (0 = DC-offset, 1 = first harmonic, ...)
 * @param   prd     Number of stored periods in the complex input sequence
 * @returns         the magnitude of the harmonic.
 */
FP32 FftGetMag(FP32 *x, INT32U n, INT32U prd);

/**
 * Get phase of a given harmonic
 *
 * @param   x       Fourier series with DC-offset, magintude and phase coefficients
 * @param   n       Harmonic of interest (1 = first harmonic, 2 = second harmonic, ...)
 * @param   prd     Number of stored periods in the complex input sequence
 * @returns         the phase of the harmonic.
 */
FP32 FftGetAng(FP32 *x, INT32U n, INT32U prd);

#endif  /* FFT_H    */
