/**
 * @file    fosc.h
 * @brief   Fast Oscillation module
 *
 * @par     Copyright:
 *
 *          Thermo Electron (Karlsruhe) GmbH, Dieselstrasse 4, 76227 Karlsruhe, Germany
 *
 *          (c) Copyright 2009 - 2010, Thermo Fisher Scientific
 *
 *          All rights reserved. Thermo Fisher Scientific source code is an unpublished work and the use of a copyright notice does
 *          not imply otherwise. This source code contains confidential, trade secret material of Thermo Fisher Scientific. Any
 *          attempt or participation in deciphering, decoding, reverse engineering or in any way altering the source code is
 *          strictly prohibited, unless the prior written consent of Thermo Fisher Scientific is obtained.
 *
 * @par     Creation Date:
 *
 *          05-Dec-2009
 *
 * @par     Last Modification:
 *
 *          01-Apr-2010
 *
 * @par     Project:
 *
 *          MARS III
 *
 * @par     Editor:
 *
 *          Microsoft Visual C++ 6.0
 *
 * @par     Compiler:
 *
 *          Microsoft Visual C++ 6.0
 *
 * @author  Uwe Bauer
 *
 * @par     Revision History:
 *
 *          Automatic angular drift compensation based on simple P-controller algorithm, FFT DC offset information and rough
 *          viscosity estimation implemented.
 */

#ifndef FOSC_H
#define FOSC_H

/*
 **********************************************************************************************************************************
 *                                                           INCLUDES                                                             *
 **********************************************************************************************************************************
 */

//#include "typedefs.h"
#include "F28335Inc.h"
#include "buf.h"
#include "opt.h"
#include "filt.h"

/* Begin UwBa   */
#include "cbuf.h"                                               /* Because of circular buffers.                                   */
#include "dft.h"                                                /* Because of single point Goertzel DFT calculation.              */
#include "src.h"                                                /* Because of torque and angle set point signal generation.       */
/* End UwBa */

/*
 **********************************************************************************************************************************
 *                                                           EXTERNS                                                              *
 **********************************************************************************************************************************
 */

#ifdef  FOSC_MODULE
#define FOSC_EXT    
#else
#define FOSC_EXT    extern
#endif

/*
 **********************************************************************************************************************************
 *                                                           DEFINES                                                              *
 **********************************************************************************************************************************
 */

/* Begin UwBa   */
#if !defined(FOSC_MIN_SMP_TM)
    #ifdef EPWMFRQ_25KHZ
        #define FOSC_MIN_SMP_TM     ((FP32) 0.000250)                 /**< Minimum possible sample time [s].                                */
    #else
        #define FOSC_MIN_SMP_TM     ((FP32) 0.000300)                 /**< Minimum possible sample time [s].                                */
    #endif
#endif

#if !defined(FOSC_MAX_NBR_SMP)
#define FOSC_MAX_NBR_SMP    ((INT32U) 256)                  /**< Maximum number of samples [1] for DFT calculation.               */
#endif
/* End UwBa     */

#define FOSC_PI              ((float32) 3.14159265358979)      /**< pi                                                               */
#define FOSC_2PI             ((float32) 6.28318530717959)      /**< 2 * pi                                                           */

// length in RAM version 128, because of memory reasons, in FLASH 256
//#define FOSC_FFT_MAX_SIZE    ((Uint32) 128)                 /**< Maximum FFT size which is a power of 2 and > 4                   */
#define FOSC_FFT_MAX_SIZE    ((Uint32) 256)                 /**< Maximum FFT size which is a power of 2 and > 4                   */
#define FOSC_FFT_MIN_SIZE    ((Uint32)  32)                 /**< Minimum FFT size which is a power of 2 and > 4                   */
#define FOSC_NO_HARMONICS    ((Uint32)   1)                 /**< Default number of harmonics calculated by the overtone analyzer. */

// ********** device variant defined in global.h *********
#ifdef VTIQ_AIR
//#define FOSC_DEV_MIN_TRQ_AMP ((float32) 2.0e-6)		/**< Device minimum torque amplitude in newton meter. */
#define FOSC_DEV_MIN_TRQ_AMP ((FP32) 5.0e-6)                /**< Device minimum torque amplitude in newton meter.                 */
#else
	#define FOSC_DEV_MIN_TRQ_AMP ((float32) 20.0e-6)	/**< Device minimum torque amplitude in newton meter. */
#endif

#define FOSC_MAX_OSC_FRQ     ((float32) 100.0)  /**< Maximum oscillation frequency in Hz.                             */
#define FOSC_MIN_OSC_FRQ     ((float32) 1.0e-4) /**< Minimum oscillation frequency in Hz.                             */
// 04.04.16 set to 4 (was 2)
#define FOSC_OPT_UPD_CY      ((float32) 4.0)    /**< Optimization update cycle is a power of 2 and < actual FFT size. */






/*
 **********************************************************************************************************************************
 *                                                       GLOBAL DATA TYPES                                                        *
 **********************************************************************************************************************************
 */

/**
 * Fast oscillation device object
 */
typedef struct fosc_dev {
    float32         gain;

    /* Begin UwBa   */
    CBUF_FP32                   sTrqCBuf;
    CBUF_FP32                   sAngleCBuf;
    DFT_GOERTZEL_SNG_PNT_FP32   sTrqDFT;
    DFT_GOERTZEL_SNG_PNT_FP32   sAngleDFT;
    INT32U                      PassCtr;
    INT32U                      PassCnts;
    SRC_SIG_GEN_FP32_T          sSigGen;
    /* End UwBa     */


    Uint32          nHarmonics;                     /**< Number of harmonics to be calculated by the overtone analysis.           */
    float32         trq;                            /**< Actual torque in Nm.                                                     */
    float32         frq;                            /**< Oscillation frequency in Hz.                                             */
    float32         isrCycleTime;                   /**< Timer ISR cycle time in seconds.                                         */
    Uint32          n;                              /**< FFT and input buffer size which is a power of 2 and > 4.                 */
    Uint32          inc;                            /**< Torque sine table increment to determine tabulated torque values.        */
    float32         minTrqAmp;                      /**< Minimum torque amplitude in Nm.                                          */
    OPT_FPROP_DEV   optObj;                         /**< Fast strain oscillation opimization object.                              */
    Uint16          enFso;                          /**< Enable fast strain oscillation.                                          */
    Uint32          fsoPassCnts;                    /**< Pass counts.                                                             */
    Uint32          fsoPassCtr;                     /**< Pass counter loaded from pass counts.                                    */
    float32         fsoSetVal;                      /**< Set point value fast strain oscillation.                                 */
    objMavg         sFiltStAmp;                     /**< Moving average filter for first harmonic strain amplitude.               */
    objMavg         sFiltTrqAmp;                    /**< Moving average filter for first harmonic torque amplitude.               */
    objMavg         sFiltDelta;                     /**< Moving average filter for first harmonic delta angle.                    */
    float32         trqDftCorr;                     /**< Angular drift compensation torque in Nm.                                 */
} FOSC_DEV;

/*
 **********************************************************************************************************************************
 *                                                        GLOBAL VARIABLES                                                        *
 **********************************************************************************************************************************
 */

/*
 **********************************************************************************************************************************
 *                                              GLOBAL INTERFACE FUNCTION PROTOTYPES                                              *
 **********************************************************************************************************************************
 */

void PtrFoscTrqAmp(FP32 val);
void PtrFoscStrainAmp(FP32 val);

void FoscStop(void);

/**
 * Interface function to configure the fast oscillation device object.
 *
 * This function must be called only once after start-up. The function assigns the data tables to the fast oscillation device
 * object and initializes the tables with start values.
 */
void FoscCfg(void);

/**
 * Interface function to set the parameters for the measurement and overtone analysis.
 *
 * This function must be called with each OscInit(). Dependent on the specified amplitude modulation function, the torque generator
 * produces the following torque output:
 *
 *      0   The torque sine generator produces a sinusoidal oscillation with specified frequency and device dependent minimum torque
 *          amplitude.
 *
 *          T(t) = FOSC_DEV_MIN_TRQ_AMP * cos(2*pi*f*t)
 *
 *      1   The torque sine generator produces a sinusoidal oscillation with specified frequency and torque amplitude. The torque
 *          amplitude must be defined with the first coefficient in the coefficient table.
 *
 *          T(t) = c[0] * cos(2*pi*f*t)
 *
 *      2   The torque sine generator produces a sinusoidal oscillation with specified frequency and sigmoidal torque amplitude
 *          modulation.
 *
 *                             c[0]
 *          T(t) = ( --------------------- + c[3] ) * cos(2*pi*f*t)
 *                        -c[2]*(t - c[1])
 *                   1 + e
 *
 *      3   The torque sine generator produces a sinusoidal oscillation with specified frequency and linear torque amplitude
 *          modulation.
 *
 *      4   The torque sine generator produces a sinusoidal oscillation with specified frequency and torque amplitude modulation
 *          based on table interpolation.
 *
 * @param   frq         Oscillation frequency in Hz. The frequency must be in the range 1.0 Hz <= frq <= 100 Hz.
 * @param   n           Number of harmonics to be calculated by the overtone analysis. The number must be greater than 0.
 * @param   funType     Amplitude function type (0 := minimum, 1 := constant and 2:= sigmoidal amplitude).
 * @param   pCoefTbl    Pointer to the coefficient table of the amplitude modulation function.
 * @param   mode        Operating mode (0 := CS-OSC, 1 := CS-FS-OSC).
 * @param   fSetStrain      Fast strain oscillation set value in radiant.
 * @param   maxTrq      Maximum torque in Nm.
 * @returns             0 if successful otherwise 1.
 */
Uint16 FoscSet (float32 frq, Uint32 unHar, float32 fAmpl, Uint16 usMeasMode, float32 fSetStrain, float32 maxTrq);

/**
 * Fast oscillation cyclic function.
 *
 * This function must be called inside the timer ISR.
 * 
 * The function calculates the actual torque dependent on the specified amplitude modulation function and the overtone analysis for
 * the specified number of harmonics.
 *
 * The magnitude and angle coefficients of the FFT series for the torque and the strain are interleaved in memory. The angle is
 * calculated in radiant. The first pair of values corresponds with the first harmonic, followed by the second and so on to higher
 * harmonics. The phase difference table starts with the first harmonic in normalized frequency order.
 *
 * @param   strain      Actual measured relative strain in radiant. The strain offset is eliminated by the overlaying calling
 *                      procedure.
 * @param   pTrqTbl     Pointer to the torque FFT coefficient table.
 * @param   pStTbl      Pointer to the strain FFT coefficient table.
 * @param   pDeltaTbl   Pointer to the phase difference table.
 * @returns             the actual torque in Nm.
 */
float32 FoscCyclic(float32 strain, float32 *pTrqTbl, float32 *pStTbl, float32 *pDeltaTbl);

/**
 * Read the ISR cycle time.
 *
 * @returns the ISR cycle time in seconds.
 */
float32 FoscGetIsrCycleTime(void);

/**
 * Read the FFT table length.
 *
 * @returns the FFT table length.
 */
Uint32 FoscGetTblLen(void);

/**
 * The function returns the status of the overtone analysis.
 *
 * @returns             1 if overtone analysis is ready otherwise 0.
 */
Uint16 FoscIsRdy(void);

/*
 **********************************************************************************************************************************
 *                                                   GLOBAL FUNCTION PROTOTYPES                                                   *
 **********************************************************************************************************************************
 */

/**
 * Set the parameters for the torque signal generator and overtone analysis.
 *
 * @param   pDev        Pointer to the used fast oscillation device instance.
 * @param   frq         Oscillation frequency in Hz. The frequency must be in the range 1.0 Hz <= frq <= 100 Hz.
 * @param   n           Number of harmonics to be calculated by the overtone analysis. The number must be greater than 0.
 * @param   funType     Amplitude function type (0 := normalized, 1 := Heaviside, 2:= sigmoidal, 3:= linear amplitude and
 *                      4 := table interpolated amplitude).
 * @param   pCoefTbl    Pointer to the coefficient table of the amplitude modulation function.
 * @param   mode        Operating mode (0 := CS-OSC, 1 := FS-OSC).
 * @param   fSetStrain  Fast strain set point in radiant.
 * @param   maxTrq      Maximum torque in Nm.
 * @returns             0 if successful otherwise 1.
 */
Uint16 FoscSetParam (FOSC_DEV *pDev, float32 frq, Uint32 unHar, float32 fAmpl, Uint16 usMeasMode, float32 fSetStrain, float32 maxTrq);

/**
 * Fast oscillation cyclic function.
 *
 * This function must be called inside the timer ISR.
 * 
 * The function calculates the actual torque dependent on the specified amplitude modulation function and the overtone analysis for
 * the specified number of harmonics.
 *
 * The magnitude and angle coefficients of the FFT series for the torque and the strain are interleaved in memory. The angle is
 * calculated in radiant. The first pair of values corresponds with the first harmonic, followed by the second and so on to higher
 * harmonics. The phase difference table starts with the first harmonic in normalized frequency order.
 *
 * @param   p           Pointer to the used fast oscillation device instance.
 * @param   strain      Actual measured relative strain in radiant. The strain offset is eliminated by the overlaying calling
 *                      procedure.
 * @param   pTrqTbl     Pointer to the torque FFT coefficient table.
 * @param   pStTbl      Pointer to the strain FFT coefficient table.
 * @param   pDeltaTbl   Pointer to the phase difference table.
 * @returns             the actual torque in Nm.
 */
float32 FoscCyclicCalc(FOSC_DEV *p, float32 strain, float32 *pTrqTbl, float32 *pStTbl, float32 *pDeltaTbl);

#endif  /* EOF FOSC_H   */
