/*
************************************************************************************************************************************
*                                              READY-TO-USE EMBEDDED BUILDING BLOCKS
*
*                             (c) Copyright 2015-2016; Thermo Electron (Karlsruhe) GmbH; Karlsruhe-Germany
*
*       All rights reserved. Protected by international copyright laws. 
*
*       TMO source code is an unpublished work and the use of a copyright notice does not imply otherwise. This source code
*       contains confidential, trade secret material of TMO. Any attempt or participation in deciphering, decoding, reverse
*       engineering or in any way altering the source code is strictly prohibited, unless the prior written consent of TMO
*       is obtained.
*
************************************************************************************************************************************
*/

/**
************************************************************************************************************************************
*
*                                                         FILE DESCRIPTION
*
* @file         list.h
* @brief        Abstract Data Type Linked List
* @details      The module provides basic abstract data types and functions for the implementation of linked lists. Linked lists
*               consist of a series of elements which are arranged in a specific order and linked together.  They are suitable for
*               managing big data volumes and realization of stacks, queues and error/exception handlers. The linked lists in this
*               module have very efficient insert and delete operations. Because of the fact that the application and associated
*               specific data size is not known the methods in this basic module uses dynamic memory allocation features.
* @author       UwBa
*/

/*
************************************************************************************************************************************
*                                                           MODULE START
************************************************************************************************************************************
*/

#ifndef LIST_H
#define LIST_H                                                  /**< LIST.H file encapsulation to avoid multiple parsing.         */

/*
************************************************************************************************************************************
*                                                          INCLUDE FILES
************************************************************************************************************************************
*/

#include <stdlib.h>                                             /* Because of NULL definition.                                    */
#include "typedef.h"                                            /* Because of target platform independend data types.             */

/*
************************************************************************************************************************************
*                                                   EXTERNAL C LANGUAGE LINKAGE
*
* Note(s) : (1) C++ compilers must 'extern'ally declare all C function prototypes & variable/object declarations for correct C
*               language linkage.
************************************************************************************************************************************
*/

#ifdef __cplusplus
extern "C" {                                                    /* See Note #1.                                                   */
#endif

/*
************************************************************************************************************************************
*                                                            EXTERNALS
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                             DEFINES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                            DATA TYPES
************************************************************************************************************************************
*/

/** List element
 */
typedef struct list_elmt {
    void                *DataPtr;                               /**< Element data pointer.                                        */
    struct list_elmt    *NextElmtPtr;                           /**< Pointer to the next list element.                            */
} LIST_ELMT;

/** Linked list
 */
typedef struct list {
    INT32U      Size;                                           /**< List size (number of list elements).                         */
    LIST_ELMT  *HeadPtr;                                        /**< Pointer to the element at the beginning of the linked list.  */
    LIST_ELMT  *TailPtr;                                        /**< Pointer to the element at the end of the linked list.        */

    void      (*Destroy)  (void *pdata);                        /**< Function pointer for data memory deallocation.               */
} LIST;

/*
************************************************************************************************************************************
*                                                         GLOBAL VARIABLES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                              MACROS
************************************************************************************************************************************
*/

#if !defined(LIST_SIZE)
#define LIST_SIZE(lp) ((lp)->Size)                              /**< Macro that returns the number of elements in a linked list.  */
#endif

#if !defined(LIST_IS_EMPTY)
#define LIST_IS_EMPTY(lp) ((lp)->Size == 0 ? 1 :0)              /**< Macro that checks if a list is empty.                        */
#endif

#if !defined(LIST_HEAD)
#define LIST_HEAD(lp) ((lp)->HeadPtr)                           /**< Macro that returns the first element in a linked list.       */
#endif

#if !defined(LIST_TAIL)
#define LIST_TAIL(lp) ((lp)->TailPtr)                           /**< Macro that returns the last element in a linked list.        */
#endif

#if !defined(LIST_IS_HEAD)
#define LIST_IS_HEAD(lp, elmt) ((elmt) == (lp)->HeadPtr ? 1 : 0)/**< Macro that checks if element is at beginning of linked list. */
#endif

#if !defined(LIST_IS_TAIL)
#define LIST_IS_TAIL(ep) ((ep)->NextElmtPtr == NULL ? 1 : 0)    /**< Macro that ckecks if element is a the end of a linked list.  */
#endif

#if !defined(LIST_DATA)
#define LIST_DATA(ep) ((ep)->DataPtr)                           /**< Macro that returns the element data in a linked list.        */
#endif

#if !defined(LIST_NEXT)
#define LIST_NEXT(ep) ((ep)->NextElmtPtr)                       /**< Macro that returns the next element of a linked list.        */
#endif

/*
************************************************************************************************************************************
*                                                       FUNCTION PROTOTYPES
************************************************************************************************************************************
*/

/** Initialization of a linked list
 *
 *  The function initializes a linked list so that it can be used by other functions and operations. During initialization the
 *  length is set to zero, head- and tail pointer are set to NULL and the destroy function is assigned.
 *
 *  @param      p_list      Pointer to a linked list
 *  @param      fnct        Pointer to a destroy function
 */
void List_Init(LIST *p_list, void (*fnct)(void *data));

/** Memory deallocation of a linked list
 *
 *  The function frees the dynamically allocated memory of the elements in a linked list. This means primarily that all elements are
 *  removed from the list. The by List_Init() given destroy function is called for the memory deallocation of the application-
 *  specific data in the elements. In case that the destroy function is set to NULL, no function is called.
 *
 *  @param      p_list      Pointer to a linked list
 */
void List_Destroy(LIST *p_list);

/** Insert an application-specific data element in a linked list
 *
 *  The function inserts right behind the given element adress a new element in the linked list. For element adress NULL, the new
 *  element is inserted at the head of the linked list. The new element contains a data pointer, the memory which is referenced
 *  via the data pointer must be valid as long as the element remains in the linked list. The memory management associated with the
 *  data pointer is the responsibility of the calling program.
 *
 *  @param      p_list      Pointer to a linked list
 *  @param      p_elmt      Pointer to a list element
 *  @param      p_data      Pointer to application-specific data
 *  @returns                ERR_SUCCESS if element is successfully inserted and ERR_ENOMEM if dynamic memory allocation failed
 */
INT32U List_InsNext(LIST *p_list, LIST_ELMT *p_elmt, const void *p_data);

/** Remove an application-specific data element in a linked list
 *
 *  The function removes the data element from a linked list, which follows the given element adress. In the case that the element
 *  adress is NULL, the data element at the head of the linked list is removed. When return from this function p_data points to the
 *  data of the currently removed element. It is the responsibility of the calling program to manage the associated data memory.
 *
 *  @param      p_list      Pointer to a linked list
 *  @param      p_elmt      Pointer to a list element
 *  @param      p_data      Pointer to application specific data
 *  @returns                ERR_SUCCESS if element is successfully removed otherwise ERR_FAILURE
 */
INT32U List_RemNext(LIST *p_list, LIST_ELMT *p_elmt, void **p_data);

/*
************************************************************************************************************************************
*                                                 EXTERNAL C LANGUAGE LINKAGE END
************************************************************************************************************************************
*/

#ifdef __cplusplus
}                                                               /* End of 'extern'al C language linkage.                          */
#endif

/*
************************************************************************************************************************************
*                                                            MODULE END
************************************************************************************************************************************
*/

#endif                                                          /* End of LIST.H header file.                                     */
