/**
 * @file    msc.h
 * @brief   Micro stress control
 *
 *          The module contains all functionalities regarding the micro stress
 *          control beginning from determination of the MSC-table, diagnosis and
 *          correction.
 *
 * @par     Copyright:
 *
 *          Thermo Electron (Karlsruhe) GmbH, Dieselstrasse 4, 
 *          76227 Karlsruhe, Germany
 *
 *          (c) Copyright 2011 - 2012, Thermo Fisher Scientific
 *
 *          All rights reserved. Thermo Fisher Scientific source code is an
 *          unpublished work and the use of a copyright notice does not imply
 *          otherwise. This source code contains confidential, trade secret
 *          material of Thermo Fisher Scientific. Any attempt or participation
 *          in deciphering, decoding, reverse engineering or in any way altering
 *          the source code is strictly prohibited, unless the prior written
 *          consent of Thermo Fisher Scientific is obtained.
 *
 * @par     Creation Date:
 *
 *          17-Jul-2012
 *
 * @par     Last Modification:
 *
 *          12-Oct-2012
 *
 * @par     Project:
 *
 *          Visco Vision K1
 *
 * @par     Editor:
 *
 *          Microsoft Visual C++ 8.0
 *
 * @par     Compiler:
 *
 *          Microsoft Visual C++ 8.0
 *
 * @author  Uwe Bauer
 *
 * @par     Revision History:
 */

#ifndef MSC_H
#define MSC_H

/*
 *******************************************************************************
 *                                  INCLUDES                                   *
 *******************************************************************************
 */

//#include "typedefs.h"
//#include "F28335Inc.h"

/*
 *******************************************************************************
 *                                 CONSTANTS                                   *
 *******************************************************************************
 */

//#define MSC_TBL_LEN ((INT32U) 256)      /**< Micro stress table length.       */

 
 /**
 * Micro stress control data structure
 */
typedef struct msc_obj{
    INT32U  nTblLen;        /**< Micro stress control table length.           */
    FP32    *pafTrq;        /**< Pointer to torque table.                     */
    FP32    *pafAbsAng;     /**< Pointer to abs angle table.                  */
    FP32    fTrqMax;        /**< Max. allowed torque during MSC measurement.  */
    FP32    fTrqDC;         /**< Torque DC-offest caused by ball bearings.    */
    INT32U  nNoFrq;         /**< Number of frequency commponents for approx.  */
    FP32    fPerc;          /**< Power percentage of the approximated table.  */
    INT32U  nSta;           /**< Status update 0:= OK and 1:=NOK.             */
    FP32    fIdxFactor;     /**< Index to determine equidistant data points.  */
    INT32U  nAqCnt;         /**< Data aquisition counter.                     */
    INT32U  nRecRdy;        /**< Data record ready 1:= complete otherwise 0.  */
} MSC_OBJ;


/*
 *******************************************************************************
 *                              GLOBAL DATA TYPES                              *
 *******************************************************************************
 */
extern MSC_OBJ tMscData;



/*
 *******************************************************************************
 *                      GLOBAL FUNCTION PROTOTYPES                             *
 *******************************************************************************
 */

/**
 * Configurate micro stress control
 *
 * The function must called before anay other function of the micro stress
 * control class can be called.
 *
 * @param   p           Pointer to the used MSC instance
 * @param   nTblLen     Table length (n > 1)
 * @param   pafTrq      Pointer to the torque table
 * @param   pafAbsAng   Pointer to the absolute angle table
 * @param   nNoFrq      Number of frequency components for FFT approximation
 * @returns             0 if all inputs arguments in the valid range
 *                      otherwise 1.
 */
void MscCfg(MSC_OBJ *p, INT32U nTblLen, FP32 *pafTrq, FP32 *pafAbsAng,
            INT32U nNoFrq);

/**
 * Initialize micro stress control data aquisition
 *
 * The function must called before the data aquisition can start.
 *
 * @param   p   Pointer to the used MSC instance
 */
void MscDatAqInit(MSC_OBJ *p);

/**
 * Micro stress control data aquisition
 *
 * The function must be called in the cyclic program. During the measurement
 * the torque and absolute angle data is checked and automatically stored in
 * equidistant absolute angle points. Once the required number of data points
 * are recorded the function returns 1 to the calling program.
 *
 * @param   p           Pointer to the used MSC instance
 * @param   fAbsAng     Current absolute angle position [rad]
 * @param   fTrq        Current tourque [Nm]
 * @returns             1 if complete data set is recorded otherwise 0.
 */
INT32U MscDatAq(MSC_OBJ *p, FP32 fAbsAng, FP32 fTrq);

/**
 * Micro stress control table calculation.
 *
 * This function must be called in the system background after the measurement
 * is completed.
 *
 * @param   p           Pointer to the used MSC instance.
 */
void MscTblCalc(MSC_OBJ *p);

/**
 * Micro stress control correction.
 *
 * This function must be called in all rheological measurment modes were MSC
 * correction is needed.
 *
 * @param   p           Pointer to the used MSC instance.
 * @param   fAbsAng     Current absolute angle position [rad]
 * @param   fTrq		Current motor torque [Nm]
 * @returns             MSC correction torque [Nm]
 */
FP32 MscCorr(MSC_OBJ *p, FP32 fAbsAng, FP32 fTrq);

float32 TblSearchMinMax(MSC_OBJ *p);

#endif  /* EOF MSC_H    */
