/**
 * @file    mth.h
 *
 *          Add-on mathematical functions
 *
 * @par     Copyright:
 *
 *          Thermo Electron (Karlsruhe) GmbH, Dieselstrasse 4,
 *          76227 Karlsrue, Germany
 *
 *          (c) Copyright 2009 - 2012, Thermo Electron
 *
 *          All rights reserved. Thermo Electron source code is an unpublished
 *          work and the use of a copyright notice does not imply otherwise.
 *          This source code contains confidential, trade secret material of
 *          Thermo Electron. Any attempt or participation in deciphering,
 *          decoding, reverse engineering or in any way altering the source
 *          code is strictly prohibited, unless the prior written consent of
 *          Thermo Electron is obtained.
 *
 * @par     Creation Date:
 *
 *          05-Dec-2009
 *
 * @par     Last Modification:
 *
 *          25-Jul-2012
 *
 * @par     Project:
 *
 *          Visco Vision K1
 *
 * @par     Editor:
 *
 *          Microsoft Visual C++ 8.0
 *
 * @par     Compiler:
 *
 *          Microsoft Visual C++ 8.0
 *
 * @author  Uwe Bauer
 *
 * @par     Revision History:
 */

#ifndef MTH_H
#define MTH_H             /**< File encapsulation to avoid multiple parsing.  */

/*
 *******************************************************************************
 *                                  INCLUDES                                   *
 *******************************************************************************
 */

//#include <math.h>
//#include "typedefs.h"


/*
 *******************************************************************************
 *                                   MACROS                                    *
 *******************************************************************************
 */

/**
 * A macro that returns the minimum of the values a and b.
 */
#if !defined(MTH_MIN)
#define MTH_MIN(a, b) (((a) < (b)) ? (a) : (b))
#endif

/**
 * A macro that returns the maximum of the values a and b.
 */
#if !defined(MTH_MAX)
#define MTH_MAX(a, b) (((a) > (b)) ? (a) : (b))
#endif

/** Macro returns the sign of x. 
 *
 *  It is defined as ((x) >= 0 ? 1 : -1). Note that with this definition the sign of zero is positive (regardless of its IEEE sign
 *  bit). 
 */
#if !defined(MTH_SIGN)
#define MTH_SIGN(x) (((x) >= 0.0) ? 1 : -1)
#endif

/** Macro that realize the Heaviside step function
 *
 *  It is defined as ((x) >= 0 ? 1 : 0). Note that with this definition the step of zero is one.
 */
#if !defined(MTH_STEP)
#define MTH_STEP(x) (((x) >= 0.0) ? 1 : 0)
#endif

/** Macro for testing odd numbers
 *
 *  This macro evaluates to 1 if n is odd and 0 if n is even. The argument n must be of integer type.
 */
#if !defined(MTH_IS_ODD)
#define MTH_IS_ODD(n)  ((n) & 1)
#endif

/** Macro for testing even numbers
 *
 *  This macro is the opposite of MTH_IS_ODD(n). It evaluates to 1 if n is even and 0 if n is odd. The argument n must be of
 *  integer type. 
 */
#if !defined(MTH_IS_EVEN)
#define MTH_IS_EVEN(n) (!(GSL_IS_ODD(n)))
#endif

/** Macro that ckecks if a integer is a power of two
 */
#if !defined(MTH_IS_PWR_TWO)
#define MTH_IS_PWR_TWO(x) (((x) != 0) && (((x) & (~(x) + 1)) == (x)))
#endif



/*
 *******************************************************************************
 *                                 CONSTANTS                                   *
 *******************************************************************************
 */

#define MTH_TRUE    ((INT08U) 1)     /**< Boolean term for a true statement.   */
#define MTH_FALSE   ((INT08U) 0)     /**< Boolean term for a false statement.  */

// 18.10.12 ersetzt durch cfPi, spart Speicher und Rechenzeit!
//#define MTH_PI      (4.0*atan(1.0)) /**< pi                                   */

#define SQRT_2	1.4142136


/*
************************************************************************************************************************************
*                                                        MACHINE CONSTANTS
*
* Note(s) : (1) Magic machine constants, mostly for the benefit of the implementation of additional math function and exception
*               handling.
************************************************************************************************************************************
*/

                                                                /*   FP64 machine constants:                                      */
#define MTH_FP64_EPS        2.2204460492503131e-16              /**< Distance from 1.0 to the next larger FP64 precision number.  */
#define MTH_FP64_EPS_SQRT   1.4901161193847656e-08              /**< Relative square root accuracy for FP64 numbers.              */
#define MTH_FP64_MIN        2.2250738585072014e-308             /**< Minimum positive FP64 value.                                 */
//#define MTH_FP64_MAX        1.7976931348623157e+308             /**< Maximum positive FP64 value.                                 */
#define MTH_FP64_MAX        3.4028234663852886e+38              /**< Maximum positive FP32 value.                                 */

                                                                /*   FP32 machine constants:                                      */
#define MTH_FP32_EPS        1.1920928955078125e-07              /**< Distance from 1.0 to the next larger FP32 precision number.  */
#define MTH_FP32_EPS_SQRT   3.4526698300124393e-04              /**< Relative square root accuracy for FP32 numbers.              */
#define MTH_FP32_MIN        1.1754943508222875e-38              /**< Minimum positive FP32 value.                                 */
#define MTH_FP32_MAX        3.4028234663852886e+38              /**< Maximum positive FP32 value.                                 */

/*
************************************************************************************************************************************
*                                                          MATH CONSTANTS
*
* Note(s) : (1) All math constants are of type long double.
************************************************************************************************************************************
*/

#define MTH_E           2.71828182845904523536                  /**< Euler's constant.                                            */
#define MTH_LN2         0.69314718055994530942                  /**< Natural logarithm of the 2.                                  */
#define MTH_LN10        2.30258509299404568402                  /**< Natural logarithm of the 10.                                 */
#define MTH_LOG2E       1.4426950408889634074                   /**< Natural logarithm of the Euler's number e to base 2.         */
#define MTH_LOG10E      0.43429448190325182765                  /**< Natural logarithm of the Euler's number e to base 10.        */
#define MTH_PI          3.14159265358979323846                  /**< The constant \f$ \pi \f$.                                    */
#define MTH_2PI         6.28318530717958647692                  /**< The constant \f$ 2 \cdot \pi \f$.                            */
#define MTH_PI_2        1.57079632679489661923                  /**< The constant \f$ {\pi \over 2} \f$.                          */
#define MTH_PI_4        0.78539816339744830962                  /**< The constant \f$ {\pi \over 4} \f$.                          */
#define MTH_1_PI        0.31830988618379067154                  /**< The constant \f$ {1 \over \pi} \f$.                          */
#define MTH_2_PI        0.63661977236758134308                  /**< The constant \f$ {2 \over \pi} \f$.                          */
#define MTH_2_SQRTPI    1.12837916709551257390                  /**< The constant \f$ {2 \over \sqrt \pi} \f$.                    */
#define MTH_SQRT2       1.41421356237309504880                  /**< The square root of 2.                                        */
#define MTH_1_SQRT2     0.70710678118654752440                  /**< The constant \f$ {1 \over \sqrt 2} \f$.                      */
#define MTH_SQRT3       1.73205080756887729353                  /**< The square root of 3.                                        */

/*
 **********************************************************************************************************************
 *                                                      CONSTANTS                                                     *
 **********************************************************************************************************************
 */


/**Default increment which is used by the root finding procedures to calculate the difference quotient.
 */
extern const FP64 MTH_DELTA;

/**Permissible error in the function value (4.0*EPS).
 */
extern const FP64 MTH_FUN_ERR;

/**Permissible relative error (128.0*EPS).
 */
extern const FP64 MTH_REL_ERR;

/**Permissible absolute error in the residual function value.
 */
extern const FP64 MTH_ABS_ERR;

/**Maximum allowed number of iterations of the root finding procedure.
 */
extern const INT32U MTH_ITER_MAX;

/**Zero function prototype pointer.
 */
typedef FP64 (*MthZeroFun) (void*, const FP64);



/*
 *******************************************************************************
 *                              GLOBAL VARIABLES                               *
 *******************************************************************************
 */

// Pi as const float in Flash Memory
extern const float cfPi_2;
extern const float cfPi;
extern const float cf2Pi;
//extern const float cf180_Pi;


/*
************************************************************************************************************************************
*                                                  ELEMENTARY FUNCTION PROTOTYPES
************************************************************************************************************************************
*/

/** Base-b logarithm 
 *
 *  The function returns the base-b logarithm of the real number a. This quantity is computed as the ratio 
 *  \f$ y = {log(a) \over log(b)} \f$.
 *
 *  @param      a   Real number.
 *  @param      b   Real base.
 *  @returns        the real base-b logarithm.
 */
FP64 Mth_LogBase(const FP64 a, const FP64 b);

/**
 * Logarithmus-Dualis
 *
 * MthLog2() calculates the logarithm of the argument x to the base of 2.
 *
 * @param   x   Argument
 * @returns     the logarithm of the argument x to the base of 2.
 */
FP64 MthLog2(FP64 x);

/**
 * Signum function.
 *
 * For each element of x, MthSign(X) returns 1 if the element is greater than
 * zero, 0 if it equals zero and -1 if it is less than zero.
 *
 * @param   x     The input value.
 * @returns       the function value.
 */
FP64 MthSign(const FP64 x);

/**
 * 1-D linear table interpolation.
 *
 * yi = MthInterp1D(X, Y, xi, n) interpolates to find yi, the value of the
 * underlying function Y at the not tabulated point xi.The vector X specifies
 * the points at which the data Y is given. Interpolation is the same operation
 * as table look-up. Described in table look-up terms, the tables are X and Y.
 * MthInterp1D looks-up the elements of xi in X, and based upon their location,
 * returns the linear interpolated value yi within the elements of Y. If xi is
 * outside the tabulated range, the last tabulated value is given out. With this
 * proceeding no exapolation is done.
 *
 * @param   X  The vector contains the arguments of the tabulated function 
 *             values.
 * @param   Y  The vector contains the function values and represents the
 *             function which must be interpolated.
 * @param   xi Not tabulated argument xi for which the associated function value
*              must be determined.
 * @param   n  Table length.
 * @returns    the interpolated function value for the given argument xi.
 */
FP32 MthInterp1D(const FP32 X[], const FP32 Y[], const FP32 xi, const INT32U n);

/**
 * Sum of elements.
 *
 * The function calculates the sum of the elements of a vector.
 *
 * @param   pVec  Startadress of the vector.
 * @param   n     Length of the vector.
 * @returns       the sum of the elements of the vector.
 */
FP32 MthVecSum(FP32 *pVec, INT32U n);


Uint16 Modulof(float value, Uint16 *iptr);

// Spline , direct from Numerical recipes
//void spline(float32 x[], float32 y[], Uint16 n, float32 yp1, float32 ypn, float32 y2[]);
//float32 splint(float xa[], float ya[], float y2a[], int n, float x);

float LimitFloat(float fVal, float fMin, float fMax);



/** Natural logarithm log(1 + x)
 *
 *  This function computes the value of log(1 + x) in a way that is accurate for small x. 
 *
 *  @param      x   The input value.
 *  @returns        the natural logarithm.
 */
FP64 Mth_Log1Plus(const FP64 x);

/** Magnitude of a complex number
 *
 *  This function computes the value of \f$ \sqrt {x^2 + y^2} \f$ in a way that avoids overflow. 
 *
 *  @param      x   The real part.
 *  @param      y   The imaginary part.
 *  @returns        the magnitude.
 */
FP64 Mth_CmplxAbs(const FP64 x, const FP64 y);

/*
************************************************************************************************************************************
*                                                  HYPERBOLIC FUNCTION PROTOTYPES
************************************************************************************************************************************
*/

/** Arccosine hyperbolic
 *
 *  @param      x   The input value.
 *  @returns        the arccosine hyperbolic.
 */
FP64 Mth_Acosh(const FP64 x);

/** Arcsine hyperbolic
 *
 *  @param      x   The input value.
 *  @returns        the arcsine hyperbolic.
 */
FP64 Mth_Asinh(const FP64 x);

/** Arctangent hyperbolic
 *
 *  @param      x   The input value.
 *  @returns        the arctangent hyperbolic.
 */
FP64 Mth_Atanh(const FP64 x);


#endif /* MTH_H */
