/**
 * @file    opt.h
 * @brief   Optimization
 *
 * @par     Copyright:
 *
 *          Thermo Electron (Karlsruhe) GmbH, Dieselstrasse 4, 76227 Karlsruhe, Germany
 *
 *          (c) Copyright 2010, Thermo Fisher Scientific
 *
 *          All rights reserved. Thermo Fisher Scientific source code is an unpublished work and the use of a copyright notice does
 *          not imply otherwise. This source code contains confidential, trade secret material of Thermo Fisher Scientific. Any
 *          attempt or participation in deciphering, decoding, reverse engineering or in any way altering the source code is
 *          strictly prohibited, unless the prior written consent of Thermo Fisher Scientific is obtained.
 *
 * @par     Creation Date:
 *
 *          11-Feb-2010
 *
 * @par     Last Modification:
 *
 *          02-Mar-2010
 *
 * @par     Project:
 *
 *          MARS III
 *
 * @par     Editor:
 *
 *          Microsoft Visual C++ 6.0
 *
 * @par     Compiler:
 *
 *          Microsoft Visual C++ 6.0
 *
 * @author  Uwe Bauer
 *
 * @par     Revision History:
 */

#ifndef OPT_H
#define OPT_H                                                   /**< File encapsulation to avoid multiple parsing.                */

/*
 **********************************************************************************************************************************
 *                                                           INCLUDES                                                             *
 **********************************************************************************************************************************
 */

//#include "typedefs.h"

/*
 **********************************************************************************************************************************
 *                                                           CONSTANTS                                                            *
 **********************************************************************************************************************************
 */

#define OPT_LRATE_PLUS  ((float32) 0.35) // was 1.05 /**< Increase factor (K > 1)          */
#define OPT_LRATE_MINUS ((float32) 0.35) // was 0.9  /**< Decrease factor (0 < K < 1)      */
#define OPT_DELTA_MAX   ((float32) 10000.0)          /**< Maximum weight update            */
#define OPT_DELTA_MIN   ((float32) 1e-6)             /**< Minimum weight update            */
#define OPT_DELTA_INIT  ((float32) 0.1)			  	 /**< Initial weight update            */
#define OPT_WEIGHT_INIT ((float32) 1.0)              /**< Initial weight                   */
#define OPT_WEIGHT_MAX  ((float32) 1e9)              /**< Maximum weight                   */
#define OPT_IDLE        ((Uint16) 1)                 /**< Idle state                       */
#define OPT_RUN         ((Uint16) 2)                 /**< Run state                        */

/*
 **********************************************************************************************************************************
 *                                                       GLOBAL DATA TYPES                                                        *
 **********************************************************************************************************************************
 */

/**
 * Fast propagation device object.
 */
#ifdef RAUS
typedef struct opt_fprop_dev {
    float32    E[2];                                               /**< Error                                                        */
    float32    dEdW[2];                                            /**< Partial derivative                                           */
    float32    delta[2];                                           /**< Step size                                                    */
    float32    dW[2];                                              /**< Weight update                                                */
    float32    W[2];                                               /**< Weight                                                       */
    float32    maxWeight;                                          /**< Maximum weight                                               */
    Uint16   state;                                              /**< State variable                                               */
} OPT_FPROP_DEV;
#endif


typedef struct opt_fprop_dev
{
    FP32    Tk;                                                 /**< Target value.                                                */
    FP32    Yk;                                                 /**< Actual value.                                                */
    FP32    Ek;                                                 /**< Error.                                                       */
    FP32    dEdWk;                                              /**< Partial error derivative.                                    */
    FP32    dEdWk_1;                                            /**< Previous partial error derivative.                           */
    FP32    Wk;                                                 /**< Weight.                                                      */
    FP32    Wk_1;                                               /**< Previous weight.                                             */
    FP32    Delta;                                              /**< Step size                                                    */                                                 
    FP32    dWk;                                                /**< Weight update.                                               */
    FP32    Wmin;                                               /**< Minimum weight.                                              */
    FP32    Wmax;                                               /**< Maximum weight.                                              */
    FP32    DeltaMin;                                           /**< Minimum step size.                                           */
    FP32    DeltaMax;                                           /**< Maximum step size.                                           */
    INT16U  state;
} OPT_FPROP_DEV;



/*
 **********************************************************************************************************************************
 *                                                   GLOBAL FUNCTION PROTOTYPES                                                   *
 **********************************************************************************************************************************
 */

/**
 * Nonlinear fast propagation optimization procedure.
 *
 * @param   p       Pointer to the used optimization instance.
 * @param   tgtVal  Target value.
 * @param   actVal  Actual value.
 * @returns         the actual weight.
 */
float32 OptFastProp(OPT_FPROP_DEV *p, float32 tgtVal, float32 actVal);

/**
 * Initialize the nonlinear fast propagation optimization procedure.
 *
 * The function initialize the step size, weight update, weight and maximum weight with default start values. The error values and
 * partial derivatives are reset to zero.
 *
 * @param   p       Pointer to the used optimization instance.
 */
void OptFastPropInit(OPT_FPROP_DEV *p);

/**
 * Set the maximum weight of the nonlinear fast propagation optimization.
 *
 * @param   p       Pointer to the used optimization instance.
 * @param   val     Maximum weight.
 * @returns         0 for valid maximum weight otherwise 0.
 */
Uint16 OptFastPropSetMaxWeight(OPT_FPROP_DEV *p, const float32 val);

#endif  /* OPT_H    */
