/*
************************************************************************************************************************************
*                                              READY-TO-USE EMBEDDED BUILDING BLOCKS
*
*                             (c) Copyright 2015-2016; Thermo Electron (Karlsruhe) GmbH; Karlsruhe-Germany
*
*       All rights reserved. Protected by international copyright laws. 
*
*       TMO source code is an unpublished work and the use of a copyright notice does not imply otherwise. This source code
*       contains confidential, trade secret material of TMO. Any attempt or participation in deciphering, decoding, reverse
*       engineering or in any way altering the source code is strictly prohibited, unless the prior written consent of TMO
*       is obtained.
*
************************************************************************************************************************************
*/

/**
************************************************************************************************************************************
*
*                                                         FILE DESCRIPTION
*
* @file         queue.h
* @brief        todo
* @details      todo.
* @version      V1.00.00
* @author       UwBa
*
************************************************************************************************************************************
* Notes         (1) The note(s) section should be omitted if no notes are given.
************************************************************************************************************************************
*/

/*
************************************************************************************************************************************
*                                                           MODULE START
************************************************************************************************************************************
*/

#ifndef QUEUE_H
#define QUEUE_H                                                 /**< QUEUE.H file encapsulation to avoid multiple parsing.        */

/*
************************************************************************************************************************************
*                                                          INCLUDE FILES
************************************************************************************************************************************
*/

#include <stdlib.h>                                             /* Because of NULL definition.                                    */

#include "typedef.h"                                            /* Because of target platform independend data types.             */
#include "list.h"

/*
************************************************************************************************************************************
*                                                   EXTERNAL C LANGUAGE LINKAGE
*
* Note(s) : (1) C++ compilers must 'extern'ally declare all C function prototypes & variable/object declarations for correct C
*               language linkage.
************************************************************************************************************************************
*/

#ifdef __cplusplus
extern "C" {                                                    /* See Note #1.                                                   */
#endif

/*
************************************************************************************************************************************
*                                                            EXTERNALS
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                             DEFINES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                            DATA TYPES
************************************************************************************************************************************
*/

typedef LIST QUEUE;                                             /**< Queue type is based on a linked list.                        */

/*
************************************************************************************************************************************
*                                                         GLOBAL VARIABLES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                              MACROS
************************************************************************************************************************************
*/

/** The macro returns the number of elements that are in the queue.
 */
#if !defined(QUEUE_SIZE)
#define QUEUE_SIZE(qp)  LIST_SIZE(qp)
#endif

/** The macro returns the data that is stored in the top element of the queue.
 */
#if !defined(QUEUE_PEEK) 
#define QUEUE_PEEK(qp) ((qp)->HeadPtr == NULL ? NULL : (qp)->HeadPtr->DataPtr)
#endif

/*
************************************************************************************************************************************
*                                                       FUNCTION PROTOTYPES
************************************************************************************************************************************
*/

/** Queue initialization
 *
 *  This function must be called before the queue can be used for any other operation and method. The argument fnct (destroy 
 *  function) allow the memory deallocation of application specific data with the call of the function Queue_Destroy(). For queues
 *  with data that should not deallocated, fnct must be set to NULL. During initialization the length is set to zero, head- and tail
 *  pointer are set to NULL and the destroy function is assigned.
 *
 *  @param  p_queue     Pointer to a queue
 *  @param  fnct        Pointer to a destroy function
 */
void Queue_Init(QUEUE *p_queue, void (*fnct)(void *data));

/** Queue deallocation
 *
 *  The function frees the dynamically allocated memory of the elements in a queue. This means primarily that all elements are
 *  removed from the queue. The by Queue_Init() given destroy function is called for the memory deallocation of each element. In
 *  case that the destroy function is set to NULL, no destroy function is called. After the call of the this function no furtger
 *  queue operations are allowed til Queue_Init() is called again.
 *
 *  @param  p_queue     Pointer to a queue
 */
void Queue_Destroy(QUEUE *p_queue);

/** Append a new element to the end of a queue
 *  
 *  The new element contains a data pointer and the memory which is referenced by the data pointer must be valid as long as the
 *  element remains in the queue. The memory management associated with the data pointer is the responsibility of the calling
 *  program.
 *
 *  @param      p_queue     Pointer to a queue
 *  @param      p_data      Pointer to application specific data
 *  @returns                ERR_SUCCESS if element is successfully inserted and ERR_ENOMEM if dynamic memory allocation failed
 */
INT32U Queue_Enqueue(QUEUE *p_queue, const void *p_data);

/** Remove top element from queue
 *
 *  Removes the top element from the queue. Upon return, the data pointer points to the data stored in the removed element. It is
 *  the responsibility of the calling program to manage the memory which is associated with the data.
 *
 *  @param      p_queue     Pointer to a queue
 *  @param      p_data      Pointer to application specific data
 *  @returns                ERR_SUCCESS if element is successfully removed otherwise ERR_FAILURE
 */
INT32U Queue_Dequeue(QUEUE *p_queue, void *p_data);

/*
************************************************************************************************************************************
*                                                 EXTERNAL C LANGUAGE LINKAGE END
************************************************************************************************************************************
*/

#ifdef __cplusplus
}                                                                                   /* End of 'extern'al C language linkage.      */
#endif

/*
************************************************************************************************************************************
*                                                            MODULE END
************************************************************************************************************************************
*/

#endif                                                          /* End of QUEUE.H header file.                                    */
