/*
************************************************************************************************************************************
*                                              READY-TO-USE EMBEDDED BUILDING BLOCKS
*
*                             (c) Copyright 2015-2016; Thermo Electron (Karlsruhe) GmbH; Karlsruhe-Germany
*
*       All rights reserved. Protected by international copyright laws. 
*
*       TMO source code is an unpublished work and the use of a copyright notice does not imply otherwise. This source code
*       contains confidential, trade secret material of TMO. Any attempt or participation in deciphering, decoding, reverse
*       engineering or in any way altering the source code is strictly prohibited, unless the prior written consent of TMO
*       is obtained.
*
************************************************************************************************************************************
*/

/**
************************************************************************************************************************************
*
*                                                         FILE DESCRIPTION
*
* @file         sig.h
* @brief        Signal Generator 
* @details      The signal generator can produce one of five different waveforms: 
*
*               1. Sine wave
*               2. Cosine wave
*               3. square wave
*               4. Sawtooth wave
*               5. Random wave
*
*               Signal parameters are expressed in Hertz.
*
* @version      V1.00.00
* @author       UwBa
*
************************************************************************************************************************************
* Notes         (1) The note(s) section should be omitted if no notes are given.
************************************************************************************************************************************
*/

/*
************************************************************************************************************************************
*                                                           MODULE START
************************************************************************************************************************************
*/

#ifndef SIG_H
#define SIG_H

/*
************************************************************************************************************************************
*                                                          INCLUDE FILES
************************************************************************************************************************************
*/

#include "typedef.h"

/*
************************************************************************************************************************************
*                                                   EXTERNAL C LANGUAGE LINKAGE
*
* Note(s) : (1) C++ compilers must 'extern'ally declare all C function prototypes & variable/object declarations for correct C
*               language linkage.
************************************************************************************************************************************
*/

#ifdef __cplusplus
extern "C" {                                                    /* See Note #1.                                                   */
#endif

/*
************************************************************************************************************************************
*                                                            EXTERNALS
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                            WAVE FORMS
************************************************************************************************************************************
*/

enum {
    SIG_SINE     = 1,                                           /**< Sine wave.                                                   */
    SIG_COSINE   = 2,                                           /**< Cosine wave.                                                 */
    SIG_SQUARE   = 3,                                           /**< Square wave.                                                 */
    SIG_SAWTOOTH = 4,                                           /**< Sawtooth wave.                                               */
    SIG_RANDOM   = 5                                            /**< Random wave.                                                 */
} ;

/*
************************************************************************************************************************************
*                                                            DATA TYPES
************************************************************************************************************************************
*/

typedef struct sig_gen_fp32_t {
    INT32U  WaveForm;                                           /**< Signal wave form: sine, cosine, square, sawtooth or random.  */
    FP32    Amp;                                                /**< Signal amplitude, the default is 1.                          */
    FP32    Frq;                                                /**< Signal frequency, the default is 1.                          */
    FP32    SmpTm;                                              /**< Sample time [s].                                             */
    FP32    Out;                                                /**< Signal generator output.                                     */
    FP32    Perd;                                               /**< Signal time period [s].                                      */
    FP32    Omega;                                              /**< Angular frequency [rad/s].                                   */
    FP32    Tm;                                                 /**< Elapsed time [s], 0 <= Tm < Perd.                            */
    FP32    Slope;                                              /**< Signal slope for sawtooth wave form generation.              */
    FP32    Offset;                                             /**< Signal offset for sawtooth wave form generation.             */
} SIG_GEN_FP32_T;

/*
************************************************************************************************************************************
*                                                         GLOBAL VARIABLES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                              MACROS
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                       FUNCTION PROTOTYPES
************************************************************************************************************************************
*/

INT32U Sig_GenCfgFP32(SIG_GEN_FP32_T *p, INT32U form, FP32 amp, FP32 frq, FP32 tm);

FP32 Sig_GenFP32(SIG_GEN_FP32_T *p);

void Sig_GenResetFP32(SIG_GEN_FP32_T *p);

INT32U Sig_GenSetAmpFP32(SIG_GEN_FP32_T *p, FP32 amp);

/*
************************************************************************************************************************************
*                                                 EXTERNAL C LANGUAGE LINKAGE END
************************************************************************************************************************************
*/

#ifdef __cplusplus
}                                                               /* End of 'extern'al C language linkage.                          */
#endif

/*
************************************************************************************************************************************
*                                                            MODULE END
************************************************************************************************************************************
*/

#endif                                                          /* End of SIG.H header file.                                      */
