/*
************************************************************************************************************************************
*                                              READY-TO-USE EMBEDDED BUILDING BLOCKS
*
*                             (c) Copyright 2015-2016; Thermo Electron (Karlsruhe) GmbH; Karlsruhe-Germany
*
*       All rights reserved. Protected by international copyright laws. 
*
*       TMO source code is an unpublished work and the use of a copyright notice does not imply otherwise. This source code
*       contains confidential, trade secret material of TMO. Any attempt or participation in deciphering, decoding, reverse
*       engineering or in any way altering the source code is strictly prohibited, unless the prior written consent of TMO
*       is obtained.
*
************************************************************************************************************************************
*/

/**
************************************************************************************************************************************
*
*                                                         FILE DESCRIPTION
*
* @file         src.h
* @brief        Generation of Signals 
* @details      The signal generator can produce one of five different waveforms: 
*
*               1. Sine wave
*               2. Cosine wave
*               3. square wave
*               4. Sawtooth wave
*               5. Random wave
*
*               Signal parameters are expressed in Hertz.
*
* @version      V1.00.00
* @author       UwBa
*
************************************************************************************************************************************
* Notes         (1) The note(s) section should be omitted if no notes are given.
************************************************************************************************************************************
*/

/*
************************************************************************************************************************************
*                                                           MODULE START
************************************************************************************************************************************
*/

#ifndef SRC_H
#define SRC_H

/*
************************************************************************************************************************************
*                                                          INCLUDE FILES
************************************************************************************************************************************
*/

#include "typedef.h"
#include "mth.h"

/*
************************************************************************************************************************************
*                                                   EXTERNAL C LANGUAGE LINKAGE
*
* Note(s) : (1) C++ compilers must 'extern'ally declare all C function prototypes & variable/object declarations for correct C
*               language linkage.
************************************************************************************************************************************
*/

#ifdef __cplusplus
extern "C" {                                                    /* See Note #1.                                                   */
#endif

/*
************************************************************************************************************************************
*                                                            EXTERNALS
************************************************************************************************************************************
*/

/*
************************************************************************************************************************************
*                                                          COMMON DEFINES
************************************************************************************************************************************
*/

#if !defined(SRC_MIN_NBR_SMP)
#define SRC_MIN_NBR_SMP ((INT16U)3)                             /**< Minimum number of samples to fulfill sampling theorem.       */
#endif

#if !defined(SRC_SIG_GEN_FP32_DEFAULT)
#define SRC_SIG_GEN_FP32_DEFAULT { \
    SRC_WF_COSINE,                 \
    1.0f,                          \
    1.0f,                          \
    SRC_UNITS_HZ,                  \
    0.001f,                        \
    0.0f,                          \
    0.999f,                        \
    ((FP32)MTH_2PI),               \
    0.0f,                          \
    0.0f,                          \
    0.0f                           \
    }
#endif

/*
************************************************************************************************************************************
*                                                            WAVE FORMS
************************************************************************************************************************************
*/

enum {
    SRC_WF_SINE     = 1,                                        /**< Sine wave.                                                   */
    SRC_WF_COSINE   = 2,                                        /**< Cosine wave.                                                 */
    SRC_WF_SQUARE   = 3,                                        /**< Square wave.                                                 */
    SRC_WF_SAWTOOTH = 4,                                        /**< Sawtooth wave.                                               */
    SRC_WF_RANDOM   = 5                                         /**< Random wave.                                                 */
} ;

/*
************************************************************************************************************************************
*                                                              UNITS
************************************************************************************************************************************
*/

enum {
    SRC_UNITS_HZ      = 1,                                      /**< Frequency in Hertz.                                          */
    SRC_UNITS_RADPSEC = 2                                       /**< Frequency in radians per seconds.                            */
} ;

/*
************************************************************************************************************************************
*                                                           SIGNAL TYPE
************************************************************************************************************************************
*/

enum {
    SRC_ST_BIPOLAR  = 1,                                        /**< Bipolar signal type.                                         */
    SRC_ST_UNIPOLAR = 2                                         /**< Unipolar signal type.                                        */
} ;

/*
************************************************************************************************************************************
*                                                            DATA TYPES
************************************************************************************************************************************
*/

/** 32-bit precision signal generator data type.
 */
typedef struct src_sig_gen_fp32_t {
    INT16U  WaveForm;                                           /**< Signal wave form: sine, cosine, square, sawtooth or random.  */
    FP32    Amp;                                                /**< Signal amplitude, the default is 1.                          */
    FP32    Frq;                                                /**< Signal frequency, the default is 1.                          */
    INT16U  Units;                                              /**< Signal units Hertz or rad/s, the default is Hertz.           */
    FP32    SmpTm;                                              /**< Sample time [s].                                             */
    FP32    Out;                                                /**< Signal generator output.                                     */
    FP32    Perd;                                               /**< Signal time period [s].                                      */
    FP32    Omega;                                              /**< Angular frequency [rad/s].                                   */
    FP32    Tm;                                                 /**< Elapsed time [s], 0 <= Tm < Perd.                            */

                                                                /*   Sawtooth wave form generation: ...                           */
    FP32    Slope;                                              /**< Signal slope for sawtooth wave form generation.              */
    FP32    Offset;                                             /**< Signal offset for sawtooth wave form generation.             */

                                                                /*   Random wave form gneration: ...                              */
    INT16U  LFSR;                                               /**< Linear feedback shift register.                              */
    INT16U  Len;                                                /**< Linear feedback shift register length (polynomial degree).   */
    INT16U  NbrFdbkStages;                                      /**< Number of feedback stages.                                   */
    INT16U  TapTbl[4];                                          /**<    */
} SRC_SIG_GEN_FP32_T;

/*
************************************************************************************************************************************
*                                                         GLOBAL VARIABLES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                              MACROS
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                       FUNCTION PROTOTYPES
************************************************************************************************************************************
*/

/** Parameterization 32-bit precision signal generator
 *
 *  @param      p           Pointer to a 32-bit precision signal generator instance.
 *  @param      wave_form   Specify the wave form: sine, cosine, square, saw tooth or radom. The parameter cannot change while a
 *                          signal generation is running.
 *  @param      amp         Specify the signal amplitude.
 *  @param      frq         Specify signal frequency. The value must be greater than zero.
 *  @param      units       Specify the signal units as Hertz or rad/sec.
 *  @param      smp_tm      Specify sample time [s]. The value must be greater than zero.
 *  @returns                ERR_EINVAL if wave form is invalid, ERR_ERANGE if frequency or sample time is out of range and 0 if
 *                          successful.
 */
INT16U Src_SigGenParamFP32(SRC_SIG_GEN_FP32_T *p, const INT16U wave_form, const FP32 amp, const FP32 frq, const INT16U units,
                           const FP32 smp_tm);

/** 32-bit precision signal generator
 *
 *  The signal generator is used to produce various signal waveforms. The signal generator can produce one of five different wave
 *  forms:
 *
 *      1. Sine wave
 *      2. Cosine wave
 *      3. Square wave
 *      4. Saw tooth wave
 *      5. Random wave
 *
 *  @param      p       Pointer to a 32-bit precision signal generator instance.
 *  @returns            various wave forms y(t) = Amp * Waveform(f, t).
 */
FP32 Src_SigGenFP32(SRC_SIG_GEN_FP32_T *p);

/** Reset 32-bit precision signal generator
 *
 *  The function resets the signal generator runtime and zeroize the output.
 *
 *  @param      p       Pointer to a 32-bit precision signal generator instance.
 */
void Src_SigGenResetFP32(SRC_SIG_GEN_FP32_T *p);

/** Set signal amplitude of 32-bit precision signal generator
 *
 *  The function is used to change the signal amplitude during a signal generation is running. A change in amplitude sign causes an
 *  additional phase shift of 180. Normally the function is called in a higher-level sequencer for a event-driven change of the
 *  signal amplitude without complete re-parameterization and initialization of the signal generator.
 *
 *  For saw tooth signal generation the characteristic internal parameters slope and offset were automatically updated.
 *
 *  For continuous amplitude manipulation the same result can be obtained by multiplying the signal generator output directly. This
 *  procedure allows an elegant and simple amplitude modulation with minimum runtime overhead.
 *
 *  @param      p       Pointer to a 32-bit precision signal generator instance.
 *  @param      amp     Signal amplitude.
 */
void Src_SigGenSetAmpFP32(SRC_SIG_GEN_FP32_T *p, const FP32 amp);

/*
************************************************************************************************************************************
*                                                 EXTERNAL C LANGUAGE LINKAGE END
************************************************************************************************************************************
*/

#ifdef __cplusplus
}                                                               /* End of 'extern'al C language linkage.                          */
#endif

/*
************************************************************************************************************************************
*                                                            MODULE END
************************************************************************************************************************************
*/

#endif                                                          /* End of SRC.H header file.                                      */
