/**
 * @file    tsg.h
 * @brief   Torque signal generator
 *
 * @par     Copyright:
 *
 *          Thermo Electron (Karlsruhe) GmbH, Dieselstrasse 4, 76227 Karlsruhe, Germany
 *
 *          (c) Copyright 2010, Thermo Fisher Scientific
 *
 *          All rights reserved. Thermo Fisher Scientific source code is an unpublished work and the use of a copyright notice does
 *          not imply otherwise. This source code contains confidential, trade secret material of Thermo Fisher Scientific. Any
 *          attempt or participation in deciphering, decoding, reverse engineering or in any way altering the source code is
 *          strictly prohibited, unless the prior written consent of Thermo Fisher Scientific is obtained.
 *
 * @par     Creation Date:
 *
 *          22-Jan-2010
 *
 * @par     Last Modification:
 *
 *          07-Apr-2010
 *
 * @par     Project:
 *
 *          MARS III
 *
 * @par     Editor:
 *
 *          Microsoft Visual C++ 6.0
 *
 * @par     Compiler:
 *
 *          Microsoft Visual C++ 6.0
 *
 * @author  Uwe Bauer
 *
 * @par     Revision History:
 */

#ifndef TSG_H
#define TSG_H

/*
 **********************************************************************************************************************************
 *                                                           INCLUDES                                                             *
 **********************************************************************************************************************************
 */

//#include "typedefs.h"

/*
 **********************************************************************************************************************************
 *                                                           CONSTANTS                                                            *
 **********************************************************************************************************************************
 */

#define TSG_STOP  ((Uint16) 0)                       /**< Stop timer and torque amplitude modulation.                              */
#define TSG_START ((Uint16) 1)                       /**< Start timer and torque amplitude modulation.                             */

/*
 **********************************************************************************************************************************
 *                                                       GLOBAL DATA TYPES                                                        *
 **********************************************************************************************************************************
 */

/**
 * Torque signal generator device object.
 */
typedef struct tsg_dev {
    float32    amp;                                    /**< Normalized amplitude.                                                    */
    float32    trq;                                    /**< Actual torque value in Nm.                                               */
    float32   *pTrqSineTbl;                            /**< Pointer to the torque sine table.                                        */
    Uint32  n;                                      /**< Table size which is a power of 2 and greater than 4.                     */
    Uint32  k;                                      /**< Common index to output tabulated sine values.                            */
    Uint32  inc;                                    /**< Table increment which is a power of 2.                                   */
    Uint16   mode;                                   /**< Operating mode of the amplitude modulator.                               */
    float32    dt;                                     /**< Time icrement in seconds.                                                */
    float32    t;                                      /**< Elapsed time in seconds.                                                 */
    //float32  (*ampModFun)(float32 *, float32);               /**< Function to execute for torque amplitude modulation.                     */
    //float32   *ampModArg;                              /**< Pointer to argument tp pass to 'ampModFun'.                              */
} TSG_DEV;

/*
 **********************************************************************************************************************************
 *                                                   GLOBAL FUNCTION PROTOTYPES                                                   *
 **********************************************************************************************************************************
 */


// 22.11.13 setting actual torque amplitude
void TsgSetTrqAmp (TSG_DEV *pDev, float32 fAmp);


/**
 * Configure the sine table for a torque signal generator.
 *
 * Normally this function is called after system start up. For valid parameter set the tabulated torque values were automatically
 * calculated. In the case of invalid parameter set the function do nothing and returns an error number.
 *
 * @param   pDev        Pointer to the used torque signal generator device instance.
 * @param   pTrqSineTbl Pointer to the torque sine table.
 * @param   n           Table size which is a power of 2 and greater than 4.
 * @returns             0 if successful otherwise 1.
 */
Uint16 TsgCfgSineTbl(TSG_DEV *pDev, float32 *pTrqSineTbl, Uint32 n);

/**
 * Set the increment to output the tabulated torque sine values.
 *
 * With the increment 1 every tabulated torque value is output.
 *
 * @param   pDev        Pointer to the used torque signal generator device instance.
 * @param   inc         Increment is at minimum 1. In the case were the increment is greater than 1 it must be a power of 2 and
 *                      smaller than the table size.
 * @returns             0 if successful otherwise 1.
 */
Uint16 TsgSetSineTblInc(TSG_DEV *pDev, Uint32 inc);

/**
 * The function set the timer resolution.
 *
 * @param   pDev        Pointer to the used torque signal generator device instance.
 * @param   res         Timer resolution in seconds. The timer resolution must be greater than zero.
 * @returns             0 if successful otherwise 1.
 */
Uint16 TsgSetTmrRes(TSG_DEV *pDev, float32 res);

/**
 * Reset of the torque signal generator.
 *
 * The function resets the sine table count, timer and changes the operating mode to idle.
 *
 * @param   pDev        Pointer to the used torque signal generator device instance.
 */
void TsgReset(TSG_DEV *pDev);

/**
 * Configure the amplitude modulator for a torque signal generator.
 *
 * This function is used to configure a amplitude modulation function associated with a torque signal generator.
 *
 * @param   pDev        Pointer to the used torque signal generator device instance.
 * @param   fun         Pointer to the amplitude modulation function.
 * @param   arg         Pointer to the arguments needed by the modulator.
 */
//void TsgCfgAmpMod(TSG_DEV *pDev, float32 (*fun)(float32 *arg, float32), float32 *arg);

/**
 * Start / stop the timer and torque amplitude modulator.
 *
 * In the stop mode the signal generator produces a torque sine wave with constant amplitude. In the start mode the timer runs free
 * and the signal generator produces a torque sine wave with modulated amplitude specified by the given modulator function.
 *
 * @param   pDev        Pointer to the used torque signal generator device instance.
 * @param   mode        Operating mode of the torque amplitude modulator.
 * @returns             0 if successful otherwise 1.
 */
Uint16 TsgStartMod(TSG_DEV *pDev, Uint16 mode);

/**
 * Calculate sine torque signal.
 *
 * @param   pDev        Pointer to the used torque signal generator device instance.
 * @returns             the torque in Nm.
 */
float32 TsgCalcTrq(TSG_DEV *pDev);

/**
 * Read actual torque amplitude.
 *
 * @param   pDev        Pointer to the used torque signal generator device instance.
 * @returns             the torque amplitude in Nm.
 */
float32 TsgGetTrqAmp(TSG_DEV *pDev);



/**
 * Heaviside amplitude modulation function
 *
 *  M(t) = c[0] for t greater or equal 0.
 *
 * @param   pCoefTbl    Pointer to the used coefficient table c.
 * @param   t           Elapsed time in seconds.
 * @returns             the torque amplitude.
 */
float32 TsgHeavisideFun(float32 *pCoefTbl, float32 t);



#endif  /* TSG_H    */
