/* ADS8329.c */
/* K. Schneider */
/* EDC */
/* Driver for ADS8329 */

//*****************************************************************************
/*! \addtogroup Treiber_ADS8329
 *! @{
 */
//*****************************************************************************
/*! \file ADS8329.c
 *
 * 	\brief Treiber fuer ADS8329
 *
 *  Created on: 04.01.2012
 *      Author: klaus.schneider
 * 		Company: EDC
 */
#include <math.h>           /* Because of trigonometric functions                     */
#include "project.h"		// all global things as Type definitions, Hardware def., ...
#include "Diagnos.h"
#include "ADS8329.h"
#include "Timer.h"


// Time-critical functions should run from RAM. Therefor they were
// loaded at Runtime from Flash to the defined RAM-Memory named "ramfuncs"
// Place sdata and rdata buffers in DMA-accessible RAM (L4 for this example)
#ifdef FLASH
#pragma CODE_SECTION(sin, "ramfuncs");
#pragma CODE_SECTION(cos, "ramfuncs");
#pragma CODE_SECTION(atan, "ramfuncs");
#pragma CODE_SECTION(sqrt, "ramfuncs");
#pragma CODE_SECTION(atan2, "ramfuncs");
//#pragma CODE_SECTION(floor, "ramfuncs");
//#pragma CODE_SECTION(modf, "ramfuncs");
#pragma CODE_SECTION(DmaCh6ReceiveCompleteIsr, "ramfuncs")
#pragma DATA_SECTION(ads_senddata, "DMARAML4")
#pragma DATA_SECTION(motor_current, "DMARAML4")
#else
#pragma DATA_SECTION(ads_senddata, "DMARAML4")
#pragma DATA_SECTION(motor_current, "DMARAML4")
#endif

union MOTORCURRENT_CONT motor_current;//!< Array, in welches der DMA die empfangenen Daten vom ADS hin speichert
unsigned int ads_senddata = ADS8329_CMR_READ_DATA;//!< Was gesendet werden muss, damit man die Daten lesen kann

volatile Uint16 usADS8329DataRec = 0;//!< Flag, welches gesetzt wird, wenn der DMA die �bertragung abgeschlossen hat

/*!	\def ADS8329_DEV_1_INIT
 * 	\brief Konfiguration des ADS8329, welcher in der Daisy Chain an erster Stelle steht
 *
 * 	F�r die Daisy Chain siehe Figure 59 im Datenblatt.
 * 
 * 	Zur Konfiguration siehe das Datenblatt ab Seite 31 und Table 4.
 */
#define ADS8329_DEV_1_INIT		(ADS8329_CFR_MANUAL_CHANNEL + \
								ADS8329_CFR_INTERNAL_OSC + \
								ADS8329_CFR_MANUAL_TRIGGER + \
								ADS8329_CFR_EOC_INT_ACTIV_LOW + \
								ADS8329_CFR_PIN_EOC + \
								ADS8329_CFR_PIN_EOC_INT_OUTPUT + \
								ADS8329_CFR_AT_NP_PWR_DOWN_DISABLED + \
								ADS8329_CFR_DIS_DEV_NAP_PWR_DOWN + \
								ADS8329_CFR_DIS_DEV_DEEP_PWR_DOWN + \
								ADS8329_CFR_DIS_TAG_BIT + \
								ADS8329_CFR_NORMAL_OPERATION)

/*!	\def ADS8329_DEV_23_INIT
 * 	\brief Konfiguration des ADS8329, welcher in der Daisy Chain an zweiter
 * 	und dritter Stelle steht
 */
#define ADS8329_DEV_23_INIT		(ADS8329_CFR_MANUAL_CHANNEL + \
								ADS8329_CFR_INTERNAL_OSC + \
								ADS8329_CFR_MANUAL_TRIGGER + \
								ADS8329_CFR_EOC_INT_ACTIV_LOW + \
								ADS8329_CFR_PIN_EOC + \
								ADS8329_CFR_PIN_CDI_INPUT + \
								ADS8329_CFR_AT_NP_PWR_DOWN_DISABLED + \
								ADS8329_CFR_DIS_DEV_NAP_PWR_DOWN + \
								ADS8329_CFR_DIS_DEV_DEEP_PWR_DOWN + \
								ADS8329_CFR_DIS_TAG_BIT + \
								ADS8329_CFR_NORMAL_OPERATION)



//*****************************************************************************
/*! \brief DMA Mcbsp-Receive-Complete Interrupt
 *
 *	author david.berthold
 *	
 *	Es wird nur das Flag gesetzt, welches Anzeigen soll, dass Daten �bernommen
 * 	wurden.
 */
//*****************************************************************************
interrupt void DmaCh6ReceiveCompleteIsr(void)		// DMA Ch6
{
	usADS8329DataRec = 1;
	// NOP for Breakpoint
	//asm(" nop");
	//GpioDataRegs.GPBTOGGLE.bit.GPIO49 = 1;
	// To receive more interrupts from this PIE group, acknowledge this interrupt	
	PieCtrlRegs.PIEACK.all = PIEACK_GROUP7;
	//PieCtrlRegs.PIEACK.all = 0xffff;
}


//*****************************************************************************
/*! \brief Initialsierung der drei ADS8329
 *
 *	Es erfolgt die Initialisierung des McBSP-B Moduls f�r Komunikation mit
 * 	Softwaregenerierung der CS-Signale, die Konfiguration der ADS gesendet,
 * 	zur�ckgelesen und verglichen. Danach erfolgt die Umkonfigruation des McBSPB
 * 	in DaisyChain mit Generierung der CS-Signale, die Initialisierung des DMA
 * 	und die Konfiguration f�r die Generierung des Conv.Start Signals.
 */
//*****************************************************************************
Uint16 ADCInit(void)
{
	ADS8329ConfigRegister_cont cCR;
	
	McbspBInit();
	
	//Schreiben der Konfiguration an die ADCs
	ADS8329Write(ADS8329_CMR_WRITE_CFR | ADS8329_DEV_23_INIT, ADS8329_DEV_2);
	ADS8329Write(ADS8329_CMR_WRITE_CFR | ADS8329_DEV_1_INIT, ADS8329_DEV_1);
	
	ADS8329Write(ADS8329_CMR_IPC_0, ADS8329_DEV_2);
	ADS8329Write(ADS8329_CMR_IPC_0, ADS8329_DEV_1);
		
	McbspBInitDaisyChain();
	//Umkonfiguration der CS-Leitungen auf McBSP-Kontrolle
	EALLOW;	
	GpioCtrlRegs.GPAMUX1.bit.GPIO15 = 3;		// GPIO15 is MFSXB pin
	GpioCtrlRegs.GPAMUX2.bit.GPIO27 = 3;		// GPIO27 is MFSXB pin
	EDIS;
	
	//Zur Kontrolle Konfiguration noch einmal auslesen
	ADS8329ReadCFRDaisyChain(&cCR);
	//Check the validity
	if(((cCR.ConfigReg1 & 0x0FFF) != ADS8329_DEV_1_INIT)\
		|| ((cCR.ConfigReg2 & 0x0FFF) != ADS8329_DEV_23_INIT)\
		|| ((cCR.ConfigReg3 & 0x0FFF) != ADS8329_DEV_23_INIT)){
		// error receive Configuration is not the sent Configuration
		return MC_ADC_RECCONF;
	}
	
	DmaCh5Ch6InitAds8329();
	
	//DMA Transmision Complete Interrupt
	EALLOW;
	PieVectTable.DINTCH6 = &DmaCh6ReceiveCompleteIsr;
	EDIS;
	IER |= M_INT7;
	PieCtrlRegs.PIEIER7.bit.INTx6 = 1;

	DmaCh5Ch6StartAds8329();
	// no eror
	return MC_NOERROR;
}


// Read Config Register
//*****************************************************************************
/*! \brief Einlesen des Konfigurationsregisters
 *
 *	Einlesen des Konfigurationsregisters aller 3 ADS8329 mit DaisyChain und
 * 	Softwaregenerierung der CS-Signale
 * 
 * \return		: Registerwert des letzten ADS8329	  
 */
//*****************************************************************************
void ADS8329ReadCFRDaisyChain(ADS8329ConfigRegister_cont* p_CfgReg)
{
//	ADS1_CS_L();
//	ADS23_CS_L();


	McbspbRegs.DXR1.all = ADS8329_CMR_READ_CFR;				// Write Command	
	p_CfgReg->ConfigReg3 = McbspBRead16();

	//McbspBWrite16(ADS8329_CMR_READ_CFR);				// Write Command
	p_CfgReg->ConfigReg2 = McbspBRead16();

	//McbspBWrite16(ADS8329_CMR_READ_CFR);				// Write Command
	p_CfgReg->ConfigReg1 = McbspBRead16();

//	ADS1_CS_H();
//	ADS23_CS_H();
}

//*****************************************************************************
/*! \brief Schreiben des Konfigurationsregisters eines ADS8329
 *
 * 	\param data	: Registerwert
 * 	\param dev  : 1 -> CS ADS8329 Nr. 1, 2(3) -> ADS8329 Nr. 2 und 3
 * 
 * 	Es wird die Konfiguration an den jeweiligen ADS gesendet, dadurch das Nr. 
 * 	2 und 3 die gleichen CS-Leitungen besitzen, k�nnen Sie nur mit dem gleichen
 * 	Wert konfiguriert werden
 */
//*****************************************************************************
void ADS8329Write(unsigned int data, unsigned char dev)
{
	if ( dev == ADS8329_DEV_1 )
	{
		ADS1_CS_L();

		McbspBWrite16(data);					// Write Command
		McbspBRead16();			// Read Dummy

		ADS1_CS_H();
	}

	if (( dev == ADS8329_DEV_2 ) || (dev == ADS8329_DEV_3))
	{
		ADS23_CS_L();

		McbspBWrite16(data);					// Write Command
		McbspBRead16();			// Read Dummy

		ADS23_CS_H();
	}

}

//*****************************************************************************
/*! \brief Initfunktion McBSP B 
 *
 *	Initialsierung des McBSP Modul B des TMS320F28335 fuer Kommunikation mit
 * 	den externen AD-Wandlern (ADS8329)	
 */
//*****************************************************************************
void McbspBInit(void)
{
	EALLOW;
	// Step 1
    McbspbRegs.SPCR2.bit.XRST = 0;		// Transmitter Reset
    McbspbRegs.SPCR1.bit.RRST = 0;		// Receiver Reset

	// Step 2
	McbspbRegs.SPCR2.bit.GRST = 0;		// Sample Generator Reset

	// Step3
	McbspbRegs.SPCR1.bit.DLB = 0;       // Disable Digital loopback mode
	McbspbRegs.SPCR1.bit.CLKSTP = 0x3; 	// Clock stop mode bits
	//McbspbRegs.SPCR1.bit.RINTM = 0x0;
	
	//McbspbRegs.SPCR2.bit.XINTM = 0x0;
	
	McbspbRegs.RCR1.bit.RWDLEN1 = 0x2;	//16bits
	McbspbRegs.RCR1.bit.RFRLEN1 = 0x0;	//1 Framelaenge
	
	//McbspbRegs.RCR2.bit.RDATDLY = 0x1;	// Receive data delay bits
	McbspbRegs.RCR2.bit.RPHASE = 0x0;	// 1 Phase
	
	McbspbRegs.XCR1.bit.XWDLEN1 = 0x2;	//16bits
	McbspbRegs.XCR1.bit.XFRLEN1 = 0x0;	//1 Framelaenge
	//McbspbRegs.XCR1.bit.XFRLEN1 = 0x1;
	//McbspbRegs.XCR1.bit.XWDLEN1 = 0x0;
	
	McbspbRegs.XCR2.bit.XPHASE = 0x0;	// 1 Phase
	//McbspbRegs.XCR2.bit.XDATDLY = 0x1;	// Transmit data delay bits
	
	
	McbspbRegs.SRGR1.bit.FWID = 2;		//Frame-synchronization pulse 2
//	McbspbRegs.SRGR1.bit.CLKGDV = 2;	//MCLK = (LOSP_FRQ)/ (CLKGDV + 1)
										//= 25MHZ
	McbspbRegs.SRGR1.bit.CLKGDV = 4;	//MCLK = (LOSP_FRQ)/ (CLKGDV + 1)
										//= 15MHZ
	
	McbspbRegs.SRGR2.bit.FSGM = 0;		// Sample rate generator transmit
	McbspbRegs.SRGR2.bit.CLKSM = 1;		// Sample rate generator input clock mode bit
	
	McbspbRegs.PCR.bit.FSXP = 1;		// Transmit frame-synchronization polarity bit
	McbspbRegs.PCR.bit.FSXM = 1;		// Transmit frame-synchronization mode bit
	McbspbRegs.PCR.bit.CLKXP = 0x1;		// Transmit clock polarity bit
	McbspbRegs.PCR.bit.CLKRP = 0x1;		// Receive clock polarity bit
	McbspbRegs.PCR.bit.CLKXM = 0x1;		// Transmit clock mode bit
	McbspbRegs.PCR.bit.SCLKME = 0;		// Sample rate generator input clock mode bit
	
	// Step 4
	McbspbRegs.SPCR2.bit.GRST = 1;		// Start Sample Generator Reset

	// Step 5
	DELAY_US(1L);
    McbspbRegs.SPCR2.bit.XRST = 1;		// Enable Transmitter
    McbspbRegs.SPCR1.bit.RRST = 1;		// Enable Receiver
	DELAY_US(1L);

	// Step 6
	McbspbRegs.SPCR2.bit.FRST = 1;		// Start Frame-Synchronization Logic
	
	//maybe delete
	McbspbRegs.SPCR2.bit.FREE = 1;		//Continue, when Debugger Halt Processor
	McbspbRegs.SPCR2.bit.SOFT = 1;
}

//*****************************************************************************
/*! \brief Initfunktion McBSP B 
 *
 *	Initialsierung des McBSP Modul B des TMS320F28335 fuer Kommunikation mit
 * 	ADS8329. Dabei wird die Sendeframel�nge auf 3 gesetzt, wodurch das
 * 	generierte CS-Signale des McBSP-Ports (MFSXB) f�r 3 16Bit Daten auf Low
 * 	bleibt.
 */
//*****************************************************************************
void McbspBInitDaisyChain(void)
{
	EALLOW;
	// Step 1
    McbspbRegs.SPCR2.bit.XRST = 0;		// Transmitter Reset
    McbspbRegs.SPCR1.bit.RRST = 0;		// Receiver Reset

	// Step 2
	McbspbRegs.SPCR2.bit.GRST = 0;		// Sample Generator Reset

	// Step3
	McbspbRegs.RCR1.bit.RWDLEN1 = 0x2;	//16bits
	McbspbRegs.RCR1.bit.RFRLEN1 = 0x2;	//1 Framelaenge
	
	McbspbRegs.XCR1.bit.XWDLEN1 = 0x2;	//16bits
	McbspbRegs.XCR1.bit.XFRLEN1 = 0x2;	//1 Framelaenge
	
	// Step 4
	McbspbRegs.SPCR2.bit.GRST = 1;		// Start Sample Generator Reset

	// Step 5
	DELAY_US(1L);
    McbspbRegs.SPCR2.bit.XRST = 1;		// Enable Transmitter
    McbspbRegs.SPCR1.bit.RRST = 1;		// Enable Receiver
	DELAY_US(1L);

	// Step 6
	McbspbRegs.SPCR2.bit.FRST = 1;		// Start Frame-Synchronization Logic
}

//*****************************************************************************
/*! \brief Schreiben von 16 Bit ueber McBSPB an ADS8329
 *
 *	\param data  : zu uebertragendes Datenwort
 * 
 * 	Es wird das �bergebenen halfword in das Senderegister geschrieben, wodurch
 * 	automatisch diese Daten gesendet werden. Es wird noch gewartet bis die
 * 	Daten erfolgreich versendet wurden.  
 */
//*****************************************************************************
void McbspBWrite16(unsigned int data)
{
	// 1x 16 Bit senden -> WORD WITDH 16, 1 FRAME
	// 16 Bit Schreiben
	// Write buffer l�schen
				
	McbspbRegs.DXR1.all = data;						// 16-bit value to send
	while(McbspbRegs.SPCR1.bit.RRDY != 1 ) { }		// Check for transmit

}

//*****************************************************************************
/*! \brief Lesen von 16 Bit ueber McBSP B
 *
 * 	Es wird zun�chst gewartet bis Daten empfangen wurden.
 *	\return empfangenes Datenwort  
 */
//*****************************************************************************
unsigned int McbspBRead16(void)
{	 
	while(McbspbRegs.SPCR1.bit.RRDY != 1 ) { }		// Check for receive
	return McbspbRegs.DRR1.all;						// read DRR1
}

//*****************************************************************************
/*! \brief Initialisierung der GPIOs f�r den McBSPB-Port
 *
 */
//*****************************************************************************
void InitMcbspbGpio(void)
{
    EALLOW;
/* Configure McBSP-A pins using GPIO regs*/
// This specifies which of the possible GPIO pins will be McBSP functional pins.
// Comment out other unwanted lines.

	//GpioCtrlRegs.GPAMUX1.bit.GPIO12 = 3;	// GPIO12 is MDXB pin (Comment as needed)
	GpioCtrlRegs.GPAMUX2.bit.GPIO24 = 3;	// GPIO24 is MDXB pin (Comment as needed)

	GpioCtrlRegs.GPAMUX2.bit.GPIO25 = 3;	// GPIO25 is MDRB pin (Comment as needed)

	GpioCtrlRegs.GPAMUX2.bit.GPIO26 = 3;	// GPIO26 is MCLKXB pin (Comment as needed)

	//GpioCtrlRegs.GPAMUX1.bit.GPIO3 = 3;		// GPIO3 is MCLKRB pin (Comment as needed)
	//GpioCtrlRegs.GPBMUX2.bit.GPIO60 = 1;	// GPIO60 is MCLKRB pin (Comment as needed)
	//GpioCtrlRegs.GPAMUX1.bit.GPIO15 = 3;	// GPIO15 is MFSXB pin (Comment as needed)
	//F�r die Init muss es ein GPIO sein, damit die CS f�r die einzelnen ADS
	//separat angesprochen werden k�nnen
	GpioCtrlRegs.GPAMUX1.bit.GPIO15 = 0;	// GPIO15 is GPIO pin (Comment as needed)
	GpioCtrlRegs.GPADIR.bit.GPIO15 = 1;

	GpioCtrlRegs.GPAMUX2.bit.GPIO27 = 0;	// GPIO27 is GPIO pin (Comment as needed)
	GpioCtrlRegs.GPADIR.bit.GPIO27 = 1;

	//GpioCtrlRegs.GPAMUX1.bit.GPIO1 = 3;		// GPIO1 is MFSRB pin (Comment as needed)
	//GpioCtrlRegs.GPBMUX2.bit.GPIO61 = 1;	// GPIO61 is MFSRB pin (Comment as needed)

/* Enable internal pull-up for the selected pins */
// Pull-ups can be enabled or disabled by the user.
// This will enable the pullups for the specified pins.
// Comment out other unwanted lines.
	GpioCtrlRegs.GPAPUD.bit.GPIO24 = 0;	    // Enable pull-up on GPIO24 (MDXB) (Comment as needed)

	GpioCtrlRegs.GPAPUD.bit.GPIO25 = 0;	    // Enable pull-up on GPIO25 (MDRB) (Comment as needed)

	GpioCtrlRegs.GPAPUD.bit.GPIO26 = 0;	    // Enable pull-up on GPIO26 (MCLKXB) (Comment as needed)

	//GpioCtrlRegs.GPAPUD.bit.GPIO3 = 0;		// Enable pull-up on GPIO3 (MCLKRB) (Comment as needed)
	//GpioCtrlRegs.GPBPUD.bit.GPIO60 = 0;	// Enable pull-up on GPIO60 (MCLKRB) (Comment as needed)
	GpioCtrlRegs.GPAPUD.bit.GPIO27 = 0;	    // Enable pull-up on GPIO27 (MFSXB) (Comment as needed)
	GpioCtrlRegs.GPAPUD.bit.GPIO15 = 0;	// Enable pull-up on GPIO15 (MFSXB) (Comment as needed)
	//GpioCtrlRegs.GPAPUD.bit.GPIO1 = 0;		// Enable pull-up on GPIO1 (MFSRB) (Comment as needed)
	//GpioCtrlRegs.GPBPUD.bit.GPIO61 = 0;	// Enable pull-up on GPIO61 (MFSRB) (Comment as needed)


/* Set qualification for selected input pins to asynch only */
// This will select asynch (no qualification) for the selected pins.
// Comment out other unwanted lines.

	GpioCtrlRegs.GPAQSEL2.bit.GPIO25 = 3;   // Asynch input GPIO25 (MDRB) (Comment as needed)
    GpioCtrlRegs.GPAQSEL2.bit.GPIO26 = 3;   // Asynch input GPIO26(MCLKXB) (Comment as needed)
 	// END OF CONVERSION for ADS8329
	GpioDataRegs.GPBSET.bit.GPIO61 = 1;
	GpioCtrlRegs.GPBDIR.bit.GPIO61 = 0;			// END OF CONVERSION for ADS8329 (Input)
	GpioCtrlRegs.GPBQSEL2.bit.GPIO61 = 0;        // Xint1 Synch to SYSCLKOUT only
	
	// GPIO61 is XINT3
	GpioIntRegs.GPIOXINT3SEL.bit.GPIOSEL = 61 - 32;	// Xint3 is GPIO61
	// Configure XINT3
	XIntruptRegs.XINT3CR.bit.POLARITY = 1;      	// Rising edge interrupt
	// Enable XINT3
	XIntruptRegs.XINT3CR.bit.ENABLE = 1;        // Enable Xint3 
	// CONVST for ADS8329 siehe DSP2833x_EPwm.c
	EDIS;
	
	//CS alle auf H
	ADS1_CS_H();
	ADS23_CS_H();
}

//*****************************************************************************
/*! \brief Initialisierung der DMA-Kan�le 5 und 6
 *
 *	author david.berthold
 *	
 *	Der DMA-Kanal 5 wird vom externen Interrupt 3 getriggert, welcher vom 
 * 	EndOfConversion des ADS Nr.1 generiert wird. Dieser Schreibt dann das
 * 	Datenempfangskommando an den McBSPB.
 * 
 * 	Der DMA-Kanal 6 wird vom McBSPB Receive getriggert und schreibt die Daten
 * 	in die union motor_current. Wo die Ergebnisse der Stromwandlung ausgelesen
 * 	werden k�nnen.
 * 
 * 	Der DMA-Kanal 6 l�st auch einen Interrupt aus, welcher event. noch entfernt
 * 	wird.
 */
//*****************************************************************************
void DmaCh5Ch6InitAds8329(void){
	EALLOW;
  // 26.11.12 now in own function DmaRegs.DMACTRL.bit.HARDRESET = 1;
  //asm(" NOP");						   // Only 1 NOP needed per Design
  
  DmaRegs.CH5.MODE.bit.CHINTE = 0;
  // Channel 5, McBSPB transmit
  DmaRegs.CH5.BURST_SIZE.all = 0;		// 1 word/burst
  DmaRegs.CH5.SRC_BURST_STEP = 0;		// no effect when using 1 word/burst
  DmaRegs.CH5.DST_BURST_STEP = 0;		// no effect when using 1 word/burst
  DmaRegs.CH5.TRANSFER_SIZE = 0;		// Interrupt every frame (127 bursts/transfer)
  DmaRegs.CH5.SRC_TRANSFER_STEP = 0;	// Move to next word in buffer after each word in a burst
  DmaRegs.CH5.DST_TRANSFER_STEP = 0;	// Don't move destination address
  DmaRegs.CH5.SRC_ADDR_SHADOW = (Uint32) &ads_senddata;			// Start address = buffer
  DmaRegs.CH5.SRC_BEG_ADDR_SHADOW = (Uint32) &ads_senddata;		// Not needed unless using wrap function
  DmaRegs.CH5.DST_ADDR_SHADOW = (Uint32) &McbspbRegs.DXR1.all;		// Start address = McBSPB DXR
  DmaRegs.CH5.DST_BEG_ADDR_SHADOW = (Uint32) &McbspbRegs.DXR1.all;	// Not needed unless using wrap function
  DmaRegs.CH5.CONTROL.bit.PERINTCLR = 1;	// Clear peripheral interrupt event flag
  DmaRegs.CH5.CONTROL.bit.SYNCCLR = 1;		// Clear sync flag
  DmaRegs.CH5.CONTROL.bit.ERRCLR = 1;	// Clear sync error flag
  DmaRegs.CH5.DST_WRAP_SIZE = 0xFFFF;		// Put to maximum - don't want destination wrap
  DmaRegs.CH5.SRC_WRAP_SIZE = 0xFFFF;		// Put to maximum - don't want source wrap
  DmaRegs.CH5.MODE.bit.SYNCE = 0;         		// No sync signal
  DmaRegs.CH5.MODE.bit.SYNCSEL = 0;       		// No sync signal
  DmaRegs.CH5.MODE.bit.CHINTE = 0;			// Disable channel interrupt
  DmaRegs.CH5.MODE.bit.PERINTE = 1;			// Enable peripheral interrupt event
  DmaRegs.CH5.MODE.bit.PERINTSEL = DMA_XINT3;		// Peripheral interrupt select = McBSP MXSYNCA
  DmaRegs.CH5.MODE.bit.CONTINUOUS = 1;		//Continius Mode an
  DmaRegs.CH5.CONTROL.bit.PERINTCLR = 1;  		// Clear any spurious interrupt flags
  

  // Channel 6, McBSPB Receive
  DmaRegs.CH6.MODE.bit.CHINTE = 0;
  DmaRegs.CH6.BURST_SIZE.all = 0;		// 1 word/burst
  DmaRegs.CH6.SRC_BURST_STEP = 0;		// no effect when using 1 word/burst
  DmaRegs.CH6.DST_BURST_STEP = 0;		// no effect when using 1 word/burst
#if((CURRENT_MEASURE_UP_DOWN == CURRENT_MEASURE_UP) || (CURRENT_MEASURE_UP_DOWN == CURRENT_MEASURE_BOTH))
  DmaRegs.CH6.TRANSFER_SIZE = CURRENT_MEASURE_TIMES_BETWEEN_FOC*(CURRENT_MEASURE_UP_DOWN+1)*3 - 1;		// Interrupt every 9 bursts/transfer
#endif
#if CURRENT_MEASURE_UP_DOWN == CURRENT_MEASURE_DOWN
  DmaRegs.CH6.TRANSFER_SIZE = CURRENT_MEASURE_TIMES_BETWEEN_FOC*3 - 1;		// Interrupt every 6 bursts/transfer
 #endif
  DmaRegs.CH6.SRC_TRANSFER_STEP = 0;	// Don't move source address
  DmaRegs.CH6.DST_TRANSFER_STEP = 1;	// Move to next word in buffer after each word in a burst
  DmaRegs.CH6.SRC_ADDR_SHADOW = (Uint32) &McbspbRegs.DRR1.all;			// Start address = McBSPA DRR
  DmaRegs.CH6.SRC_BEG_ADDR_SHADOW = (Uint32) &McbspbRegs.DRR1.all;		// Not needed unless using wrap function
  DmaRegs.CH6.DST_ADDR_SHADOW = (Uint32) &motor_current.all.array[0];		// Start address = Receive buffer (for McBSP-A)
  DmaRegs.CH6.DST_BEG_ADDR_SHADOW = (Uint32) &motor_current.all.array[0];	// Not needed unless using wrap function
  DmaRegs.CH6.CONTROL.bit.PERINTCLR = 1;	// Clear peripheral interrupt event flag
  DmaRegs.CH6.CONTROL.bit.SYNCCLR = 1;		// Clear sync flag
  DmaRegs.CH6.CONTROL.bit.ERRCLR = 1;		// Clear sync error flag
  DmaRegs.CH6.DST_WRAP_SIZE = 0xFFFF;		// Put to maximum - don't want destination wrap
  DmaRegs.CH6.SRC_WRAP_SIZE = 0xFFFF;		// Put to maximum - don't want source wrap
  DmaRegs.CH6.MODE.bit.CHINTE = 1;			// Enable channel interrupt
  DmaRegs.CH6.MODE.bit.CHINTMODE = 1;		// Interrupt at end of transfer
  DmaRegs.CH6.MODE.bit.PERINTE = 1;			// Enable peripheral interrupt event
  DmaRegs.CH6.MODE.bit.PERINTSEL = DMA_MREVTB;  // Peripheral interrupt select = McBSP MRSYNCA
  DmaRegs.CH6.MODE.bit.CONTINUOUS = 1;		//Continius Mode an
  DmaRegs.CH6.CONTROL.bit.PERINTCLR = 1;  		// Clear any spurious interrupt flags
  
  DmaRegs.DEBUGCTRL.bit.FREE = 1;
  EDIS;
}

//*****************************************************************************
/*! \brief Start der DMA-Kan�le 5 und 6
 *
 *	author david.berthold
 *
 */
//*****************************************************************************
void DmaCh5Ch6StartAds8329 (void)
{
  EALLOW;
  DmaRegs.CH5.CONTROL.bit.RUN = 1;	         // Start DMA Transmit from McBSP-B
  DmaRegs.CH6.CONTROL.bit.RUN = 1;           // Start DMA Receive from McBSP-B
  EDIS;
}


/*!
\fn		Uint16* PtrADS8329DataRec(void)
\brief  Read ADS9329 Data-Receive-Flag
\author	K�ster
\date	22.03.2013
\param
\return	void
\details Changes:
*/
volatile Uint16* PtrADS8329DataRec(void)
{
	return &usADS8329DataRec;
}


//*****************************************************************************
/*! @}
 *	Close the Doxygen group.
 */
//*****************************************************************************

