/*!
\file	control.c
\brief	functions for control routines like PI-Controllers

\author	Koester
\date	14.10.2011
\details
	changes:	28.01.14 ControlPirPreSet not used at the moment, so commented out


*/


/* Include files ************************************************************/

#include <math.h>						/* define mathemetical functions */
#include "project.h"
#include "control.h"
#include "Device.h"
#include "Measmode.h"

/* Macro definitions ********************************************************/

/* Global Prototypes ********************************************************/

/* Global variables *********************************************************/

#ifdef FLASH
#pragma CODE_SECTION( ControlPiR, "ramfuncs");
#pragma CODE_SECTION( ControlPi, "ramfuncs");
#endif


/*!
\fn		void ControlPiRInit(void)
\brief  Reset parameters for PI-Controller with Proportional Gain
		in the Back-Loop (PIr-Controller). 

\author	Kster
\date	16.07.2013
\param[in]
\return	void

\details Changes:

*/  
void ControlPiRInit(t_stcPiRcontrol *tCrPtr)
{
	// control difference actual and old
	tCrPtr->fXd = 0.0;
	tCrPtr->fXdAlt = 0.0;
	// all output values, I-Term, I-Term-old, P-Term, Total-Output
	tCrPtr->fYkI=0;
	tCrPtr->fYkIold=0;
	tCrPtr->fYkPr = 0;
	tCrPtr->fYk = 0.0;
}


/*!
\fn		void ControlPirPreSet(t_stcPiRcontrol *tCrPtr)
\brief  Pre setting of the PI-Controller after Mode changes

\author	Kster
\date	16.07.2013
\param[in]
\return	void

\details 
	Changes: 28.01.14 not used at the moment, so commented out
*/  
#ifdef RAUS
void ControlPirPreSet(t_stcPiRcontrol *tCrPtr)
{
	// control difference actual and old
	tCrPtr->fXd = 0.0;
	tCrPtr->fXdAlt = 0;
	// Parameter CR-Regler init.
	tCrPtr->fYkI=0;
	tCrPtr->fYkIold=0;
	tCrPtr->fYkPr = 0.0;
}
#endif

/*!
\fn		float ControlPiR(t_stcPiRcontrol *tCrPtr, float p_fSetValue, float p_fMeasValue)
\brief  PI-Controller with Proportional Gain in the Back-Loop (PIr-Controller).

\author	Kster
\date	16.07.2013
\param[in]	tCrPtr -> pointer to controller data
			p_fSetValue -> actual Set-value
			p_fMeasValue -Y actual measure value

\return	Output value Yk

\details Changes:	21.12.10 Test Duration RELEASE -> rough. 1,05s

*/  
float ControlPiR(t_stcPiRcontrol *tCrPtr, float p_fSetValue, float p_fMeasValue)
{

	// Regeldifferenz berechnen
	tCrPtr->fXd = p_fSetValue - p_fMeasValue;

	// neuen P-Anteil berechnen
	tCrPtr->fYkPr = tCrPtr->fKPr * p_fMeasValue;
	
	// Anteile aufsummieren
	tCrPtr->fYk = tCrPtr->fYkI - tCrPtr->fYkPr;

	// Anti-Wind-Up measure
	if( ((tCrPtr->fYk >= tCrPtr->fYkMax) && (tCrPtr->fXd >= 0.0))
	||  ((tCrPtr->fYk <= -tCrPtr->fYkMax) && (tCrPtr->fXd <= 0.0)) )
	{
		/* I-Regler anhalten */  
		tCrPtr->fXdAlt = 0;
	}
	else
	{
		// I-Algorithmus acc. trapezodial rule
		tCrPtr->fYkI = tCrPtr->fYkIold + tCrPtr->fD0I * (tCrPtr->fXd + tCrPtr->fXdAlt);
		tCrPtr->fYk = tCrPtr->fYkI - tCrPtr->fYkPr;
		// 
		if( tCrPtr->fYk > tCrPtr->fYkMax )
		{
			tCrPtr->fYkI = tCrPtr->fYkMax + tCrPtr->fYkPr;
			tCrPtr->fYk = tCrPtr->fYkMax;
		}
		//
		if( tCrPtr->fYk < -(tCrPtr->fYkMax) )
		{
			tCrPtr->fYkI = tCrPtr->fYkPr - tCrPtr->fYkMax;
			tCrPtr->fYk = ( 0 - tCrPtr->fYkMax );
		}
		// save values for next sampling period
		tCrPtr->fYkIold = tCrPtr->fYkI;
		tCrPtr->fXdAlt = tCrPtr->fXd;
	}
	return (tCrPtr->fYk);
}



/*!
\fn		float ControlPi(t_stcPiRcontrol *tCrPtr, float p_fSetValue, float p_fMeasValue)
\brief  PI-Controller

\author	Kster
\date	16.07.2013
\param[in]	tCrPtr -> pointer to controller data

\return	Output value Yk

\details Changes:	21.12.10 Test Duration RELEASE -> rough. 1,05s

*/  
float ControlPi(tStcPiData* p_Data)
{
	// Control difference
	p_Data->fXd = p_Data->fSet - p_Data->fAct;
	// proportional output
	p_Data->fYkP = p_Data->fKp * p_Data->fXd;
	// integral output
	p_Data->fYkI = p_Data->fYkI + (p_Data->fKi * p_Data->fYkP) +(p_Data->fKc * p_Data->fSatErr);
	//
	p_Data->fYk = p_Data->fYkP + p_Data->fYkI;
	//
	p_Data->fOut = p_Data->fYk;
	//
	if( p_Data->fOut < p_Data->fOutMin)
	{
		p_Data->fOut = p_Data->fOutMin;
	}
	if( p_Data->fOut > p_Data->fOutMax)
	{
		p_Data->fOut = p_Data->fOutMax;
	}
	// saturate difference
	p_Data->fSatErr	= p_Data->fOut - p_Data->fYk;
	// return output, manipulated variable
	return (p_Data->fOut);
}
