/*!
\file	Correction.c
\brief	here we find the functions for all corrections of the measuring motor.
		This are fricton, micro Torque MTC, micr stress MSC,
		inertia and compliance.
		The basics were transfered from Mars-3 project.

\author	Koester
\date	28.01.2010
\details
		changes:
		02.05.12 Friction correction for ball bearings implemented
		22.10.12 MSC-Correction new in msc.c
		01.07.13 Friction correction with linear regression
		07.11.13 Friction correction with quadratic regression
		09.10.13 MSC AVG calculation over 5 rounds
		28.01.14 small correction in CorrLoadConstTable
		05.03.14 friction correction with spline interpolation
		06.03.14 friction correction back to point to point interpolation
		12.03.14 friction array a little bit smaller, 28 values enough
		07.09.15 Friction table length, 46 -> was 28
        09.12.15 Friction with additinal correction of temperature housing

*/

/* Include files **************************************************************/
#include "Project.h"		// all global things as Type definitions, Hardware def., ...
#include "Device.h"
#include "Uc1dsp.h"
#include "correction.h"
#include "control.h"
#include "mth.h"
#include "SsiComm.h"
#include "Measmode.h"
#include "Crmode.h"
#include "msc.h"

/* Macro definitions **********************************************************/
// for each data or constant tabel transfer we have a maximum float values
#define NO_CONST_PER_TRANSFER 14

#define MSC_TBL_LEN ((Uint16) 257)		// Micro stress table length
#define FRI_TBL_LEN ((Uint16) 46)		// Friction table length, 07.09.15 46 -> was 28
#define INE_TBL_LEN ((Uint16) 3)		// Inertia table length
#define COM_TBL_LEN ((Uint16) 2)		// Compliance table length
#define GE1_TBL_LEN ((Uint16) 33)		// Compliance table length
#define GE2_TBL_LEN ((Uint16) 33)		// Compliance table length

#define TEMP_COEFF_ALU 0.0039           // theoretical value is 0,0039 [1/K]


// copy to RAM in Flash-Mode, for faster running
#ifdef FLASH
#pragma CODE_SECTION( FrictionCorr, "ramfuncs");
#endif



/* Types **********************************************************************/

struct	// data and Correction table Inertia-Correction
{
	float32 afC[INE_TBL_LEN];		// [1]-Device, [2]-Sensor
} tInertia;


struct	// data and Correction table Fristion-Correction
{
	float32 afC[FRI_TBL_LEN];
} tFriction;

struct	// data and Correction table MSC-Correction
{
	Uint16	usState;
	Uint16	usAvgCnt;
	float32 fAvgTrqDc;
	float32 afC[MSC_TBL_LEN];			// 257
	//float32 afBuf[(MSC_TBL_LEN/2)+1];	// 129
	float32 *pafBuf;
} tMsc;

struct	// data and Correction table Inertia-Correction
{
	float32 afC[COM_TBL_LEN];		// [1]-Device, [2]-Sensor
} tCompliance;

struct	// data and Correction table G1-Correction
{
	float32 afC[GE1_TBL_LEN];
} tG1;

struct	// data and Correction table G1-Correction
{
	float32 afC[GE2_TBL_LEN];
} tG2;


/* Local Prototypes ***********************************************************/

/* Global variables ***********************************************************/

extern t_stcDev tDev;          // Details zum Geraet, device.c
// Details zum aktuellen Sensor. A-, M-Faktor....
extern t_stcSensor tSensor;
t_CorrectionMode tCorrMode;


/* Functions ******************************************************************/

/*
	\brief	This function loads a receive data table into the dedecated table.
			It could be a complete or a part of a table.
			The use of the table is marked with the Interrupt-Vector (IV), the
			Start-Index (position) in the table is marked in the ID.
			With every transfer 14 float value can be received or sent.
			The maximum table length is controlled and in case of too much values
			for the adressed table an error message will be replied!

	\Fn 	CorrLoadConstTable(void)
	\date	19.09.2012
	\author	Koester
	\param[IN] the input data will be load from the SSI Receive Buffer
	\return
	
	Changes:
*/
void CorrLoadConstTable(void)
{
	t_Const tConst;
	Uint16 usPos, usLength;
 	// pointer to constants
	void *pfData;
	// Daten aus Empfangsbuffer laden!
	GetSsiData(&tConst);
	// calculate position in table, where to load
	usPos = tConst.usID;
	// depends on IV, which table to load
	switch(tConst.usIV)
	{
		case IV_SEND_FRICTIONCONST:
		case IV_REQ_FRICTIONCONST:
			pfData = &tFriction.afC[usPos];
			usLength = FRI_TBL_LEN;
		break;
		case IV_SEND_MSCCONST:
		case IV_REQ_MSCCONST:
			pfData = &tMsc.afC[usPos];
			usLength = MSC_TBL_LEN;
		break;
		case IV_SEND_SENSCONST:
		case IV_REQ_SENSCONST:
			pfData = &tSensor.afC[usPos];
			usLength = SENCONST_TBL_LEN; 
		break;
		case IV_SEND_DEVCONST:
		case IV_REQ_DEVCONST:
			// Device Constants, like TYPE, ENCODER, ....
			pfData = &tDev.afC[usPos];
			usLength = DEVCONST_TAB_LEN;
			// several values have to be recalculated then
            tDev.fKmotor = (1.0/tDev.afC[CAL1_MOTOR]) * SQRT_2;
		break;
		case IV_SEND_G1CONST:
		case IV_REQ_G1CONST:
			pfData = &tG1.afC[usPos];
			usLength = GE1_TBL_LEN;
		break;
		case IV_SEND_G2CONST:
		case IV_REQ_G2CONST:
			pfData = &tG2.afC[usPos];
			usLength = GE2_TBL_LEN;
		break;
		default:
			// no known IV here -> Error message
			tConst.usState = CONST_E_IV;
		break;
	}
	// 28.01.2014 todo, must be multiplication?
	if( (tConst.usID+tConst.usN) > usLength )
	{
		tConst.usState = CONST_E_LENGTH;
	}
	else
	{
		// IV for writing are below 179 (130.....140)
		if( tConst.usIV < 179 )
		{
			// Datenstruct in Array kopieren
			memcpy (pfData, &tConst.af[0], tConst.usN * sizeof(float32));
		}
		// IV for reading are higher then 179 (180.....190)
		else
		{
			// Datenstruct in Array kopieren
			memcpy (&tConst.af[0], pfData, tConst.usN * sizeof(float32));
		}
		tConst.usState = CONST_OK;

	}
	// Daten als Antwort senden!
	StartSsiSendData(&tConst);
	
	// ******************** following procedures ********************

}



/*
	\brief	This function loads the correction modes.
			Every mode can be set to ON or OFF with this data.
	\Fn 	CorrLoadConstTable(void)
	\date	19.09.2012
	\author	Koester
	\param[IN] the input data will be load from the SSI Receive Buffer
	\return
	
	Changes:
*/
void CorrLoadModeTable(void)
{
	// Load all Correction Modes from Receive-Buffer of SSI!
	GetSsiData(&tCorrMode);
}


/*
\brief		Initialize Correction data after program start
			to have defined states. This data will be overwritten
			soon after program start with the stored data received
			from the master.
\Fn			SetCondCorrections()
\date		20.12.2006
\param
\return
\details	Changes:
*/
void SetCondCorrections(void)
{
	// set the length at the first member
	tFriction.afC[0] = FRI_TBL_LEN;
	// reset MSC-Correction table
	CorrResetBuffer(&tFriction.afC[0]);

	// set the length at the first member
	tG1.afC[0] = GE1_TBL_LEN;
	// reset G1-Correction table
	CorrResetBuffer(&tG1.afC[0]);

	// set the length at the first member
	tG2.afC[0] = GE2_TBL_LEN;
	// reset G1-Correction table
	CorrResetBuffer(&tG2.afC[0]);

	// set the length at the first member
	tInertia.afC[0] = INE_TBL_LEN;
	// reset MSC-Correction table
	CorrResetBuffer(&tInertia.afC[0]);

	// set the length at the first member
	tMsc.afC[0] = MSC_TBL_LEN;	
	// reset MSC-Correction table
	CorrResetBuffer(&tMsc.afC[0]);

	//tMsc.afBuf[0] = (MSC_TBL_LEN/2)+1;	// 129
	// reset MSC temporary Buffer
	//CorrResetBuffer(&tMsc.afBuf[0]);
	// and DC Offset
	tMsc.fAvgTrqDc = 0.0;

	// initialize Msc data, to work correct, when table is empty
	MscCfg(&tMscData, 2, PtrCorrMscTbl(1), PtrCorrMscTbl(1), 1);

	// todo
	tCompliance.afC[0] = 0.0;

	tMsc.usState = 0;
	tMsc.usAvgCnt = 0;

	// disable corrections at this time
	tCorrMode.usFri = 0;
	tCorrMode.usMsc = 0;
	tCorrMode.usCom = 0;
	tCorrMode.usG1G2 = 0;
}


/*!
\brief		This function is configurating the filter for Torque
			when a Friction Detection measurement is starting.
		
\fn			float32 FriCfg (float32 fFreq)
\author		K�ster, Ralf
\date		26.10.2012
\param[IN]	fFreq, frequency for Torque filter setting
\return		Correction Torque
\details	Changes:	
*/
void FriCfg(float32 fFreq)
{
	// Reset Average Torque measure data
	*PtrTorqueAVGMeasValue() = 0.0;
	//

}

void FriStartDatAq(void)
{
	t_MeasSetVal tSet;
	// Read Set Data from Receive Buffer SSI
	GetSsiData(&tSet);
	// Set FOLP-Filter to 10Hz, AVG-Filter to received Time from Master
	// tSet.fNoFreq contains the complete measuring time (e.g. 10sec)
	// calculate the AVG time from measuring time 10sec -> 0,500sec
	// 18.02.14 raus, messen mit Standardeinstellung 250Hz 250ms
	//tSet.fNoFreq = tSet.fNoFreq / 20.0;
	//TorqueSetCondCalc(F10HZ, *PtrSamplingTime(), tSet.fNoFreq);
	// Preset filter to actual value
	//TorqueInitFilter( *PtrTorqueMeasValue() );
}


/*!
\fn		float* PtrFrictionBconst(void)
\brief  Read or Write actual Regression Constant B (there are A, B, C)
		Write with:	*PtrFrictionBconst() = float;
		Read with:	= *PtrFrictionBconst(); 

\author	K�ster
\date	21.04.2016
\param[in]
\return	void

\details Changes:

*/  
float* PtrFrictionBconst(void)
{
#ifdef VTIQ_AIR
	return &tFriction.afC[14];
#else
	return &tFriction.afC[22];
#endif
}





/*!
\brief		Friction Correction with 1-Dimensonal table
			The correction data of ball bearings are stored in a 1-dimensonal
			table. An actual Omega-value is used to calculate the exact correction
			torque at this Omega point.
			The method is linear interpolation from point to point!

			Duration when running in RAM: �s!
			
\fn			float32 FrictionCorr (float32 fOmega)
\author		K�ster, Ralf
\date		02.05.2012
\param[IN]	float32 fOmega
\return		Correction Torque
\details	Changes:	
*/
float32 FrictionCorr (float32 fOmega)
{
    Uint32 n;
    float32 fMcorr;

	if( tCorrMode.usFri == 1 )
	{   
	    // table length in first float const, at the moment  n=13, but only 10 table values
	    n = ((Uint32) *(tFriction.afC )) - 3;
        // correction of motor temperature but only if Reference temp. is ok!
        if( tCorrMode.fMotorTempRef > -10.0)
        {
            // Interpolation between the tabulated values
            fMcorr = MthInterp1D( (tFriction.afC +1), (tFriction.afC+n+1 ), fOmega, n);
            // correction of the correction Torque because of drift aluminium resistance (housing)
            fMcorr = fMcorr * (1 - ( (*PtrMotorTemp() - tCorrMode.fMotorTempRef) * TEMP_COEFF_ALU));
            return fMcorr;
        }
        else
        {
		    // Interpolation between the tabulated values
    	    return MthInterp1D( (tFriction.afC +1), (tFriction.afC+n+1 ), fOmega, n);
        }
	}
	else
	{
		return 0.0;
	}

}

/*!
\brief		G� Correction with 1-Dimensonal table
			The correction data of G� for ball bearing effects are stored in a 1-dimensonal
			table. An actual Phi-value is used to calculate the exact correction for G�
			at this Phi point.
			The method is linear Interpolation!
			Duration when running in RAM: �s!
			
\fn			float32 G1Corr (float32 fPhiAmpl)
\author		K�ster, Ralf
\date		15.11.2013
\param[IN]	float32 fPhiAmpl
\return		Correction G1
\details	Changes:	
*/
float32 G1Corr (float32 fPhiAmpl)
{
    Uint32 n;

	//tCorrMode.usG1G2 = 0;
	if( tCorrMode.usG1G2 == 1 )
	{   
	    // table length in first float const
	    n = ((Uint32) *(tG1.afC ));
		// linear Interpolation in array
    	return MthInterp1D( (tG1.afC +1), (tG1.afC +1 + n ), fPhiAmpl, n);
	}
	else
	{
		return 0.0;
	}

}

/*!
\brief		G��-Correction with 1-Dimensonal table
			The correction data of G�� for ball bearing effects are stored in a 1-dimensonal
			table. An actual Phi-value is used to calculate the exact correction for G��
			at this Phi point.
			The method is linear Interpolation!
			Duration when running in RAM: �s!
			
\fn			float32 G2Corr (float32 fPhiAmpl)
\author		K�ster, Ralf
\date		15.11.2013
\param[IN]	float32 fPhiAmpl
\return		Correction G2
\details	Changes:	
*/
float32 G2Corr (float32 fPhiAmpl)
{
    Uint32 n;

	//tCorrMode.usG1G2 = 0;
	if( tCorrMode.usG1G2 == 1 )
	{   
	    // table length in first float const
	    n = ((Uint32) *(tG2.afC ));
		// linear Interpolation in array
    	return MthInterp1D( (tG2.afC +1), (tG2.afC +1 + n ), fPhiAmpl, n);
	}
	else
	{
		return 0.0;
	}

}




/*
   \brief      
   \Fn   InertiaCorrection()
   \date		20.12.2006
    
   \param[IN]  zu korrigierendes Drehmoment
   \return

   Changes:

*/
float32 InertiaCorrection (int p_nMode, float32 p_fSlope, float32 p_fOmega)
{
	float32 l_fSlope;
	float32 l_fTorqueAcc;

	// 02.12.13 always activ, not to be controlled with Rheowin settings
	tCorrMode.usIneRot = 1;

	// Korrektur verwenden?
	if (tCorrMode.usIneRot == 1)
	{
		// Massentraegheitskorrektur lineare Rampe durchfuehren
		if (p_nMode == 1)
		{
			l_fSlope = p_fSlope;
		}
		// Massentraegheitskorrektur logar. Rampe durchfuehren
		else
		{
			// Steigung = log(10) * n * SteigungRampe
			l_fSlope = 1.0 * p_fOmega* p_fSlope;
		}

		if (fabs(l_fSlope) > 1E-8)
		{
			// Momentanteil fuer Beschleunigung
			l_fTorqueAcc = ( *PtrInertiaDevice() + *PtrInertiaSensor() ) * l_fSlope;
			return l_fTorqueAcc;
		}
	}
	return 0.0;

}   



/*!
\fn		float* PtrMscState
\brief  Deliver pointer to actual state of MSC-Detection
		If a MSC-Detection measurement is running, the state
		is always showing the ongoing state of it.
\author	K�ster
\date	16.10.2012
\param[in]
\return	void
\details Changes:

*/  

Uint16* PtrMscState(void)
{
	return &tMsc.usState;
}

Uint16* PtrMscCorrMode(void)
{
	return &tCorrMode.usMsc;
}

Uint16* PtrMscAvgCnt(void)
{
	// return the adress of the Avg-Counter
	return &tMsc.usAvgCnt;
}

void CorrMscClearAvgBuffer(void)
{
	// we need for calculation the Average values a table half as long as the MSC table
	tMsc.pafBuf = (float32 *)calloc( ((MSC_TBL_LEN/2)+1), sizeof(float32));
	// and DC Offset
	tMsc.fAvgTrqDc = 0.0;
	//tMsc.afBuf[0] = (MSC_TBL_LEN/2)+1;	// 129
	*tMsc.pafBuf = (MSC_TBL_LEN/2)+1;
	// reset MSC temporary Buffer
	//CorrResetBuffer(&tMsc.afBuf[0]);
	CorrResetBuffer(tMsc.pafBuf);

}

/*!
\fn		float* PtrCorrMscTbl(Uint16 pusN)
\brief  Deliver pointer to required Msc Array 
\author	K�ster
\date	16.10.2012
\param[in]
\return	void
\details Changes:
*/  
float32* PtrCorrMscTbl(Uint16 pusN)
{
	// return adress of requ. value
	return &tMsc.afC[pusN];
}


float32* PtrCorrMscDc(void)
{
	// return adress of requ. value
	return &tMsc.fAvgTrqDc;
}


/*!
\fn		void CorrResetBuffer(*ptr)
\brief  Reset float array
		Presetting array with 0.0, in the first member the length is necessary
		Ex.: when unL = 257, 256 members will be filled
\author	K�ster
\date	23.09.2013
\param[in]
\return	void
\details Changes:
		04.04.14	begin filling at the second element
*/  
void CorrResetBuffer(float32 *pafBuf)
{
	int	i;
	Uint16 unL;
	// length always in first member
	unL = (Uint16)(*pafBuf+0);
	// and increment pointer to the second element
	pafBuf++;
	for( i = 1; i<unL; i++)
	{
		// fill actual member
		*pafBuf = 0.0;
		// and increment pointer
		pafBuf++;
	}
}

/*!
\fn		void MscTblAddAvgBuffer(void)
\brief  
\author	K�ster
\date	24.09.2013
\param[in]
\return	void
\details Changes:
*/ 
void MscTblAddAvgBuffer(void)
{
	Uint16 i;
	// Average DC Offset
	tMsc.fAvgTrqDc = tMsc.fAvgTrqDc + tMscData.fTrqDC;

	for( i = 1; i<(Uint16)tMsc.afC[0]+1; i++)
	{
		// add MSC table result to AVG Buffer
		//tMsc.afBuf[i] = tMsc.afBuf[i] + tMsc.afC[(Uint16)tMsc.afC[0]+i];
		*(tMsc.pafBuf+i) = *(tMsc.pafBuf+i) + tMsc.afC[(Uint16)tMsc.afC[0]+i];
	}	
}

void CorrMscTblAvgCalc(void)
{
	Uint16 i;
	tMsc.fAvgTrqDc = tMsc.fAvgTrqDc / (float32)MSC_AVG;
	for( i = 1; i<(Uint16)tMsc.afC[0]+1; i++)
	{
		// add MSC table result to AVG Buffer
		//tMsc.afC[(Uint16)tMsc.afC[0]+i] = tMsc.afBuf[i] / (float32)MSC_AVG;
		tMsc.afC[(Uint16)tMsc.afC[0]+i] = *(tMsc.pafBuf+i) / (float32)MSC_AVG;
	}	
	free(tMsc.pafBuf);
}


/*
   Title:      
   Function:   ComplianceCorrection()
   Datum:		28.01.2010
   Beschreibung:aus den Amplituden der FFT-Analyse wird die
				Winkelamplitude der Probe durch Herausrechnen
				der Motorcompliance (Federkonstante der Welle)
				ermittelt!
   Parameter:	p_fPhi0 -> Winkelampl. aus FFT
				p_M0 -> M-Amplitude aus FFT
   				 
   R�ckgabewert:

   ge�ndert:

*/
float ComplianceCorrection(float p_fPhi0, float p_fM0)
{
	float l_fV;
	// todo rad/Nm, wieder raus, wenn �C1 implementiert
	//*PtrSensorCompliance() = 0.0057;
	if (tCorrMode.usCom == 1)
	{   
		l_fV = fabs(p_fPhi0 - p_fM0 * ( *PtrSensorCompliance() ));
		return l_fV;  
	}
	else
	{
		// R�ckgabewert ist die Winkelamplitude der Probe
		return p_fPhi0;  
	}
}
