/*!
\file	Crmode.c
\brief	here we find the functions for the CR-Mode
		CR means speed control of the motor.

\author	Koester
\date	19.10.2011
\details
		changes:
		19.09.13 CR-Controller output limit from 0,1001 to 0,1050
		06.02.14 CR-Controller output limit from 0,1050 to 0,1002	
				 Limit Torque after corrections
		27.02.14 Friction correction depends on Set-Value not FOLP-Value
				 Bugfix, noise with lower speed (0,01rpm)
		10.08.15 Kp und D0I einstellbar ueber GK[8] und [9]
		12.08.15 Test mit adaptiver Regelung
		05.11.15 Sollwertintegrator f�r VTiQ-Air eingef�hrt
     28.04.16 Bugfix, bei FR-Detection ohne Inertia-Korrektur obw. Rmp-MOde
     23.06.16 Test mit 0,01sec Rampe fuer neues Luftlager
       
*/

// for Compiler control
#define CRMODE
#include <math.h>
#include "Project.h"		// all global things as Type definitions, Hardware def., ...#include "Device.h"
#include "Uc1dsp.h"
#include "Diagnos.h"
#include "control.h"
#include "Measmode.h"
#include "Crmode.h"
#include "timer.h"
#include "filt.h"
#include "dsv.h"
#include "omega.h"
#include "correction.h"
#include "angle.h"
#include "mth.h"
#include "Device.h"
#include "msc.h"
#include "Test.h"
#include "Pmsm.h"
#include "control.h"


/* Macro definitions ********************************************************/

// Parameters for MovingAverageFilter
#define TRG_MAVG_LENGTH     250

// Parameters for Speed-Controller

// ********** device variant defined in global.h *********
#ifdef VTIQ_AIR
	#define VTIQAIR_KP_CR		0.0073		// 0.0073 bei 250Hz
	#define VTIQAIR_D0I_CR		0.000024 	// war 0.000006 Luftlager, war 0.000030

#else
	// neu ab 01.04.16 ermittelt durch Ralf K�ster, neu ab 21.09.15 ermittelt durch Uwe Bauer
	#define VTIQ_KP_CR			0.02		// war 0,0379 war 0.0073
	#define VTIQ_D0I_CR			0.000200	// war 0.000322 war 000042
#endif


#ifdef NANO
 // max. Torque 
 #define YK_MAX_CR		0.01
#else
 // max. Torque 
 #define YK_MAX_CR		0.1002
#endif


/* Local Prototypes *********************************************************/
#ifdef FLASH
#pragma CODE_SECTION( CrMode, "ramfuncs");
#endif


/* Types ********************************************************************/

struct
{
	float32 afC[9];
} tCrD0i = {8, 0.05, 0.1, 0.2, 0.5, 0.0025, 0.0005, 0.00025, 0.00004}; 


// Struktur Daten nur f�r CR-Mode
typedef struct
{
	float	fOmega;
	float	fOmegaMinus1;
	float	fMRaw;
	float	fMRawMinus1;
	float	fMCorr;
	float	fTrqAvg;           // Torque FOLP-Filter output
	float	fTrqFolp;          // Torque FOLP-Filter output
  float   fTrqMavg;
  float   fAcc;
	float	fMsoll;
	float	fMMax;
	float	fSetVal;
	float	fSetValAct;
	float	fSetValOld;
	float	fSetValIntegr;
	float	fSetValIncr;
    float   fTimeAvg;           // timer for mixing diff. AVG Signals
    float   fTimeInc;
	Uint16	unSetVcnt;
    Uint16  unState;            // state CR-mode, 0 - new setvalue, 1 - setvalue within 5%
} t_stcCrMode;

/* Global variables *********************************************************/

// data for speed PI-Control, PI-Controller with Kp backwards
t_stcPiRcontrol tPiCr = {0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0};

// data for filtered data torque and speed
t_FOLPF sFOLPFTorque;
t_stcAvgIncMem tAvgTrq;
t_EmphSig tEmphTrq;
t_stcFiltMavg tTrqMavg;
 
// data for f�r CR-Mode control
t_stcPiRcontrol tCrControl;
t_stcCrMode tCrMode = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,0,0};
// data for calculating acceleration
t_stcOmgAccel tCrAcc;


/* Functions ****************************************************************/

/*!
\fn		float* PtrCrTorqueRaw(void)
\brief  Pointer to unfiltered value of torque
		with this function it is possible to write or to read this value
*/
float* PtrCrTorqueRaw(void)
{
	return &tCrMode.fMRaw;
}

/*!
\fn		float* PtrCrTorqueFolp(void)
\brief  Pointer to FOLP filtered value of torque
		with this function it is possible to write or to read this value
		write: *PtrCrTorqueFolp() = val;
		read:  val = *PtrCrTorqueFolp();
*/
float* PtrCrTorqueFolp(void)
{
	return &tCrMode.fTrqFolp;
}


//dito actual Setvalue state
Uint16* PtrCrState(void)
{
	return &tCrMode.unState;
}



/*!
\fn		void CrModeSetTrqFilter(void)
\brief  initializes a data object for Torque measurement over a 1/4 turn
        with a Moving-Average-Filter
\author	K�ster
\date	20.01.2016
\param	
\return	void
\detail Changes:
*/
void CrModeSetTrqFilter(void)
{
    InitSigMavg(&tTrqMavg);
}

/*!
\fn		void CrModeResetTrqFilter(void)
\brief  resets a data object for Torque measurement over a 1/4 turn
        with a Moving-Average-Filter
\author	K�ster
\date	20.01.2016
\param	
\return	void
\detail Changes:
*/
void CrModeResetTrqFilter(void)
{
    ResetSigMavg(&tTrqMavg);
}


/*!
\fn		void SetCondCrMode(void)
\brief  Initialisierungsroutine fuer CS-Regler bei Betriebsartaenderung
\author	K�ster
\date	25.10.2011
\param[in]
\return	
Changes:	dd.mm.yyyy
*/  
void SetCondCrMode(void)
{
    // initialize calculation accelleration
    OmgInitSigAccLinReg(&tCrAcc);
}

/*!
\fn		void TorqueSetCondCalc(float32 p_fFreq, float32 p_fSamplT, float32 p_fMeasTime)
\brief  Set data filters for Torque during in CR-Mode
\author	K�ster
\date	26.10.2012
\param	actual Torque
\return	void
\detail Changes:
*/
void TorqueSetCondCalc(float32 p_fFreq, float32 p_fSamplT, float32 p_fMeasTime)
{
	// Signalgl�ttung Parametrieren und Zust�nde initialisieren
	ParamFOLPF(&sFOLPFTorque, p_fFreq, p_fSamplT * 1e-06);
	SetCondAvgIncMem(&tAvgTrq, (Uint16)(p_fMeasTime*1.0e06/p_fSamplT) );
}

void TorqueSetCondAvg(float32 p_fSamplT, float32 p_fMeasTime)
{
	SetCondAvgIncMem(&tAvgTrq, (Uint16)(p_fMeasTime*1.0e06/p_fSamplT) );
}

/*!
\fn		void TorqueInitFilter(float32 p_fValue)
\brief  Initialize data filters with initial value
\author	K�ster
\date	26.10.2012
\param	actual Torque
\return	void
\detail Changes:
*/
void TorqueInitFilter(float32 p_fValue)
{
	// FOLP set with actual value to avoid switching effects
	InitCondFOLPF(&sFOLPFTorque, p_fValue);
	// dito AVG
	InitCondAvgIncMem(&tAvgTrq, p_fValue);
    InitCondAvgIncMem(&tAvgTrq, 0.0);

    // dito Emphasis function
	InitCondEmphasisSignal(&tEmphTrq, ( 200.0e-06 / TIME_750MS), *PtrMeasTorqueOld() );
    // dito Mavg filter
    tTrqMavg.unState = 0;
    // mixing timer Avg value reset
    tCrMode.fTimeAvg = 0.0;
    // time for 1/4 turn
    //tCrMode.fTimeAvg = (0.25 * ((float32) cf2Pi / *PtrMeasSetvalue())) / 0.0002;
    tCrMode.fTimeInc = *PtrMeasSetvalue() * 127.324e-06;
    //InitCondSigMavg(&tTrqMavg);

}



/*!
\fn		void CrmodeInit(void)
\brief  Initialize CR-Mode, parameters for controller and max. values
\author	K�ster
\date	16.07.2013

\param[in]

\return	void

Changes:

*/
void CrmodeInit(void)
{
	// Set conditions and parameters for CR-Controller, use PI-R control
#ifdef VTIQ_AIR
//	tPiCr.fD0I = *PtrCrD0();
//	tPiCr.fKPr = *PtrCrKP();
    // 22.04.16, from DeviceConstants for VTiQ too
	    tPiCr.fD0I = VTIQAIR_D0I_CR;
	    tPiCr.fKPr = VTIQAIR_KP_CR;
#else
    // 22.04.16, from DeviceConstants for VTiQ too
    if( (*PtrCrKP() > 0.0001) && (*PtrCrKP() < 0.1) )
    {
        // check for resonable values
      tPiCr.fD0I = *PtrCrD0();
	    tPiCr.fKPr = *PtrCrKP();
    }
    else
    {
	    tPiCr.fD0I = VTIQ_D0I_CR;
	    tPiCr.fKPr = VTIQ_KP_CR;
    }
#endif
	tPiCr.fYkMax = YK_MAX_CR;
	// Reset all controller values
	ControlPiRInit(&tPiCr);
}


/*
   Title:         CR Regler initialisieren
   Function:      CrmodePreSetPiC(void)
   Datum:         13.02.2007
   Beschreibung:  Pre setting of the PI-Controller after Mode changes
   Parameter:     tCrPtr -> Pointer auf Regelparameter
   R�ckgabewert:  ---

   ge�ndert:

*/
void CrmodePreSetPiC(void)
{
    // 22.04.16, also at VTiQ
//#ifdef VTIQ_AIR
	tPiCr.fD0I = *PtrCrD0();
	tPiCr.fKPr = *PtrCrKP();
//#endif
	tCrMode.fMMax = *PtrTorqueMax() * TORQUE_LIMIT;
	// Presetting of PIr controller
	ControlPiR(&tPiCr, *PtrOmegaFolp(), *PtrOmegaFolp() );
	// both I and P the same value
	tPiCr.fYkI = tPiCr.fYkIold = tPiCr.fYkPr;
	tCrMode.unSetVcnt = 0;

	// 03.11.15, delete values for setvalue integrator
	tCrMode.fSetValAct = 0.0;
	tCrMode.fSetValIncr = 0.0;
	tCrMode.fSetValIntegr = 0.0;
	tCrMode.fSetValOld = 0.0;
	tCrMode.fSetVal = 0.0;
    // 25.01.16 setvalue not yet reached
    tCrMode.unState = FALSE;
    tCrMode.fTimeAvg = 0.0;
    tCrMode.fTimeInc = *PtrMeasSetvalue() * 127.324e-06;

}

/*
   Title:         CR Regelparameter berechnen
   Function:      CrmodeCalcPiC(void)
   Datum:         28.10.2015
   Beschreibung:  Pre setting of the PI-Controller after Mode changes
   Parameter:     tCrPtr -> Pointer auf Regelparameter
   R�ckgabewert:  ---

   ge�ndert:

*/
void CrmodeCalcPiC(void)
{
#ifdef VTIQ_AIR
	// y = ax-b , a=556,6666, b=-0,01775
	tPiCr.fKPr = 556.6666 * (*PtrInertiaSensor() + *PtrInertiaDevice()) -0.01775;
	tPiCr.fKPr = LimitFloat(tPiCr.fKPr, 0.007 , VTIQAIR_KP_CR );
	// y = ax-b a=3,0, b=-0,000105
	tPiCr.fD0I = 3.0 * (*PtrInertiaSensor() + *PtrInertiaDevice()) - 0.000105;
	tPiCr.fD0I = LimitFloat(tPiCr.fD0I, 0.000002 , VTIQAIR_D0I_CR );
#else
	// y = a10^{(}bx) max dev:2.314968E-9, a=0.05551296, b=-4735.860
	tPiCr.fKPr = 0.055 * pow(10.0, -4735.86 * (*PtrInertiaSensor() + *PtrInertiaDevice()) );
	//tPiCr.fKPr = -340.0 * ( *PtrInertiaSensor() + *PtrInertiaDevice() ) +0.049;
	tPiCr.fKPr = LimitFloat(tPiCr.fKPr, 0.008 , VTIQ_KP_CR );
	// y = a10^{(}bx) max dev:1.021699E-5 a=7.493005E-4, b=-1.048395E4
	tPiCr.fD0I = 0.00074 * pow(10.0, -10483.95 * (*PtrInertiaSensor() + *PtrInertiaDevice()) );
	//tPiCr.fD0I = -5.64 * ( *PtrInertiaSensor() + *PtrInertiaDevice() ) +0.000604;
	tPiCr.fD0I = LimitFloat(tPiCr.fD0I, 0.000012 , VTIQ_D0I_CR );
#endif
}



void CrmodeIntSetValue(float p_fSollwertw)
{
	if( tCrMode.fSetVal == p_fSollwertw )
	{
		// no, check state of integration
		if(	tCrMode.fSetValAct == tCrMode.fSetVal)
		{
			// nothing to do
			asm(" nop");
		}
		else
		{
			// actual Setvalue is not still Setvalue -> Integration
			tCrMode.fSetValAct = tCrMode.fSetValAct + tCrMode.fSetValIncr;
			// end of integration when difference is lower than 0.5 rad/s
			if( fabs(tCrMode.fSetValAct - tCrMode.fSetVal) < 7.5 )
			{
				tCrMode.fSetValAct = tCrMode.fSetVal;
			}
		}
	}
	else
	{
		// save old setvalue
		tCrMode.fSetValOld = tCrMode.fSetVal;
		// store new setvalue
		tCrMode.fSetVal = p_fSollwertw;
		// calculate integrator interval , integrate within 500 msec
		//tCrMode.fSetValIncr = (tCrMode.fSetVal - tCrMode.fSetValOld) / 500.0;
		if( tCrMode.fSetVal >= tCrMode.fSetValOld)
		{
			// 0.104719755 -> 0,3 s, 0.062831853 -> 0,5 s
			tCrMode.fSetValIncr = 0.05235988;  // 0.052359 -> 0,6 sec
 			// 23.06.16 Test ohne Rampe fuer neues Luftlager
 			//tCrMode.fSetValIncr = 3.1415927;  //  -> 0,01 sec
		}
		else
		{
			tCrMode.fSetValIncr = -0.05235988;
		}
		// actual Setvalue calculation
		tCrMode.fSetValAct = tCrMode.fSetValAct + tCrMode.fSetValIncr;
	}

}


/*
   Title:         CR-Mode
   Function:      CrMode()
   Datum:           13.03.2007
   Beschreibung:   
                   
   Parameter:     p_fSetValue -> Sollwert

   R�ckgabewert:  ---

   ge�ndert:
*/

extern float fTest;


void CrMode(float32 p_fSollwertw)
{
	
#ifdef VTIQ_AIR
	// new setvalue?
	// setvalue -> Integrator
	CrmodeIntSetValue(p_fSollwertw);
#else
  tCrMode.fSetValAct = p_fSollwertw;
#endif

    // calculate acceleration with linear regression
    tCrMode.fAcc = OmgSigAccLinReg(&tCrAcc, *PtrOmegaAvg());

	// ********************** Control ********************************

	// call PIr-Controller and store calculated torque
	tCrMode.fMRaw = ControlPiR(&tPiCr, 	tCrMode.fSetValAct, *PtrOmegaFolp() );

    // limit to max. Torque
	tCrMode.fMRaw = LimitFloat(tCrMode.fMRaw, -( *PtrTorqueMax() ) , *PtrTorqueMax() );

	// store to Corrected value   
	tCrMode.fMCorr = tCrMode.fMRaw;
	
	// ********************** Corrections ***************************
    // active Inertia correction
    if( (*PtrMeasRmpMode() == RMP_ON_LIN) || (*PtrMeasRmpMode() == RMP_ON_LOG) )
    {
		// 28.04.2016 not while FR-Detection
		if( *PtrMeasActDataMode() != FRI_START_AQU)
        {
		    // 07.03.16 new with acceleration from regression function
		    tCrMode.fMCorr = tCrMode.fMCorr + InertiaCorrection(1, tCrMode.fAcc , *PtrMeasSetvalue() );
        }
	}
    // active friction correction
	// raus 27.02.14, noise with lower speed  if( *PtrOmegaFolp() >=0 )
	if(	tCrMode.fSetValAct >=0 )
	{
		tCrMode.fMCorr = tCrMode.fMCorr + FrictionCorr( fabs(tCrMode.fSetValAct) );
	}
	else
	{
		tCrMode.fMCorr = tCrMode.fMCorr - FrictionCorr( fabs(tCrMode.fSetValAct) );
	}
    
    // active MSC correction
	if( *PtrMscCorrMode() == 1 )
	{
    	//tCrMode.fMCorr = tCrMode.fMCorr + MscCorr(&tMscData,*PtrAngleAbs(), tCrMode.fMCorr) + *PtrCorrMscDc();
    	tCrMode.fMCorr = tCrMode.fMCorr + MscCorr(&tMscData,*PtrAngleAbs(), tCrMode.fMCorr);
	}
	
	// limit to max. Torque
	tCrMode.fMCorr = LimitFloat(tCrMode.fMCorr, -(tCrMode.fMMax), tCrMode.fMMax );

	// now the output value is complete
	tCrMode.fMsoll = tCrMode.fMCorr;

	// ********************** Filters *******************************
	// if there is no ramp, emphasis torque signal after jumps
	if ( *PtrMeasRmpMode() == RMP_OFF )
	{
		if ( (Uint16)*PtrRawdataFlag() == 1)
		{
			// no emphased signal
			tCrMode.fMRaw = tCrMode.fMRaw;
		}
		else
		{
			// no emphased signal
			//tCrMode.fMRaw = tCrMode.fMRaw;
			// emphases Signal after Jump in SetValue, to reduce spikes in torque
			tCrMode.fMRaw = EmphasisSignal(&tEmphTrq, tCrMode.fMRaw);	
		}
	}

	// filter torque with Folp-Filter and AVG-Filter
	tCrMode.fTrqFolp = FOLPF(&sFOLPFTorque, tCrMode.fMRaw );
	//tCrMode.fTrqAvg = tCrMode.fTrqFolp;
	tCrMode.fTrqAvg = AvgIncMem(&tAvgTrq, tCrMode.fTrqFolp);

    // Sollwert erreicht?, MAVG initialisieren
    if( tCrMode.unState == FALSE )
    {
        // im Bereich von 1% des Sollwertes ?, dann MAVG initialisieren
        if( fabs((p_fSollwertw - *PtrOmegaAvg()) / p_fSollwertw) < 0.01 )
        {
            // 25.01.16  Setvalue reached, clear semaphore
            tCrMode.unState = TRUE;
            // Set value reached, init. Mavg-Filter
            InitCondSigMavg(&tTrqMavg);
        }
        tCrMode.fTrqMavg = tCrMode.fTrqAvg;
    }
    else
    {
        // True -> setvalue reached, calculate Mavg-Filter over 1/4 turn
        tCrMode.fTrqMavg = SignalCalcMavg(&tTrqMavg, tCrMode.fTrqAvg, *PtrAngleAbs());

        // dependent from speed mix the filtered values within a defined time
        tCrMode.fTimeAvg = tCrMode.fTimeAvg + tCrMode.fTimeInc;
        if(tCrMode.fTimeAvg >= 1.0)
        {
            tCrMode.fTimeAvg = 1.0;
        }
        tCrMode.fTrqMavg = (tCrMode.fTimeAvg * tCrMode.fTrqMavg) + ((1.0 - tCrMode.fTimeAvg) * tCrMode.fTrqAvg);
        
        // but if out 1% again, reinitialize. MAVG 
        if( fabs((p_fSollwertw - *PtrOmegaAvg()) / p_fSollwertw) > 0.03 )
        {
            // 18.02.16  Setvalue reached, clear semaphore
            tCrMode.unState = FALSE;
            tCrMode.fTimeAvg = 0.0;
            // Set value reached, init. Mavg-Filter
            //InitCondSigMavg(&tTrqMavg);
        }
        
    }
  

	// ********************** Save Data *****************************
	// for test and development use, send rawdata?
	if ( (Uint16)*PtrRawdataFlag() == 1)
	{
		*PtrTorqueMeasValue() = tCrMode.fMRaw;
		*PtrOmegaMeasValue() = *PtrOmegaRaw();
        
	}
	// for test and development use, send rawdata?
	else if ( (Uint16)*PtrRawdataFlag() == 2)
	{
        // store indicated value Torque
        *PtrTorqueMeasValue() = tCrMode.fTrqAvg;
		// store indicated value Omega
		*PtrOmegaMeasValue() = *PtrOmegaAvg();
	}

	else
	{
        // store indicated value Torque
        *PtrTorqueMeasValue() = tCrMode.fTrqMavg;
		// store indicated value Omega
		*PtrOmegaMeasValue() = *PtrOmegaAvg();
	}

	// while friction detection is running, fill also this variable
	*PtrTorqueAVGMeasValue() = tCrMode.fTrqAvg;
	
	// store indicated value Angle
	*PtrPhiMeasValue() = *PtrAngleRel();
      	
	// store indicated value absolute Angle
	*PtrPhiAbsMeasValue() = *PtrAngleAbs();

#ifdef TEST_DEBUG
  *PtrMeasTestF6() = *PtrFocTestI1();
  *PtrMeasTestF7() = *PtrFocTestI2();
  *PtrMeasTestF8() = *PtrFocTestI3();
#endif

	// store actual values for next sample period
	// 04.12.13 tCrMode.fOmegaMinus1 = *PtrOmegaAvg();
	tCrMode.fOmegaMinus1 = *PtrOmegaFolp();
	tCrMode.fMRawMinus1 = tCrMode.fMRaw;

	// Store actual Omega for e.G. Mode changes
	*PtrOmegaOld() = *PtrOmegaAvg();
	*PtrMeasTorqueOld() =  tCrMode.fTrqAvg;

	// ***************** Output to motor control ********************

   	*PtrFocSetCurrent() = tCrMode.fMsoll *  ( *PtrCal1Motor() );
	//*PtrFocSetCurrent() = (tCrMode.fMsoll) *  (1 / *PtrCal1Motor() ) * SQRT_2;

}
