/*!
\file	CsfastOsc.c
\brief	functions for Fast-Oscillation

\author	Koester
\date	28.01.2010
\details
	changes:	01.10.13 modification for VTiQ
				25.03.14 Meas.Mode CDOSC is in use, remove Ampl.-Function for VTiQ
				15.04.14 correction phase, coming from number of table values
           11.05.16 Test, because of bug in �C-1, Friction value B  not available
           31.05.16 back to use the regression result for Friction correction
           06.07.16, for VTiQ although correct Compliance-Correction
*/

/* Include files ************************************************************/
#include <math.h>
#include "project.h"		// all global things as Type definitions, Hardware def., ...#include "Device.h"
#include "mth.h"
#include "Uc1dsp.h"
#include "Diagnos.h"
#include "SsiComm.h"
#include "Measmode.h"
#include "filt.h"
#include "dsv.h"
#include "omega.h"
#include "correction.h"
#include "angle.h"
#include "CsFastOsc.h"
#include "fosc.h"
#include "omega.h"
#include "Device.h"
#include "Pmsm.h"
#include "msc.h"
#include "sig.h"


// Prototype statements for functions found within this file.


/* Macro definitions ********************************************************/

// VTiQ fix value, only the first harmonics is interesting
#define NO_HARMONICS	1

#ifdef FLASH
//#pragma CODE_SECTION(atan, "ramfuncs");
#endif

/* Local Prototypes *********************************************************/
void CalcOscRheoData(float32 p_fMm0, float32 p_fPhiM0, float32 p_fDelta0, float32 p_fFosc, int i);

/* Types ********************************************************************/

typedef struct
{
	Uint16			usSweepMode;
	float32			fMsoll;		// Output Set-Value Torque
	float32			fW;			// w angular frequency, for reduction calculation time
	float32			fW2;		// w�, for reduction calculation time
	float32			fQAM;		// Ratio A/M Sensor for calculation
	t_stcFOscSet	tSet;		// Set data from Master
	t_stcOscData	tData;		// Measuring Data Raw
	t_stcOscFftData	tFftData;	// Measuring Data FFT
}t_stcFOscMode;



/* Global variables *********************************************************/
t_stcFOscMode tFosc;
t_stcAvgIncMem tAvgPhiAbs;


/* Functions ****************************************************************/

/*!
\fn		Uint16* PtrFoscSweepMode(void)
\brief  Read or Write actual Sampling-Time 
		Write with:	*PtrFoscSweepMode() = Uint16;
		Read with:	= *PtrFoscSweepMode(); 
		depending of the receive Interrupt it is an Initialization or only a new
		Set-Value of a Sweep (Amp. or f-Sweep)
		This is an important information when re-initialize the fos modul.

\author	K�ster
\date	20.03.2014
\param[in]
\return	void

\details Changes:

*/  
Uint16* PtrFoscSweepMode(void)
{
	return &tFosc.usSweepMode;
}
float* PtrFoscPhiAbs(void)
{
	return &tFosc.tData.fPhiAbs;
}

float* PtrFoscPhiRaw(void)
{
	return &tFosc.tData.fPhiRaw;
}

float* PtrFoscPhi(void)
{
	return &tFosc.tData.fPhi;
}

float* PtrFoscTrq(void)
{
	return &tFosc.tData.fTorque;
}


/*!
\fn		void InitFastOscillation(void)
\brief  Initialisierungsroutine FastOsc. Variablen
\author	K�ster
\date	28.01.2010

\param[in]

\return	

Changes:	

*/  
void InitFastOscillationData(void)
{
	tFosc.usSweepMode = 0;
	tFosc.tSet.usIV = IV_CSOSCFASTINIT;
	tFosc.tSet.usID = 1;
	tFosc.tSet.usMode = CSOSC;
	/* 1Hz voreinstellen */
	tFosc.tSet.fFreq = 1.0;
	
	/* Delete Amplitude-Array */
	tFosc.tSet.fAmpl[0] = 0.0;	
	tFosc.fMsoll = 0.0;
	tFosc.tSet.usAmpMode = 1;
	tFosc.tSet.fDuration = 0.0;
	tFosc.tSet.fFStrain = 0.0;
	tFosc.tSet.usFStrain = 0;

	/* Mode and Tabel length */
	tFosc.tData.usIV = IV_OSCDATARAW;
	tFosc.tData.usID = 0;
	tFosc.tData.usMode = 0;
	tFosc.tData.usTabLength = 0;

	/* Rawdata */
	tFosc.tData.fTorqueRaw = 0.0;	
	tFosc.tData.fPhiRaw = 0.0;
	tFosc.tData.fPhiAbs = 0.0;

//#ifdef RAUS
	/* Rheologic data */
	/* Amplitudes from FFT for n frequencies*/
	tFosc.tFftData.usIV = IV_OSCDATAFFT;
	tFosc.tFftData.afTorqueFFT[0] = 0.0;
	tFosc.tFftData.afPhiFFT[0] = 0.0;	
	tFosc.tData.fTorque = 0.0;
	tFosc.tData.fPhi = 0.0;
	tFosc.tData.fG1 = 0.0;
	tFosc.tData.fG2 = 0.0;
	tFosc.tData.fTau0 = 0.0;
	tFosc.tData.fGamma0 = 0.0;
	tFosc.tData.fDelta = 0.0;
	tFosc.tData.fEtaStern = 0.0;
	tFosc.tData.fEta1 = 0.0;
	tFosc.tData.fEta2 = 0.0;
	tFosc.tData.fK = 0.0;
	/* Delta has only the half length */
	tFosc.tFftData.afDeltaRFFT[0] = 0.0;	
//#endif
}


/*!
\fn		void CsFastOscInit(void)
\brief  Initialie Function for CS-FAST-Oscillation
\author	K�ster
\date	28.01.2010
\param[in]
\return	
Changes:	
*/  
void CsFastOscInit(void)
{
	// Load Set data from SSI Receive-Buffer, this contains the new Amplitude CS or CD
	GetSsiData(&tFosc.tSet);
	// set actual measuring state (is used sometimes when switching to other states)
	*PtrMeasMode() = tFosc.tSet.usMode;
	
	// walking over this function means new measurement, switch off Sweep
	tFosc.usSweepMode = 0;	
	// ********** Init CS Fast Oscillation itsself **********
	// Interface function to configure the fast oscillation device object
	FoscCfg();	
	// initialize Fastoscillation
	// -> Inputs:	frequency, n Harmonisc,	Amplitude CS-Mode,		CD-Mode 0-1,		Amplitude CD-Mode,	Maximum Torque
	FoscSet(tFosc.tSet.fFreq, NO_HARMONICS, tFosc.tSet.fAmpl[0], *PtrMeasMode(), tFosc.tSet.fFStrain, *PtrTorqueMax() );
	// this is useful during Interrupt, for reducing calculation time
	tFosc.fW = (float32) cf2Pi * tFosc.tSet.fFreq;
	tFosc.fW2 = tFosc.fW * tFosc.fW;
	tFosc.fQAM = *PtrSensorA() / *PtrSensorM();
	// here is actual already STOP

	// ********** Set the Measurement Conditions **********
	// set calculated sample time (e.G. 5Hz -> 128 points SamplTime. is 1562.5�s)
	*PtrSamplingTime() = 1e06 * FoscGetIsrCycleTime();

	// CsFastOsc Data, send calculated tabel length (important for the Rawdata window in Rheowin)
	tFosc.tData.usTabLength = (FoscGetTblLen() );
	// no special Data-Mode here, this tells the master, whether the first period is over
	tFosc.tData.usMode = FALSE;

	// Phi-Abs as averaged value over one period, the MovingAverageFilter needs the number of points
	SetCondAvgIncMem(&tAvgPhiAbs, (float32)tFosc.tData.usTabLength );
	// initialize the filter with the actual value
	InitCondAvgIncMem(&tAvgPhiAbs, *PtrAngleAbs() );	// Absolutwinkel
	// set Constant Omega-Calculation (Omega calc. is always running in the background)
    SetConstOmegaCalc( *PtrSamplingTime() );
}



/*!
\fn		void CsOscControl(void)
\brief  Vorgabe Drehmoment im CSOSC Mode
\author	K�ster
\date	10.01.2010

\param[in]

\return	

Changes:

*/  
void CsFastOscControl(void)
{
	//GpioDataRegs.GPASET.bit.GPIO30 = 1;
	// for master, semaphore that first cycle is over (==1) (or not == 0)
	tFosc.tData.usMode = FoscIsRdy();

    //if(tFosc.tData.usMode == 1)
    //{
				//NOP for Breakpoint
	//			asm(" nop");
    //}
	
	// call Oscillation control modul all xxx �s
	tFosc.fMsoll = FoscCyclic( *PtrAngleRel(),tFosc.tFftData.afTorqueFFT, tFosc.tFftData.afPhiFFT, tFosc.tFftData.afDeltaRFFT);

	// Rawdata to special channels, for Rawdata-View Rheowin
	tFosc.tData.fTorqueRaw = tFosc.fMsoll;
	tFosc.tData.fPhiRaw = *PtrAngleRel();

	// Rheowin data absolute angle as average result over one period
	// 17.11.15 testweise 
	tFosc.tData.fPhiAbs = AvgIncMem(&tAvgPhiAbs, *PtrAngleAbs() );
	
	// active Tmc-Correction
	//if( *PtrMscCorrMode() == 1 )
	//{
	//	tFosc.fMsoll = tFosc.fMsoll + MscCorr(&tMscData, *PtrAngleAbs(), tFosc.fMsoll);
	//}


	// ***************** Output to motor control ********************
	//*PtrFocSetCurrent() = tFosc.fMsoll *  ( *PtrCal1Motor() );

	// calculate rheological data from the FFT results
	CalcOscRheoData(tFosc.tFftData.afTorqueFFT[0],tFosc.tFftData.afPhiFFT[0], tFosc.tFftData.afDeltaRFFT[0], tFosc.tSet.fFreq, 0);
 	//GpioDataRegs.GPACLEAR.bit.GPIO30 = 1;
}



/*!
\fn		void SetCondFastOscStop(void)
\brief  Stop FastOsc.
\author	K�ster
\date	09.02.2010

\param[in]

\return	

Changes:

*/  
void SetCondFastOscStop(void)
{
	// Set meas.-Mode in Meas-Control to CCROT (stop the motor)
	*PtrMeasMode() = CCROT;
	*PtrMeasSetvalue() = 0.0;
	tFosc.tSet.usMode = 0;
	tFosc.tSet.fAmpl[0] = 0.0;
	tFosc.fMsoll = 0.0;
	// free memory in Heap
	FoscStop();
}


/*!
\fn		void SetCondFastOscAmp(void)
\brief  new Amplitude for Amplitude-Sweep
\author	K�ster
\date	09.02.2010

\param[in]

\return	

Changes:

*/  
void SetCondFastOscAmp(void)
{
	// Load Set data from SSI Receive-Buffer, this contains the new Amplitude CS or CD
	GetSsiData(&tFosc.tSet);
	if( tFosc.tSet.usMode == CSOSC )
	{
		PtrFoscTrqAmp(tFosc.tSet.fAmpl[0]);
	}
	else if( tFosc.tSet.usMode == CDOSC )
	{
		PtrFoscStrainAmp(tFosc.tSet.fFStrain);
	}
	else
	{
	}
}


/*!
\fn		void SetCondFastOscFrq(void)
\brief  new Frequency for F-Sweep
\author	K�ster
\date	09.02.2010

\param[in]

\return	

Changes:

*/ 
void SetCondFastOscFrq(void)
{
	// Load Set data from SSI Receive-Buffer, this contains the new Amplitude CS or CD
	GetSsiData(&tFosc.tSet);
	// set actual measuring state (is used sometimes when switching to other states)
	*PtrMeasMode() = tFosc.tSet.usMode;
	// not a new initialization, this is a sweep
	tFosc.usSweepMode = 1;
	// config in Sweep mode is without calloc memory and without initialization of control
	FoscCfg();
	// initialize Fastoscillation
	// -> Inputs:	frequency,	n frequ.,	Amplitude CS-Mode,		CD-Mode 0-1, Amplitude CD-Mode,		Maximum Torque
	FoscSet(tFosc.tSet.fFreq, NO_HARMONICS, tFosc.tSet.fAmpl[0], *PtrMeasMode(), tFosc.tSet.fFStrain, *PtrTorqueMax() );
	// this is useful during Interrupt, for reducing calculation time
	tFosc.fW = (float32) cf2Pi * tFosc.tSet.fFreq;
	tFosc.fW2 = tFosc.fW * tFosc.fW;

	// ********** Set the Measurement Conditions **********
	// set calculated sample time (e.G. 5Hz -> 128 points SamplTime. is 1562.5µs)
	*PtrSamplingTime() = 1e06 * FoscGetIsrCycleTime();

	// CsFastOsc Data, send calculated tabel length (important for the Rawdata window in Rheowin)
	tFosc.tData.usTabLength = (FoscGetTblLen() );
	// no special Data-Mode here, this tells the master, whether the first period is over
	tFosc.tData.usMode = FALSE;

	// Phi-Abs as averaged value over one period, the MovingAverageFilter needs the number of points
	SetCondAvgIncMem(&tAvgPhiAbs, (float32)tFosc.tData.usTabLength );
	// initialize the filter with the actual value
	InitCondAvgIncMem(&tAvgPhiAbs, *PtrAngleAbs() );	// Absolutwinkel
	// set Constant Omega-Calculation (Omega calc. is always running in the background)
    SetConstOmegaCalc( *PtrSamplingTime() );
}


/*!
\fn		void SendSsiMeasData(void)
\brief  write measuring data to Send-Buffer
\author	K�ster
\date	01.10.2011
\param[in]
\return	void

Changes:

*/  
void SendSsiOscData(void)
{
	// write data into the Ssi-Send-Buffer
	StartSsiSendData(&tFosc.tData);
}



/*!
\fn	CalcOscRheoData()
\brief	Here we calculate the rheological data from the FFT results and
		do the TMC- and G-Corrections. The names are according the documentation:
		"MARS RS-RT Oszillations Dokumentation v.1.0.doc"
		the notes (e.G.3.16) refer to the calculations there.
\author	K�ster
\date	01.10.2011
\param[in]	p_fMm0 -> Drehmomentamplitude aus FFT
			p_fPhiM0 -> Winkelamplitude aus FFT
   			p_fFosc -> Osc. Frequenz
\return	void
Changes:
        07.12.15 new DFT calculation with OSC, no more Delta-correction
        04.03.16 new compliance correction according to document
                 "DIN_Osz Kap_1_bis_11 Ergebnis 2013-03-27.doc"
        06.07.16 new compliance correction also for VTiQ

*/  
void CalcOscRheoData(float32 p_fMm0, float32 p_fPhiM0, float32 p_fDelta0, float32 p_fFosc, int i)
{
	float32 fPhiP0;
	float32 fMPhiQ;
	float32 fTau0, fGamma0, fDeltaCorr;
	float32 fG1, fG2;
  float32 fG1corr, fG2corr;
  float32 fHelp1, fHelp2, fHelp3;
  float32 fKcomp;

  // 06.07.16, for VTiQ although correct Compliance-Correction
  // fetch compliance constant
  fKcomp = *PtrSensorCompliance();
	// take angle amplitude from FFT
	fPhiP0 = p_fPhiM0;

	// check angle for Zero, because of division !!!
	if (fPhiP0 < 1e-8)
	{
	  fPhiP0 = 1e-8;
	  tFosc.tData.fPhi = 1e-8;
	}
  /* Winkelamplitude der Probe berechnen (3.16) auch Compliance-Korrektur genannt */
  // 06.07.16, for VTiQ although correct Compliance-Correction
	//fPhiP0 = ComplianceCorrection(p_fPhiM0, p_fMm0);

	/* Schubspannung = A-Faktor * gemessene Drehmomentamplitude */
	fTau0 = *PtrSensorA() * p_fMm0;
	
	/* Deformation = M-Faktor * Winkelamplitude der Probe */
	fGamma0 = *PtrSensorM() * fPhiP0;

	/* Quotient aus Drehmomentamplitude und Winkelampl. Probe f�r weitere Berechnungen */
	fMPhiQ = p_fMm0 / fPhiP0;

  // 07.12.15 air bearing no more Delta correction
  // 14.03.16 ball bearing also no more Delta correction
  fDeltaCorr = p_fDelta0;

	// calculate G� with Inertia-Correction when activ (according to 3.12)
	fG1 = tFosc.fQAM * (fMPhiQ * cos(fDeltaCorr) + ( tFosc.fW2 * ( *PtrInertiaSensor() + *PtrInertiaDevice())) ) ;

	// calculate G'' (according 3.13) with Friction-Correction when activ
  // 06.07.16, for VTiQ although correct Compliance-Correction
  // 12.11.15, Test with friction correction but only VTiQ-Air
  // G�� = ( A/M * M0/Phi0 * sin(delta) ) - (A/M * w * 1,65e-05)
  // 31.05.16 , back to use the regression result

#ifdef VTIQ_AIR
  fG2 = tFosc.fQAM * (fMPhiQ * sin(fDeltaCorr) - (tFosc.fW * ( *PtrFrictionBconst() )) );
#else
  fG2 = tFosc.fQAM * (fMPhiQ * sin(fDeltaCorr));
#endif

  // 03.03.16 new compliance correction, only VTiQ-Air
  // 06.07.16, for VTiQ although correct Compliance-Correction
  // for simple calculation and to avoid double calculations do some "helps"
  fHelp1 = fKcomp*fG1/tFosc.fQAM;
  fHelp2 = fKcomp*fG2/tFosc.fQAM;
  // only as help variable
  fHelp3 = ( (1-fHelp1)*(1-fHelp1) + fHelp2*fHelp2 );
  // "DIN_Osz Kap_1_bis_11 Ergebnis 2013-03-27.doc" 6.23
  fG1corr = ( (fG1*(1-fHelp1))-fHelp2*fG2 ) / fHelp3;
  // "DIN_Osz Kap_1_bis_11 Ergebnis 2013-03-27.doc" 6.24
  fG2corr = fG2 / fHelp3;
  // to standard variable, always positiv values
  fG1 = fabs(fG1corr);
  fG2 = fabs(fG2corr);
  // "DIN_Osz Kap_1_bis_11 Ergebnis 2013-03-27.doc" 6.25    
  tFosc.tData.fDelta = atan(fG2/fG1);

  // VTiQ also G' G'' correction
#ifndef VTIQ_AIR
	// VTiQ with G� G�� Correction
	fG1 = fabs( fG1 - tFosc.fQAM * G1Corr(fPhiP0) );	
	fG2 = fabs( fG2 - tFosc.fQAM * G2Corr(fPhiP0) );
#endif
	// store results here
	tFosc.tData.fDelta = atan2(fG2, fG1);
	tFosc.tData.fTau0 = fTau0;
	tFosc.tData.fGamma0 = fGamma0;

	// Delta depends from...???
	if (tFosc.tData.fDelta < (cfPi / 2.0) )
	{
	  tFosc.tData.fTau0 = fGamma0 * fG1 / cos(tFosc.tData.fDelta);
	}
	else
	{
	  tFosc.tData.fTau0 = fGamma0 * fG2 / sin(tFosc.tData.fDelta);
	}
	//
	tFosc.tData.fTorque = tFosc.tData.fTau0 / *PtrSensorA();
  //tFosc.tData.fTorque = p_fMm0;
	
	tFosc.tData.fPhi = fPhiP0;
	tFosc.tData.fG1 = fG1;
	tFosc.tData.fG2 = fG2;
//	tFosc.tData.fEta1 = fG1 / tFosc.fW;
//	tFosc.tData.fEta2 = fG2 / tFosc.fW; 
//	tFosc.tData.fEtaStern = sqrt(tFosc.tData.fEta1 * tFosc.tData.fEta1 + tFosc.tData.fEta2 * tFosc.tData.fEta2);
}

