/*!
\file	Csmode.c
\brief	here we find the functions for the CS-Mode
		CS means Torque control of the EC-Motor

\author	Koester
\date	19.10.2011
\details
	changes:	28.01.14 set torque output to zero, when Set-Value < 0,000001Nm
		
*/

// for Compiler control
#define CSMODE

#include "Project.h"		// all global things as Type definitions, Hardware def., ...#include "Device.h"
#include "Uc1dsp.h"
#include "Diagnos.h"
#include "control.h"
#include "Measmode.h"
#include "Crmode.h"
#include "timer.h"
#include "dsv.h"
#include "filt.h"
#include "omega.h"
#include "correction.h"
#include "angle.h"
#include "math.h"
#include "mth.h"
#include "Device.h"
#include "msc.h"
#include "Pmsm.h"
#include "Test.h"


/* Macro definitions ********************************************************/


/* Local Prototypes *********************************************************/
#ifdef FLASH
#pragma CODE_SECTION( CsMode, "ramfuncs");
#endif

/* Types ********************************************************************/
// Data strcture for CS-Mode
typedef struct
{
	int16	nMode;			// Mode,not clear what use at the moment
    Uint16  unState;        // state CS-mode, 0 - new setvalue, 1 - 
	float32	fOmega;			// actual used Omega while CS-Mode for e.g. friction-corr.
	float32	fMRaw;			// Torque without any corrections and not filtered
	float32	fMCorr;			// corrected Setvalue
	float32	fMmeas;			// Measuring value, e.G. for Display, Rheowin
    float32 fOmgMavgOut;    // Mittelwert Omega ueber 1/4 Umdrehung
    float32 fTime;          // process time from Start Cs-Mode or new setvalue
	float32	fMsoll;			// actual used Setvalue
    float32 fAcc;
} t_stcCsMode;


/* Global variables *********************************************************/

// data for CS-Mode
t_stcCsMode tCsMode = {0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
// data for calculating acceleration
t_stcOmgAccel tCsAcc;
//
t_stcFiltMavg tOmgMavg;

/* Functions ****************************************************************/


//dito actual Setvalue state
Uint16* PtrCsState(void)
{
	return &tCsMode.unState;
}



/*!
\fn		void CsModeSetOmgFilter(void)
\brief  initializes a data object for Omega measurement over a 1/4 turn
        with a Moving-Average-Filter
\author	Kster
\date	20.01.2016
\param	
\return	void
\detail Changes:
*/
void CsModeSetOmgFilter(void)
{
    // init. this object for Omega
    InitSigMavg(&tOmgMavg);
}

/*!
\fn		void CsModeResetOmgFilter(void)
\brief  resets a data object for Omega measurement over a 1/4 turn
        with a Moving-Average-Filter
\author	Kster
\date	20.01.2016
\param	
\return	void
\detail Changes:
*/
void CsModeResetOmgFilter(void)
{
    // reset function for this object Omega
    ResetSigMavg(&tOmgMavg);
}




/*!
\fn		void CsmodeInit(void)
\brief  initialize CS-Mode
\author	Kster
\date	25.10.2011

\param[in]

\return	void

Changes:

*/

void CsmodeInit(void)
{
	// NOP for Breakpoint
	asm(" nop");
}


/*!
\fn		void SetCondCsMode(void)
\brief  Initialisierungsroutine fuer CS-Regler bei Betriebsartaenderung
\author	Kster
\date	25.10.2011
\param[in]
\return	
Changes:	dd.mm.yyyy
*/  
void SetCondCsMode(void)
{
    // no reset while ramps, because accelaration then always zero, thats wrong
    if( *PtrMeasRmpMode() == RMP_OFF )
    {
        // initialize calculation accelleration
        OmgInitSigAccLinReg(&tCsAcc);
    }
    // reset process time
    tCsMode.fTime = 0.0;
    tCsMode.unState = FALSE;
}



/*!
\fn		void CsMode(void)
\brief  Vorgabe Drehmoment -> Drehmomentregelung
\author	Kster
\date	01.10.2011
\param[in]
\return	void

Changes:

*/  

void CsMode(float32 p_fSollwertw)
{
	// ********** CS-Messung mit Momentenvorgabe **********
    
    // time in this process, at the moment calculated with 2 ms sampling time
    tCsMode.fTime = tCsMode.fTime + 0.002;

	// take set torque
	tCsMode.fMRaw = p_fSollwertw;
	// load actual filtered Omega
	tCsMode.fOmega = *PtrOmegaAvg();
    
    // calculate acceleration with linear regression
    tCsMode.fAcc = OmgSigAccLinReg(&tCsAcc, tCsMode.fOmega);

    // passive J-Correction of Torque, input is Omega
    if( (*PtrMeasRmpMode() == RMP_ON_LIN) || (*PtrMeasRmpMode() == RMP_ON_LOG) )
	{
        //tCsMode.fMmeas = tCsMode.fMRaw - InertiaCorrection(1, *PtrOmegaAccelaration(), tCsMode.fOmega);
        tCsMode.fMmeas = tCsMode.fMRaw - InertiaCorrection(1, tCsMode.fAcc, tCsMode.fOmega);
    }
	else
	{
		// no ramp, no correction
		tCsMode.fMmeas = tCsMode.fMRaw;
	}

    // activ friction correction
	tCsMode.fMCorr = tCsMode.fMRaw + FrictionCorr( tCsMode.fOmega );

	// active MSC correction
	if( *PtrMscCorrMode() == 1 )
	{
    	// activ MSC-correction, Magnet System Correction
    	tCsMode.fMCorr = tCsMode.fMCorr + MscCorr(&tMscData, *PtrAngleAbs(), tCsMode.fMCorr);
	}

	// Maxgrenzen pruefen und einhalten
    if ( tCsMode.fMCorr > (*PtrTorqueMax() * TORQUE_LIMIT) )
    {
        tCsMode.fMCorr = (*PtrTorqueMax() * TORQUE_LIMIT);
    }
    if ( tCsMode.fMCorr < - (*PtrTorqueMax() * TORQUE_LIMIT) )
    {
        tCsMode.fMCorr = - (*PtrTorqueMax() * TORQUE_LIMIT);
    }
 
	// Ausgabewert Drehmoment speichern
	tCsMode.fMsoll = tCsMode.fMCorr;


    // Sollwert erreicht?, MAVG initialisieren
    if( tCsMode.unState == FALSE )
    {
        // 
        if( (tCsMode.fTime >= 1.0) && (tCsMode.fAcc < 0.1) )
        {
            // 28.01.16
            tCsMode.unState = TRUE;
            // Set value reached, init. Mavg-Filter
            InitCondSigMavg(&tOmgMavg);
        }
        tCsMode.fOmgMavgOut = tCsMode.fOmega;
    }
    else
    {
        // True -> setvalue reached, start with Mavg-Filter
        tCsMode.fOmgMavgOut = SignalCalcMavg(&tOmgMavg, tCsMode.fOmega, *PtrAngleAbs());
        // but if bigger then 0,05 rad/s, reinitialize. MAVG (something happend in substance)
        if( (tCsMode.fTime >= 3.0) && (fabs(tCsMode.fAcc) > 0.15) )
        {
            // 22.02.16  Setvalue reached, clear semaphore
            tCsMode.unState = FALSE;
            tCsMode.fTime = 0.0;
        }

    }

	// ********************** Save Data *****************************
	// for test and development use, send rawdata?
	if ( (Uint16)*PtrRawdataFlag() == 1)
	{
		*PtrTorqueMeasValue() = tCsMode.fMmeas;
		*PtrOmegaMeasValue() = *PtrOmegaRaw();
	}
	// for test and development use, send rawdata?
	else if ( (Uint16)*PtrRawdataFlag() == 2)
	{
        // store indicated value Torque
        *PtrTorqueMeasValue() = tCsMode.fMmeas;
		// store indicated value Omega
		*PtrOmegaMeasValue() = *PtrOmegaAvg();
	}

	else
	{
      	// Anzeige-, Messwert Drehmoment speichern
	    *PtrTorqueMeasValue() = tCsMode.fMmeas;
        // Anzeige-, Messwert Omega speichern
	    *PtrOmegaMeasValue() = tCsMode.fOmgMavgOut;
	}

	// Anzeige-, Messwert Winkel speichern
	*PtrPhiMeasValue() = *PtrAngleRel();
    // Anzeige-, Messwert Absolutwinkel speichern
	*PtrPhiAbsMeasValue() = *PtrAngleAbs();
    //*PtrPhiAbsMeasValue() = tCsMode.fAcc;

	//keep CR-Control actual, for the next change to CR-Mode
	CrmodePreSetPiC();

	#ifdef RAUS
	// todo 27.03.13 Testausgabe diverses FOC Gedns!!
	if( TestDaten(0.0, PROOF) == FALSE )
		{
		// Testdaten senden
		TestDaten (0.0, SEND);
	}
	#endif

	// Store actual Omega for e.G. Mode changes
	*PtrOmegaOld() = tCsMode.fOmega;

#ifdef NANO
    if( tCsMode.fOmega > *PtrOmegaMax() )
    {
        // limitieren auf 100 Nm
        tCsMode.fMsoll = 0.01;
    }
#endif

	// ********** Calibration: Current[Bit]=f(Torque) [Nm] **********
	if( fabs(tCsMode.fMRaw) > 0.0000001)
	{
		// calculate motor current only if set-value > 0,1Nm
       	*PtrFocSetCurrent() = tCsMode.fMsoll *  ( *PtrCal1Motor() );
		//*PtrFocSetCurrent() = tCsMode.fMsoll *  (1 / *PtrCal1Motor() ) * SQRT_2;
	}
	else
	{
		// alway zero when set-value < 1Nm
		*PtrFocSetCurrent() = 0.0;
	}
	
}

