/*!
\file	IntSvc.c
\brief	Interrupt Service Routinen
		Hier befinden sich die ISR- Routinen des Timer0.-Int sowie
		des Ext.-Int. (vom ).

\author	Koester
\date	11.10.2011

\details
	changes:	12.06.12 Timer-1 Hwi function integrated from Timer.c
				29.01.14 modifikation Int.-Handling Timer-0 und EPWM1 (FOC)
				24.03.14 Functions for Osc Sweeps added, Mode CDOSC added
				15.09.14 Amplitude-Sweep although with Start-Stop
				03.03.14 Angle sampling now in FOC interrupt HwiTimer0()
				17.04.15 GPIO49 toggle nur bei bekanntem Kommnado in SwiMasterCommand()
				
*/

/* Include files ************************************************************/
//#include "F28335Inc.h"
//#include "DSP2833x_EPwm_defines.h"		//OK to use 280x file
#include <math.h>
#include "Project.h"		// all global things as Type definitions, Hardware def., ...#include "Device.h"
//#include "C28x_FPU_FastRTS.h"
#include "control.h"
#include "IntSvc.h"
#include "McbspDma.h"
#include "Uc1Dsp.h"
#include "Timer.h"
#include "Epwm.h"
#include "Measmode.h"
#include "IP2000.h"
#include "filt.h"
#include "dsv.h"
#include "omega.h"
#include "angle.h"
#include "CsFastOsc.h"
#include "Crmode.h"
#include "Csmode.h"
#include "Ccmode.h"
#include "fosc.h"
#include "SsiComm.h"
#include "Device.h"
#include "version.h"
#include "Diagnos.h"
#include "Test.h"
#include "correction.h"
#include "Pmsm.h"
#include "SpiA.h"

extern Uint16* PtrRootTskSem(void);

/* Macro definitions ********************************************************/

/* Quittierung Interrupte vom  TRUE oder FALSE */
#define	INT_QUIT_TRUE ((int16) 1)
#define	INT_QUIT_FALSE  ((int16) 0)

/* Semaphore Interrupte TRUE oder FALSE */
#define	INT_SEM_TRUE ((Uint16) 1)
#define	INT_SEM_FALSE  ((Uint16) 0)

/* Types ********************************************************************/


/* Local Prototypes *********************************************************/
void Swi00_RotInit(void);
void Swi01_RotStop(void);
void Swi02_RotSet(void);
void Swi21_IpResetAngle(void);

#ifdef FLASH
#pragma CODE_SECTION( HwiTimer0, "ramfuncs");
#pragma CODE_SECTION( HwiTimer1, "ramfuncs");
#pragma CODE_SECTION( HwiTimer2, "ramfuncs");
#endif



/* Global variables *********************************************************/

// 05.10.10, neu volatile, damit auch immer korrekt gelesen wird
volatile t_stcHwiData tHwiTimer0, tHwiTimer2, tSwiMasterComm;

/* Functions ****************************************************************/


#ifdef RAUS
void ClearHwiTimer0Semaphor(void)
{
	tHwiTimer0.usSemaphor = INT_SEM_FALSE;
}   

unsigned short GetHwiTimer0Semaphor(void)
{
	return (tHwiTimer0.usSemaphor);
}   
#endif



/*!
\fn	Uint32*  PtrHwiTimer1Cnt(void)
\brief  Read or Write actual Sampling-Time 
		Write with:	*PtrHwiTimer1Cnt() = Uint32;
		Read with:	= *PtrHwiTimer1Cnt(); 

\author	K�ster
\date	18.04.2015
\param[in]
\return	void

\details Changes:

*/  

volatile Uint32* PtrHwiTimer0Cnt(void)
{
	return &tHwiTimer0.ulCnt;
}

//dito actual counter value Timer-2
volatile Uint32* PtrHwiTimer2Cnt(void)
{
	return &tHwiTimer2.ulCnt;
}

//dito actual counter value Timer-2
volatile Uint16* PtrSwiMasterIntVec(void)
{
	return &tSwiMasterComm.usIntVector;
}



/*!
\fn		void InitHwiInterrupts()
\brief  Hardware-Interrupts initialization
		1. Step: Write the Int-Function adress to the PieVectTable
		2. Step: Enable the selected CPU-Int. in IER-Register
		3. Step: Enable the selected Int. in the PIEIER-Register
		4. Step: Enable the needed Pie Block in the PIE-Control-Register
		5. Step: Enable global Interrupts in the CPU-Register ST1.0


\param[in]

*/ 
void InitHwiInterrupts(void)
{
	// 1. Step
	EALLOW;  // This is needed to write to EALLOW protected registers
	PieVectTable.TINT0 = &HwiTimer0;
	PieVectTable.XINT13 = &HwiTimer1;
	PieVectTable.TINT2 = &HwiTimer2;

	PieVectTable.DINTCH1 = &HwiDmaCh1Transmit;
	PieVectTable.DINTCH2 = &HwiDmaCh2Receive;
	EDIS;    // This is needed to disable write to EALLOW protected registers

	// Timer-1 f�r �berwachung PWM Endstufe, vorerst auf 1000�s
	//tHwiTimer1.usSemaphor = 0;
	//tHwiTimer1.ulCnt = 0;
	Timer1InitStart(1000.0);
	// initilization. over, stop at once
	StopCpuTimer1();
	CpuTimer1Regs.TCR.bit.FREE = 0;//Stop Timer, wenn ein Breakpoint erreicht wird
	CpuTimer1Regs.TCR.bit.SOFT = 0;
	
	// Da wir nach Ger�testart mit CCROT anfangen, hier 2ms einstellen
	tHwiTimer0.usSemaphor = 0;
	tHwiTimer0.ulCnt = 0;
	tHwiTimer2.ulCnt = 0;

	Timer0InitStart(2000.0);

	// todo 24.03.15 Timer-0 gleich wieder anhalten, der wird sp�tergestartet
	StopCpuTimer0();
		
	// 2. Step
	// Enable CPU Int1, CPU-Timer 0, PIE Group 1
	IER |= M_INT1;
  	// Enable CPU Int13, CPU-Timer 1, no PIE Group
  	IER |= M_INT13;
	// Enable CPU Int14, CPU-Timer 2, no PIE Group
	IER |= M_INT14;
	// Enable CPU Int7, DMA Ch-1, DMA CH-2, PIE Group 7
	IER |= M_INT7;

	// 3. Step
	// Enable TINT0 in the PIE: Group 1 interrupt 7
	PieCtrlRegs.PIEIER1.bit.INTx7 = 1;
	// Enable PIE Group 7, INT 1 (DMA CH1 Transmit)
	PieCtrlRegs.PIEIER7.bit.INTx1 = 1;
	// Enable PIE Group 7, INT 2 (DMA CH2 Receive)
	PieCtrlRegs.PIEIER7.bit.INTx2 = 1;
	
	// 4. Step
	// Enable the PIE block
	PieCtrlRegs.PIECTRL.bit.ENPIE = 1;
	
	// 5. Step
	// Enable global Interrupts and higher priority real-time debug events:
	EINT;   // Enable Global interrupt INTM
	ERTM;   // Enable Global realtime interrupt DBGM

}




/*!
\fn	void 
\brief	This Timer is in use for a definied counting of PWM Errors.
		The Counter starts from HWI

\param[in]

*/
interrupt void HwiTimer1(void)
{
	//NOP for Breakpoint
	asm(" nop");

	// EDC ****************************************
	if( *PtrEpwmErrCnt() > 0)
	{
		*PtrEpwmErrCnt() = 0;
		CpuTimer1Regs.TCR.bit.TSS = 1;
	}
	//Timerint Bit loeschen
	CpuTimer1Regs.TCR.bit.TIF = 1;
	// The CPU acknowledges the interrupt.
	EDIS;

	// EDC ****************************************
	
	// diagnos counters
	//CpuTimer1.InterruptCount++;
	//tHwiTimer1.ulCnt++;
}


/*!
\fn		void 
\brief  

\param[in]

*/    
interrupt void HwiTimer2(void)
{
	// Int runs every 0.1ms
	// hier wird Int3 (EPWM1) und Int13 (Timer-0) freigegeben. Diese m�ssen ungest�rt laufen.
	// Global Disable all Interrupts
	//DINT;
   	// Enable CPU int3, hier EPWM1_INT und INT1 Timer-0
	IER |= M_INT3;
	IER |= M_INT1;
	// 16.01.14 Enable CPU int7, DMA6 Analog Current Measure
	IER |= M_INT7;
	// Enable global Interrupts and higher priority real-time debug events:
	EINT;   // Enable Global interrupt INTM
	ERTM;   // Enable Global realtime interrupt DBGM
	
	// check for new command from master and do....
	SwiMasterCommand();
	tHwiTimer2.ulCnt ++;
	// ********** bis hier Zeitmessung **********

}



/*!
\fn			interrupt void HwiTimer0(void)
\brief  
			Interrupt function for Timer-0, which controls the different measuring
			modes. Within this functions the INT3 and INT7 have to be enabled immediatly,
			because they should have the highest priority.
			The main task is position detection -> omega calculation -> motor control
			Int runs every 200�s (CR), 2000�s (CS), roughly 625-39063�s (OSC 0,1-50Hz)
			see also:
			http://processors.wiki.ti.com/index.php/Interrupt_Nesting_on_C28x

\param[in]

\details
	changes:	29.01.14 modifikation Int.-Handling Timer-0 und EPWM1 (FOC)
				03.03.14 Angle sampling now in FOC interrupt because of Real-Time sampling

*/    
interrupt void HwiTimer0(void)
{
	// ********** Read This *****************************************************
	// 	The CPU blocks new hardware interrupts until the interrupt service routine
	// (ISR) begins. In addition, the IER bit is cleared (in step 10) before the ISR
	// begins. Therefore, an interrupt from the same source cannot disturb the
	// ISR until the IER bit is set again by the ISR.
	// All INT's are disabled, this is done by hardware. The higher priority 
	// Interrupts are enabled manually here, to allow interrupting this HwiTimer0
	//  interrupt by modification of IER of the CPU.
	// Finally Global interrupt are enabled immediatly -> INTM
	// **************************************************************************

	//	DINT (INTM = 1 <-> asm(" setc INTM") ) is not neccessary, this is done by the CPU;

	//tHwiTimer0.usSemaphor = 1;
  	// Enable CPU int3, hier EPWM1_INT -> FOC interrupt
	IER |= M_INT3;
	// 16.01.14 Enable CPU int7, DMA6 Analog Current Measure
	IER |= M_INT7;
	// Acknowledge this interrupt to receive more interrupts from group 1
	PieCtrlRegs.PIEACK.all = PIEACK_GROUP1;
	// Enable global Interrupts and higher priority real-time debug events:
	EINT;   // Enable Global interrupt "clr INTM"
	ERTM;   // Enable Global realtime interrupt DBGM

	// ********** ab hier Zeitmessung **********

  	// mit dem Z�hlerstand der GEMAC-Interpolation Winkel berechnen 03.03.14 in FOC
	//AngleCalculation( *PtrFocIp2000Cnt() );
	// gleich Drehzahl berechnen, l�uft immer mit
	OmegaCalculation();

	// call control according measuring mode
	switch ( *PtrMeasMode() )
    {	case CSOSC:
		case CDOSC:
			// call CsFastOsc-Mode
			CsFastOscControl();
		break;
	    case CSROT:
			// call CS-Mode
			CsMode( *PtrMeasSetvalue() );
		break;
	    case CRROT:
			// call CR-Mode
			CrMode( *PtrMeasSetvalue() );
		break;
		case CCROT:
			// call CC-Mode, not really a measuring mode
			CcMode( *PtrMeasSetvalue() );
		break;

		default:
			//CcMode();
		break;
	}
	// Interrupts des Timer-0 z�hlen, hierzu hier r�cksetzen
	//CpuTimer0.InterruptCount++;
	tHwiTimer0.ulCnt++;

	// ********** bis hier Zeitmessung **********
	//tHwiTimer0.usSemaphor = 0;
}




/*!
\fn		void Swi00_RotInit()
\brief  Software-Interrupt00

\param[in]

*/

void Swi00_RotInit(void)
{
	// Timer0 sofort anhalten, sonst haut wom�glich der Timer in die Initialisierung
	Timer0Stop();	
	/* Messung Rotation initialisieren, denkbar sind CRROT, CSROT, CCROT */
	MeasRotInit();
	// Timer-0 Stop, Abtastzeit an Timer0 �bergeben. Timer-0 Start
	SetCondTimer0( *PtrSamplingTime() );
	// transfer actual data to SSI-Buffer
	SendSysData();


}


/*!
\fn		void Swi01_RotStop()
\brief  Software-Interrupt01

\param[in]

*/
void Swi01_RotStop(void)
{
	// first stop Timer0, to stop other running application Modes (..CCROT, CSROT, CRROT etc.)
	Timer0Stop();
	// reset the parameters for Rotation Modes
	MeasStop();
	// Timer-0 Stop, Abtastzeit an Timer0 �bergeben. Timer-0 Start
	SetCondTimer0( *PtrSamplingTime() );
	// Ende des Messjobs anzeigen
	// transfer actual data to SSI-Buffer
	SendSsiMeasData();
}



/*!
\fn		void Swi02_RotSet()
\brief  Software-Interrupt02

\param[in]

*/    
void Swi02_RotSet(void)
{
	// change new Set-value, Mode, and others
	MeasRotSet();
}

/*!
\fn		void Swi02_RotSet()
\brief  Software-Interrupt02

\param[in]

*/    
void Swi03_RotData(void)
{
	// transfer actual data to SSI-Buffer
	SendSsiMeasData();

}


/*!
\fn		void Swi05_FastOscInit()
\brief  Software-Interrupt05

\param[in]

*/
void Swi05_FastOscInit(void)
{
	// Timer0 sofort anhalten, sonst haut wom�glich der Timer in die Initialisierung
	Timer0Stop();

	// 14.04.2010 Winkel Reset an diese Stelle f�r beide Messarten
	SetAngleOffset();

	// Get parameters from �C-1 via SSI and initialize Fast Oscillation CS or CD mode
	CsFastOscInit();

	// Timer-0 Stop, Abtastzeit an Timer0 �bergeben. Timer-0 Start
	SetCondTimer0( *PtrSamplingTime() );

	// vorerst Rot-Daten senden, transfer actual data to SSI-Buffer
	SendSysData();

}



/*!
\fn		void Swi06_FastOscAmp()
\brief  Software-Interrupt06

\param[in]

\details
	changes:	15.09.14 Amplitude-Sweep although with Start-Stop

*/
void Swi06_FastOscAmp(void)
{
	// Timer0 sofort anhalten, sonst haut wom�glich der Timer in die Initialisierung
	Timer0Stop();
	// 15.09.14, neue Amplitude vorgeben aber �ber Frequenz Funktion, um alles zu initialisieren
	//SetCondFastOscAmp();
	SetCondFastOscFrq();
	// Timer-0 Start
	SetCondTimer0( *PtrSamplingTime() );
	// 
	SendSsiOscData();
}


/*!
\fn		void Swi10_FastOscFrq()
\brief  Software-Interrupt10

\param[in]

*/
void Swi10_FastOscFrq(void)
{
	// Timer0 sofort anhalten, sonst haut wom�glich der Timer in die Initialisierung
	Timer0Stop();
	// set new Frequency
	SetCondFastOscFrq();
	// Timer-0 Start
	SetCondTimer0( *PtrSamplingTime() );
	//	
	SendSsiOscData();
}


/*!
\fn		void Swi07_FastOscStop()
\brief  Software-Interrupt07

\param[in]

*/
void Swi07_FastOscStop(void)
{
	/* und Timer wieder geradebiegen */
	Timer0Stop();
	/* FastOsc. Daten zur�cksetzen */
	SetCondFastOscStop();
	/* Abtastzeit einstellen wieder auf 2ms */
	*PtrSamplingTime() = ST_2000US;
	// Abtastzeit an Timer0 �bergeben
	SetCondTimer0( *PtrSamplingTime() );
	SendSsiOscData();
}


/*!
\fn		void Swi08_GetOscDataRaw()
\brief  Software-Interrupt08, Oszillation Daten in SSI puffer schreiben

\param[in]

*/
void Swi08_GetOscDataRaw(void)
{
	SendSsiOscData();
}


/*!
\fn		void Swi20_StartIpRefSearch()
\brief  Software-Interrupt20

\param[in]

*/
void Swi20_StartIpRefSearch (void)
{
	// Daten Winkelmessung r�cksetzen
	ResetValueAngleMeas();
	// Set FOC to Start Mode
	FocSetToRefMode();
	// 21.10.15, Reset diagnose data from IP2000
	*PtrDsIP() = IP_NOERROR;
	// todo, do not know to send other infos at the moment
	SendSysData();
	//NOP for Breakpoint
	asm(" nop");

}

/*!
\fn		void Swi21_IpResetAngle()
\brief  Software-Interrupt21

\param[in]

*/
void Swi21_IpResetAngle(void)
{
	// Messwert schon mal null setzen
	*PtrPhiMeasValue() = 0.0;
	/* Winkeloffset speichern f�r Relativwinkelberechnung */
	SetAngleOffset();
	// todo, do not know to send other infos at the moment
	SendSysData();
}


/*!
\fn		void Swi32_LoadConstTable()
\brief  Software-Interrupt32, a part of any correction table
		has been received and has to be stored!

\param[in]

*/
void Swi32_LoadConstTable(void)
{
	// Load data or correction tabel into detected array
	CorrLoadConstTable();

}

/*!
\fn		void Swi35_LoadConstTable()
\brief  Software-Interrupt32, the correction mode table has been received.
		Nothing more to do then to store in Correction module.
\param[in]

*/
void Swi35_LoadCorrMode(void)
{
	// Load the different correction modes
	CorrLoadModeTable();
	// todo, do not know to send other infos at the moment
	SendSysData();
	//NOP for Breakpoint
	asm(" nop");
}

/*!
\fn		void Swi87_MscAnalyse()
\brief  Software-Interrupt87, Start Command for MSC Calculation table after having
		finished Data-Aquisition.
\param[in]

*/
void Swi87_MscAnalyse(void)
{
	// todo semaphore fpr MscTblCalc() in main.c
	//*PtrMscState() = MSC_START_ANALYSE;
	// Dummy Quittierung senden
	*PtrMeasActDataMode() = MSC_DATA_CAL;
	// transfer actual data to SSI-Buffer
	SendSsiMeasData();
	
	// semaphore fpr MscTblCalc() in main.c
	*PtrMscState() = MSC_END;

}

/*!
\fn		void Swi88_MscAnalyse()
\brief  Software-Interrupt87, Start Command for MSC Calculation table after having
		finished Data-Aquisition.
\param[in]

*/
void Swi88_MscStartAquisition(void)
{
	// semaphore for MscDatAqInit() in main.c
	*PtrMscState() = MSC_START_AQU;
	// transfer actual data to SSI-Buffer
	SendSsiMeasData();

}

/*!
\fn		void Swi89_FriStartAquisition()
\brief  Software-Interrupt89, Start Command for Filter-Reset Torque
		while Friction Detection measurement is ongoing.
\param[in]

*/
void Swi89_FriStartAquisition(void)
{
	// semaphore for Filter Reset in Data-Filter Torque
	//*PtrFriState() = FRI_START_AQU;
	// Reset Torque filters for Friction detection measurement
	FriStartDatAq();

	// transfer actual data to SSI-Buffer
	SendSsiMeasData();
}



/*!
\fn		void Swi95()
\brief  Software-Interrupt94
\param[in]
*/
void Swi95_SetAngleConstant (void)
{
	//NOP for Breakpoint
	asm(" nop");
	FocSetTestAngleConstant();
	// todo, do not know to send other infos at the moment
	SendSysData();
	//NOP for Breakpoint
	asm(" nop");
}


/*!
\brief Select Function from Master Command
\fn SwiMasterCommand(void)
\author K�ster, Ralf
\date 19.12.2011
\details	With every Command Data-Set from the Master-�C via SSI
			there is received a Software-Interrupt-Vektor, which describes
			the kind of command and/or data!
			Here this Vector is interpreted an the according function call
			will be performed.

\param[IN] void
\return void
	Changes:	

*/

void SwiMasterCommand(void)
{
	
	// Has been a record received?
	if( *PtrSsiRecComm() == TRUE )
	{
		// Vorgabedaten lesen aus Empfangsbuffer
		// Int-Vector reading
		tSwiMasterComm.usIntVector = *PtrSsiRecBuffer();
		// Set Semaphore for Data-Receive
		*PtrSsiRecComm() = FALSE;
		if( (tSwiMasterComm.usIntVector >= 100) && ( tSwiMasterComm.usIntVector <= 200) )
		{
			GpioDataRegs.GPBTOGGLE.bit.GPIO49 = 1;
		}
				
		switch(tSwiMasterComm.usIntVector)
		{
			case IV_ROTINIT:
				Swi00_RotInit();
				break;
			case IV_ROTSTOP:
				Swi01_RotStop();
				break;
			case IV_ROTSET:
				Swi02_RotSet();
				break;
			case IV_ROTDATA:
				Swi03_RotData();
				break;
			case IV_CSOSCFASTINIT:
				Swi05_FastOscInit();
				break;
			case IV_CSOSCFASTAMP:
				Swi06_FastOscAmp();
				break;
			case IV_CSOSCFASTSTOP:
				Swi07_FastOscStop();
				break;
			case IV_OSCDATARAW:
				Swi08_GetOscDataRaw();
				break;
//			case IV_OSCDATAFFT:
//				Swi09_GetOscDataFft();
//				break;
			case IV_CSOSCFASTFRQ:
				Swi10_FastOscFrq();
				break;
			case IV_START_REFSEARCH:
				Swi20_StartIpRefSearch();
				break;
			case IV_RESET_ANGLE:
				Swi21_IpResetAngle();
				break;
	
			//case IV_INIT_DEVCONST:
				//Swi30_Init_DevConst();
				//break;
			case IV_SEND_DEVCONST:
			case IV_SEND_FRICTIONCONST:
			case IV_SEND_MSCCONST:
			case IV_SEND_G1CONST:
			case IV_SEND_G2CONST:
			case IV_SEND_SENSCONST:
			case IV_REQ_DEVCONST:
			case IV_REQ_FRICTIONCONST:
			case IV_REQ_MSCCONST:
			case IV_REQ_G1CONST:
			case IV_REQ_G2CONST:
			case IV_REQ_SENSCONST:
				Swi32_LoadConstTable();
				break;
			case IV_SEND_CORRECTION_MODE:
				Swi35_LoadCorrMode();
				break;
//				Swi80_SendConstTable();
//				break;

			case IV_MSC_START_ANALYSE:
				Swi87_MscAnalyse();
				break;

			case IV_MSC_START_AQ:
				Swi88_MscStartAquisition();
				break;
			// Start Data Aquisition Friction Detection after achievement of Set-value
			case IV_FRI_START_AQ:
				Swi89_FriStartAquisition();
				break;

			case IV_SET_ANGLE_CONSTANT:
				Swi95_SetAngleConstant();
				break;

			case 117:
				//NOP for Breakpoint
				asm(" nop");
				break;

			case IV_INIT_FOC:
				//NOP for Breakpoint
				asm(" nop");
				// FOC_CUROFFSET = 1, starts FOC
				*PtrFocState()= 1;
				// starts FOC and the initialization of the PWM
				//*PtrRootTskSem() = 1;
				// transfer actual data to SSI-Buffer
				SendSsiMeasData();

				break;
			
			case IV_SYSTEMDATA:			// Systemdaten senden
				SendSysData();
				break;
			default:
				// unknown Interrupt-Vektor
				//*PtrDsComm() = ERR_SSI_IV;
				*PtrDsComm() = *PtrDsComm() +1;
				SendSysData();
				//NOP for Breakpoint
				asm(" nop");
				break;
		}
	}
}

