/*!
\file	InP2000.c
\brief	Functions for Gemac IP2000 control
		The IP2000 is special counter device for handling encoder-sensors
		This device is controlled by the C 28F335 via SPI-A interface.
		(see here also , http://www.ti.com/lit/ug/sprueu3a/sprueu3a.pdf)

\author	Koester
\date	11.10.2012

\details
	changes: 

*/

/* Include files ************************************************************/
#include <math.h>
#include "Project.h"		// all global things as Type definitions, Hardware def., ...#include "Device.h"
#include "GCIP2000.h"
#include "Ip2000.h"
#include "SpiA.h"
#include "Test.h"
#include "Timer.h"
#include "Diagnos.h"

/* Macro definitions ********************************************************/
// Initwerte GC-IP-2000
#define GCIP_CFG1_INIT		0x77771050
#define GCIP_CFG2_INIT		0x00008000	// 0000 0000 0000 0000 1000 0000 0000 0000
#define GCIP_CNTRLG_INIT	0x04000400	// 0000 0100 0000 0000 0000 0100 0000 0000
#define GCIP_CNTRLO_INIT	0x00000000	// 0000 0000 0000 0000 0000 0000 0000 0000
#define GCIP_RSV_INIT		0x0000		// 0000 0000 0000 0000


/* Types ********************************************************************/

// Data for IP2000 Modul
typedef struct
{
   Uint16   unError;		//< Super-error: maybe System doesn't work
   int32	lCnt;			//< Measurement Value
   Uint16	usData[3];
   Uint16	usReference;
}
t_IpData;


/* Local Prototypes *********************************************************/


/* Global variables *********************************************************/
t_IpData	volatile tIp2000 = {0,0,0,0,0,0};

// copy to RAM in Flash-Mode, for faster running
#ifdef FLASH
#pragma CODE_SECTION(Ip2000RecvDataSpiRxIsr, "ramfuncs");
#pragma CODE_SECTION(IP2000GetTrigData, "ramfuncs");
#pragma CODE_SECTION(Ip2000GetReferenzImpuls, "ramfuncs");
#pragma CODE_SECTION(Ip2000SendDataExtInt5StartIsr, "ramfuncs");
#endif


/* Functions ****************************************************************/


/*!
\fn		int32* PtrIp2000Cnt(void)
\brief		

			 
\param[in]

*/
//dito counter value
volatile int32* PtrIp2000Cnt(void)
{
	return &tIp2000.lCnt;
}


//*****************************************************************************
/*! \brief Interrupt fr das Senden der Daten ber SPI
 *
 *	author david.berthold
 *	
 *	Es wird durch den Komperator der IP2000 getriggert, welches den aktuellen
 * 	Zhlerstand in ein Register (MVAL) zwischenspeichert. Diese Funktion wird
 * 	etwas spter als die Triggerung ausgefhrt und schreibt in das SPI-Sende-
 * 	register, dass der Delfino das MVAL register lesen mchte.
 *	Dauer ca. 100ns! (optimize for speed -mf5)
 *
 * 	\return		None
 */
//*****************************************************************************
interrupt void Ip2000SendDataExtInt5StartIsr(void)
{
	//Werte in Puffer schreiben
	//SpiaRegs.SPITXBUF = (GCIP_OPC_RD0<<12)|(GCIP_OPC_HWA<<8)|(GCIP_ADR_CNT);
	SpiaRegs.SPITXBUF = (GCIP_OPC_RD0<<12)|(GCIP_OPC_HWA<<8)|(GCIP_ADR_MVAL);
	SpiaRegs.SPITXBUF = ((unsigned int)(GCIP_OPC_RD1<<12));
	SpiaRegs.SPITXBUF = ((unsigned int)(GCIP_OPC_NOP<<12));
	PieCtrlRegs.PIEACK.all = PIEACK_GROUP12; // To receive more interrupts from this PIE group, acknowledge this interrupt
}


//*****************************************************************************
/*! \brief Interrupt fr das Auslesen der empfangenen Daten ber SPI
 *
 *	author david.berthold
 *	
 *	Wenn die Daten aus dem Sendepuffer alle gesendet wurden, fllt sich 
 * 	automatisch der Empfangspuffer mit den Daten. Diese Daten werden aus dem
 * 	Puffer gelesen und in einem Array zwischengespeichert.
 *	Dauer ca. 270ns! (optimize for speed -mf5)
 *
 * 	\return		Beschreibung des zurckgegebenen Wertes
 */
//*****************************************************************************
interrupt void Ip2000RecvDataSpiRxIsr(void)
{
	//empfangene Werte auslesen
	tIp2000.usData[0] = SpiaRegs.SPIRXBUF;
	tIp2000.usData[1] = SpiaRegs.SPIRXBUF;
	tIp2000.usData[2] = SpiaRegs.SPIRXBUF;

	SpiaRegs.SPIFFRX.bit.RXFFINTCLR = 1;
	//Es wird geprft, ob im Puffer des IP noch weitere Daten zur 
	//Abholung bereit stehen. Wenn ja wird wieder der SPI-Puffer
	//beschrieben, dass diese Daten gelesen werden
	if(GpioDataRegs.GPBDAT.bit.GPIO50 == 0)
	{
		//Ip2000SendDataExtInt5StartIsr(); oder besser:
		//Werte in Puffer schreiben
		SpiaRegs.SPITXBUF = (GCIP_OPC_RD0<<12)|(GCIP_OPC_HWA<<8)|(GCIP_ADR_MVAL);
		SpiaRegs.SPITXBUF = ((unsigned int)(GCIP_OPC_RD1<<12));
		SpiaRegs.SPITXBUF = ((unsigned int)(GCIP_OPC_NOP<<12));
	}
	PieCtrlRegs.PIEACK.all = PIEACK_GROUP6; // To receive more interrupts from this PIE group, acknowledge this interrupt
}

//*****************************************************************************
/*! \brief Wenn IP2000 Referenzsignal ausgibt, wird dies in dieser Var
 * 	gespeichert
 *
 *	Dauer ca. 60ns! (optimize for speed -mf5)
 * 	author david.berthold
 */
//*****************************************************************************
interrupt void Ip2000GetReferenzImpuls(void)
{
	tIp2000.usReference = 1;
	// 30.01.14 Diagnosezwecke
	tIp2000.unError++;
	*PtrDsIP() = *PtrDsIP() | IP_REFOK;
	// todo 03.03.15 nach 1x erkannt, sperren, da nicht mehr notwendig
	PieCtrlRegs.PIEIER12.bit.INTx2 = 0;
	PieCtrlRegs.PIEACK.all = PIEACK_GROUP12; // To receive more interrupts from this PIE group, acknowledge this interrupt
}




/*!
\fn		long IP2000GetCntData(void)
\brief  Lesen des Zhlerstandes des IP-DSP
			Gelesen wird der Zhlerstand
			 
\param[in]

*/
long IP2000GetCntData(void)
{
	return(tIp2000.lCnt);
}


//*****************************************************************************
/*! \brief Initialisierung des IP2000
 *
 *	author david.berthold
 *	
 * Initialisierung des IP2000. Es werden dabei folgende Punkte vorgenommen:
 * 		- Generierung des Taktes (25MHz)
 * 		- Reset auf H
 * 		- Initialisierung der Register des IP2000, welche ber die oben aufgefhrten Macros beschrieben werden
 * 		- Interrupteinstellungen fr Senden/Empfangen 
 *
 * 	\return		ErrorCodes siehe Datei [@ref error_codes.h]
 */
//*****************************************************************************
Uint16 Ip2000Init(void)
{

	unsigned long recdata;
	unsigned long timeout;
	// Setup APWM mode on CAP4, set period and compare register fr die Takterzeugung
	//von 25MHz fr IP2000	
	ECap4Regs.ECCTL2.bit.CAP_APWM = 1;	// Enable APWM mode
	ECap4Regs.CAP1 = 0x00005;			// Set Period value
	ECap4Regs.CAP2 = 0x0003;			// Set Compare value
	ECap4Regs.ECCLR.all = 0x0FF;		// Clear pending interrupts
	ECap4Regs.ECCTL1.bit.FREE_SOFT = 0x2;
	ECap4Regs.ECCTL2.bit.TSCTRSTOP = 1;	//Start Timer

	// Hier wird die SPI-Schnittstelle initialisiert
	SpiAInit();

	//  GPIO-77 = Reset IP2000
	//Reset an IP2000 auf H ber Transistor an TMS
	GpioDataRegs.GPCCLEAR.bit.GPIO77= 1;

	DELAY_US(100);

	// read the Asic-ID, if not IP2000 (0x9) then error
	recdata = SpiARead32(GCIP_ADR_STAT);
	if((recdata & 0x00F00000) != 0x00900000){
		return IP_RECEVID;
	}
	DELAY_US(1);
	
	//read register EEPSTAT
	timeout = 0;
	do
	{
		recdata = SpiARead16(GCIP_ADR_EEP);
		timeout++;
		DELAY_US(1);
	// wait if EEPBSY = 1, but only 100 readings (EEPBSY must always be 0)
	}while((recdata & 0x800) && (timeout < 100));
	if(timeout == 100)
	{
		return IP_EETIME;
	}
	// no EEPROM must be found EEPEXIST = 0
	if((recdata & 0x100) != 0){
		return IP_EECONF;
	}
	
	// CFG-1 (0x0C-0x0F) -> IP-Rate, Reference-mark, and others
	SpiAWrite32(GCIP_CFG1_INIT, GCIP_ADR_CFG1);
	DELAY_US(1);
	recdata = SpiARead32(GCIP_ADR_CFG1);
	if(recdata != GCIP_CFG1_INIT){
		return IP_CFG1;
	}

	// CFG-2 (0x10) -> ASYNC=1 -> DISCTL=0 (siganl control activ), and others
	SpiAWrite32(GCIP_CFG2_INIT, GCIP_ADR_CFG2);
	DELAY_US(1);
	recdata = SpiARead32(GCIP_ADR_CFG2);
	if(recdata != GCIP_CFG2_INIT){
		return IP_CFG2;
	}

	SpiAWrite32(GCIP_CNTRLG_INIT, GCIP_ADR_CNTRLG);
	DELAY_US(1);
	//Register CNTRLG wird intern geregelt, deshalb nicht zurcklesen

	SpiAWrite32(GCIP_CNTRLO_INIT, GCIP_ADR_CNTRLO);
	DELAY_US(1);
	//Register GCIP_ADR_CNTRLO wird intern geregelt, deshalb nicht zurcklesen

	SpiAWrite32(GCIP_RSV_INIT, GCIP_ADR_RSV);
	DELAY_US(1);
	recdata = SpiARead32(GCIP_ADR_RSV);
	if(recdata != GCIP_RSV_INIT){
		return IP_RSVCONF;
	}
	
	DELAY_US(1);
	//Laut Datenblatt soll 2mal MVAL gelesen werden
	recdata = SpiARead32(GCIP_ADR_MVAL);
	
	DELAY_US(1);
	recdata = SpiARead32(GCIP_ADR_MVAL);
	DELAY_US(1);
	//Reset Counter
	SpiAWrite8(GCIP_CMD_RESCNT, GCIP_ADR_CMD);
	
	//Umkonfiguration auf interruptgesteuertes Senden/Empfangen
	SpiAInitInterruptSendRecv();
	
	//Wenn 3 Bytes Empfangen wurden (MVAL) dann wird Int. generiert und die Werte in ein Array zw.-gespeichert
	EALLOW;
	PieVectTable.SPIRXINTA = &Ip2000RecvDataSpiRxIsr;
	EDIS;
	IER |= M_INT6;
	PieCtrlRegs.PIEIER6.bit.INTx1 = 1;

	//Das einlesen des Referenzsignals erfolgt durch einen externen Interrupt
	EALLOW;
	GpioIntRegs.GPIOXINT4SEL.bit.GPIOSEL = 52 - 32;// Xint4 is GPIO52
	EDIS;
	// Configure XINT4
	//XIntruptRegs.XINT4CR.bit.POLARITY = 0;      // falling edge interrupt
	XIntruptRegs.XINT4CR.bit.POLARITY = 1;      // rising edge interrupt
	// Enable XINT4
	XIntruptRegs.XINT4CR.bit.ENABLE = 1;        // Enable Xint1
	
	EALLOW;
	PieVectTable.XINT4 = &Ip2000GetReferenzImpuls;
	EDIS;
	PieCtrlRegs.PIEIER12.bit.INTx2 = 1;

	//Im DSP/Controller Mode des Ip gibt dieser einen Impuls aus,
	//wenn im Triggerregister ein Datum anliegt
	EALLOW;
	GpioIntRegs.GPIOXINT5SEL.bit.GPIOSEL = 50 - 32;// Xint5 is GPIO50
	EDIS;
	// Configure XINT5
	XIntruptRegs.XINT5CR.bit.POLARITY = 0;      // falling edge interrupt
	//XIntruptRegs.XINT5CR.bit.POLARITY = 1;      // rising edge interrupt
	// Enable XINT5
	XIntruptRegs.XINT5CR.bit.ENABLE = 1;        // Enable Xint1
	
	EALLOW;
	PieVectTable.XINT5 = &Ip2000SendDataExtInt5StartIsr;
	EDIS;
	PieCtrlRegs.PIEIER12.bit.INTx3 = 1;
	IER |= M_INT12;
	tIp2000.lCnt = 0;
	return IP_NOERROR;
}


//*****************************************************************************
/*! \brief Lesen der empfangenen Werte von der FOC
 *
 * 	\param ip2000_angle rckgabe der Zhlerwerte
 *
 *	author david.berthold
 *	
 *	Die empfangenen Werte werden in der ISR in einem Array zwischengespeichert.
 * 	In dieser Funktion werden die empfangenen Daten aus dem Array heraussortiert.
 * 	
 *
 * 	\return		ErrorCodes siehe Datei [@ref error_codes.h]
 */
//*****************************************************************************
//long lMesswert;
void IP2000GetTrigData(float32* fAngle)
{
	//unsigned long ulCnt;
	long lMesswert;

	//Es wird geprft, ob im Puffer des IP noch weitere Daten zur Abholung bereit stehen.
	//Wenn ja wird wieder der SPI-Puffer beschrieben, dass diese Daten gelesen werden
	if(GpioDataRegs.GPBDAT.bit.GPIO50 == 0)
	{
		//Werte in Puffer schreiben
		SpiaRegs.SPITXBUF = (GCIP_OPC_RD0<<12)|(GCIP_OPC_HWA<<8)|(GCIP_ADR_MVAL);
		SpiaRegs.SPITXBUF = ((unsigned int)(GCIP_OPC_RD1<<12));
		SpiaRegs.SPITXBUF = ((unsigned int)(GCIP_OPC_NOP<<12));
	}
	// store Error-Flag IP2000 to diagnose modul, an error occured, no details!		
	*PtrDsIP() = *PtrDsIP() | (tIp2000.usData[1] & IP_ERR);
	//Datenbernahme
	//ulCnt = ((unsigned long)(tIp2000.usData[2])<<16 ) | tIp2000.usData[1]; 
	lMesswert = (long)( ( (unsigned long)(tIp2000.usData[2])<<16 ) | tIp2000.usData[1] )>>2;
	// IP2000 liefert bei CW negative Werte
	//tIp2000.lCnt = -lMesswert;
	// Berechnung in ABSolute
	if(lMesswert < 0)
	{
		lMesswert = lMesswert + IP2000_MAX_CNT;
	}
	// IP2000 liefert bei CW negative Werte
	tIp2000.lCnt = IP2000_MAX_CNT - lMesswert;
	// Calculation in float, 1/5e06 = 200-09, 0...... 1.0000
	*fAngle = (float)( lMesswert) * 200e-09;
}

#ifdef RAUS
//*****************************************************************************
/*! \brief Bestimmung, ob die Referenzmarke ueberfahren wurde
 *
 * 	\param fCntValue aktueller Zaehlerwert
 *
 *	author david.berthold
 *
 * 	\return	 0 Referenzpunkt nicht berfahren
 * 	\return	 1 Referenzpunkt berfahren Zaehler zaehlt aufwaerts(Vorwaerts drehen)
 * 	\return	-1 Referenzpunkt berfahren Zaehler zaehlt rueckwaerts(Rueckwaerts drehen)
 */
//*****************************************************************************
int16 Ip2000CntRounds(void)
{
	Uint16 return_value = 0;
	return_value = tIp2000.usReference;
	// Semaphore Reference mark has been passed
	tIp2000.usReference = 0;
	return return_value;
}
#endif
