/*!
\file	Measmode.c
\brief	here we find the functions for the main control of all
		measure modes and data collection

\author	Koester
\date	19.10.2011
\details
	changes:	19.10.15, set also the AVG-Filter depth at Motor-Stop
                03.02.16, while Ramp-Mode, set Avg-Filter to 1% of Ramp-Time
	

*/

#define MEASMODE	/* compiler control */

#include "Project.h"		// all global things as Type definitions, Hardware def., ...
#include "mth.h"
#include "Uc1dsp.h"
#include "control.h"
#include "SsiComm.h"
#include "Measmode.h"
#include "Device.h"
#include "Crmode.h"
#include "Csmode.h"
#include "Ccmode.h"
#include "timer.h"
#include "filt.h"
#include "dsv.h"
#include "omega.h"
#include "version.h"
#include "correction.h"
#include "angle.h"
#include "math.h"
#include "Diagnos.h"
#include "msc.h"
#include "Test.h"


#ifdef FLASH
#pragma CODE_SECTION( MeasRotSet, "ramfuncs");
#pragma CODE_SECTION( MeasChangeMode, "ramfuncs");
#pragma CODE_SECTION( MeasCheckChanges, "ramfuncs");
#endif


/* Macro definitions ********************************************************/
#define	RESET	((Uint16) 0)
#define	VALUE	((Uint16) 1)


/* Types ********************************************************************/

// bei Messelementnderungen Statuswerte
enum eModeChange
	{
		CH_NONE = 0,
		CH_MODE = 1,
		CH_RAMP = 2,
		CH_SETV = 3
	};


// Struktur Daten fr allgemeine Messeinstellungen
typedef struct
{
	Uint16	usState;		// actual meas-Mode for internal use, e.g. ERROR-Messages
	Uint16	usDataMode;		// actual mode for special activated data modes like e.g. MSC
	Uint16	usFiltMode;		// actual Filter Mode
	Uint16	usModeOld;		// Meas.-Mode from one cycle before
	Uint16	usRmpOld;		// Ramp-Mode from one cycle before
	float32	fSamplingTime;	// sampling Time Motor-Control -> Timer-0
	float32	fSetOld;		// old Set-Value
	float32 fTrqOld;		// last Torque value from last sampling period

	t_MeasSetVal tSet;		// Sollwerte Messung
	t_MeasData tData;

} t_stcMeasuringMode;



/* Global variables *********************************************************/


// Datenstruktur fr CR-Mode
extern t_stcPiRcontrol tCrControl;


// Struktur Daten fr allgemeine Messeinstellungen anlegen

// Infos ber das gerade laufende Messelement
t_stcMeasuringMode tMeas;

/* Functions ****************************************************************/

/*!
\fn		float* PtrSamplingTime(void)
\brief  Read or Write actual Sampling-Time 
		Write with:	*PtrSamplingTime() = float;
		Read with:	= *PtrSamplingTime(); 

\author	Kster
\date	02.01.2012
\param[in]
\return	void

\details Changes:

*/  
float* PtrSamplingTime(void)
{
	return &tMeas.fSamplingTime;
}


//dito actual Meas.-Mode like CR, CS, CC....
Uint16* PtrMeasMode(void)
{
	return &tMeas.tSet.usMode;
}

//dito actual Meas.-State
Uint16* PtrMeasState(void)
{
	return &tMeas.usState;
}

//dito actual Data Mode
Uint16* PtrMeasActDataMode(void)
{
	return &tMeas.usDataMode;
}

//dito last Meas.-Mode from last Sampling Cycle
Uint16* PtrMeasModeOld(void)
{
	return &tMeas.usModeOld;
}

//dito actual Set-Value (for every Mode)
float* PtrMeasSetvalue(void)
{
	return &tMeas.tSet.fSetValue;
}

//dito actual Ramp Mode
Uint16* PtrMeasRmpMode(void)
{
	return &tMeas.tSet.usRmpMode;
}

// dito Set value DataMode, for special functions during measurement (e.G. MSC, Friction...)
Uint16* PtrDataMode(void)
{
	return &tMeas.tSet.usDataMode;
}

//dito Meas. Value Motortemperature from master
float32* PtrMotorTemp(void)
{
	return &tMeas.tSet.fMotorTemp;
}


//dito last Set-Value from last Sampling Cycle
float* PtrMeasSetvalueOld(void)
{
	return &tMeas.fSetOld;
}

//dito last Set-Value from last Sampling Cycle
float* PtrMeasTorqueOld(void)
{
	return &tMeas.fTrqOld;
}

//dito Meas. Value Angle [rad]
float* PtrPhiMeasValue(void)
{
	return &tMeas.tData.fPhi;
}

//dito Meas. Value Angle absolut 0....2*PI [rad]
float* PtrPhiAbsMeasValue(void)
{
	return &tMeas.tData.fPhiAbs;
}

//dito Meas. Value Torque [Nm]
float* PtrTorqueMeasValue(void)
{
	return &tMeas.tData.fTorque;
}

//dito Meas. Value Torque with deeper filter [Nm]
float* PtrTorqueAVGMeasValue(void)
{
	return &tMeas.tData.fTorqueAvg;
}

//dito Meas. Value Omega [rad/s]
float* PtrOmegaMeasValue(void)
{
	return &tMeas.tData.fOmega;
}


Uint16* PtrMeasDataState(void)
{
	return &tMeas.tData.usState;
}


Uint16* PtrMeasFilterMode(void)
{
	return &tMeas.usFiltMode;
}

//#ifdef RAUS
float* PtrMeasTestF6(void)
{
	return &tMeas.tData.f6;
}

float* PtrMeasTestF7(void)
{
	return &tMeas.tData.f7;
}

float* PtrMeasTestF8(void)
{
	return &tMeas.tData.f8;
}

float* PtrMeasTestF9(void)
{
	return &tMeas.tData.f9;
}
//#endif




/*!
\fn		void MeasmodeInit(void)
\brief  Initialisierungsroutine Rot Variablen!
		Alle Steuervariablen werden definiert init. um einen
		fehlerlosen Start zu ermglichen.
\author	Kster
\date	24.10.2011
\param
\return	
\details Changes:

*/  
void MeasmodeInit(void)
{
	#ifdef RAUS
	// todo Testdaten lschen
	TestDaten (0.0, CLEAR);	
	#endif

	// Meas. System and Data initializing
	tMeas.usState = MM_STOP;
	tMeas.usDataMode = NORMALMODE;
	tMeas.usFiltMode = 0;
	tMeas.usRmpOld = RMP_OFF;
	tMeas.fSetOld = 0.0;
	tMeas.fTrqOld = 0.0;
	// Vorgabewerte
	tMeas.tSet.usIV = IV_ROTINIT;
	tMeas.tSet.usID = 0;
	// at program start no mode, no motor control
	tMeas.tSet.usMode = NONE;
	tMeas.usModeOld = tMeas.tSet.usMode;
	tMeas.tSet.fSetValue = 0.00;
	tMeas.tSet.fMotorTemp = 20.0;
	tMeas.tSet.fTableLen = 1.0;
	tMeas.tSet.fNoFreq = 1.0;
	tMeas.tSet.fRampendauer = 60.0;
	tMeas.tSet.af[0] = 1.22;
	tMeas.tSet.af[4] = 1.2;
	tMeas.tSet.usRmpMode = RMP_OFF;
	tMeas.tSet.usDataMode = NORMALMODE;

	tMeas.tData.usIV = IV_ROTDATA;
	tMeas.tData.usID = 0x0001;
	tMeas.tData.fOmega = 0.0;
	tMeas.tData.fPhi = 0.0;
	tMeas.tData.fPhiAbs = 0.0;
	tMeas.tData.fTorque = 0.0;
	tMeas.tData.fTorqueAvg = 0.0;
	// show Mode in Measdata
	tMeas.tData.usState = MM_STOP;
	// show Error State hardware, FALSE == no ERROR
	tMeas.tData.usDiagn = FALSE;
	tMeas.tData.f6 = 0.0;
	tMeas.tData.f7 = 0.0;
	tMeas.tData.f8 = 0.0;
	tMeas.tData.f9 = 0.0;
	tMeas.tData.af[0] = 0.0;
	tMeas.tData.af[1] = 0.0;
	tMeas.tData.af[2] = 0.0;
	tMeas.tData.af[3] = 0.0;
	tMeas.tData.af[4] = 0.0;

	// zum Test Systemdata in Transmit-Buffer schreiben
	StartSsiSendData(&tMeas.tData);
	// CR-Rot und CS-Rot Mode init.
	CrmodeInit();
	CsmodeInit();
	CcModeInit();


	// Wir sind bei Start im CCROT Mode,Abtastzeit einstellen
	*PtrSamplingTime()= ST_2000US;	
	/* genderte Abtastzeit in Omegaberechnung bercksichtigen */
	SetConstOmegaCalc(ST_2000US);
	
	// Now we can start Timer-0, all is initialized!                
	/* Timer0 Stop, Abtastzeit an Timer0 bergeben, Timer0 Start */
	SetCondTimer0( ST_2000US );

}	// *** end MeasmodeInit(void)


/*!
\fn		void MeasCheckChanges(void)
\brief  Test of Changes in Set-Values after receiving a Swi02_RotSet() command!
		Changes could be in Mode, Ramp-Mode or Set-Value!
\author	Kster
\date	11.01.2012

\param[in]
\return	Uint16 Value with State of change

Changes:
*/
Uint16 MeasCheckChanges(void)
{
	// Change of Mode has most consequences, ramp and Set-Value changes will be respected too
	if( tMeas.usModeOld != tMeas.tSet.usMode)
	{
		return (CH_MODE);
	}
	// Change of Set-Value has no conseq. at the moment 10.01.2012
	else if( tMeas.fSetOld != tMeas.tSet.fSetValue)
	{
		return (CH_SETV);
	}
	// Change of Ramp-Mode has only conseq. to filters
	else if( tMeas.usRmpOld != tMeas.tSet.usRmpMode)
	{
		return (CH_RAMP);
	}
	// no change should deliver only datas
	else
	{
		return (CH_NONE);
	}
}

/*!
\fn		void MeasSetCondFilters(void)
\brief  Change the Filter settings when a change in Ramp-Mode has been received!
		This function will be called when a Change was detected!
\author	Kster
\date	11.01.2012

\param[in]
\return	

Changes:
*/
void MeasSetCondFilters(Uint16 p_usState)
{
	// what is the new Ramp-Mode?
	switch (p_usState)
	{
		// Reset to zero
		case RESET_DATA_FILT:
			// set all values to 0.0
			OmegaInitFilter(0.0);
			TorqueInitFilter(0.0);
		break;
		// preset with actual values
		case PRESET_DATA_FILT:
            if( tMeas.tSet.usRmpMode == 0 )
			{
			    // Preset Filters with actual values
//			    OmegaInitFilter(tMeas.tData.fOmega);
			    // todo 12.11.13 no update bei change in Set-Value
//			    TorqueInitFilter(tMeas.tData.fTorque);
				OmegaInitFilter( * PtrOmegaFolp() );
				TorqueInitFilter(* PtrCrTorqueRaw() );
            }
		break;
		// set conditions of the data filters
		case SETCON_DATA_FILT:
			// what is the new Ramp-Mode?
			switch (tMeas.tSet.usRmpMode)
			{
				case RMP_ON_LIN:
				case RMP_ON_LOG:
					// NOP for Breakpoint
					asm(" nop");
					
                    // 03.02.16, while Ramp-Mode, set Avg-Filter to 1,5% of Ramp-Time
					// 
					OmegaSetCondAvg(tMeas.fSamplingTime, tMeas.tSet.fRampendauer * 0.015);
					// 
					TorqueSetCondAvg(tMeas.fSamplingTime, tMeas.tSet.fRampendauer * 0.015);
					// Preset Filters with actual values
					OmegaInitFilter(tMeas.tData.fOmega);
					TorqueInitFilter(tMeas.tData.fTorque);
				break;
				case RMP_OFF:
					// only MovAvg-Filter update
					OmegaSetCondAvg(tMeas.fSamplingTime, TIME_250MS);
					TorqueSetCondAvg(tMeas.fSamplingTime, TIME_250MS);
					// Preset Filters with actual values
//					OmegaInitFilter(tMeas.tData.fOmega);
//					TorqueInitFilter(tMeas.tData.fTorque);
					OmegaInitFilter( * PtrOmegaFolp() );
					TorqueInitFilter(* PtrCrTorqueRaw() );

				break;
				default:
				break;
			}
		break;

		default:
		break;
	}
}




/*!
\fn		void MeasChangeMode(void)
\brief  Initialisierungsroutine fuer Regler bei Betriebsartaenderung
\author	Kster
\date	11.01.2012
\param[in]
\return	
Changes:
*/  
void MeasChangeMode(void)
{
    //volatile int i;

	// what is the new Mode?
	switch (tMeas.tSet.usMode)
	{
		// Hint: old mode could be CCROT or CRROT
		case CSROT:
			// generell mit 2 ms abtasten
			tMeas.fSamplingTime = ST_2000US;
            // several initializations for CS-Mode
            //SetCondCsMode();
			break;

		// Hint: old mode could be CCROT or CSROT
		case CRROT:
			// generell mit 1 ms abtasten
			tMeas.fSamplingTime = ST_200US;
			// Preallocation CR-Controller due to Mode change
			CrmodePreSetPiC();
            if ( (Uint16)*PtrRawdataFlag() != 2)
			{
			    // Calculate Control parameters dependend from Mass
			    CrmodeCalcPiC();
            }
			break;

		// Hint: old mode could be CSCROT or CRROT
		case CCROT :
			// Filter Update nicht notwendig, da kein reg Betriebsmode
			// Abtastzeit einstellen
			tMeas.fSamplingTime = ST_2000US;				
			break;
	  default :
			break;
	  }
	// mit neuer Abtastzeit Omegaberechnung initialisieren
	SetConstOmegaCalc( tMeas.fSamplingTime );

	/* Timer0 Stop, Abtastzeit an Timer0 bergeben, Timer0 Start */
//	SetCondTimer0( tMeas.fSamplingTime );
}



/*!
\fn		void MeasRotInit(void)
\brief  Initialisierungsroutine fuer CS-Rot. und CR-Rot. Modus
\author	Kster
\date	25.10.2011
\param[in]
\return	
Changes:
*/  


void MeasRotInit(void)
{
	Uint32	usL;
	#ifdef RAUS
	// todo 27.03.13 Testdaten lschen
	TestDaten (0.0, CLEAR);	
	#endif

	// Vorgabedaten lesen aus Empfangsbuffer
	GetSsiData(&tMeas.tSet);
	// Meas. System and Data initializing
	tMeas.usState = MM_NORMAL;
	
	// Overwrite actual data-mode if special mode is selected
	switch(tMeas.tSet.usDataMode)
	{
		case MSC_INIT_AQU:
			usL = (Uint32)tMeas.tSet.fTableLen;
			// write table length in MSC table too
			*PtrCorrMscTbl(0) = tMeas.tSet.fTableLen;
			// Configuration of MSC Detection, ptr. to MSC-Obj, MSC-Table and number of frequencies
			// 22.10.2012, for number of frequencies -> immer eins mehr...:-)	
			MscCfg(&tMscData, usL, PtrCorrMscTbl(usL+1), PtrCorrMscTbl(1), (1+(Uint32)tMeas.tSet.fNoFreq) );
			// for averaging several MSC detections reset the counter here
			*PtrMscAvgCnt() = 0;
			// for averaging several MSC detections reset the Buffer here
			CorrMscClearAvgBuffer();
		break;
		case FRI_INIT_AQU:
			//FriCfg( tMeas.tSet.fNoFreq );
			// update State in Meas-Data, data aqu. is running
			tMeas.usDataMode = FRI_DATA_AQU;
		break;
		default:
		break;
	}
	
	// Initialisierungen je nach Betriebsart
	switch(tMeas.tSet.usMode)
	{
		case CSROT:
			// Abtastzeit einstellen
			*PtrSamplingTime()= ST_2000US;
		break;

		case CRROT:
			// Abtastzeit einstellen
			*PtrSamplingTime() = ST_200US;
            if ( (Uint16)*PtrRawdataFlag() != 2)
			{
			    // Calculate Control parameters dependend from Mass
			    CrmodeCalcPiC();
            }
		break;

		case CCROT:
			// Abtastzeit einstellen
			*PtrSamplingTime()= ST_2000US;
		break;

		default:
			tMeas.usState = MM_E_MODE;
		break;

	}
	// 03.03.15 mit neuer Abtastzeit Omegaberechnung initialisieren
	SetConstOmegaCalc( tMeas.fSamplingTime );
	
	// Filter initialisieren mit Reset Filter
	MeasSetCondFilters(RESET_DATA_FILT);
	// reset acceleration values for Inertia-Correction
	OmegaResetAccelaration();
    // delete old torque value
	tMeas.fTrqOld = 0.0;

    // 15.02.16 Set MAVG-Filter 1-4 turn and allocate memory
    CrModeSetTrqFilter();
    CsModeSetOmgFilter();

	// store Angle offset for Relative Angle calculation
	SetAngleOffset();

	// set Measuring values to Zero
	tMeas.tData.fTorque = 0.0;
	tMeas.tData.fTorqueAvg = 0.0;
	tMeas.tData.fOmega = 0.0;
	tMeas.tData.fPhi = 0.0;

	// normal or error message have to be sent to the master
	tMeas.tData.usDiagn = tMeas.usState;

	// save old Set-Value
	tMeas.fSetOld = tMeas.tSet.fSetValue;
	tMeas.usRmpOld = tMeas.tSet.usRmpMode;
	tMeas.usModeOld = tMeas.tSet.usMode;
    
	// reset several Diagnos States (delete Error message)
	*PtrDsMotorCon() = MC_NOERROR;
}	// end MeasRotInit(void)


/*!
\fn		void MeasRotSet(void)
\brief  after receiving a Swi02_RotSet() command this function is called to check
		the new Set-Values. Changes could be in Mode, Ramp-Mode or Set-Value or
		in case of e.g. *-Time-Elements no changes.
		This function has to deliver only the Meas.-data in this case.

\author	Kster
\date	10.01.2012
\param[in]
\return	
Changes:
*/  
void MeasRotSet(void)
{
	Uint16 usChstate;

	// Vorgabedaten lesen aus Empfangsbuffer
	GetSsiData(&tMeas.tSet);

	asm(" nop");
	if( tMeas.tSet.usRmpMode == 1)
	{
		// NOP for Breakpoint
		asm(" nop");
		asm(" nop");
		asm(" nop");
		asm(" nop");
		asm(" nop");
	}
	// e.g. Overspeed or others, then stop the motor with Set-Value
	if( tMeas.usState == MM_STOP )
	{
		tMeas.tSet.fSetValue = 0.0;
		tMeas.tSet.usMode = CCROT;
	}

	usChstate = MeasCheckChanges();
	// Initialisierungen je nach Betriebsart
	switch(usChstate)
	{
		// Change in Mode, most influence to control, filters, mode
		case CH_MODE:
			// Timer 0 Stop, because of change in sampl. time
			StopCpuTimer0();
			// change controllers and Sampling time
			MeasChangeMode();
			// Set Filter Conditions dependence from Sampling-Time, Ramp-Mode
			MeasSetCondFilters(SETCON_DATA_FILT);
            // 25.01.16 new Setvalue, clear semaphore
            *PtrCrState() = FALSE;
            // several initializations for CS-Mode and CR-Mode
            SetCondCsMode();
            SetCondCrMode();
			/* Timer0 Stop, Abtastzeit an Timer0 bergeben, Timer0 Start */
			SetCondTimer0( tMeas.fSamplingTime );
		break;
		// Change in Ramp mode and or Set-Value
		case CH_RAMP:
			// Timer 0 Stop, because of change in sampl. time
			StopCpuTimer0();
			// Set Filter Conditions dependence from Sampling-Time, Ramp-Mode
			MeasSetCondFilters(SETCON_DATA_FILT);
			/* Timer0 Stop, Abtastzeit an Timer0 bergeben, Timer0 Start */
			SetCondTimer0( tMeas.fSamplingTime );
		break;
		// Change only in Set-Value, only filter presetting
		case CH_SETV:
			// Set Filter Conditions dependence from Sampling-Time, Ramp-Mode
			MeasSetCondFilters(PRESET_DATA_FILT);
            // 25.01.16 new Setvalue, clear semaphore
            *PtrCrState() = FALSE;
            // several initializations for CS-Mode and CR-Mode
            SetCondCsMode();
		break;

		case CH_NONE:
			// Abtastzeit einstellen
		break;
		default:
		break;
	}
	// actual data mode as handshake to master
	tMeas.tData.usState = tMeas.usDataMode;
	// normal or error message have to be sent to the master
	tMeas.tData.usDiagn = tMeas.usState;

	// neueste Messdaten in Transmit-Buffer schreiben
	StartSsiSendData(&tMeas.tData);
	
	// save Mode for next Set-Command
	tMeas.usModeOld = tMeas.tSet.usMode;
	tMeas.usRmpOld = tMeas.tSet.usRmpMode;
	// save old Set-Value
	tMeas.fSetOld = tMeas.tSet.fSetValue;

}	// MeasRotSet(void)




/*!
\fn		void MeasStop(void)
\brief  Beenden von Rot. und Osz. Modi

\param[in]

*/    
void MeasStop(void)
{
	// *** Set Values Reset ***
	// Reset Mode	
	tMeas.tSet.usMode = CCROT;
    // Setvalue Zero
	tMeas.tSet.fSetValue = 0.0;
	// no Ramp
	tMeas.tSet.usRmpMode = RMP_OFF;
	// Data with filterd values
	tMeas.tSet.usDataMode = TRUE;
	// show Mode in Measdata
	tMeas.tData.usState = MM_STOP;
	// Abtastzeit einstellen
	*PtrSamplingTime()= ST_2000US;
	// Drehzahlmessung wg. Abtastzeit anpassen
	SetConstOmegaCalc( *PtrSamplingTime() );
	// 19.10.15, set also the AVG-Filter depth
	OmegaSetCondAvg(tMeas.fSamplingTime, TIME_250MS);
	TorqueSetCondAvg(tMeas.fSamplingTime, TIME_250MS);
	// Filter initialisieren mit Reset Filter
	MeasSetCondFilters(RESET_DATA_FILT);
    
    CsModeResetOmgFilter();
    // 15.02.16 reset the moving average filter torque in CR-Mode
    CrModeResetTrqFilter();
	//Daten fr Rheowin init.
	tMeas.tData.fTorque = 0.0;
	tMeas.tData.fOmega = 0.0;
	tMeas.tData.fPhi = 0.0;

}



/*!
\fn		void SendSsiMeasData(void)
\brief  Messdaten in Sendepuffer bertragen
\author	Kster
\date	01.10.2011
\param[in]
\return	void

Changes:

*/  
void SendSsiMeasData(void)
{
	// in Transmit-Buffer schreiben
	StartSsiSendData(&tMeas.tData);
}





#ifdef RAUS
void MeasCreateTestTorqueData(Uint16 unTblLen, float32 *pafTrq)
{
	Uint16 i;
	float32 fTrq, fAngStep;
	const float32 fTrqAmp = 0.000125;
    fAngStep = (cf2Pi / unTblLen);
    for (i = 0; i < unTblLen; i++)
    {
		//fTrq = 	0.000990 + (fTrqAmp * ( sin(i*fAngStep))) + (0.5 * fTrqAmp * ( sin(4*i*fAngStep)));
		fTrq = 	0.000990 + (fTrqAmp * ( sin(4*i*fAngStep)));
        *(pafTrq+i) = fTrq;
    }
}
#endif

