/*!
\file	omega.c
\brief	Drehzahlberechnung / Calculation Omega

\author	Koester
\date	17.01.2008

\details
 changes:	08.09.09 Rohdatenvorfilter Omega auch richtig vorbelegen
			22.12.09 Variablen an Style-Guide angepasst (float double)
			03.03.15 Initialisierung FOC Interruptzaehler fuer exakte
					   Phi Abtastung in SetConstOmegaCalc()
        06.09.16 use device constant instead of fix value for check Omega max.
        21.09.16 back again to fixed value 200.0 rad/s

*/

/* Include files ************************************************************/
#include <math.h>    /* wegen Betragsfunktion fabs */
#include "project.h"		// all global things as Type definitions, Hardware def., ...
#include "Uc1dsp.h"
#include "mth.h"
#include "filt.h"
#include "Measmode.h"
#include "angle.h"
#include "dsv.h"
#include "omega.h"
#include "Diagnos.h"
#include "Pmsm.h"
#include "device.h"



/* Macro definitions ********************************************************/

/* Local Prototypes *********************************************************/
#ifdef FLASH
#pragma CODE_SECTION( OmegaCalculation, "ramfuncs");
//#pragma CODE_SECTION( PtrOmegaRaw, "ramfuncs");
//#pragma CODE_SECTION( PtrOmegaFolp, "ramfuncs");
//#pragma CODE_SECTION( PtrOmegaAvg, "ramfuncs");
//#pragma CODE_SECTION( PtrOmegaOld, "ramfuncs");
#endif

/* Types ********************************************************************/
struct
{
    int nActual;
    float32 fKOmegaCalc;      // zur Berechnung in rad/s aus Bit
    float32 fRaw;             // aktueller Rohwert, ohne jede Filterung
    float32 fOld;             // vorangegangener Wert
    float32 fFolpOut;         // Ausgang Tiefpassfilter 1.Ordnung
	float32 fAvg;			// avg-filterd value
    float32 fARaw;            // aktueller Wert Beschleunigung
	float32 fAvgM1;			// avg-filtered value from last sample period		

} tOmega;

/* Local Prototypes *********************************************************/

/* Global variables *********************************************************/

/* Filterstruktur f�r Regressionsfilter Phi anlegen */
//t_RegFilterStruct tREGFilterOmega;
//#pragma DATA_SECTION(tPhiMemory, ".SDRAM")
//#pragma DATA_SECTION(tOmegaLinReg, ".SDRAM")
//t_DataMemoryStruct		tPhiMemory;
/* Struktur f�r lineare Regression Winkel/Zeit */
//t_LinRegressionStruct	tOmegaLinReg;

t_FOLPF	sFOLPFOmega;			// First-Order-Low-Pass-Filter (FOLP) Omega
t_stcAvgIncMem tAvgOmega;		// Moving-Average-Filter (MAV) Omega


/* Functions ****************************************************************/



/*!
\fn		float* PtrOmegaRaw(void)
\brief  Read or write Omega raw data (not filtered)
		Read with:	*PtrOmegaRaw() = float;
		Write with:	float = *PtrOmegaRaw(); 

\author	K�ster
\date	01.10.2012
\param[in]
\return	void
\details Changes:

*/  
float* PtrOmegaRaw(void)
{
	return &tOmega.fRaw;
}

//dito Omega FOLP filtered
float* PtrOmegaFolp(void)
{
	return &tOmega.fFolpOut;
}

//dito Omega filtered with MovingAverigeFilter
float* PtrOmegaAvg(void)
{
	return &tOmega.fAvg;
}

// dito Omega from Last sample cycle (Omega-1)
float* PtrOmegaOld(void)
{
	return &tOmega.fOld;
}


// dito Angle accelaration
float* PtrOmegaAccelaration(void)
{
	return &tOmega.fARaw;
}


/*!
\fn		void CsInitSigAccLinReg(t_stcAccelCs *p_stcSig)
\brief  initialize Acceleration calculation in CS-Mode.
        In CS-Mode we calculate always the acceleration of the motor with
        a linear regression over the speed, at the moment with 25 values
        which leads to 25*0,002 sec -> 50 ms!
        This function initializes the objects for this function.
        
\author	K�ster
\date	28.01.2016

\param[in]

\return	void

Changes:

*/
void OmgInitSigAccLinReg(t_stcOmgAccel *p_stcSig)
{
    // calculate PhiAbs Index for data array Mavg-Filter
    p_stcSig->unIdx = 0;
    p_stcSig->unDecFac = (Uint16)((ACCEL_TIME / (float)ACCEL_LENGTH / (*PtrSamplingTime()/ 1e06) ) +0.5);
    p_stcSig->unLen = ACCEL_LENGTH;
    // data buffer not yet FULL
    p_stcSig->unState = FALSE;
    p_stcSig->fSmpTm = ACCEL_TIME / (float)ACCEL_LENGTH;
    // set the LinReg-Filter object
    FiltLinRegInit(&p_stcSig->objSigLinReg, p_stcSig->afDatBuf, p_stcSig->unLen);
}


/*!
\fn		float32 CsSigAccLinReg(t_stcAccelCs *p_stcSig, float32 p_fActSig)
\brief  calculate Acceleration in CS-Mode.
        In CS-Mode we calculate always the acceleration of the motor with
        a linear regression over the speed, at the moment with 25 values
        which leads to 25*0,002 sec -> 50 ms!
        This function caculates and stores the acceleration.
        
\author	K�ster
\date	28.01.2016

\param[in]

\return	void

Changes:

*/
float32 OmgSigAccLinReg(t_stcOmgAccel *p_stcSig, float32 p_fActSig)
{
    //float32 fOut=0.0;
    p_stcSig->unIdx++;
    // next sample?
    if( p_stcSig->unIdx == p_stcSig->unDecFac )
    {
        p_stcSig->unIdx = 0;    
        // calculate gradient of stored data, x-> sampling time, y-> omega
        p_stcSig->fOut = FilterLinReg (&p_stcSig->objSigLinReg, p_stcSig->fSmpTm, p_fActSig);
    }
    // return old value or new value
    return p_stcSig->fOut;
}





/*!
\fn		void OmegaSetCondCalc(float32 p_fFreq, float32 p_fSamplT, float32 p_fMeasTime)
\brief  Set data filters for Torque during in CR-Mode
\author	K�ster
\date	26.10.2012
\param	actual Torque
\return	void
\detail Changes:
*/
void OmegaSetCondCalc(float32 p_fFreq, float32 p_fSamplT, float32 p_fMeasTime)
{
	// Signalgl�ttung Parametrieren und Zust�nde initialisieren
	ParamFOLPF(&sFOLPFOmega, p_fFreq, p_fSamplT * 1e-06);
	SetCondAvgIncMem(&tAvgOmega, (Uint16)(p_fMeasTime*1.0e06/p_fSamplT) );
}

void OmegaResetAccelaration(void)
{
	tOmega.fAvgM1 = 0.0;
	tOmega.fARaw = 0.0;
}


void OmegaSetCondAvg(float32 p_fSamplT, float32 p_fMeasTime)
{
	SetCondAvgIncMem(&tAvgOmega, (Uint16)(p_fMeasTime*1.0e06/p_fSamplT) );
}


/*!
\fn		void TorqueInitFilter(float32 p_fValue)
\brief  Initialize data filters with initial value
\author	K�ster
\date	26.10.2012
\param	actual Torque
\return	void
\detail Changes:
*/
void OmegaInitFilter(float32 p_fValue)
{
	// vorbelegen
	InitCondFOLPF(&sFOLPFOmega, p_fValue);
	InitCondAvgIncMem(&tAvgOmega, p_fValue);
}


/*
   Title:         Absolutwinkel lesen aus Winkelstruktur
   Function:      SetConstOmegaCalc()
   Datum:		   13.02.2007
   Beschreibung:   
                  aus der 
   Parameter:     Abtastzeit
   R�ckgabewert:  ---

   ge�ndert:		03.03.15 Initialisierung FOC Interruptzaehler fuer exakte Phi Abtastung
*/
void SetConstOmegaCalc(float p_fTabTast)
{
   /* Konstante zur Berechnung Omega in rad/s aus Bit aktualisieren */
   tOmega.fKOmegaCalc = cf2Pi / ( (float) *PtrConstAngleMeas() * p_fTabTast / 1e06 );
	// Zaehler fuer FOC Routine zur Winkelabtastung in Echtzeit zur Rohwertberechnung Omega
	*PtrFocIntCnt() = (Uint16) ( (p_fTabTast / EPWM_PERIOD_US)+1 );
}




/*
   Title:         Absolutwinkel lesen aus Winkelstruktur
   Function:      OmegaCalculation()
   Datum:		   13.02.2007
   Beschreibung:  Lesen des zuletzt berechneten 
                  aus der 
   Parameter:     ---
   R�ckgabewert:  Absolutwinkel [rad] ungefiltert

   ge�ndert:
*/

void OmegaCalculation (void)
{
//	float32 dAngle;

	// erste Berechnung in rad/s aus Inkrementen als Rohwert
	tOmega.fRaw = tOmega.fKOmegaCalc * (float32) *PtrAngleOmegaRaw();
	// check Omega for max. speed and Stop if too high

	// Tiefpassfilter 1.Ordn., zur Gl�ttung Drehzahl
	tOmega.fFolpOut = FOLPF(&sFOLPFOmega, tOmega.fRaw);
	// 06.09.16 use device constant instead of fix value, 21.09.16 back again to fixed value
  if( fabs( tOmega.fFolpOut) > 330.0 )
	{
		// update Diagnos-State
		*PtrDsMotorCon() = MC_N_MAX;
		// signal for stopping the motor
		*PtrMeasState() = MM_STOP;

	}
	tOmega.fAvg = AvgIncMem(&tAvgOmega, tOmega.fFolpOut);

}

/*
   Title:         Calculation Accelaration Motor
   Function:      AngleAccelaration()
   Datum:		  03.07.2013
   Beschreibung:   
   Parameter:     ---
   R�ckgabewert:

   ge�ndert:
*/
#ifdef RAUS
void AngleAccelaration (void)
{
	if ( *PtrMeasMode() == CRROT )
	{
		tOmega.fARaw = (*PtrMeasSetvalue() - tOmega.fAvgM1) / TIME_1000MS;
		tOmega.fAvgM1 = *PtrMeasSetvalue();
	}
	else if ( *PtrMeasMode() == CSROT)
	{
		//tOmega.fARaw = ( tOmega.fAvg - tOmega.fAvgM1) / TIME_250MS;
		tOmega.fARaw = ( tOmega.fFolpOut - tOmega.fAvgM1) / TIME_1000MS;
		tOmega.fAvgM1 = tOmega.fFolpOut;
	}
}
#endif



#ifdef RAUS

/*
   Title:         
   Function:      InitCondOmegaRegression()
   Datum:		   09.05.2007
   Beschreibung:   
                   
   Parameter:     ---
   R�ckgabewert:

   ge�ndert:
*/
void InitCondOmegaRegression(t_LinRegressionStruct *ptLrd, int nLength)
{
   /* Regression Phi -> Omega initialisieren */
   ptLrd->fSumX =0.0;
   ptLrd->fSumY =0.0;
   ptLrd->fSumXx =0.0;
   ptLrd->fSumXy =0.0;
   ptLrd->fB =0.0;
   ptLrd->fA =0.0;
   /* Anfangswert dyn. Regression festlegen */
   ptLrd->nActLength = 0;
   /* Hier wird die max. verwendete Speichertiefe festgelegt, aber nie mehr als 250 */
   ptLrd->nMaxLength = nLength;
}



/*
   Title:         
   Function:      ResetCondOmegaRegression()
   Datum:		   09.05.2007
   Beschreibung:  Zur�cksetzen der Regression auf kleine Tiefe, die zuletzt
                  gemessenen werte werden umgespeichert an den Anfang des
                  Speicherbereiches!
   Parameter:     ---
   R�ckgabewert:

   ge�ndert:
*/
void ResetCondOmegaRegression(int p_nLength)
{
	/* Winkel f�r Regression nullen */
	//SetAngleOffsetRegression();
	/* Regression Phi -> Omega initialisieren*/
	//InitCondOmegaRegression(&tOmegaLinReg, p_nLength);
	/* Speicher f�r Winkelringpuffer initialisieren */
	//SetCondXYDataMemory(&tPhiMemory, p_nLength);
	/* auch den Zeitz�hler zur�cksetzen (u.a. g�nstig wg. Rundungsfehlern) */
	tOmega.fTime = 0.0;
	/* Winkelwert mit Zeitstempel speichern */
	//StoreXYData(&tPhiMemory, tOmega.fTime, 0.0);
	SetCondEWMAF(&objOmegaRawEWMAF, 0.0, 10);
	/* 08.09.09 Rohdatenvorfilter Omega auch richtig vorbelegen */
	InitCondEWMAF(&objOmegaRawEWMAF,0.0);
	/* 12.04.10 auch Filter f�r Acceleration */
	SetCondEWMAF(&objAccelarationEWMAF, 0.0, 10);
	/* 08.09.09 Rohdatenvorfilter Omega auch richtig vorbelegen */
	InitCondEWMAF(&objAccelarationEWMAF,0.0);
}
#endif

