/*!
\file	Pmsm.c
\brief	function for FOC, field oriented vector control

\author	Schneider (EDC), Koester
\date	17.01.2008
\details

 changes:	22.07.2013	Spezialtest f�r WCM, n fix auf 5rpm
			06.08.2013	Totzeit bei EPWM Ausgabe von 70 auf 120ns
						FocWriteEpwmValues();
			14.01.2014	Variable names of HPWM to Haake standard
			15.01.2014	PWM disable when ADC data missing in Foc
						(FocCalculateMotorCurrent(void) )
			29.01.2014	modifikation Int.-Handling Timer-0 und EPWM1 (FOC)
			06.02.2014	limit current controller from 5.0 to 5.1 A
			03.03.2014	in FocAngleCalibration() limit-values check with tolerances
			03.03.2015	integration AngleCalculation in FOC interrupt
						for better real-time results
			15.09.2015  all OSC modes, angle calculation every sample, bugfix
			23.10.2015	extended time for VTiQ-Air magnet search
            11.04.2016  test auch Kugellagergeraete mit 3-Phasen Motorstrom
            26.04.2016  select 2 or 3 phases motor current, dependent from PWM frequency
*/

// Include header files used in the main function
#include <math.h>
#include "project.h"		// all global things as Type definitions, Hardware def., ...
#include "mth.h"
//#include "C28x_FPU_FastRTS.h"
#include "Pmsm.h"
#include "Epwm.h"						// ePWM software driver
//#include "DSP2833x_EPwm_defines.h"		// DSP2833x EPwm Defines Headerfile Include File
//#include "SFO_V5.h"     				// Special Functions High Resolution PWM
#include "ADS8329.h"					// Software driver ADC ADS8329
#include "Ip2000.h"						// Software driver for GC-IP2000
#include "Diagnos.h"
#include "dsv.h"
#include "Measmode.h"
#include "mth.h"
#include "dsv.h"
#include "angle.h"
#include "control.h"
#include "device.h"
#include "Uc1dsp.h"
#include "version.h"
#include "SsiComm.h"
#include "Test.h"

/* Compiler Settings *********************************************************/

// uncomment, if using SVGEN-, comment if using Inverse-CLark-Park-Transformation
#define SVGEN_TRANSFORMATION
// uncomment, if using Diagnose functions like filtered motor currents a.o.
//#define DIAGNOSE_FOC

// Prototype statements for functions found within this file.
#ifdef FLASH
#pragma CODE_SECTION(FieldOrientedControl_ISR, "ramfuncs");
#pragma CODE_SECTION(FocIqIdControl, "ramfuncs");
#pragma CODE_SECTION(FocInvParkTrf, "ramfuncs");
#pragma CODE_SECTION(FocSvgen, "ramfuncs");
#pragma CODE_SECTION(FocWriteEpwmValues, "ramfuncs");
#pragma CODE_SECTION(FocCalculateAngles, "ramfuncs");
#pragma CODE_SECTION(FocCalculateMotorCurrent, "ramfuncs");

#endif


/* Local Prototypes *********************************************************/
void FocCalculateAngles(void);
void FocCalculateMotorCurrent(void);
//void FocDq0Trf(void);
void FocClarkParkTrf(void);
void FocInvParkTrf(void);
void FocIqIdControl(void);
void FocReferenceControl(void);
void FocSetAngleCalVectors(void);
void FocSvgen(void);
//void FocInvParkClarke(void);
void FocWriteEpwmValues(float32 fA,float32 fB, float32 fC);

/* Macro definitions ********************************************************/
#define	DEBUGPIN_H()	GpioDataRegs.GPASET.bit.GPIO30 = 1
#define	DEBUGPIN_L()	GpioDataRegs.GPACLEAR.bit.GPIO30 = 1
#define	DEBUGPIN_T()	GpioDataRegs.GPATOGGLE.bit.GPIO30 = 1


// several mathematical constants (float) which are useful for short calculation time
#define PI			3.1415927	// thats PI
#define K_2PI_3	2.0943951	// 2.0*Pi/3.0
#define K_2_3 		0.6666667	// 2.0/3.0
#define K_1_3 		0.3333333	// 1.0/3.0
#define K_1_SQRT3	0.5773503	// 1/SQRT(3)
#define K_SQRT3_2	0.8660254	// SQRT(3)/2.0

// Motor
#define MOTOR_POLEPAIRS	4.0 

// several pre-defined motor-currents for calibration
#define IDREF_0		0.0		// immer NULL, kann ggfs. Id= f(T) ersetzt werden
#define IPARK_DS_0		0.0		// Festwert bei Referenzlauf f�r Feld-Istwert
#define IPARK_QS_050	0.05	// Festwert bei Referenzlauf f�r Strom-Istwert 50mA
#define I_ANGLECAL		0.05	// I-peak for Angle calibration 50mA

// Modes for FOC control
#define FOC_IDLE		0	// at start in Idle mode until command from �C1
#define FOC_CUROFFSET	1	// Calibration of motor-currents is running
#define FOC_REFERENCE	2	// Reference search encoder
#define FOC_ANGLECAL	3	// Angle calibration
#define FOC_NORMAL		4	// Normal motor-control mode

// Angle increment for refrence detection encoder
//#ifdef VTIQ_AIR
	#define PHI_INC_REF		(120e-6)
//#else
//	#define PHI_INC_REF		(80e-6)
//#endif

// 03.03.14, with tolerances, because of unaccurate assembled magnets
#define MECH_15_DEGREE		0.042666666	// 15/360 ( 60�elec * 22,5�mech / 90�elec )
//#define ONE_SECTOR_MECH	0.041666666	// 15/360 ( 60�elec * 22,5�mech / 90�elec )
#define MECH_30_DEGREE		0.083333333	// 30/360 ( 120�elec * 22,5�mech / 90�elec )
#define MECH_45_DEGREE		0.125000000	// 45/360 ( 180�elec * 22,5�mech / 90�elec )

// Reference Search
#define REF_MIN			0
#define REF_MAX			2		// 03.03.15 no more 3, but now 2

// ***** Counter values for start procedures
// 26.04.16, dependent from PWM frequency
#ifdef EPWMFRQ_25KHZ
	// Current Calibration
	#define MOTOR_CURRENT_CNT	12500
	// Angle calibration constants
	#define ANGCAL_CNT1			50000		//2 * ANGCAL_CNT1	// = 1,5sec
	#define ANGCAL_CNT2			100000		//2 * ANGCAL_CNT1	// = 3,0sec
	#define ANGCAL_CNT3			150000		//3 * ANGCAL_CNT1	// = 4,5sec
#else
	// Current Calibration
	#define MOTOR_CURRENT_CNT	50000
	// Angle calibration constants
	#define ANGCAL_CNT1			100000		//2 * ANGCAL_CNT1	// = 2,0sec
	#define ANGCAL_CNT2			200000		//2 * ANGCAL_CNT1	// = 4,0sec
	#define ANGCAL_CNT3			300000		//3 * ANGCAL_CNT1	// = 6,0sec
#endif

#define ANGCAL_VEC_U0		0
#define ANGCAL_VEC_U60		1



/* Types ********************************************************************/

// Data for Foc Modul
typedef struct
{
	Uint16	usState;		// State for FOC
	Uint16	usIntCnt;		// for real-time control we count the FOC Int calls
	Uint16	usIntCntLimit;	// for Omega Calculation in FOC Interrupt
	int16	nRcnt;			// Rounds Counter (0.....)
	float32	fAngle;			// Angle (0...1) from IP2000
	float32 fPhiNmech;		// normalized mechanical Abs.- Angle 0...1
	float32 fPhiNelec;		// normalized electrical Angle
	float32 fPhiElec;		// electric Angle 0...2PI
	float32 fPhiRef;		// Help Angle for Encoder-Reference-Search
	float32 fSetCurrent;	// Set value Torque
	int32	lIp2000Cnt;		// Counter as long from IP2000 for real-time angle meas.
}t_Foc;


// Sinus PWM
typedef struct{
	float32 SinPhi;			// Output Clarke-Transformation Sinus of Angle
	float32 CosPhi;			// Output Clarke-Transformation Cosinus of Angle
	
	float32 ClarkeAlpha;	// Output Clarke-Transformation Alpha
	float32 ClarkeBeta;		// Output Clarke-Transformation Beta
	float32 ParkDS;			// Output Park-Transformation Ds
	float32 ParkQS;			// Output Park-Transformation Qs

	float32 InvParkAlpha;	// Output Inverse-Park-Transformation Alpha
	float32 InvParkBeta;	// Output Inverse-Park-Transformation Beta
	float32 InvParkDS;		// Input Inverse-Park-Transformation DS
	float32 InvParkQS;		// Input Inverse-Park-Transformation Qs

	float32 fTa;			// Output Inverse-Clarke-Park-Transformation Phase-A
	float32 fTb;			// Output Inverse-Clarke-Park-Transformation Phase-B
	float32 fTc;			// Output Inverse-Clarke-Park-Transformation Phase-C
}t_FocData;


// Data for Motor Current
typedef struct
{
	//Uint16	usState;		// State for motor current
	Uint32	ulCnt;			// counter for offset detection
	Uint16	usI1;			// motor-current-1 as 16-Bit integer
	Uint16	usI2;			// motor-current-2 as 16-Bit integer
	Uint16	usI3;			// motor-current-3 as 16-Bit integer
	float32	fI1;			// motor-current-1 as float
	float32	fI2;			// motor-current-2 as float
	float32	fI3;			// motor-current-3 as float
	int	nI1Off;				// DC-Offset motor-current-1
	int nI2Off;				// DC-Offset motor-current-2
	int nI3Off;				// DC-Offset motor-current-3
	//float32	fI1Folp;			// motor-current-1 as float
	//float32	fI2Folp;			// motor-current-2 as float
	//float32	fI3Folp;			// motor-current-3 as float
	//float32 fI0;			// assymmetric Motor-current I0 from DQ0-Transformation
	//long long llC1Sum;		// Help data for Offset-calculation 
	//long long llC2Sum;		// 
	//long long llC3Sum;		// 
}t_MC;


// Data for SVGEN Space-VectorGgeneration
typedef struct
{
	Uint16	nSector;
	float32 fVa;
	float32 fVb;
	float32 fVc;
	float32 fT1;
	float32 fT2;
	float32 fTmpSv1;
	float32 fTmpSv2;

	float32 fUalpha;
	float32 fUbeta;
	float32 fTa; 
	float32 fTb;
	float32 fTc;
}t_FocSvGen;

// Data for Angle Calibration
typedef struct
{
	Uint32	ulCnt;
	Uint16	usVec;
	int16	nDir;
	Uint16	usRndV0;
	Uint16	usRndV60;
	float32	fAngV0;
	float32	fAngV60;
	float32 fAngA;
	float32 fCalAng;
	//float32 fTest;

}t_FocAngCal;

// Data for EPWM output
typedef struct
{
	float32	fPWMPeriod;
	// Scale-Factor for Micro-Edge-Positioner of the High-Resolution-PWM-Modul, see SPRU924F.pdf
	float32	fMEP_SF;
	Uint16	unTotzeit;
}t_FocEPWM;


/* Global variables *********************************************************/

// if this is activ, the data areas are in a part of RAM which works a little bit faster
#ifdef RAUS
//#ifndef FLASH
// put into Fast-RAM arrea when running in RAM
#pragma DATA_SECTION(tFoc, "FASTRAML0")
#pragma DATA_SECTION(tMc, "FASTRAML0")
#pragma DATA_SECTION(tFocData, "FASTRAML0")
#pragma DATA_SECTION(tSvGen, "FASTRAML0")
#pragma DATA_SECTION(tFocAc, "FASTRAML0")
#pragma DATA_SECTION(ePWM, "FASTRAML0")
#endif

/* instance all data for FOC **************************************************/

// data for FOC control
t_Foc	tFoc = {0, 0, 0, 0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0};

// data for motor current
t_MC	tMc;
//t_MC	tMc = {0, 32768, 32768, 32768, 0.0, 0.0, 0.0, 0, 0, 0};

// data for FOC moduls Park, Clarke, Svgen
t_FocData tFocData;
//t_FocData tFocData = {0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0};
// data for Angle calibration
t_FocAngCal tFocAc;
//t_FocAngCal tFocAc = {0, 0, 0, 0, 0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};

// data for Testfunctions
t_MeasSetVal tFocTest;
//t_MeasSetVal tFocTest = {0, 0, 0, 0, 0, 0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0} ;

// Instance a Space Vector PWM modulator. This modulator generates a, b and c
// phases based on the d and q stationery reference frame inputs
//t_FocSvGen tSvGen;
t_FocSvGen tSvGen = {0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.5,0.5,0.5};

// Scale-Factor for Micro-Edge-Positioner of the High-Resolution-PWM-Modul, see SPRU924F.pdf
t_FocEPWM tFocEpwm;

// data for ID and IQ PI-Controls
//tStcPiData tPiId;
//tStcPiData tPiIq;
tStcPiData tPiId = {0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0};
tStcPiData tPiIq = {0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0};



// First-order-low-pass filters for motor currents
//t_FOLPF	sFOLPFI1;			// First-Order-Low-Pass-Filter (FOLP) Omega
//t_FOLPF	sFOLPFI2;			// First-Order-Low-Pass-Filter (FOLP) Omega
//t_FOLPF	sFOLPFI3;			// First-Order-Low-Pass-Filter (FOLP) Omega

// ------------------------------------------------------------------------------
//    High Resolution PWM (see more at see SPRU924F.pdf)
// ------------------------------------------------------------------------------

//SFO_V5.lib including, HRPWM Reference Guide, Appendix B SFO LIB Software
//Step 1
//Step 2
#define PWM_CH 7	// 6 HRPWM Channels, EPWM1A, EPWM2A, EPWM3A, EPWM4A, EPWM5A, EPWM6A + 1 Dummy
//Step 3
//int MEP_ScaleFactor[PWM_CH] = {0,0,0,0,0,0,0};									//dummy pointer in location EPWM[0]
volatile struct EPWM_REGS *ePWM[PWM_CH] = {&EPwm1Regs, &EPwm1Regs, &EPwm2Regs, &EPwm3Regs, &EPwm4Regs, &EPwm5Regs, &EPwm6Regs};
//dummy pointer in location EPWM[0]



// ------------------------------------------------------------------------------
//    End of Defines and Variables
// ------------------------------------------------------------------------------


/* Functions ****************************************************************/


/*!
\fn		float* PtrFocSetCurrent(void)
\brief  Read or Write actual Motor current from different Motor-Control modes
		Write with:	*PtrFocSetCurrent() = float;
		Read with:	= *PtrFocSetCurrent(); 
\author	K�ster
\date	04.03.2013
\param[in]
\return	void
\details Changes:
*/  
float* PtrFocSetCurrent(void)
{
	return &tFoc.fSetCurrent;
}


//dito counter value Interrupt FOC, to calculate speed exactly
Uint16* PtrFocState(void)
{
	return &tFoc.usState;
}

//dito counter value Interrupt FOC, to calculate speed exactly
Uint16* PtrFocIntCnt(void)
{
	tFoc.usIntCnt = 0;
	return &tFoc.usIntCntLimit;
}



//dito counter value
int32* PtrFocIp2000Cnt(void)
{
	return &tFoc.lIp2000Cnt;
}


float* PtrFocTestAngle(void)
{
	return &tFoc.fAngle;
}

#ifdef TEST_DEBUG
float* PtrFocTestI1(void)
{
	return &tMc.fI1;
}

float* PtrFocTestI2(void)
{
	return &tMc.fI2;
}

float* PtrFocTestI3(void)
{
	return &tMc.fI3;
}
#endif


void FocSetTestAngleConstant(void)
{
	// NOP for Breakpoint
	asm(" nop");
	//GetSsiData(&tFocTest);
	//tFocAc.fTest = tFocTest.fSetValue;
	// NOP for Breakpoint
	asm(" nop");
}



/*!
\fn		void FocAngleCalibration(void)
\brief  After Device-Start the Angle-calbration-counter has been set to Zero.
		In this function the magnets were searched an their position relativ
		to the Reference-Signal of the encoder, calibrated and stored.
		The counter is updated every FOC cycle which is 20�s, so that all 2 sec.
		the next vector is selected and the according angle measured.
		1. Vektor-0�
		2. after 2sec measure Angle-V0, set Vector-60�
		3. after 4sec measure Angle-V60, set Vector-0�
		4. after 6sec calculate Angle.Offset and store

\author	K�ster/Schneider-EDC
\date	04.03.2013
\param[in]
\return	
Changes:
*/  
void FocAngleCalibration()
{
	float32 fAngleDiff;
	tFocAc.ulCnt++;
	// store the crossing of the reference-mark, to detect the overflow in angle
	switch (tFocAc.ulCnt)
	{
		case 1:
			// Move rotor in direction of vector U0 
			tFocAc.usVec = ANGCAL_VEC_U0;
		break;			
		case ANGCAL_CNT1:	// after 2.0s
			// store number of Reference-Signal passings for detecting overflow
			tFocAc.usRndV0 = abs(tFoc.nRcnt);				
			// store mechanical Angle at Vector-0, range for 360� is 0...1
			tFocAc.fAngV0 = 1 - tFoc.fAngle;
			// set 2nd vector to 60�
			tFocAc.usVec = ANGCAL_VEC_U60;

		break;

		case ANGCAL_CNT2:	// after 4.0s
			// store number of Reference-Signal passings for detecting overflow
			tFocAc.usRndV60 = abs(tFoc.nRcnt);				
			tFocAc.fAngV60 = 1 - tFoc.fAngle;
			// calculate angle difference between Vector-0 and Vector-60
			fAngleDiff = tFocAc.fAngV0 - tFocAc.fAngV60;
			// no encoder overflow				
			if( tFocAc.usRndV60 == tFocAc.usRndV0 )
			{
				// dependend of the two Vector angles we can detect the direction
				if( fAngleDiff < 0 )
				{
					tFocAc.nDir = -1;
					// Position Vector-0 plus 45� mechanical angle ???
					tFocAc.fAngA = tFocAc.fAngV0 + MECH_45_DEGREE;
				}
				else
				{
					tFocAc.nDir = 1;
					// Position Vector-0 minus 30� mechanical angle ???
					tFocAc.fAngA = tFocAc.fAngV0 - MECH_30_DEGREE;
				}
			}
			// encoder overflow				
			else
			{
				// 03.03.14 now with tolerances
				if( (( tFocAc.fAngV0 >= 0.0 ) && ( tFocAc.fAngV0 <= MECH_15_DEGREE )) &&
					(( tFocAc.fAngV60 >= ( 1.0-MECH_15_DEGREE ) ) && ( tFocAc.fAngV60 <= 1.0 ))	) 
				{
					// (0.0 < Angle-V0 < 0.042666) and (0.957 < Angle-V60 < 1.0)
					tFocAc.nDir = 1;
					tFocAc.fAngA = tFocAc.fAngV0 - MECH_30_DEGREE;
				}
				else if( (( tFocAc.fAngV0 >= ( 1.0-MECH_15_DEGREE ) ) && ( tFocAc.fAngV0 <= 1.0 )) &&
					(( tFocAc.fAngV60 >= 0.0 ) && ( tFocAc.fAngV60 <= MECH_15_DEGREE ))	)
				{
					// (0.957 < Angle-V0 < 1.0) and (0.0 < Angle-V60 < 0.042666)					
					tFocAc.nDir = -1;
					tFocAc.fAngA = tFocAc.fAngV0 + MECH_45_DEGREE;    // 10
				}
				else
				{
					// an error has been detected, store in diagnose modul
					*PtrDsMotorCon() = MC_REF_MAGNET;
				}
			}
					// Move rotor back in dircetion of vector U0
			tFocAc.usVec = ANGCAL_VEC_U0;
		break;
		case ANGCAL_CNT3:	// after 6.0s
			// enough with waiting, store calibrated angle
			tFocAc.fCalAng = tFocAc.fAngA;
			*PtrSysMagPosition() = tFocAc.fCalAng;
		break;

		case (ANGCAL_CNT3+1):
			// 4. Step after START is Normal mode
			tFoc.usState = FOC_NORMAL;
			tFocAc.usVec = ANGCAL_VEC_U0;
			// set CPU diagnosis to Reference found
			*PtrDsCpu() = CPU_MAGRDY;
			StartCpuTimer0();

		break;
		default:
		break;
	}
}


/*!
\fn		void FocCalMotorCurrOffset(void)
\brief  This function contained the measuring cycle for the motorcurrent offset
		at the beginning of the device start. This was cancelled and replaced by
		the load of the stored motor-current offsets!
		The the motor-current offsets (caused by the OPV's) cause a Torque failure
		that must be corrected. This offests will be detected manually in a test
		procedure and then stored in the EEPROM of the master.
		We load them after device-start!

\author	K�ster/Schneider-EDC
\date	04.03.2013
\param[in]
\return	
Changes:
*/  
void FocCalMotorCurrOffset(void)
{
	// increment counter, to wait for the right time :-)
	tMc.ulCnt++;
	// time over, n values are measured, calculate the offset error
	if( tMc.ulCnt == MOTOR_CURRENT_CNT )
	{
		// use the stored values from inital test procedure
		tMc.nI1Off = (int) *PtrIoff1Ampl();
		tMc.nI2Off = (int) *PtrIoff2Ampl();
		tMc.nI3Off = (int) *PtrIoff3Ampl();
		// todo 22.07.13 Sonderversion WCM, keine Kalibrierung Encoder und Magnetsystem
		// ready, switch to Reference-Search, this means never come back to this function
		tFoc.usState = FOC_REFERENCE;
		//tFoc.usState = FOC_NORMAL;
	}
	else
	{
	}
}																		 


/*!
\fn		void FocInit(void)
\brief  Initilization function for the FOC control, Current controllers

\author	K�ster/Schneider-EDC
\date	04.03.2013
\param[in]
\return	
Changes:
*/

void FocInit(void)
{
#ifdef NANO
	tPiId.fKp = 0.01;    // 0.03
	tPiId.fKi = 0.01;   // 0.015
	tPiId.fKc = 0.054474;
	tPiId.fSatErr = 0.0;
	// limit Output Current to 5.1 Amp�re = 3,6062 Aeff. (roughly 0,108Nm)
	tPiId.fOutMax = 4.5;
	tPiId.fOutMin = -4.5;
	// Iq	
	tPiIq.fKp = 0.01;
	tPiIq.fKi = 0.01;
	tPiIq.fKc = 0.054474;
	tPiIq.fSatErr = 0.0;
	// limit Output Current to 5.1 Amp�re = 3,6062 Aeff. (roughly 0,108Nm)
	tPiIq.fOutMax = 4.5;
	tPiIq.fOutMin = -4.5;
#else
	// Initialize the PI-Controller for Id and Iq
	tPiId.fKp = 0.055;
	tPiId.fKi = 0.085;
	tPiId.fKc = 0.054474;
	tPiId.fSatErr = 0.0;
	// limit Output Current to 5.1 Amp�re = 3,6062 Aeff. (roughly 0,108Nm)
	tPiId.fOutMax = 5.1;
	tPiId.fOutMin = -5.1;
	// Iq	
	tPiIq.fKp = 0.055;
	tPiIq.fKi = 0.085;
	tPiIq.fKc = 0.054474;
	tPiIq.fSatErr = 0.0;
	// limit Output Current to 5.1 Amp�re = 3,6062 Aeff. (roughly 0,108Nm)
	tPiIq.fOutMax = 5.1;
	tPiIq.fOutMin = -5.1;
#endif
	// EPWM Time-Base counter Debug Mode "Stop when counter completes a whole cycle"
	// (see also TI SPRUG04A.pdf)
	EPwm1Regs.TBCTL.bit.FREE_SOFT = 0x01;
	
	// values for calculating the EPWM outputs
	// DeadTime, here value 9 representing 120ns in this application
	tFocEpwm.unTotzeit = PWM_TOTZEIT120NS_2;
	// here value 1500, representing 120ns in this application
	tFocEpwm.fPWMPeriod = EPWM_COUNTER_PERIOD;
	// here set the Scale-Factor for Micro-Edge-Positioner MEP to 88 in this application
	tFocEpwm.fMEP_SF = 2*(float32)(CPU_CLOCK) / (float32)(MEP_STEP_SIZE);

	// Reset Angle/Magnet Calibration Reference Counter
	tFocAc.ulCnt = 0;
	tFocAc.usVec = ANGCAL_VEC_U0;

	// data for real-time counter sampling
	tFoc.lIp2000Cnt = 0;
	tFoc.usIntCnt = 0;

	// counter init. for Motor-Current calibration at Device-Start
	tMc.ulCnt = 0;
		
	// Reference search activ
	//tFoc.usState = FOC_CUROFFSET;
	tFoc.usState = FOC_IDLE;

#ifdef DIAGNOSE_FOC
	// Todo Testweise Filter f�r Diagnosemesswerte
	ParamFOLPF(&sFOLPFI1, F10HZ, TIME_20US);
	ParamFOLPF(&sFOLPFI2, F10HZ, TIME_20US);
	ParamFOLPF(&sFOLPFI3, F10HZ, TIME_20US);
	InitCondFOLPF(&sFOLPFI1, 0.0);
	InitCondFOLPF(&sFOLPFI2, 0.0);
	InitCondFOLPF(&sFOLPFI3, 0.0);
	// Test Angle Current calibration
	tFocAc.fTest = 0.93750;
#endif
}


/*!
\fn		void FocSetToRefMode(void)
\brief  

\author	K�ster/Schneider-EDC
\date	04.03.2013
\param[in]
\return	
Changes:
*/
void FocSetToRefMode(void)
{
	// Reference search activ
	tFoc.usState = FOC_CUROFFSET;
	tMc.ulCnt = 0;
	//tMc.llC1Sum = 0;
	//tMc.llC2Sum = 0;
	//tMc.llC3Sum = 0;
	// Reset Refrence-Angle Counter
	tFoc.nRcnt = 0;
	tFoc.fPhiRef = 0.0;
	// Reset Refrence-Magnet Counter
	tFocAc.ulCnt = 0;
	tFocAc.usVec = ANGCAL_VEC_U0;
	// set CPU diagnosis to init
	*PtrDsCpu() = CPU_RUN;
}


/*!
\fn		void FocCalculateMotorCurrent(void)
\brief  

\author	K�ster/Schneider-EDC
\date	04.03.2013
\param[in]
\return	
Changes:
*/
void FocCalculateMotorCurrent(void)
{
	float fHelp;
	// has ADS9329 modul receive data for motor currents?
	if( *PtrADS8329DataRec() == 1)
	{
		// yes data received, reset the Flag
		*PtrADS8329DataRec() = 0;
		// 0 -> 5,2083 A, 32768 -> 0,0000 A, 65535 -> -5,2082 A
		tMc.usI1 = 65536 - motor_current.separate.current1_measure1_up;
		tMc.usI2 = 65536 - motor_current.separate.current2_measure1_up;
		tMc.usI3 = 65536 - motor_current.separate.current3_measure1_up;

		// Direction = CCW, change currents of Phase-1 and Phase-2
		if(tFocAc.nDir == -1)
		{
			fHelp = tMc.usI1;
			tMc.usI1 = tMc.usI2;
			tMc.usI2 = fHelp;
		}
	}
	else
	{
		// no interrupt ADS8329 with new data, must be an error, message to diagnose modul
		// 15.01.14 do not Stop PWM at the moment	(-> stop PWM)
		//PWMDisable();
		// set diagnose state ADC_NODATA
		*PtrDsMotorCon() = MC_ADC_NODATA;
		//*PtrDsPwm() = *PtrDsPwm() +1;
		// signal for stopping the motor
		//*PtrMeasState() = MM_STOP;
	}
	// Fetch calibration constants from Device-Data and calculate the motor current
	// 0 -> -5,2083 A, 32768 -> 0,0000 A, 65535 -> +5,2082 A
	tMc.fI1 = ((float)( tMc.usI1 - ( *PtrIoff1Ampl() ) - 32768 )) * ( *PtrIcal1Ampl() );
	tMc.fI2 = ((float)( tMc.usI2 - ( *PtrIoff2Ampl() ) - 32768 )) * ( *PtrIcal2Ampl() );
	tMc.fI3 = ((float)( tMc.usI3 - ( *PtrIoff3Ampl() ) - 32768 )) * ( *PtrIcal3Ampl() );

#ifdef DIAGNOSE_FOC
	tMc.fI1Folp = FOLPF(&sFOLPFI1, tMc.fI1);
	tMc.fI2Folp = FOLPF(&sFOLPFI2, tMc.fI2);
	tMc.fI3Folp = FOLPF(&sFOLPFI3, tMc.fI3);
#endif

}


/*!
\fn		void FocCalculateAngles(void)
\brief  

\author	K�ster/Schneider-EDC
\date	04.03.2013
\param[in]
\return	
Changes:
*/
void FocCalculateAngles(void)
{
	float fHelp;
	// calculate normalized mechanical Absolute-Angle Phi-abs, value range: 0...1 (0..360�)
	if(tFoc.usState < FOC_NORMAL)
	{
		// ?
		// tFoc.fPhiNmech = 1 - tFoc.fAngle - tFocAc.fCalAng;
     // Test
     tFoc.fPhiNmech = 1 - tFoc.fAngle - tFocAc.fCalAng - *PtrPolAngle();
	}
	else
	{
		// todo
		//tFoc.fPhiNmech = 1 - tFoc.fAngle - tFocAc.fCalAng;
     // Test
     tFoc.fPhiNmech = 1 - tFoc.fAngle - tFocAc.fCalAng - *PtrPolAngle();
		// 22.07.13. Spezialtest f�r WCM, n fix auf 5rpm
		//tFoc.fPhiNmech = tFoc.fPhiNmech  + 1666.6666666e-09;	// 5 rpm
		//tFoc.fPhiNmech = tFoc.fPhiNmech  + 0.00002;			// 60 rpm
		
		//tFoc.fPhiNmech = tFocAc.fTest; // zur Stromkalibrierung
		
		//tFoc.fPhiNmech = 1.00000;		//360,00� -> electr. 360�
		//tFoc.fPhiNmech = 0.9791667;	//352,50� -> electr. 330�
		//tFoc.fPhiNmech = 0.9583333;	//345,00� -> electr. 300�
		//tFoc.fPhiNmech = 0.93750;		//337,50� -> electr. 270� -> I1 = max.
		//tFoc.fPhiNmech = 0.9166667;	//330,00� -> electr. 240�
		//tFoc.fPhiNmech = 0.90625;		//326,25� -> electr. 210�
		//tFoc.fPhiNmech = 0.87500;		//315,00� -> electr. 180�
		//tFoc.fPhiNmech = 0.8541667;	//307,50� -> electr. 150� -> I3 = max.
		//tFoc.fPhiNmech = 0.8333333;	//300,00� -> electr. 120�
		//tFoc.fPhiNmech = 0.81250;		//292,50� -> electr. 90�
		//tFoc.fPhiNmech = 0.7708333;	//277,50� -> electr. 30� -> I2 = max.
		//tFoc.fPhiNmech = 0.81250;		//270,00� -> electr. 0�

		//tFoc.fPhiNmech =  + 333.333e-09;
		//tFoc.fPhiNmech = tFoc.fPhiNmech + 0.000235;
	}


	if ( tFoc.fPhiNmech < 0.0 )
	{
		tFoc.fPhiNmech = tFoc.fPhiNmech + 1.0;
	}	
	if ( tFoc.fPhiNmech >= 1.0 )
	{
		tFoc.fPhiNmech = tFoc.fPhiNmech - 1.0;
	}
	// calculate electric angle from mechanical
	fHelp = MOTOR_POLEPAIRS * tFoc.fPhiNmech;
	// Phi-elec = (Phi-mech * PolePairs) - floor(Phi-mech * PolePairs)
	tFoc.fPhiNelec = fHelp - floor(fHelp);
	tFoc.fPhiElec = tFoc.fPhiNelec * cf2Pi;
}


/*!
\fn		void FocReferenceControl(void)
\brief  

\author	K�ster/Schneider-EDC
\date	04.03.2013
\param[in]
\return	
Changes:
*/
void FocReferenceControl(void)
{
		// calculate an artifical angle to make a rotating magnetic field
		// values between 0...1 representing 0...2PI
		tFoc.fPhiRef = tFoc.fPhiRef + PHI_INC_REF;
		if ( tFoc.fPhiRef <= 0.0 )
		{
			tFoc.fPhiRef = tFoc.fPhiRef + 1.0;
		}		
		if ( tFoc.fPhiRef >= 1.0 )
		{
			tFoc.fPhiRef = tFoc.fPhiRef - 1.0;
		}
		
		#ifdef RAUS
		// bei Referenzlauf Feld wie immer Null
		tFocData.InvParkDS = IPARK_DS_0;
		// bei Referenzlauf Torque-Istwert fest
		tFocData.InvParkQS = IPARK_QS_050;				
				
		tFocData.SinPhi = sin((double)(tFoc.fPhiRef * cf2Pi));		// 6.28 = 2 * phi
		tFocData.CosPhi = cos((double)(tFoc.fPhiRef * cf2Pi));
		#endif
		// we calculate direct the SVGEN inputs
		tSvGen.fUalpha = I_ANGLECAL*sin((double)(tFoc.fPhiRef * cf2Pi));	// 2pi/ 360
		tSvGen.fUbeta = I_ANGLECAL*-cos((double)(tFoc.fPhiRef * cf2Pi));

}


/*!
\fn		void FocSetAngleCalVectors(void)
\brief  For the calibration of the magnetic field relative to the Reference of
		the angle measuring system we control two vector positions of the rotor.
		This brings the rotor to two fixed positions, which are measured an the
		different angle can be calculated relativ to the reference mark of
		the angle system.
		The motor is moved 60� electrical angle, means 15� mechanical.
		( 360� electrical angle == 90� mechanical )

\author	K�ster/Schneider-EDC
\date	04.03.2013
\param[in]
\return	
Changes:
*/
void FocSetAngleCalVectors(void)
{
		//Sector 1, Spannungsvektor U_0
		if( tFocAc.usVec == ANGCAL_VEC_U0 )
		{
			tSvGen.fUalpha = I_ANGLECAL*sin(210*0.017453);	// 2pi/ 360
			tSvGen.fUbeta = I_ANGLECAL*-cos(210*0.017453);
		}

		//Zeit warten bis sich Winkel eingestellt hat
		
		//Sector 3, Spannungsvektor U_60
		if( tFocAc.usVec == ANGCAL_VEC_U60 )
		{
			tSvGen.fUalpha = I_ANGLECAL*sin(150*0.017453);	// 2pi/ 360
			tSvGen.fUbeta = I_ANGLECAL*-cos(150*0.017453);
		}
}



/*!
\fn		void FocClarkParkTrf(void)
\brief  Clark-Park-Transformation.
		With the actual angle and motorcurrents the CLARK- and PARK-Transformation is calculated.
		Only the current-values of two phases are involved.
\author	K�ster
\date	01.02.2013
\param	uses only local data-structures
\return	
\details Changes:

*/  
void FocClarkParkTrf(void)
{
	// ********** Trogonometric Functions Sine & Cosine **********
	//calculate sin/cos of electric angle
	tFocData.SinPhi = sin(tFoc.fPhiElec);
	tFocData.CosPhi = cos(tFoc.fPhiElec);

	// ********** CLARKE-Transformation **********

// 26.04.16, select 2 or 3 phases, dependent from PWM frequency
#ifdef EPWMFRQ_25KHZ
	// 03.08.15 Test EDC Berechnung aus 3 Str�men
	tFocData.ClarkeAlpha = 0.3333333 * ((2 * tMc.fI1) - tMc.fI2 - tMc.fI3);
	tFocData.ClarkeBeta = K_1_SQRT3 * (tMc.fI2 - tMc.fI3);
#else
	// 27.06.16 Test auch f�r 50 kHz
	//tFocData.ClarkeAlpha = 0.3333333 * ((2 * tMc.fI1) - tMc.fI2 - tMc.fI3);
	//tFocData.ClarkeBeta = K_1_SQRT3 * (tMc.fI2 - tMc.fI3);

    //	Clarke-Alpha = I1
  tFocData.ClarkeAlpha = tMc.fI1;
	 // Clarke-Beta = 1/SQRT(3) * I1 + 2/SQRT(3) * I2
  tFocData.ClarkeBeta = K_1_SQRT3 * (tMc.fI1 + 2.0 * tMc.fI2);
#endif

	// ********** PARK-Transformation **********
	// Park-Transformation, Ds = a*cos + �*sin
	tFocData.ParkDS = (tFocData.ClarkeAlpha * tFocData.CosPhi) + (tFocData.ClarkeBeta * tFocData.SinPhi);
	// Park-Transformation, Qs = -a*sin + �*cos
	tFocData.ParkQS = (tFocData.ClarkeBeta * tFocData.CosPhi) - (tFocData.ClarkeAlpha * tFocData.SinPhi);
}


/*!
\fn		void FocInvParkTrf(void)
\brief  Inverse-Park-Transformation.
		From Ds and Qs Inputs the actual Alpha and Beta Inverse-Park-Transformation is calculated.

\author	K�ster
\date	01.02.2013
\param	uses only local data-structures
\return	
\details Changes:

*/ 
void FocInvParkTrf(void)
{
  	// Inverse-Park-Transformation, Alpha = Ds*cos - Qs*sin
  	tSvGen.fUalpha = tFocData.InvParkDS * tFocData.CosPhi - tFocData.InvParkQS * tFocData.SinPhi;
	// Inverse-Park-Transformation, Beta = Qs*cos + Ds*sin
 	tSvGen.fUbeta = tFocData.InvParkQS * tFocData.CosPhi + tFocData.InvParkDS * tFocData.SinPhi;
}


/*!
\fn		void FocIqIdControl(void)
\brief  

\author	K�ster/Schneider-EDC
\date	04.03.2013
\param[in]
\return	
Changes:
*/
void FocIqIdControl(void)
{
		// Setvalue Id
		tPiId.fSet = (IDREF_0);
     // Actual value Id
		tPiId.fAct = tFocData.ParkDS;
		// call Id controller
		tFocData.InvParkDS = ControlPi(&tPiId);
		// Setvalue Iq
		tPiIq.fSet = tFoc.fSetCurrent;
     // Actual value Iq
		tPiIq.fAct = tFocData.ParkQS;
     // call Iq controller
		tFocData.InvParkQS = ControlPi(&tPiIq);

		#ifdef RAUS
		// todo 27.03.13 Sprungantwort aufzeichnen zur Sichtbarmachung in Rheowin
		if( tFoc.fSetCurrent >= 0.01 )
		{
			// nur wenn noch Platz
			if( TestDaten(0.0, PROOF) == TRUE )
 			{
				// Testdaten speichern
				TestDaten (tFocData.ParkQS, SET);
			}
		}
		#endif

}



/*!
\fn		void FocSvgen(void)
\brief  Space-Vector-Generation (Raumzeigermodulation)
		(http://de.wikipedia.org/wiki/Raumzeigermodulation)
		
\author	K�ster
\date	01.02.2013
\param	uses only local data-structures
\return	
\details Changes:

*/ 
void FocSvgen(void)
{
		// initialize Sector
		tSvGen.nSector = 0;																	
		// this reduces calculation time		
		tSvGen.fTmpSv1 = 0.5 * (tSvGen.fUbeta);
		tSvGen.fTmpSv2 = K_SQRT3_2 * tSvGen.fUalpha;
		
		// ********** Inverse CLARKE-Transformation **********
		// modified version transforms U-Beta instead of U-Alpha -> 90� phase-shift
		tSvGen.fVa = tSvGen.fUbeta;																
		tSvGen.fVb = -tSvGen.fTmpSv1 + tSvGen.fTmpSv2;													
		tSvGen.fVc = -tSvGen.fTmpSv1 - tSvGen.fTmpSv2;

		/* 60 degree Sector determination */											
		if (tSvGen.fVa>0.0) tSvGen.nSector = 1;													
		if (tSvGen.fVb>0.0) tSvGen.nSector = tSvGen.nSector+2;											
		if (tSvGen.fVc>0.0) tSvGen.nSector = tSvGen.nSector+4;											
		
		/* X,Y,Z (Va,Vb,Vc) calculations X = Va, Y = Vb, Z = Vc */						
		tSvGen.fVa = tSvGen.fUbeta;																
		tSvGen.fVb = tSvGen.fTmpSv1 + tSvGen.fTmpSv2;													
		tSvGen.fVc = tSvGen.fTmpSv1 - tSvGen.fTmpSv2;	
		
		/* Sector 0: this is special case for (Ualpha,Ubeta) = (0,0)*/					
		switch(tSvGen.nSector)																	
		{																				
			case 0:																		
				tSvGen.fTa = 0.5;															
				tSvGen.fTb = 0.5;															
				tSvGen.fTc = 0.5;															
				break;																		
			case 1:   /*Sector 1: t1=Z and t2=Y (abc ---> Tb,Ta,Tc)*/					
				tSvGen.fT1 = tSvGen.fVc;																	
				tSvGen.fT2 = tSvGen.fVb;																	
				tSvGen.fTb = 0.5 * ((1.0-tSvGen.fT1-tSvGen.fT2)); 												
				tSvGen.fTa = tSvGen.fTb+tSvGen.fT1;			 	/* taon = tbon+t1		*/						
				tSvGen.fTc = tSvGen.fTa+tSvGen.fT2;			  	/* tcon = taon+t2		*/						
				break;																		
			case 2:	  /* Sector 2: t1=Y and t2=-X (abc ---> Ta,Tc,Tb)*/ 				
				tSvGen.fT1 = tSvGen.fVb;																	
				tSvGen.fT2 = -tSvGen.fVa;																	
				tSvGen.fTa= 0.5 * ((1.0-tSvGen.fT1-tSvGen.fT2)); 												
				tSvGen.fTc = tSvGen.fTa+tSvGen.fT1;				/*  tcon = taon+t1		*/						
				tSvGen.fTb = tSvGen.fTc+tSvGen.fT2;				/*  tbon = tcon+t2		*/						
				break;																		
			case 3:	  /* Sector 3: t1=-Z and t2=X (abc ---> Ta,Tb,Tc)*/					
				tSvGen.fT1 = -tSvGen.fVc;																	
				tSvGen.fT2 = tSvGen.fVa;																	
				tSvGen.fTa= 0.5 * ((1.0-tSvGen.fT1-tSvGen.fT2)); 												
				tSvGen.fTb = tSvGen.fTa+tSvGen.fT1;				/*	tbon = taon+t1		*/						
				tSvGen.fTc = tSvGen.fTb+tSvGen.fT2;				/*	tcon = tbon+t2		*/						
				break;																		
			case 4:	  /* Sector 4: t1=-X and t2=Z (abc ---> Tc,Tb,Ta)*/					
				tSvGen.fT1 = -tSvGen.fVa;																	
				tSvGen.fT2 = tSvGen.fVc;																	
				tSvGen.fTc= 0.5 * ((1.0-tSvGen.fT1-tSvGen.fT2)); 												
				tSvGen.fTb = tSvGen.fTc+tSvGen.fT1;				/*	tbon = tcon+t1		*/						
				tSvGen.fTa = tSvGen.fTb+tSvGen.fT2;				/*	taon = tbon+t2		*/						
				break;																		
			case 5:	  /* Sector 5: t1=X and t2=-Y (abc ---> Tb,Tc,Ta)*/					
				tSvGen.fT1 = tSvGen.fVa;																	
				tSvGen.fT2 = -tSvGen.fVb;					/*	tbon = (1-t1-t2)/2	*/						
				tSvGen.fTb= 0.5 * ((1.0-tSvGen.fT1-tSvGen.fT2)); 												
				tSvGen.fTc = tSvGen.fTb+tSvGen.fT1;				/*	taon = tcon+t2		*/						
				tSvGen.fTa = tSvGen.fTc+tSvGen.fT2;																
				break;																		
			case 6:	  /* Sector 6: t1=-Y and t2=-Z (abc ---> Tc,Ta,Tb)*/				
				tSvGen.fT1 = -tSvGen.fVb;																	
				tSvGen.fT2 = -tSvGen.fVc;																	
				tSvGen.fTc= 0.5 * ((1.0-tSvGen.fT1-tSvGen.fT2)); 												
				tSvGen.fTa = tSvGen.fTc+tSvGen.fT1;				/*	taon = tcon+t1		*/						
				tSvGen.fTb = tSvGen.fTa+tSvGen.fT2;				/*	tbon = taon+t2		*/						
				break;																		
		}
		if(tSvGen.fTa >= 1.0){
			tSvGen.fTa = 1.0;
		}
		if(tSvGen.fTb >= 1.0){
			tSvGen.fTb = 1.0;
		}
		if(tSvGen.fTc >= 1.0){
			tSvGen.fTc = 1.0;
		}
}


#ifndef SVGEN_TRANSFORMATION
void FocInvParkClarke(void)
{
	// ********** Trogonometric Functions Sine & Cosine **********	
	// calculate sin & cos from actual angle
	tFocData.SinPhi = sin((double)((tFoc.fPhiNelec - 0.5) * 6.283185307));
	tFocData.CosPhi = cos((double)((tFoc.fPhiNelec - 0.5) * 6.283185307));

	// ********** Inverse PARK-Transformation **********
	// Inverse-Park-Transformation, Alpha = Ds*cos - Qs*sin
	tFocData.InvParkAlpha = tFocData.InvParkDS * tFocData.CosPhi - tFocData.InvParkQS * tFocData.SinPhi;
	// Inverse-Park-Transformation, Beta = Qs*cos + Ds*sin
	tFocData.InvParkBeta = tFocData.InvParkQS * tFocData.CosPhi + tFocData.InvParkDS * tFocData.SinPhi;

	// ********** Inverse CLARKE-Transformation **********
	// Va = U-alpha
	tFocData.fTa = tFocData.InvParkAlpha;
	// Vb = Sqrt(3)/2 * Ubeta - 1/2 *Ualpha
	tFocData.fTb = K_SQRT3_2 * tFocData.InvParkBeta - tFocData.InvParkAlpha * 0.5;
	// Vb = -Sqrt(3)/2 * Ubeta - 1/2 *Ualpha
	tFocData.fTc = -K_SQRT3_2 * tFocData.InvParkBeta - tFocData.InvParkAlpha * 0.5;

	// calculate Input for SVGEN module
	tSvGen.fTa = (tFocData.fTa * K_1_SQRT3 + 0.5);
	tSvGen.fTb = (tFocData.fTb * K_1_SQRT3 + 0.5);
	tSvGen.fTc = (tFocData.fTc * K_1_SQRT3 + 0.5);

	// check maximum values
	if(tSvGen.fTa >= 1.0){
		tSvGen.fTa = 1.0;
	}
	if(tSvGen.fTb >= 1.0){
		tSvGen.fTb = 1.0;
	}
	if(tSvGen.fTc >= 1.0){
		tSvGen.fTc = 1.0;
	}


}
#endif



void FocWriteEpwmValues(float32 fA,float32 fB, float32 fC)
{
	Uint16 comp_value1, comp_value4, comp_value2, comp_value5, comp_value3, comp_value6;
	Uint16 intpart1, intpart2, intpart3, intpart4, intpart5, intpart6;
	
	// actual: PWM_TOTZEIT_2 = 9, see epwm.h
  //         fPWMPeriod = 1500
	intpart1 = (Uint16)(fA*tFocEpwm.fPWMPeriod+tFocEpwm.unTotzeit);
	comp_value1 = (Uint16)( ((fA*tFocEpwm.fPWMPeriod+tFocEpwm.unTotzeit)- intpart1)* (tFocEpwm.fMEP_SF) );
	intpart2 = (Uint16)(fA*tFocEpwm.fPWMPeriod-tFocEpwm.unTotzeit);
	comp_value2 = (Uint16)( ((fA*tFocEpwm.fPWMPeriod-tFocEpwm.unTotzeit)- intpart2)* (tFocEpwm.fMEP_SF) );
	intpart3 = (Uint16)(fB*tFocEpwm.fPWMPeriod+tFocEpwm.unTotzeit);
	comp_value3 = (Uint16)( ((fB*tFocEpwm.fPWMPeriod+tFocEpwm.unTotzeit)- intpart3)* (tFocEpwm.fMEP_SF) );
	intpart4 = (Uint16)(fB*tFocEpwm.fPWMPeriod-tFocEpwm.unTotzeit);
	comp_value4 = (Uint16)( ((fB*tFocEpwm.fPWMPeriod-tFocEpwm.unTotzeit)- intpart4)* (tFocEpwm.fMEP_SF) );
	intpart5 = (Uint16)(fC*tFocEpwm.fPWMPeriod+tFocEpwm.unTotzeit);
	comp_value5 = (Uint16)( ((fC*tFocEpwm.fPWMPeriod+tFocEpwm.unTotzeit)- intpart5)* (tFocEpwm.fMEP_SF) );
	intpart6 = (Uint16)(fC*tFocEpwm.fPWMPeriod-tFocEpwm.unTotzeit);
	comp_value6 = (Uint16)( ((fC*tFocEpwm.fPWMPeriod-tFocEpwm.unTotzeit)- intpart6)* (tFocEpwm.fMEP_SF) );


	EPwm1Regs.CMPA.half.CMPA = intpart1 - 1;
	EPwm2Regs.CMPA.half.CMPA = intpart2 - 1;
	EPwm3Regs.CMPA.half.CMPA = intpart3 - 1;
	EPwm4Regs.CMPA.half.CMPA = intpart4 - 1;
	EPwm5Regs.CMPA.half.CMPA = intpart5 - 1;
	EPwm6Regs.CMPA.half.CMPA = intpart6 - 1;
	//
	EPwm1Regs.CMPA.half.CMPAHR = ( ( ( (comp_value1) ) << 8 ) + 0x180 );
	EPwm2Regs.CMPA.half.CMPAHR = ( ( ( (comp_value2) ) << 8 ) + 0x180 );
	//EPWM3A
	EPwm3Regs.CMPA.half.CMPAHR = ( ( ( (comp_value3) ) << 8 ) + 0x180 );
	//EPWM4A
	EPwm4Regs.CMPA.half.CMPAHR = ( ( ( (comp_value4) ) << 8 ) + 0x180 );
	//EPWM5A
	EPwm5Regs.CMPA.half.CMPAHR = ( ( ( (comp_value5) ) << 8 ) + 0x180 );	//Clock: 1713
	//EPWM6A
	EPwm6Regs.CMPA.half.CMPAHR = ( ( ( (comp_value6) ) << 8 ) + 0x180 );
	
}


/*!
\fn		interrupt void FieldOrientedControl_ISR(void)
\brief  Field Orientated Control for EC Motor
		Wiki/TI:
		The Field Orientated Control (FOC)consists of controlling the stator currents
		represented by a vector. This control is based on projections which
		transform a three-phase time and speed dependent system into a
		two coordinate (d and q co-ordinates) time invariant system.
		These projections lead to a structure similar to that of a DC machine control.
		Field orientated controlled machines need two constants as input
		references: the torque component (aligned with the q co-ordinate) and the flux
		component (aligned with d co-ordinate).
		As Field Orientated Control is simply based on projections the control structure
		handles instantaneous electrical quantities. This makes the control accurate
		in every working operation (steady state and transient) and independent
		of the limited bandwidth mathematical model.		
		
\author	K�ster
\date	01.02.2013
\param	uses only local data-structures
\return	
\details
	changes:	29.01.14 modifikation Int.-Handling Timer-0 und EPWM1 (FOC)

*/
interrupt void FieldOrientedControl_ISR(void)
{
	float fHelp;
	//	Global Disable all Interrupts
	//	DINT (INTM = 1 <-> asm(" setc INTM") ) is not neccessary, this is done by the CPU;

	// To receive more interrupts from this PIE group, acknowledge this interrupt
	PieCtrlRegs.PIEACK.all = PIEACK_GROUP3;
	// disable M_INT1, M_INT14, Timer-0, Timer-2 should have lower priority
	IER = IER & 0xFFFE;			

	// 29.01.14 comment out, wait with enabling Int. until end of function
	//EINT;
	//ERTM;   // Enable Global realtime interrupt DBGM
	// time measuring with Debug-Pin
	DEBUGPIN_H();

	// take Ip2000 counter value always at the same sampling point (all 20 or 40 �s)
	tFoc.lIp2000Cnt = *PtrIp2000Cnt();
	// Int. counter incremental, this is for calculating the speed from angle values
	tFoc.usIntCnt++;
	// 15.09.15, all OSC modes, angle calculation every sample
	if( (*PtrMeasMode() == CSOSC) || (*PtrMeasMode() == CDOSC) )
	{
		// time over (e.g. 200 or 2000 �s) then calculate the angle and "raw" speed
		AngleCalculation(tFoc.lIp2000Cnt);
		tFoc.usIntCnt = 1;
	}
	// counter limit is controlled from outside in omega.c, this fixes the meas. time
	else if( tFoc.usIntCnt == tFoc.usIntCntLimit)
	{
		// time over (e.g. 200 or 2000 �s) then calculate the angle and "raw" speed
		AngleCalculation(tFoc.lIp2000Cnt);
		tFoc.usIntCnt = 1;
	}
	else
	{
		// NOP for Breakpoint
		asm(" nop");
	}

	// Get Angle data (0....1) from IP2000
	IP2000GetTrigData(&tFoc.fAngle);

	// count rounds for Reference-Run
	//tFoc.ulRcnt += Ip2000CntRounds();
	//tFoc.ulRcnt = *PtrAngleRounds();

	// calculate the electric and mechanical angle use all tFoc data
	FocCalculateAngles();

	// read and calculate motor currents 1-3
	FocCalculateMotorCurrent();
	
	// different operating modes for Initial. and Normal mode	
	switch (tFoc.usState)
	{
		// 1. Step after START is Offset-Detection of current channels
		case FOC_CUROFFSET:
			// calculate Current Offsets of all 3 phases as average of 250000 values
			FocCalMotorCurrOffset();
			// set SVGEN outputs always to zero (50% duty-cycle) during current calibration
			tSvGen.fTa = 0.50;
			tSvGen.fTb = 0.50;
			tSvGen.fTc = 0.50;
		break;

		// 2. Step after START is Reference-Detection of encoder
		case FOC_REFERENCE:
			tFoc.nRcnt = (int16)*PtrAngleRounds();
			// Reference found?
			if( abs(tFoc.nRcnt) == REF_MAX )
			{
				// set CPU diagnosis to Reference found
				*PtrDsCpu() = CPU_REFRDY;
				// yes, switch to next step
				tFoc.usState = FOC_ANGLECAL;
			}
			// bei Referenzsuche schnell noch Parameter anpassen...
			FocReferenceControl();
			// Space-Vector Generation
			FocSvgen();
		break;

		// 3. Step after START is Angle-Calibration of magnetic system
		case FOC_ANGLECAL:
			// count rounds for reference search and angle calibration
			tFoc.nRcnt = (int16)*PtrAngleRounds();
			// search the magnetic system and calculate the Angle-Offset to the Angle-Reference
			FocAngleCalibration();
			// Calculate the vectors for 0� and 60� and the SVGEN inputs Ualpha and Ubeta
			FocSetAngleCalVectors();
			// Space-Vector Generation SVGEN
			FocSvgen();
			// wrong direction?
			if(tFocAc.nDir == -1)
			{
				fHelp = tSvGen.fTa;
				tSvGen.fTa = tSvGen.fTb;
				tSvGen.fTb = fHelp;
			}
		break;
		
		// FOC normal mode, with DQ0-Transformation and SVGEN-Transformation 
		case FOC_NORMAL:
			// *** wahlweise DQ0- oder Clark-Park-Transformation
			// Alternativ to DQ0-Tr., 3-phase currents are transformed in a 2-axis system
			FocClarkParkTrf();
			// PID Control of Iq and Id
			FocIqIdControl();
		
		// *** wahlweise SVGEN- oder Inverse-Clark-Park-Transformation
		#ifdef SVGEN_TRANSFORMATION
			// Inverse Park-Transformation
			FocInvParkTrf();
			// Space-Vector Generation
			FocSvgen();
		#else
			// Inverse park- and clark Transformation
			FocInvParkClarke();
		#endif			

		break;
		default:
		break;
	}
	// calculate EPWM-Outputs
	FocWriteEpwmValues(tSvGen.fTa, tSvGen.fTb, tSvGen.fTc);
	
	// Enable more interrupts from this timer
	EPwm1Regs.ETCLR.bit.INT = 1;
	DEBUGPIN_L();
	DINT;
	IER |= M_INT1;
	//IER |= M_INT14;
	EINT;
	ERTM;   // Enable Global realtime interrupt DBGM

// Acknowledge interrupt to recieve more interrupts from PIE group 3
//	PieCtrlRegs.PIEACK.all = PIEACK_GROUP3;

}





//===========================================================================
// No more.
//===========================================================================

//*****************************************************************************
/*! @}
 *	Close the Doxygen group.
 */
//*****************************************************************************
