/*!
\addtogroup SsiMcbsp
\file	SsiComm.c
\brief	Synchronus serial Interface fuctions.
		This function use the MCBSP-A in SPI-Slave Mode to transfer data
		FROM and TO the Master-Controller. The DMA-Controller is hooked direct
		to the Receive- and Transmit-Buffer of this modul.
		Functions are publiced in Ssicomm.h!
\author	Koester
\date	22.02.2012

\details
 changes:

*/
#define SSICOMM	/* zur Compilersteuerung */

#include "Project.h"		// all global things as Type definitions, Hardware def., ...
#include "Uc1dsp.h"
#include "Measmode.h"
#include "timer.h"
#include "math.h"
#include "SsiComm.h"
#include "McbspDma.h"

#ifdef FLASH
#pragma CODE_SECTION( GetSsiData, "ramfuncs");
#pragma CODE_SECTION( StartSsiSendData, "ramfuncs");
#endif


/* Macro definitions ********************************************************/


#define	NOSSCERROR		0x0000
#define	ERRORCOMM		0x0001
#define	ERRORHANDSHAKE	0x0002
#define	ERRORTRANSMIT	0x0004
#define	ERRORRECEIVE	0x0010


/* Types ********************************************************************/

/* Local Prototypes *********************************************************/

/* Functions ****************************************************************/


typedef struct
{
	Uint16	usRecKom;					// Semaphore Int. Data received
	Uint16	usTraKom;					// Semaphore Int. Data sent
	Uint16	unError;					// Error state
	Uint16	usRecBuffer[SSIBUFSIZE];	// Receive-Buffer
	Uint16	usTraBuffer[SSIBUFSIZE];	// Send-Buffer
}t_stcSsiStatus;

// Place sdata and rdata buffers in DMA-accessible RAM (L4 for this example)
#ifdef FLASH
	#pragma DATA_SECTION(tSsi, "DMARAML4")
#else
	#pragma DATA_SECTION(tSsi, "DMARAML4")
#endif

t_stcSsiStatus tSsi;



/* Functions ****************************************************************/

/*!
\brief  Pointer to SSI-Receive-Buffer
		with this function the Adress of the SSI Receive-Buffer can be read.
\fn		Uint16* PtrSsiRecBuffer(void) 

\author Kster, Ralf
\date	22.02.2012
\param[IN] void
\return	&tSsi.usRecBuffer[0]

Changes:	

*/
Uint16* PtrSsiRecBuffer(void)
{
	// return the Adress of the SSI Buffer
	return &tSsi.usRecBuffer[0];
}



/*!
\brief  Pointer to SSI-Transmit-Buffer

\fn		Uint16* PtrSsiTraBuffer(void) 

\author Kster, Ralf
\date	22.02.2012
\param[IN] void
\return	&tSsi.usTraBuffer[0]

Changes:	

*/ 
Uint16* PtrSsiTraBuffer(void)
{
	// return the Adress of the SSI Buffer
	return &tSsi.usTraBuffer[0];
}



/*!
\brief  Pointer to SSI-Receive-State
		with this function the State of an SSI Receive can be read.
		If == TRUE, there were new data arrived via EDMA-SSI from the Master!

\fn		Uint16* PtrSsiRecComm(void) 

\author Kster, Ralf
\date	22.02.2012
\param[IN] void
\return	void

Changes:	

*/ 
Uint16* PtrSsiRecComm(void)
{
	return &tSsi.usRecKom;
}


/*!
\brief  Pointer to SSI-Transmit-State
		with this function the State of an SSI Tranfer can be read.
		If == TRUE, the data are transfered via SSI to the Master!

\fn		Uint16* PtrSsiTraComm(void) 

\author Kster, Ralf
\date	22.02.2012
\param[IN] void
\return	void

Changes:	

*/ 
Uint16* PtrSsiTraComm(void)
{
	return &tSsi.usTraKom;
}


/*!
\brief  Initialisation of the SSI data area
		This synchronous serial init. function defines all variables of the
		SSI modul which exchanges data with the Master-C via Mcbsp-Spi-Port

\fn		void SsiInit(void) 

\author Kster, Ralf
\date	24.02.2012
\param[IN] void
\return	void

Changes:	

*/ 
void SsiInit(void)
{
	tSsi.usTraKom = FALSE;
	// reset Receive-Semaphore
	tSsi.usRecKom = FALSE;
	// reset Error state
	tSsi.unError = 0;
	// write start-data to the Sendbuffer
	//SsiSendVersion();
}


/*!		Load data from SSI Receive-Buffer
\fn		Uint16* PtrSsiRecComm(void) 
\brief  with this function the Receive data will be written to the
		handed over data memory!

\param[in] Pointer to the Receive Data Memory

*/ 
void GetSsiData(void *p_ptData)
{
	// Datenstruct in Sendearray kopieren 
	memcpy (p_ptData, &tSsi.usRecBuffer[0], SSIBUFSIZE);
}



/*!
\brief  Initialisation of the SSI data area
		with this function the Send-Data will be written to the
		handed over data memory!

\fn		void StartSsiSendData(void *p_ptData) 

\author Kster, Ralf
\date	24.02.2012
\param[IN] void
\return	void

Changes:	

*/ 
void StartSsiSendData(void *p_ptData)
{
	// copy data to the Send-Buffer
	memcpy (&tSsi.usTraBuffer[0], p_ptData, SSIBUFSIZE);
	asm(" nop"); 		   
	// DMA starten !!!
	DmaCh1Ch2Start();
	// Set Semaphor that data transfer process has been started
	*PtrSsiTraComm() = TRUE;
}


/*!
\brief  Initialisation of the SSI data area
		with this function the handed over Data will be written to the
		Receive-Buffer, e.g. for init. or Test!

\fn		void InitSsiRecBuffer(void *p_ptData) 

\author Kster, Ralf
\date	24.02.2012
\param[IN] void
\return	void

Changes:	

*/ 
void InitSsiRecBuffer(void *p_ptData)
{
	// Datenstruct in Sendearray kopieren (Adresse 0-71, Lnge 16 Words)
	memcpy (&tSsi.usRecBuffer[0], p_ptData, SSIBUFSIZE);
}



