/**
 * @file    buf.c
 * @brief   Buffer module
 *
 *          The buffer is an ordered collection of data values. Before a new data value is stored at the head of the buffer all
 *          data elements are moved by one position. The shift operation deletes the oldest value at the tail of the buffer. 
 *
 * @par     Copyright:
 *
 *          Thermo Electron (Karlsruhe) GmbH, Dieselstrasse 4, 76227 Karlsruhe, Germany
 *
 *          (c) Copyright 2009 - 2010, Thermo Fisher Scientific
 *
 *          All rights reserved. Thermo Fisher Scientific source code is an unpublished work and the use of a copyright notice does
 *          not imply otherwise. This source code contains confidential, trade secret material of Thermo Fisher Scientific. Any
 *          attempt or participation in deciphering, decoding, reverse engineering or in any way altering the source code is
 *          strictly prohibited, unless the prior written consent of Thermo Fisher Scientific is obtained.
 *
 * @par     Creation Date:
 *
 *          05-Dec-2009
 *
 * @par     Last Modification:
 *
 *          27-Jan-2010
 *
 * @par     Project:
 *
 *          MARS III
 *
 * @par     Editor:
 *
 *          Microsoft Visual C++ 6.0
 *
 * @par     Compiler:
 *
 *          Microsoft Visual C++ 6.0
 *
 * @author  Uwe Bauer
 *
 * @par     Revision History:
 */

/*
 **********************************************************************************************************************************
 *                                                           INCLUDES                                                             *
 **********************************************************************************************************************************
 */
#include "project.h"		// all global things as Type definitions, Hardware def., ...
#include "buf.h"

/*
 **********************************************************************************************************************************
 *                                                        GLOBAL FUNCTIONS                                                        *
 **********************************************************************************************************************************
 */

/*
 * Initialize an empty buffer
 */
void BufInit (BUF_QUEUE *p, float32 *data, const Uint32 n, void (*cpyFun)(float32 *, float32 *, Uint32))
{
    p->tail   = data;
    p->size   = n;
    p->cpyFun = cpyFun;
    
    BufReset(p);
}   /* BufInit()    */

/*
 * Reset buffer
 */
void BufReset (BUF_QUEUE *p)
{
    Uint32 k;

    for (k=0; k<p->size; k++)
    {
        *(p->tail + k) = ((float32) 0.0);
    }

    p->cnt  = ((Uint32) 0);
    p->full = ((Uint16)  0);
    p->rdy  = ((Uint16)  0);
}   /* BufReset()    */

/*
 * Insert a new value at the head of the buffer
 */
void BufInsNxt (BUF_QUEUE *p, const float32 val)
{
    Uint32 k;

    if (p->full)
    {
//#pragma MUST_ITERATE(128,128,1);
        for (k = 0; k < p->size - 1; k++)
        {
            *(p->tail + k) = *(p->tail + k + 1);
        }
        *(p->tail + p->size - 1) = val;
    }
    else
    {
        *(p->tail + p->cnt) = val;
        p->cnt = p->cnt + 1;

        if (p->cnt == p->size)
        {
            p->full = ((Uint16) 1);
        }
    }

    p->rdy = ((Uint16) 1);
}   /* BufInsNxt()  */

/*
 * Read buffer load status
 */
Uint16 BufIsFull (const BUF_QUEUE *p)
{
    return p->full;
}   /* BufIsFull()  */

/*
 * Read buffer count
 */
Uint32 BufGetCnt (const BUF_QUEUE *p)
{
    return p->cnt;
}   /* BufGetCnt()  */

/*
 * Copy buffer data
 */
void BufCpy (BUF_QUEUE *p, float32 *dst)
{
    p->cpyFun(p->tail, dst, p->size);
    
    p->rdy = ((Uint16) 0);
}   /* BufCpy() */

/*
 * Buffer data ready
 */
Uint16 BufIsDataRdy (const BUF_QUEUE *p)
{
    return p->rdy;
}   /* BufIsDataRdy()   */
