/*
************************************************************************************************************************************
*                                              READY-TO-USE EMBEDDED BUILDING BLOCKS
*
*                             (c) Copyright 2015-2016; Thermo Electron (Karlsruhe) GmbH; Karlsruhe-Germany
*
*       All rights reserved. Protected by international copyright laws. 
*
*       TMO source code is an unpublished work and the use of a copyright notice does not imply otherwise. This source code
*       contains confidential, trade secret material of TMO. Any attempt or participation in deciphering, decoding, reverse
*       engineering or in any way altering the source code is strictly prohibited, unless the prior written consent of TMO
*       is obtained.
*
************************************************************************************************************************************
*/

/**
************************************************************************************************************************************
*
*                                                         FILE DESCRIPTION
*
* @file         cbuf.c
* @brief        Circular Buffers
* @details      At circular buffers the new input values are continuously written into the memory. A pointer points to the memory
*               location in which the newest input value is written. When the last element of the memory is reached, the write
*               operation is proceeding with the first memory element. The new value x(n) overwrites the oldest value x(n-N) in the
*               memory and the shifting operation is not necessary.
* @version      V1.00.00
* @author       UwBa
*
************************************************************************************************************************************
* Notes         (1) The note(s) section should be omitted if no notes are given.
************************************************************************************************************************************
*/

/*
************************************************************************************************************************************
*                                                          INCLUDE FILES
************************************************************************************************************************************
*/

//#include <stdlib.h>                                             /* Because of calloc.                                             */

#include "Project.h"		// all global things as Type definitions, Hardware def., ...
#include "cbuf.h"
//#include "typedef.h"                                            /* Because of target platform independent data types.             */
#include "err.h"                                                /* Because of error numbers and error handler.                    */
#include "Diagnos.h"

/*
************************************************************************************************************************************
*                                                   EXTERNAL C LANGUAGE LINKAGE
*
* Note(s) : (1) C++ compilers must 'extern'ally declare all C function prototypes & variable/object declarations for correct C
*               language linkage.
************************************************************************************************************************************
*/

#ifdef __cplusplus
extern "C" {                                                    /* See Note #1.                                                   */
#endif

#ifdef FLASH
#pragma CODE_SECTION(CBuf_WrtFP32, "ramfuncs");
#endif

/*
************************************************************************************************************************************
*                                                          LOCAL DEFINES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                         LOCAL CONSTANTS
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                         LOCAL DATA TYPES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                           LOCAL TABLES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                         LOCAL VARIABLES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                     LOCAL FUNCTION PROTOTYPES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
************************************************************************************************************************************
*                                                         GLOBAL FUNCTIONS
************************************************************************************************************************************
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                            32-BIT PRECISION CIRCULAR BUFFER FUNCTIONS
************************************************************************************************************************************
*/

/*
 *  Configurate a 32-bit precision circular buffer
 */
INT16U CBuf_CfgFP32 (CBUF_FP32 *p, const INT16U len)
{
    if (CBUF_MIN_LEN > len)                                                         /* Check buffer length.                       */
    {
        //ERR_HDLR("CBuf_CfgFP32(): Buffer length below limit!", ERR_EBADLEN);
    }
    else
    {
        p->len = len;                                                               /* Take over memory size.                     */
    }

    p->elmt = (FP32 *)calloc(p->len, sizeof(FP32));

    if (!p->elmt)
    {
        //ERR_HDLR("CBuf_CfgFP32(): Dynamic memory allocation failed!", ERR_ENOMEM);  /* Memory allocation failed.                  */
    }

    p->idx  = ((INT16U) 0);                                                         /* Reset memory index.                        */
    p->uFull = ((INT16U) 0);                                                        /* buffer is empty                            */

    return ERR_SUCCESS;                                                             /* Memory allocation successful.              */
}   /* CBuf_CfgFP32()   */

/*
 *  Memory de-allocation of a 32-bit precision circular buffer
 */
void CBuf_DestroyFP32 (CBUF_FP32 *p)
{
    free(p->elmt);
}   /* CBuf_DestroyFP32()   */

/*
 *  Initializing a 32-bit precision circular buffer with a start value
 */
INT16U CBuf_InitFP32 (CBUF_FP32 *p, const FP32 val)
{
    volatile INT16U k;

    if (Err_IsNaN(val))                                                     /* Check if start value is not-a-number.              */
    {
        //ERR_HDLR("CBuf_InitFP32(): Start value not-a-number!", ERR_ENAN);
    }

    if (Err_IsInf(val))                                                     /* Check if start value is infinite.                  */
    {
        //ERR_HDLR("CBuf_InitFP32(): Start value is infinite!", ERR_EOVRFLW);
    }

    for (k = 0; k < p->len; k++)
    {
        CBuf_WrtFP32(p, val);
    }
    p->idx  = ((INT16U) 0);                                                 /* Reset memory index.                        */
    p->uFull = ((INT16U) 0);                                                /* buffer is empty                            */

    return ERR_SUCCESS;
}   /* CBuf_InitFP32()  */

/*
 *  Write value into a 32-bit precision circular buffer
 */
void CBuf_WrtFP32 (CBUF_FP32 *p, const FP32 val)
{
   *(p->elmt + p->idx) = val;                                               /* Insert new value.                                  */

   p->idx++;                                                                /* Point to next element.                             */

   if (p->idx >= p->len)                                                    /* Increment modulo N.                                */
   {
       p->idx = 0;
       p->uFull = ((INT16U) 1);                                             /* buffer is full                                     */
   }
} /* CBuf_WrtFP32()   */


/*
 *  Read specific data element from a 32-bit precision circular buffer
 */
FP32 CBuf_RdFP32 (CBUF_FP32 *p, const INT16U n)
{
    volatile INT32S dly;

    dly = p->idx - 1 - n;

    if (0 > dly)
    {
        dly += p->len;
    }

    return *(p->elmt + dly);                                                /* Return n-th time delayed value.                    */
}   /* CBuf_RdFP32()    */


/*
 * Read buffer load status
 */
Uint16 CBufIsFull (const CBUF_FP32 *p)
{
    return p->uFull;
}   /* CBufIsFull()  */



/*
************************************************************************************************************************************
*                                            64-BIT PRECISION CIRCULAR BUFFER FUNCTIONS
************************************************************************************************************************************
*/
#ifdef RAUS
/*
 *  Configure a 64-bit precision circular buffer
 */
INT16U CBuf_Cfg (CBUF *p, const INT16U len)
{
    if (CBUF_MIN_LEN > len)                                                         /* Check buffer length.                       */
    {
        ERR_HDLR("CBuf_Cfg(): Buffer length below limit!", ERR_EBADLEN);
    }
    else
    {
        p->len = len;                                                               /* Take over buffer length.                   */
    }

    p->elmt = (FP64 *)calloc(p->len, sizeof(FP64));

    if (!p->elmt)
    {
        ERR_HDLR("CBuf_Cfg(): Dynamic memory allocation failed!", ERR_ENOMEM);      /* Memory allocation failed.                  */
    }

    p->idx  = ((INT16U) 0);                                                         /* Reset memory index.                        */

    return ERR_SUCCESS;                                                             /* Memory allocation successful.              */
}   /* CBuf_Cfg()   */

/*
 *  Memory de-allocation of a 64-bit precision circular buffer
 */
void CBuf_Destroy (CBUF *p)
{
    free(p->elmt);
}   /* CBuf_Destroy()   */

/*
 *  Initializing a 64-bit precision circular buffer with a start value
 */
INT16U CBuf_Init (CBUF *p, const FP64 val)
{
    volatile INT16U k;                                                      /* Common run index.                                  */

    if (Err_IsNaN(val))                                                     /* Check if start value is not-a-number.              */
    {
        ERR_HDLR("CBuf_Init(): Start value not-a-number!", ERR_ENAN);
    }

    if (Err_IsInf(val))                                                     /* Check if start value is infinite.                  */
    {
        ERR_HDLR("CBuf_Init(): Start value is infinite!", ERR_EOVRFLW);
    }

    for (k = 0; k < p->len; k++)                                            /* Buffer initialization loop.                        */
    {
        CBuf_Wrt(p, val);
    }

    return ERR_SUCCESS;                                                     /* Buffer initialization successful.                  */
}   /* CBuf_Init()  */

/*
 *  Write value into a 64-bit precision circular buffer
 */
void CBuf_Wrt (CBUF *p, const FP64 val)
{
  *(p->elmt + p->idx) = val;                                                /* Insert new value.                                  */
    
    p->idx = (p->idx + 1) % p->len;                                         /* Point to next element.                             */
}   /* CBuf_Wrt()   */

/*
 *  Read specific data element from a 64-bit precision circular buffer
 */
FP64 CBuf_Rd (CBUF *p, const INT16U n)
{
    volatile INT16U k;                                                      /* Offset                                             */

    k = (p->idx - n - 1 + p->len) % p->len;                                 /* Decrement offset by modulo of the buffer length.   */

    return *(p->elmt + k);                                                  /* Return n-th time delayed value.                    */
}   /* CBuf_Rd()    */
#endif





/*
************************************************************************************************************************************
************************************************************************************************************************************
*                                                         LOCAL FUNCTIONS
************************************************************************************************************************************
************************************************************************************************************************************
*/

/*
************************************************************************************************************************************
*                                                 EXTERNAL C LANGUAGE LINKAGE END
************************************************************************************************************************************
*/

#ifdef __cplusplus
}                                                                                   /* End of 'extern'al C language linkage.      */
#endif
