/*
************************************************************************************************************************************
*                                              READY-TO-USE EMBEDDED BUILDING BLOCKS
*
*                             (c) Copyright 2015-2016; Thermo Electron (Karlsruhe) GmbH; Karlsruhe-Germany
*
*       All rights reserved. Protected by international copyright laws. 
*
*       TMO source code is an unpublished work and the use of a copyright notice does not imply otherwise. This source code
*       contains confidential, trade secret material of TMO. Any attempt or participation in deciphering, decoding, reverse
*       engineering or in any way altering the source code is strictly prohibited, unless the prior written consent of TMO
*       is obtained.
*
************************************************************************************************************************************
*/

/**
************************************************************************************************************************************
*
*                                                         FILE DESCRIPTION
*
* @file         cmplx.c
* @brief        Complex Math
* @details      The functions in this module provide support for complex numbers and associated math. The implemented algorithms
*               take care to avoid unnecessary intermediate underflows and overflows, allowing the functions to be evaluated over
*               as much of the complex plane as possible.
* @version      V1.00.00
* @author       UwBa
*
************************************************************************************************************************************
* Notes         (1) The note(s) section should be omitted if no notes are given.
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                          INCLUDE FILES
************************************************************************************************************************************
*/

                                                                /* Because of:                                                    */
#include <math.h>                                               /* sin, cos, asin, acos, atan, atan2, sinh, cosh, tanh, log, ...  */
#include "project.h"											/* exp, pow, fabs, sqrt                                           */
#include "cmplx.h"
#include "err.h"                                                /* Because of error numbers and error handler.                    */
#include "mth.h"                                                /* Mth_Log1Plus, Mth_CmplxAbs, Mth_Acosh, Mth_Atanh, ...          */
                                                                /* MTH_MIN, MTH_MAX, ...                                          */
                                                                /* MTH_FP64_MAX, MTH_LN10, MTH_PI, MTH_PI_2                       */

/*
************************************************************************************************************************************
*                                                   EXTERNAL C LANGUAGE LINKAGE
*
* Note(s) : (1) C++ compilers must 'extern'ally declare all C function prototypes & variable/object declarations for correct C
*               language linkage.
************************************************************************************************************************************
*/

#ifdef __cplusplus
extern "C" {                                                                        /* See Note #1.                               */
#endif

/*
************************************************************************************************************************************
*                                                          LOCAL DEFINES
************************************************************************************************************************************
*/

#ifdef FLASH
#pragma CODE_SECTION(Cmplx_Angle, "ramfuncs");
#pragma CODE_SECTION(Cmplx_Abs, "ramfuncs");
#endif


/*
************************************************************************************************************************************
*                                                         LOCAL CONSTANTS
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                         LOCAL DATA TYPES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                           LOCAL TABLES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                         LOCAL VARIABLES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                     LOCAL FUNCTION PROTOTYPES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
************************************************************************************************************************************
*                                                         GLOBAL FUNCTIONS
************************************************************************************************************************************
************************************************************************************************************************************
*/

/*
************************************************************************************************************************************
*                                                    CONSTRUCT COMPLEX NUMBERS
************************************************************************************************************************************
*/

/*
 *  Construct complex data from Cartesian coordinates.
 */
CMPLX_FP64 Cmplx_Rect (const FP64 x, const FP64 y)
{
    CMPLX_FP64  z;

    CMPLX_SET(&z, x, y);

    return z;
}   /* Cmplx_Rect() */

/*
 *  Construct complex data from polar coordinates.
 */
CMPLX_FP64 Cmplx_Polar (const FP64 r, const FP64 theta)
{
    CMPLX_FP64  z;

    CMPLX_SET(&z, r * cos(theta), r * sin(theta));

    return z;
}   /* Cmplx_Polar()    */

/*
************************************************************************************************************************************
*                                                  PROPERTIES OF COMPLEX NUMBERS
************************************************************************************************************************************
*/

/*
 *  Complex phase angle.
 */
FP64 Cmplx_Angle (const CMPLX_FP64 z)
{
    volatile FP64 x;
    volatile FP64 y;

    x = CMPLX_REAL(z);
    y = CMPLX_IMAG(z);

    if ((0.0 == x) && (0.0 == y))
    {
        return 0.0;
    }
    else
    {
        return atan2(y, x);
    }
}   /* Cmplx_Angle()    */

/*
 *  Complex magnitude.
 */
FP64 Cmplx_Abs (const CMPLX_FP64 z)
{
    volatile FP64 x;
    volatile FP64 y;
    volatile FP64 x_abs;
    volatile FP64 y_abs;
    volatile FP64 min;
    volatile FP64 max;
    volatile FP64 rat;
    volatile FP64 root;

    x = CMPLX_REAL(z);
    y = CMPLX_IMAG(z);

    if ((0.0 == x) && (0.0 == y))
    {
        return 0.0;
    }
    else
    {
        x_abs = fabs(x);
        y_abs = fabs(y);
        min   = MTH_MIN(x_abs, y_abs);
        max   = MTH_MAX(x_abs, y_abs);
        rat   = min/max;
        root  = sqrt(1.0 + rat*rat);

        if (max < MTH_FP64_MAX/root)
        {
            return (max*root);
        }
        else
        {
            //ERR_HDLR_VAL("Cmplx_AbsFP64(): Overflow error", ERR_EOVRFLW, MTH_FP64_MAX);
            return MTH_FP64_MAX;
        }
    }
}   /* Cmplx_Abs()  */

/*
 *  Complex square magnitude.
 */
FP64 Cmplx_AbsSqr (const CMPLX_FP64 z)
{
    volatile FP64 x;
    volatile FP64 y;
    
    x = CMPLX_REAL(z);
    y = CMPLX_IMAG(z);

    if ((0.0 == x) && (0.0 == y))
    {
        return 0.0;
    }
    else
    {
        /** @todo   Exception handling overflow. */
        return ((x * x)  + (y * y));
    }
}   /* Cmplx_AbsSqr()   */

/*
 *  Natural logarithm of the magnitude of the complex number.
 */
FP64 Cmplx_LogAbs (const CMPLX_FP64 z)
{
    volatile FP64 x_abs;
    volatile FP64 y_abs;
    volatile FP64 max;
    volatile FP64 rat;

    x_abs = fabs(CMPLX_REAL(z));
    y_abs = fabs(CMPLX_IMAG(z));

    if (x_abs >= y_abs)
    {
        max = x_abs;
        rat = y_abs / x_abs;
    }
    else
    {
        max = y_abs;
        rat = x_abs / y_abs;
    }

    return log(max) + 0.5 * Mth_Log1Plus(rat * rat);                                /* Handle underflow when u is close to 0.     */
}   /* Cmplx_LogAbs()   */

/*
************************************************************************************************************************************
*                                                   COMPLEX ARITHMETIC OPERATORS
************************************************************************************************************************************
*/

/*
 *  Negative complex number
 */
CMPLX_FP64 Cmplx_Neg (const CMPLX_FP64 a)
{
    CMPLX_FP64 z;

    CMPLX_SET(&z, -CMPLX_REAL(a), -CMPLX_IMAG(a));

    return z;
}   /* Cmplx_Neg()  */

/*
 *  Complex conjugate
 */
CMPLX_FP64 Cmplx_Conj (const CMPLX_FP64 a)
{
    CMPLX_FP64 z;

    CMPLX_SET(&z, CMPLX_REAL(a), -CMPLX_IMAG(a));

    return z;
}   /* Cmplx_Conj() */

/*
 *  Complex inverse
 */
CMPLX_FP64 Cmplx_Inv (const CMPLX_FP64 a)
{
    volatile FP64   s;
       CMPLX_FP64   z;

    /** @todo: Avoid division by zero   */
    s = 1.0 / Cmplx_Abs(a);

    CMPLX_SET(&z, (CMPLX_REAL(a) * s) * s, -(CMPLX_IMAG(a) * s) * s);

    return z;
}   /* Cmplx_Inv()  */

/*
 *  Additon of two complex numbers.
 */
CMPLX_FP64 Cmplx_Add (const CMPLX_FP64 z1, const CMPLX_FP64 z2)
{
    CMPLX_FP64 s;

    CMPLX_SET(&s, CMPLX_REAL(z1) + CMPLX_REAL(z2), CMPLX_IMAG(z1) + CMPLX_IMAG(z2));

    return s;
}   /* Cmplx_Add()  */

/*
 *  Addition of a complex and a real number.
 */
CMPLX_FP64 Cmplx_AddReal (const CMPLX_FP64 z, const FP64 x)
{
    CMPLX_FP64 s;

    CMPLX_SET(&s, CMPLX_REAL(z) + x, CMPLX_IMAG(z));

    return s;
}   /* Cmplx_AddReal()  */

/*
 *  Addition of a complex and a imaginary number.
 */
CMPLX_FP64 Cmplx_AddImag (const CMPLX_FP64 z, const FP64 y)
{
    CMPLX_FP64 s;

    CMPLX_SET(&s, CMPLX_REAL(z), CMPLX_IMAG(z) + y);

    return s;
}   /* Cmplx_AddImag()  */

/*
 *  Subtraction of two complex numbers.
 */
CMPLX_FP64 Cmplx_Sub (const CMPLX_FP64 z1, const CMPLX_FP64 z2)
{
    CMPLX_FP64 d;

    CMPLX_SET(&d, CMPLX_REAL(z1) - CMPLX_REAL(z2), CMPLX_IMAG(z1) - CMPLX_IMAG(z2));

    return d;
}   /* Cmplx_Sub()  */

/*
 *  Subtraction of a complex and a real number.
 */
CMPLX_FP64 Cmplx_SubReal (const CMPLX_FP64 z, const FP64 x)
{
    CMPLX_FP64 d;

    CMPLX_SET(&d, CMPLX_REAL(z) - x, CMPLX_IMAG(z));

    return d;
}   /* Cmplx_SubReal()  */

/*
 *  Subtraction of a complex and a imaginary number.
 */
CMPLX_FP64 Cmplx_SubImag (const CMPLX_FP64 z, const FP64 y)
{
    CMPLX_FP64 d;

    CMPLX_SET(&d, CMPLX_REAL(z), CMPLX_IMAG(z) - y);

    return d;
}   /* Cmplx_SubImag()  */

/*
 *  Multiply complex numbers.
 */
CMPLX_FP64 Cmplx_Mul (const CMPLX_FP64 a, const CMPLX_FP64 b)
{
       CMPLX_FP64 z;
    volatile FP64 ar, ai, br, bi;

    ar = CMPLX_REAL(a);
    ai = CMPLX_IMAG(a);
    br = CMPLX_REAL(b);
    bi = CMPLX_IMAG(b);

    CMPLX_SET(&z, (ar * br) - (ai * bi), (ar * bi) + (ai * br));
    
    return z;
}   /* Cmplx_Mul()  */

/*
 *  Multiply a complex number with a real factor.
 */
CMPLX_FP64 Cmplx_MulReal (const CMPLX_FP64 a, const FP64 x)
{
    CMPLX_FP64 z;

    CMPLX_SET(&z, x * CMPLX_REAL(a), x * CMPLX_IMAG(a));

    return z;
}   /* Cmplx_MulReal()  */


/*
 *  Multiply a complex number with a imaginary factor.
 */
CMPLX_FP64 Cmplx_MulImag (const CMPLX_FP64 a, const FP64 y)
{
    CMPLX_FP64 z;

    CMPLX_SET(&z, -y * CMPLX_IMAG(a), y * CMPLX_REAL(a));

    return z;
}   /* Cmplx_MulImag()  */

/*
 *  Divide complex numbers.
 */
CMPLX_FP64 Cmplx_Div (const CMPLX_FP64 a, const CMPLX_FP64 b)
{
       CMPLX_FP64   z;
    volatile FP64   ar, ai, br, bi, s, zr, zi;

    ar = CMPLX_REAL(a);
    ai = CMPLX_IMAG(a);
    br = CMPLX_REAL(b);
    bi = CMPLX_IMAG(b);

    s = 1.0 / Cmplx_AbsSqr(b);

    zr = ((ar * br) + (ai * bi)) * s;
    zi = ((ai * br) - (ar * bi)) * s;

    CMPLX_SET(&z, zr, zi);

    return z;
}   /* Cmplx_Div()  */

/*
 *  Divide a complex number with a real number.
 */
CMPLX_FP64 Cmplx_DivReal (const CMPLX_FP64 a, const FP64 x)
{
    CMPLX_FP64 z;

    CMPLX_SET(&z, CMPLX_REAL(a) / x, CMPLX_IMAG(a) / x);

    return z;
}   /* Cmplx_DivReal()  */

/*
 *  Divide a complex number with an imaginary number.
 */
CMPLX_FP64 Cmplx_DivImag (const CMPLX_FP64 a, const FP64 y)
{
    CMPLX_FP64 z;

    CMPLX_SET(&z, CMPLX_IMAG(a) / y, -CMPLX_REAL(a) / y);

    return z;
}   /* Cmplx_DivImag()  */

/*
************************************************************************************************************************************
*                                                   ELEMENTARY COMPLEX FUNCTIONS
************************************************************************************************************************************
*/

/*
 *  Square root of a complex number.
 */
CMPLX_FP64 Cmplx_Sqrt (const CMPLX_FP64 a)
{
       CMPLX_FP64 z;
    volatile FP64 x, y, w, t, vi;

    x = CMPLX_REAL(a);
    y = CMPLX_IMAG(a);

    if ((0.0 == x) && (0.0 == y))
    {
        CMPLX_SET(&z, 0.0, 0.0);
    }
    else
    {
        if (x >= y)
        {
            t = y / x;
            w = sqrt(x) * sqrt(0.5 * (1.0 + sqrt(1.0 + t * t)));
        }
        else
        {
            t = x / y;
            w = sqrt(y) * sqrt(0.5 * (t + sqrt(1.0 + t * t)));
        }

        if (0.0 <= x)
        {
            CMPLX_SET(&z, w, y / (2.0 * w));
        }
        else
        {
            vi = (0.0 <= y) ? w : -w;
            CMPLX_SET(&z, y / (2.0 * vi), vi);
        }
    }

    return z;
}   /* Cmplx_Sqrt() */

/*
 *  Complex square root of a real number.
 */
CMPLX_FP64 Cmplx_SqrtReal (const FP64 x)
{
    CMPLX_FP64 z;

    if (0.0 <= x)
    {
        CMPLX_SET(&z, sqrt(x), 0.0);
    }
    else
    {
        CMPLX_SET(&z, 0.0, sqrt(-x));
    }

    return z;
}   /* Cmplx_SqrtReal() */

/*
 *  Complex exponent.
 */
CMPLX_FP64 Cmplx_Exp (const CMPLX_FP64 a)
{
       CMPLX_FP64 z;
    volatile FP64 rho;
    volatile FP64 theta;

    rho   = exp(CMPLX_REAL(a));
    theta = CMPLX_IMAG(a);

    CMPLX_SET(&z, rho * cos(theta), rho * sin(theta));

    return z;
}   /* Cmplx_Exp()  */

/*
 *  Complex power.
 */
CMPLX_FP64 Cmplx_Pow (const CMPLX_FP64 a, const CMPLX_FP64 b)
{
       CMPLX_FP64 z;
    volatile FP64 logr;
    volatile FP64 theta;
    volatile FP64 br;
    volatile FP64 bi;
    volatile FP64 rho;
    volatile FP64 beta;

    if ((0.0 == CMPLX_REAL(a)) && (0.0 == CMPLX_IMAG(a)))
    {
        CMPLX_SET(&z, 0.0, 0.0);
    }
    else
    {
        logr  = Cmplx_LogAbs(a);
        theta = Cmplx_Angle(a);
        br    = CMPLX_REAL(b);
        bi    = CMPLX_IMAG(b);
        rho   = exp((logr * br) - (bi * theta));
        beta  = (theta * br) + (bi * logr);

        CMPLX_SET(&z, rho * cos(beta), rho * sin(beta));
    }

    return z;
}   /* Cmplx_Pow()  */

/*
 *  Real power.
 */
CMPLX_FP64 Cmplx_PowReal (const CMPLX_FP64 a, const FP64 x)
{
       CMPLX_FP64 z;
    volatile FP64 logr;
    volatile FP64 theta;
    volatile FP64 rho;
    volatile FP64 beta;

    if ((0.0 == CMPLX_REAL(a)) && (0.0 == CMPLX_IMAG(a)))
    {
        CMPLX_SET(&z, 0.0, 0.0);
    }
    else
    {
        logr  = Cmplx_LogAbs(a);
        theta = Cmplx_Angle(a);
        rho   = exp(logr * x);
        beta  = theta * x;
        
        CMPLX_SET(&z, rho * cos(beta), rho * sin(beta));
    }

    return z;
}   /* Cmplx_PowReal()  */

/*
 *  Complex natural logarithm
 */
CMPLX_FP64 Cmplx_Log (const CMPLX_FP64 a)
{
       CMPLX_FP64 z;
    volatile FP64 logr;
    volatile FP64 theta;

    logr  = Cmplx_LogAbs(a);
    theta = Cmplx_Angle(a);

    CMPLX_SET(&z, logr, theta);

    return z;
}   /* Cmplx_Log()  */

/*
 *  Complex base-10 logarithm
 */
CMPLX_FP64 Cmplx_Log10 (const CMPLX_FP64 a)
{
    return Cmplx_DivReal(Cmplx_Log(a), MTH_LN10);
}   /* Cmplx_Log10()    */

/*
 *  Complex base-b logarithm 
 */
CMPLX_FP64 Cmplx_LogBase (const CMPLX_FP64 a, const CMPLX_FP64 b)
{
    return Cmplx_Div(Cmplx_Log(a), Cmplx_Log(b));
}   /* Cmplx_LogBase()  */

/*
************************************************************************************************************************************
*                                                 COMPLEX TRIGONOMETRIC FUNCTIONS
************************************************************************************************************************************
*/

/*
 *  Complex sine
 */
CMPLX_FP64 Cmplx_Sin (const CMPLX_FP64 a)
{
    volatile FP64 x;
    volatile FP64 y;
       CMPLX_FP64 z;

    x = CMPLX_REAL(a);
    y = CMPLX_IMAG(a);

    if (0.0 == y)
    {
        CMPLX_SET(&z, sin(x), 0.0);                             /* Avoid returing negative zero (-0.0) for the imaginary part.    */
    }
    else
    {
        CMPLX_SET(&z, sin(x) * cosh(y), cos(x) * sinh(y));
    }
    
    return z;
}   /* Cmplx_Sin()  */

/*
 *  Complex cosine
 */
CMPLX_FP64 Cmplx_Cos (const CMPLX_FP64 a)
{
    volatile FP64 x;
    volatile FP64 y;
       CMPLX_FP64 z;

    x = CMPLX_REAL(a);
    y = CMPLX_IMAG(a);

    if (0.0 == y)
    {
        CMPLX_SET(&z, cos(x), 0.0);                             /* Avoid returing negative zero (-0.0) for the imaginary part.    */
    }
    else
    {
        CMPLX_SET(&z, cos(x) * cosh(y), sin(x) * sinh(-y));
    }
    
    return z;
}   /* Cmplx_Cos()  */

/*
 *  Complex tangent
 */
CMPLX_FP64 Cmplx_Tan (const CMPLX_FP64 a)
{
    volatile FP64 x;
    volatile FP64 y;
    volatile FP64 c;
    volatile FP64 d;
    volatile FP64 u;
    volatile FP64 s;
    volatile FP64 t;
       CMPLX_FP64 z;

    x = CMPLX_REAL(a);
    y = CMPLX_IMAG(a);

    if (1.0 > fabs(y))
    {
        d = pow(cos(x), 2.0) + pow(sinh(y), 2.0);

        CMPLX_SET(&z, 0.5 * sin(2.0 * x) / d, 0.5 * sinh (2.0 * y) / d);
    }
    else
    {
        u = exp(-y);
        c = 2.0 * u / (1 - pow(u, 2.0));
        d = 1.0 + pow(cos(x), 2.0) * pow(c, 2.0);
        s = pow (c, 2.0);
        t = 1.0 / tanh(y);

        CMPLX_SET(&z, 0.5 * sin(2 * x) * s / d, t / d);
    }

    return z;
}   /* Cmplx_Tan()  */

/*
 *  Complex secant
 */
CMPLX_FP64 Cmplx_Sec (const CMPLX_FP64 a)
{
    CMPLX_FP64 z;

    z = Cmplx_Cos(a);

    return Cmplx_Inv(z);
}   /* Cmplx_Sec()  */

/*
 *  Complex cosecant
 */
CMPLX_FP64 Cmplx_Csc (const CMPLX_FP64 a)
{
    CMPLX_FP64 z;

    z = Cmplx_Sin(a);

    return Cmplx_Inv(z);
}   /* Cmplx_CscFP64()  */

/*
 *  Complex cotangent
 */
CMPLX_FP64 Cmplx_Cot (const CMPLX_FP64 a)
{
    CMPLX_FP64 z;

    z = Cmplx_Tan(a);

    return Cmplx_Inv(z);
}   /* Cmplx_Cot()  */

/*
************************************************************************************************************************************
*                                                 INVERSE COMPLEX TRIGONOMETRIC FUNCTIONS
************************************************************************************************************************************
*/

/*
 *  Complex arcsine
 */
CMPLX_FP64 Cmplx_Asin (const CMPLX_FP64 z)
{
    volatile FP64 x;
    volatile FP64 y;
    volatile FP64 x_abs;
    volatile FP64 y_abs;
    volatile FP64 r;
    volatile FP64 s;
    volatile FP64 a;
    volatile FP64 b;
    volatile FP64 y_abs_sqr;
    volatile FP64 real;
    volatile FP64 imag;
    volatile FP64 a_crossover;
    volatile FP64 b_crossover;
    volatile FP64 d;
    volatile FP64 a_px;
    volatile FP64 am1;
       CMPLX_FP64 c;

    x = CMPLX_REAL(z);
    y = CMPLX_IMAG(z);

    if (0.0 == y)
    {
        c = Cmplx_AsinReal(x);
    }
    else
    {
        x_abs = fabs(x);
        y_abs = fabs(y);

        r = Mth_CmplxAbs(x_abs + 1.0, y_abs);
        s = Mth_CmplxAbs(x_abs - 1.0, y_abs);

        a         = 0.5 * (r + s);
        b         = x_abs / a;
        y_abs_sqr = y_abs * y_abs;

        a_crossover = 1.5;
        b_crossover = 0.6417;

        if (b_crossover >= b)
        {
            real = asin(b);
        }
        else
        {
            if (1.0 >= x_abs)
            {
                d    = 0.5 * (a + x_abs) * (y_abs_sqr / (r + x_abs + 1.0) + (s + (1.0 - x_abs)));
                real = atan(x_abs / sqrt(d));
            }
            else
            {
                a_px = a + x_abs;
                d    = 0.5 * (a_px / (r + x_abs + 1.0) + a_px / (s + (x_abs - 1.0)));
                real = atan(x_abs / (y_abs * sqrt(d)));
            }
        }

        if (a_crossover >= a)
        {
            if (1.0 > x_abs)
            {
                am1 = 0.5 * (y_abs_sqr / (r + (x_abs + 1.0)) + y_abs_sqr / (s + (1.0 - x_abs)));
            }
            else
            {
                am1 = 0.5 * (y_abs_sqr / (r + (x_abs + 1.0)) + (s + (x_abs - 1.0)));
            }

            imag = Mth_Log1Plus(am1 + sqrt(am1 * (a + 1.0)));
        }
        else
        {
            imag = log(a + sqrt ((a * a) - 1.0));
        }

        CMPLX_SET(&c, (0.0 <= x) ? real : -real, (0.0 <= y) ? imag : -imag);
    }

    return c;
}   /* Cmplx_Asin()   */

/*
 *  Complex arcsine of the real number 
 */
CMPLX_FP64 Cmplx_AsinReal (const FP64 a)
{
    CMPLX_FP64 z;

    if (1.0 >= fabs(a))
    {
        CMPLX_SET(&z, asin(a), 0.0);
    }
    else
    {
        if (0.0 > a)
        {
            CMPLX_SET(&z, -MTH_PI_2, Mth_Acosh(-a));
        }
        else
        {
            CMPLX_SET(&z, MTH_PI_2, -Mth_Acosh(a));
        }
    }

    return z;
}   /* Cmplx_AsinReal() */

/*
 *  Complex arccosine
 */
CMPLX_FP64 Cmplx_Acos (const CMPLX_FP64 z)
{
    volatile FP64 x;
    volatile FP64 y;
    volatile FP64 x_abs;
    volatile FP64 y_abs;
    volatile FP64 r;
    volatile FP64 s;
    volatile FP64 a;
    volatile FP64 b;
    volatile FP64 y_abs_sqr;
    volatile FP64 real;
    volatile FP64 imag;
    volatile FP64 a_crossover;
    volatile FP64 b_crossover;
    volatile FP64 d;
    volatile FP64 a_px;
    volatile FP64 am1;
       CMPLX_FP64 c;

    x = CMPLX_REAL(z);
    y = CMPLX_IMAG(z);

    if (0.0 == y)
    {
        c = Cmplx_AcosReal(x);
    }
    else
    {
        x_abs = fabs(x);
        y_abs = fabs(y);

        r = Mth_CmplxAbs(x_abs + 1.0, y_abs);
        s = Mth_CmplxAbs(x_abs - 1.0, y_abs);

        a         = 0.5 * (r + s);
        b         = x_abs / a;
        y_abs_sqr = y_abs * y_abs;

        a_crossover = 1.5;
        b_crossover = 0.6417;

        if (b_crossover >= b)
        {
            real = acos(b);
        }
        else
        {
            if (1.0 >= x_abs)
            {
                d    = 0.5 * (a + x_abs) * (y_abs_sqr / (r + x_abs + 1.0) + (s + (1.0 - x_abs)));
                real = atan(sqrt(d) / x_abs);
            }
            else
            {
                a_px = a + x_abs;
                d    = 0.5 * (a_px / (r + x_abs + 1.0) + a_px / (s + (x_abs - 1.0)));
                real = atan((y_abs * sqrt(d)) / x_abs);
            }
        }

        if (a_crossover >= a)
        {
            if (1.0 > x_abs)
            {
                am1 = 0.5 * (y_abs_sqr / (r + (x_abs + 1.0)) + y_abs_sqr / (s + (1.0 - x_abs)));
            }
            else
            {
                am1 = 0.5 * (y_abs_sqr / (r + (x_abs + 1.0)) + (s + (x_abs - 1.0)));
            }

            imag = Mth_Log1Plus(am1 + sqrt(am1 * (a + 1.0)));
        }
        else
        {
            imag = log(a + sqrt ((a * a) - 1.0));
        }

        CMPLX_SET(&c, (0.0 <= x) ? real : (MTH_PI - real), (0.0 <= y) ? -imag : imag);
    }

    return c;
}   /* Cmplx_Acos() */

/*
 *  Complex arccosine of the real number 
 */
CMPLX_FP64 Cmplx_AcosReal (const FP64 a)
{
    CMPLX_FP64 z;

    if (1.0 >= fabs(a))
    {
        CMPLX_SET(&z, acos(a), 0.0);
    }
    else
    {
        if (0.0 > a)
        {
            CMPLX_SET(&z, MTH_PI, -Mth_Acosh(-a));
        }
        else
        {
            CMPLX_SET(&z, 0.0, Mth_Acosh(a));
        }
    }
    
    return z;
}   /* Cmplx_AcosReal() */

/*
 *  Complex arctangent
 */
CMPLX_FP64 Cmplx_Atan (const CMPLX_FP64 z)
{
    volatile FP64 x;
    volatile FP64 y;
    volatile FP64 r;
    volatile FP64 u;
    volatile FP64 imag;
    volatile FP64 a;
    volatile FP64 b;
       CMPLX_FP64 c;

    x = CMPLX_REAL(z);
    y = CMPLX_IMAG(z);

    if (0.0 == y)
    {
        CMPLX_SET(&c, atan(x), 0.0);
    }
    else
    {
        /* FIXME: This is a naive implementation which does not fully
         take into account cancellation errors, overflow, underflow
         etc.  It would benefit from the Hull et al treatment. */

        r = Mth_CmplxAbs(x, y);

        u = (2.0 * y) / (1.0 + (r * r));

        /* FIXME: the following cross-over should be optimized but 0.1
         seems to work ok */

        if (0.1 > fabs(u))
        {
            imag = 0.25 * (Mth_Log1Plus(u) - Mth_Log1Plus(-u));
        }
        else
        {
            a    = Mth_CmplxAbs(x, y + 1.0);
            b    = Mth_CmplxAbs(x, y - 1.0);
            imag = 0.5 * log(a / b);
        }

        if (0.0 == x)
        {
            if (1.0 < y)
            {
                CMPLX_SET(&c, MTH_PI_2, imag);
            }
            else if (-1.0 > y)
            {
                CMPLX_SET(&c, -MTH_PI_2, imag);
            }
            else
            {
                CMPLX_SET(&c, 0.0, imag);
            }
        }
        else
        {
            CMPLX_SET(&c, 0.5 * atan2((2.0 * x), ((1.0 + r) * (1.0 - r))), imag);
        }
    }

    return c;
}   /* Cmplx_Atan() */

/*
 *  Complex arcsecant 
 */
CMPLX_FP64 Cmplx_Asec (const CMPLX_FP64 a)
{
    CMPLX_FP64 z;

    z = Cmplx_Inv(a);

    return Cmplx_Acos(z);
}   /* Cmplx_Asec() */

/*
 *  Complex arcsecant of a real number
 */
CMPLX_FP64 Cmplx_AsecReal (const FP64 a)
{
    CMPLX_FP64 z;

    if ((-1.0 >= a) || (1.0 <= a))
    {
        CMPLX_SET(&z, acos(1.0 / a), 0.0);
    }
    else
    {
        if (0.0 <= a)
        {
            CMPLX_SET(&z, 0.0, Mth_Acosh(1.0 / a));
        }
        else
        {
            CMPLX_SET(&z, MTH_PI, -Mth_Acosh(-1.0 / a));
        }
    }

    return z;
}   /* Cmplx_AsecReal() */

/*
 *  Complex arccosecant
 */
CMPLX_FP64 Cmplx_Acsc (const CMPLX_FP64 a)
{
    CMPLX_FP64 z;

    z = Cmplx_Inv(a);

    return Cmplx_Asin(z);
}   /* Cmplx_Acsc() */

/*
 *  Complex arccosecant of a real number
 */
CMPLX_FP64 Cmplx_AcscReal (const FP64 a)
{
    CMPLX_FP64 z;

    if ((-1.0 >= a) || (1.0 <= a))
    {
        CMPLX_SET(&z, asin(1.0 / a), 0.0);
    }
    else
    {
        if (0.0 <= a)
        {
            CMPLX_SET(&z, MTH_PI_2, -Mth_Acosh(1.0 / a));
        }
        else
        {
            CMPLX_SET(&z, -MTH_PI_2, Mth_Acosh(-1.0 / a));
        }
    }

    return z;
}   /* Cmplx_AcscReal() */

/*
 *  Complex arccotangent 
 */
CMPLX_FP64 Cmplx_Acot (CMPLX_FP64 a)
{
    CMPLX_FP64 z;

    if ((0.0 == CMPLX_REAL(a)) && (0.0 == CMPLX_IMAG(a)))
    {
        CMPLX_SET(&z, MTH_PI_2, 0.0);
    }
    else
    {
        z = Cmplx_Inv(a);
        z = Cmplx_Atan(z);
    }

    return z;
}   /* Cmplx_Acot() */

/*
************************************************************************************************************************************
*                                                   COMPLEX HYPERBOLIC FUNCTIONS
************************************************************************************************************************************
*/

/*
 *  Complex hyperbolic sine
 */
CMPLX_FP64 Cmplx_Sinh (CMPLX_FP64 a)
{
    volatile FP64 x;
    volatile FP64 y;
       CMPLX_FP64 z;

    x = CMPLX_REAL(a);
    y = CMPLX_IMAG(a);

    CMPLX_SET(&z, sinh(x) * cos (y), cosh(x) * sin (y));

    return z;
}   /* Cmplx_Sinh() */

/*
 *  Complex hyperbolic cosine
 */
CMPLX_FP64 Cmplx_Cosh (CMPLX_FP64 a)
{
    volatile FP64 x;
    volatile FP64 y;
       CMPLX_FP64 z;

    x = CMPLX_REAL(a);
    y = CMPLX_IMAG(a);

    CMPLX_SET(&z, cosh(x) * cos(y), sinh(x) * sin (y));

    return z;
}   /* Cmplx_Cosh() */

/*
 *  Complex hyperbolic tangent
 */
CMPLX_FP64 Cmplx_Tanh (CMPLX_FP64 a)
{
    volatile FP64 x;
    volatile FP64 y;
    volatile FP64 d;
    volatile FP64 f;
       CMPLX_FP64 z;

    x = CMPLX_REAL(a);
    y = CMPLX_IMAG(a);

    if (1.0 > fabs(x))
    {
        d = pow(cos(y), 2.0) + pow(sinh(x), 2.0);
        
        CMPLX_SET(&z, (sinh(x) * cosh(x)) / d, (0.5 * sin(2.0 * y)) / d);
    }
    else
    {
        d = pow(cos(y), 2.0) + pow (sinh(x), 2.0);
        f = 1.0 + pow(cos(y) / sinh(x), 2.0);

        CMPLX_SET(&z, 1.0 / (tanh(x) * f), (0.5 * sin(2.0 * y)) / d);
    }
    
    return z;
}   /* Cmplx_Tanh() */

/*
 *  Complex hyperbolic secant
 */
CMPLX_FP64 Cmplx_Sech (CMPLX_FP64 a)
{
    CMPLX_FP64 z;
    
    z = Cmplx_Cosh(a);

    return Cmplx_Inv(z);
}   /* Cmplx_Sech() */

/*
 *  Complex hyperbolic cosecant
 */
CMPLX_FP64 Cmplx_Csch (CMPLX_FP64 a)
{
    CMPLX_FP64 z;
    
    z = Cmplx_Sinh(a);

    return Cmplx_Inv(z);
}   /* Cmplx_Csch() */

/*
 *  Complex hyperbolic cotangent
 */
CMPLX_FP64 Cmplx_Coth (CMPLX_FP64 a)
{
    CMPLX_FP64 z;
    
    z = Cmplx_Tanh(a);

    return Cmplx_Inv(z);
}   /* Cmplx_Coth() */

/*
************************************************************************************************************************************
*                                               INVERSE COMPLEX HYPERBOLIC FUNCTIONS
************************************************************************************************************************************
*/

/*
 *  Complex hyperbolic arcsine 
 */
CMPLX_FP64 Cmplx_Asinh (const CMPLX_FP64 a)
{
    CMPLX_FP64 z;
    
    z = Cmplx_MulImag(a, 1.0);
    z = Cmplx_Asin(z);
    z = Cmplx_MulImag(z, -1.0);

    return z;
}   /* Cmplx_Asinh()    */

/*
 *  Complex hyperbolic arccosine 
 */
CMPLX_FP64 Cmplx_Acosh (const CMPLX_FP64 a)
{
    CMPLX_FP64 z;

    z = Cmplx_Acos(a);
    z = Cmplx_MulImag(z, CMPLX_IMAG(z) > 0.0 ? -1.0 : 1.0);

    return z;
}   /* Cmplx_Acosh()    */

/*
 *  Complex hyperbolic arccosine of the real number
 */
CMPLX_FP64 Cmplx_AcoshReal (const FP64 a)
{
    CMPLX_FP64 z;

    if (1.0 <= a)
    {
        CMPLX_SET(&z, Mth_Acosh(a), 0.0);
    }
    else
    {
        if (-1.0 <= a)
        {
            CMPLX_SET(&z, 0.0, acos(a));
        }
        else
        {
            CMPLX_SET(&z, Mth_Acosh(-a), MTH_PI);
        }
    }

    return z;
}   /* Cmplx_AcoshReal()    */

/*
 *  Complex hyperbolic arctangent 
 */
CMPLX_FP64 Cmplx_Atanh (const CMPLX_FP64 a)
{
    CMPLX_FP64 z;

    if (0.0 == CMPLX_IMAG(a))
    {
        return Cmplx_AtanhReal(CMPLX_REAL(a));
    }
    else
    {
        z = Cmplx_MulImag(a, 1.0);
        z = Cmplx_Atan(z);
        z = Cmplx_MulImag(z, -1.0);
        
        return z;
    }
}   /* Cmplx_Atanh()    */

/*
 *  Complex hyperbolic arctangent of the real number
 */
CMPLX_FP64 Cmplx_AtanhReal (const FP64 a)
{
    CMPLX_FP64 z;

    if ((-1.0 < a) && (1.0 > a))
    {
        CMPLX_SET(&z, Mth_Atanh(a), 0.0);
    }
    else
    {
        CMPLX_SET(&z, Mth_Atanh(1.0 / a), (a < 0.0) ? MTH_PI_2 : -MTH_PI_2);
    }

    return z;
}   /* Cmplx_AtanhReal()    */

/*
 *  Complex hyperbolic arcsecant
 */
CMPLX_FP64 Cmplx_Asech (const CMPLX_FP64 a)
{
    CMPLX_FP64 z;
    
    z = Cmplx_Inv(a);

    return Cmplx_Acosh(z);
}   /* Cmplx_Asech()    */

/*
 *  Complex hyperbolic arccosecant 
 */
CMPLX_FP64 Cmplx_Acsch (const CMPLX_FP64 a)
{
    CMPLX_FP64 z;
    
    z = Cmplx_Inv(a);

    return Cmplx_Asinh(z);
}   /* Cmplx_Acsch()    */

/*
 *  Complex hyperbolic arccotangent 
 */
CMPLX_FP64 Cmplx_Acoth (const CMPLX_FP64 a)
{
    CMPLX_FP64 z;
    
    z = Cmplx_Inv(a);

    return Cmplx_Atanh(z);
}   /* Cmplx_Acoth()    */

/*
************************************************************************************************************************************
************************************************************************************************************************************
*                                                         LOCAL FUNCTIONS
************************************************************************************************************************************
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                 EXTERNAL C LANGUAGE LINKAGE END
************************************************************************************************************************************
*/

#ifdef __cplusplus
}                                                                                   /* End of 'extern'al C language linkage.      */
#endif
