/*
************************************************************************************************************************************
*                                              READY-TO-USE EMBEDDED BUILDING BLOCKS
*
*                             (c) Copyright 2015-2016; Thermo Electron (Karlsruhe) GmbH; Karlsruhe-Germany
*
*       All rights reserved. Protected by international copyright laws. 
*
*       TMO source code is an unpublished work and the use of a copyright notice does not imply otherwise. This source code
*       contains confidential, trade secret material of TMO. Any attempt or participation in deciphering, decoding, reverse
*       engineering or in any way altering the source code is strictly prohibited, unless the prior written consent of TMO
*       is obtained.
*
************************************************************************************************************************************
*/

/**
************************************************************************************************************************************
*
*                                                         FILE DESCRIPTION
*
* @file         dft.c
* @brief        Discrete Fourier Transform
* @details      @todo
* @version      V1.00.00
* @author       UwBa
*
************************************************************************************************************************************
* Notes         (1) The note(s) section should be omitted if no notes are given.
************************************************************************************************************************************
*/

/*
************************************************************************************************************************************
*                                                          INCLUDE FILES
************************************************************************************************************************************
*/

#include <math.h>                                               /* Because of sin and cos.                                        */
#include "Project.h"		// all global things as Type definitions, Hardware def., ...
#include "dft.h"
#include "cbuf.h"
//#include "typedef.h"                                            /* Because of target platform independent data types.             */
#include "mth.h"                                                /* Because of math constant pi.                                   */
#include "cmplx.h"                                              /* Because of complex math abs and angle.                         */
#include "err.h"                                                /* Because of error handler and error numbers.                    */

/*
************************************************************************************************************************************
*                                                   EXTERNAL C LANGUAGE LINKAGE
*
* Note(s) : (1) C++ compilers must 'extern'ally declare all C function prototypes & variable/object declarations for correct C
*               language linkage.
************************************************************************************************************************************
*/

#ifdef __cplusplus
extern "C" {                                                    /* See Note #1.                                                   */
#endif

#ifdef FLASH
#pragma CODE_SECTION(DFT_GoertzelSngPntFP32, "ramfuncs");
#endif


/*
************************************************************************************************************************************
*                                                          LOCAL DEFINES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                         LOCAL CONSTANTS
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                         LOCAL DATA TYPES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                           LOCAL TABLES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                         LOCAL VARIABLES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                     LOCAL FUNCTION PROTOTYPES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
************************************************************************************************************************************
*                                                         GLOBAL FUNCTIONS
************************************************************************************************************************************
************************************************************************************************************************************
*/

/*
************************************************************************************************************************************
*                                                    32-BIT PRECISION GOERTZEL
************************************************************************************************************************************
*/

/*
 *  Setup 32-bit precision single point Goertzel DFT algorithm
 */
INT16U DFT_GoertzelSngPntSetupFP32 (DFT_GOERTZEL_SNG_PNT_FP32 *p, CBUF_FP32 *ptr_cbuf)
{
    if (!ptr_cbuf)
    {
        //ERR_HDLR("DFT_GoertzelSngPntSetupFP32(): Invalid pointer to circular buffer instance!", ERR_EFAULT);
    }

    p->PtrCBuf  = ptr_cbuf;

    return ERR_SUCCESS;
}   /* DFT_GoertzelSngPntSetupFP32()    */

/*
 *  Configurate 32-bit precision single point Goertzel algorithm
 */
INT16U DFT_GoertzelSngPntCfgFP32 (DFT_GOERTZEL_SNG_PNT_FP32 *p, const FP32 trg_frq, const FP32 min_smp_tm, const INT16U max_nbr_smp)
{
    FP32    omega;                                                                  /* Normalized angular frequency               */
    FP32    per;                                                                    /* Oscillation period [s]                     */

    if (DFT_GOERTZEL_MIN_NBR_SMP > ((INT16U)(1.0f / (trg_frq * min_smp_tm))))
    {
        //ERR_HDLR("DFT_GoertzelSngPntCfgFP32(): Number of samples below limit!",ERR_FAILURE);
    }

    p->TrgFrq     = trg_frq;                                                        /* Take over target frequency [Hz]            */

    per           = 1.0f / p->TrgFrq;                                               /* Calculate oscillation period [s]           */
    p->NbrSmp     = (INT16U)floor(per / min_smp_tm);                                /* Calculate unlimited number of samples      */
    p->DecFactor  = (INT16U)ceil(((FP32)p->NbrSmp) / ((FP32)max_nbr_smp));          /* Calculate decimation factor                */
    p->NbrSmp     = (INT16U)floor(((FP32)p->NbrSmp) / ((FP32)p->DecFactor));        /* Calculate limited number of samples        */
    p->SmpTm      = per / ((FP32)(p->NbrSmp));                                      /* Calculate sample time [s]                  */
    p->CycleTmISR = p->SmpTm / ((FP32)(p->DecFactor));                              /* Calculate ISR cycle time [s]               */

    p->SngFrqPnt  = (INT16U)(0.5f + (((FP32)p->NbrSmp) * p->TrgFrq * p->SmpTm));    /* Calculate DFT frequency index of interest  */
    omega         = (FP32)(MTH_2PI * ((FP32)p->SngFrqPnt)) / ((FP32)p->NbrSmp);     /* Calculate normalized angular frequency     */

    p->Coeff[1]   = (FP32)sin(omega);                                               /* Calculate second-order coefficients        */
    p->Coeff[2]   = (FP32)cos(omega);
    p->Coeff[0]   = 2.0f * p->Coeff[2];

    p->ScFactor   = ((FP32)p->NbrSmp) / 2.0f;                                       /* Calculate scaling factor                 */
    p->ScFac_1    = 2.0f / ((FP32)p->NbrSmp);
    p->ScKfrq     = (MTH_2PI * p->TrgFrq * p->SmpTm);

    return ERR_SUCCESS;
}   /* DFT_GoertzelSngPntCfgFP32()  */

/*
 *  32-bit precision Goertzel algorithm which computes a single point in the discrete Fourier transform
 */
void DFT_GoertzelSngPntFP32 (DFT_GOERTZEL_SNG_PNT_FP32 *p)
{
    volatile INT16S      i;                                                          /* Common run index                                   */
    volatile CMPLX_FP64  X;                                                          /* Complex DFT point value                            */
    volatile FP32  q[3];                                                       /* State variables for iterative feedback calculation */
    volatile FP32  in;                                                         /* Input value.                                       */

    q[0] = 0.0;                                                             /* Zeroize state variables                            */
    q[1] = 0.0;
    q[2] = 0.0;

    for(i = p->NbrSmp -1; 0 <= i; i--)                                      /* Iterative feedback calculation                     */
    {
        CBUF_RD(p->PtrCBuf, i, in);
        q[0] = (p->Coeff[0] * q[1]) - q[2] + in;
        q[2] = q[1];
        q[1] = q[0];
    }
    //CMPLX_REAL_SET(&X, (q[1] - q[2] * p->Coeff[2]) / p->ScFactor);          /*Feedforward complex DFT point value calculation     */
    //CMPLX_IMAG_SET(&X, (       q[2] * p->Coeff[1]) / p->ScFactor);
    CMPLX_REAL_SET(&X, (q[1] - q[2] * p->Coeff[2]) * p->ScFac_1);          /*Feedforward complex DFT point value calculation     */
    CMPLX_IMAG_SET(&X, (       q[2] * p->Coeff[1]) * p->ScFac_1);
    p->SigMag   = (FP32)Cmplx_Abs(X);                                       /* Calculate signal magnitude                         */

    //p->SigAngle = (FP32)(Cmplx_Angle(X) + (MTH_2PI * p->TrgFrq * p->SmpTm));/* Calculate signal angle [rad]                       */
    p->SigAngle = (FP32)(Cmplx_Angle(X) + p->ScKfrq);
} /* DFT_GoertzelSngPntFP32()   */

/*
************************************************************************************************************************************
************************************************************************************************************************************
*                                                         LOCAL FUNCTIONS
************************************************************************************************************************************
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                 EXTERNAL C LANGUAGE LINKAGE END
************************************************************************************************************************************
*/

#ifdef __cplusplus
}                                                               /* End of 'extern'al C language linkage.                          */
#endif
