/*!
\file	dsv.c
\brief	functions for data storing and filter

\author	Koester
\date	01.10.2012
\details
	changes:

*/


/* Include files ************************************************************/

#define	DSV_C
#include <math.h>    /* wegen Betragsfunktion fabs */
#include "Project.h"		// all global things as Type definitions, Hardware def., ...
#include "dsv.h"
#include "filt.h"


/* Macro definitions ********************************************************/
#define MAXMEM_AVG_FILTER	(Uint16)16384


#ifdef FLASH
#pragma CODE_SECTION( FOLPF, "ramfuncs");
#pragma CODE_SECTION( AvgIncMem, "ramfuncs");
#endif


/* Types ********************************************************************/

/* Functions ****************************************************************/


/*!
\fn		void InitCondEmphasisSignal(t_EmphSig *pEmph, float32 val, float32 offset)
\brief  Initialization for the Emphasis function

\author	K�ster
\date	12.11.2013
\param[in]
\return	void

\details Changes:
*/
void InitCondEmphasisSignal(t_EmphSig *pEmph, float32 val, float32 offset)
{
	// set Emphases step val
	pEmph->fStep = val;
	// meanwhile max. Emphasis is 100%
	pEmph->fMaxEmph = 1.0;
	// the actual
	pEmph->fEmph = 0.0;
	// an offset, that emphasis only the rise in the input-signal
	pEmph->fOffset = offset;
}


/*!
\fn		float32 EmphasisSignal(t_EmphSig *pEmph, float32 val)
\brief  The emphasis function emphases a signal for a fixed time which is 
		described in the fStep variable. The fEmph value represents at least
		the relation (in Percents) of the input signal.
		The function supresses glitches may be caused e.g. through Set-Value changes.

\author	K�ster
\date	12.11.2013
\param[in]
\return	void

\details Changes:
*/
float32 EmphasisSignal(t_EmphSig *pEmph, float32 val)
{
	// increment emphasis with stepwidth
	pEmph->fEmph = pEmph->fEmph + pEmph->fStep;
	// maximum (e.g. 100%) is reached?
	if(pEmph->fEmph >= pEmph->fMaxEmph)
	{
		// stay at maximum e.g. 1,000
		pEmph->fEmph = pEmph->fMaxEmph;
		//return (val * pEmph->fEmph);
		// return the actual value without change
		return (val);
	}
	else
	{
		// calculate emphasis value
		// val = offset + (val-offset)*emph
		return ( pEmph->fOffset + ( (val-pEmph->fOffset) * pEmph->fEmph) );
		// return only 1% of the actual value
		//return ( pEmph->fOffset + ( (val-pEmph->fOffset) * 0.01) );
	}
}



float32 FOLPF(t_FOLPF *ptr, float32 val)
{                                
   ptr->uk = val;
   ptr->yk = ptr->d0*(ptr->uk + ptr->uk_1) + ptr->c1*ptr->yk_1;
   
   ptr->yk_1 = ptr->yk;
   ptr->uk_1 = ptr->uk;
   
   return ptr->yk;
}

void InitCondFOLPF(t_FOLPF *ptr, float32 val)
{
   ptr->yk = ptr->yk_1 = ptr->uk = ptr->uk_1 = val;
   return;
}

void ParamFOLPF(t_FOLPF *ptr, float32 F, float32 T)
{
   ptr->Fp = F;
   ptr->Ts = T;
   
   ptr->d0 = ptr->Ts/(ptr->Ts + 2.0/ptr->Fp);
   ptr->c1 = -(ptr->Ts - 2.0/ptr->Fp)/(ptr->Ts + 2.0/ptr->Fp);
   return;
}


/* 
 * Initialisierung der Zustandsgroessen des Mittelwertbildners mit anwachsendem
 * Gedaechtnis
 */
void InitCondAvgIncMem(t_stcAvgIncMem *ps, float value)
{
	// aktuellen und alten Mittelwert mit dem vorgegebenen Startwert belegen
	ps->yk = ps->yk_1 = value;
	// Gedaechtnis zuruecksetzen
	ps->fK = 1.0;
	return;
}

/*
 * Vorgabe der maximal zulaessigen Gedaechtnisbreite des Mittelwertbildner mit 
 * anwachsendem Gedaechtnis
 */
Uint16 SetCondAvgIncMem(t_stcAvgIncMem *ps, Uint16 unMemSize)
{
	/*
	 * Bewirkt das vorgegebene Gedaechtnis eine Mittelwertbildung und ist der
	 * maximal zulaessige Zahlenbereich nicht ueberschritten ?
	 */
	if ((2.0 <= unMemSize) && (MAXMEM_AVG_FILTER >= unMemSize))
	{
		/*
		 *	Die maximale Gedaechtnisbreite uebernehmen und erfolgreiche Parameter-
		 * uebernahme anzeigen.
		 */
		ps->unMmax = unMemSize;
		// aktuellen und alten Mittelwert mit dem vorgegebenen Startwert belegen
		ps->yk = ps->yk_1 = 0.0;
		// Gedaechtnis zuruecksetzen
		ps->fK = 1.0;
		return OK;
	}
	else
	{
		/*
		 *	Die alte maximale Gedaechtnisbreite beibehalten und falsche Parameter-
		 *	vorgabe anzeigen.
		 */
		return NOK;
	}
}

/*
 * Mittelwertbildner mit anwachsendem Gedaechtnis
 */
float32 AvgIncMem(t_stcAvgIncMem *ps, float32 value)
{
	//	aktuellen Messwert einlesen
	ps->uk = value;

	//	neuen Mittelwert mittels rekursiver Summenform berechnen
	ps->yk = ps->yk_1 + (ps->uk - ps->yk_1) / ps->fK;

	//	aktuellen Mittelwert fuer die rekursive Berechnung als alten
	//	Mittelwert speichern
	ps->yk_1 = ps->yk;

	// bis 25 nur um 0.005 erh�hen, das ist k = 25 bei 1 sec
   if( ps->fK < 25.0 )
	{
		//	Gedaechtnis anwachsen lassen
		ps->fK = ps->fK + 0.05;
	}

   else if( ps->fK < (float)ps->unMmax )   //	max. zulaessiges Gedaechtnis noch nicht erreicht ?
	{
		//	Gedaechtnis anwachsen lassen
		ps->fK = ps->fK + 0.2;
	}
	else
	{
		//	Begrenzung auf das max. zulaessigen Gedaechtnis, dieses Verhalten ist
		//	mit einem exponetielle nachlassendem Gedaechtnis gleich zu setzen
		ps->fK = (float)ps->unMmax;
	}
   // aktuellen Mittelwert ausgeben
   return ps->yk;
}






