/*
************************************************************************************************************************************
*                                              READY-TO-USE EMBEDDED BUILDING BLOCKS
*
*                             (c) Copyright 2015-2016; Thermo Electron (Karlsruhe) GmbH; Karlsruhe-Germany
*
*       All rights reserved. Protected by international copyright laws. 
*
*       TMO source code is an unpublished work and the use of a copyright notice does not imply otherwise. This source code
*       contains confidential, trade secret material of TMO. Any attempt or participation in deciphering, decoding, reverse
*       engineering or in any way altering the source code is strictly prohibited, unless the prior written consent of TMO
*       is obtained.
*
************************************************************************************************************************************
*/

/**
************************************************************************************************************************************
*
*                                                         FILE DESCRIPTION
*
* @file         err.c
* @brief        Exception / Error Handler
* @details      This module contains functions to report and handle errors. By examining the status information returned by every
*               function of the ready-to-use embedded building blocks the user can determine whether it succeeded or failed, and if
*               it failed the user can find out what the precise cause of failure was.
* @version      V1.00.00
* @author       UwBa
*
************************************************************************************************************************************
* Notes         (1) The note(s) section should be omitted if no notes are given.
************************************************************************************************************************************
*/

/*
************************************************************************************************************************************
*                                                          INCLUDE FILES
************************************************************************************************************************************
*/

#define ERR_MODULE
#include "Project.h"											// all global things as Type definitions, Hardware def., ...
#include "err.h"


/*
************************************************************************************************************************************
*                                                   EXTERNAL C LANGUAGE LINKAGE
*
* Note(s) : (1) C++ compilers must 'extern'ally declare all C function prototypes & variable/object declarations for correct C
*               language linkage.
************************************************************************************************************************************
*/

#ifdef __cplusplus
extern "C" {                                                    /* See Note #1.                                                   */
#endif

/*
************************************************************************************************************************************
*                                                          LOCAL DEFINES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                         LOCAL CONSTANTS
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                         LOCAL DATA TYPES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                           LOCAL TABLES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                         LOCAL VARIABLES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                     LOCAL FUNCTION PROTOTYPES
************************************************************************************************************************************
*/

/** Divide real numbers
 *
 *  @param      x   Real numerator.
 *  @param      y   Real denominator.
 *  @returns        the real quotient.
 */
FP64 Err_Div(const FP64 x, const FP64 y);

/*
************************************************************************************************************************************
************************************************************************************************************************************
*                                                         GLOBAL FUNCTIONS
************************************************************************************************************************************
************************************************************************************************************************************
*/

/*
************************************************************************************************************************************
*                                                    ERROR & EXCEPTION HANDLING
************************************************************************************************************************************
*/

/*
 *  Error handler
 */
 #ifdef RAUS
INT32S Err_Hdlr (const CHAR *reason, const CHAR *file, INT32U line, INT32U no)
{
    ERR_HDLR_REC *p_rec;

    if (ERR_HDLR_QUEUE_SIZE_MAX <= QUEUE_SIZE(&Err_Hdlr_Queue))                 /* Maximum allowed error queue size exceeded?     */
    {
        Queue_Dequeue(&Err_Hdlr_Queue, &p_rec);                                 /* Remove oldest error record from queue.         */

        free(p_rec);                                                            /* Deallocate memory for oldest error record.     */
    }

    if (NULL == (p_rec = (ERR_HDLR_REC *)malloc(sizeof(ERR_HDLR_REC))))         /* Allocate memory for new error record.          */
    {
        return ERR_ENOMEM;                                                      /* Dynamic memory allocation failed.              */
    }
                                                                                /* Create new error record:                       */
    p_rec->reason = reason;                                                     /* Reason for the error.                          */
    p_rec->file   = file;                                                       /* File in which the error occure.                */
    p_rec->line   = line;                                                       /* Line number in file.                           */
    p_rec->no     = no;                                                         /* Error number.                                  */

    Queue_Enqueue(&Err_Hdlr_Queue, p_rec);                                      /* Append new error record to the queue.          */ 

    return ERR_SUCCESS;
} /* Err_Hdlr() */
#endif
/*
************************************************************************************************************************************
*                                                   INFINITIES AND NOT-A-NUMBER
************************************************************************************************************************************
*/

/*
 *  Positive infinity
 */
FP64 Err_PosInf (void)
{
    return Err_Div(+1.0, 0.0);
}   /* Err_PosInf() */

/*
 *  Negative infinity
 */
FP64 Err_NegInf (void)
{
    return Err_Div(-1.0, 0.0);
}   /* Err_NegInf() */

/*
 *  Not-A-Number
 */
FP64 Err_NaN (void)
{
    return Err_Div(0.0, 0.0);
}   /* Err_NaN()    */

/*
 *  Check if Not-a-Number
 */
INT32S Err_IsNaN (const FP64 x)
{
    volatile INT32S status;

    status = (x != x);

    return status;
}   /* Err_IsNaN()  */

/*
 *  Check if number is infinite
 */
INT32S Err_IsInf (const FP64 x)
{
    volatile FP64   y;
    volatile INT32S s;

    y = x - x;
    s = (y != y);

    if (s && (0 < x))
    {
        return +1;
    }
    else if (s && (0 > x))
    {
        return -1;
    }
    else
    {
        return 0;
    }
}   /* Err_IsInf()  */

/*
 *  Check if number is finite
 */
INT32S Err_Finite (const FP64 x)
{
    volatile INT32S status;
    volatile FP64   y;
    
    y = (x - x);
    
    status = (y == y);
    
    return status;
}   /* Err_Finite() */

/*
************************************************************************************************************************************
************************************************************************************************************************************
*                                                         LOCAL FUNCTIONS
************************************************************************************************************************************
************************************************************************************************************************************
*/

/*
 *  Divide real numbers
 */
FP64 Err_Div (const FP64 x, const FP64 y)
{
    return (x / y);
}   /* Err_Div()    */

/*
************************************************************************************************************************************
*                                                 EXTERNAL C LANGUAGE LINKAGE END
************************************************************************************************************************************
*/

#ifdef __cplusplus
}                                                                                   /* End of 'extern'al C language linkage.      */
#endif
