/**
 * @file    fft.c
 * @brief   Fast Fourier transformation module
 *
 *          The module contains C-functions for general purpose signal-processing in the frequency domain. These routines are
 *          typically used in computationally intensive real-time applications where optimal execution speed is critical. The
 *          module contains the following routines:
 *
 *              - Radix 2 forward FFT with complex input
 *              - Bit-reverse the FFT output
 *              - Generate the FFT coefficient table
 *
 * @par     Copyright:
 *
 *          Thermo Electron (Karlsruhe) GmbH, Dieselstrasse 4, 76227 Karlsruhe, Germany
 *
 *          (c) Copyright 2009 - 2010, Thermo Fisher Scientific
 *
 *          All rights reserved. Thermo Fisher Scientific source code is an unpublished work and the use of a copyright notice does
 *          not imply otherwise. This source code contains confidential, trade secret material of Thermo Fisher Scientific. Any
 *          attempt or participation in deciphering, decoding, reverse engineering or in any way altering the source code is
 *          strictly prohibited, unless the prior written consent of Thermo Fisher Scientific is obtained.
 *
 * @par     Creation Date:
 *
 *          05-Dec-2009
 *
 * @par     Last Modification:
 *
 *          18-Jul-2012
 *
 * @par     Project:
 *
 *          K1
 *
 * @par     Editor:
 *
 *          Microsoft Visual C++ 8.0
 *
 * @par     Compiler:
 *
 *          Microsoft Visual C++ 8.0
 *
 * @author  Uwe Bauer
 *
 * @par     Revision History:
 */

/*
 **********************************************************************************************************************************
 *                                                           INCLUDES                                                             *
 **********************************************************************************************************************************
 */

#include <math.h>                                                       /* Because of trigonometric functions                     */
#include "project.h"		// all global things as Type definitions, Hardware def., ...
#include "fft.h"
#include "mth.h"                                                        /* Because of minimum operation                           */

#ifdef FLASH
#pragma CODE_SECTION( FftGenTwfRx2, "ramfuncs");
#pragma CODE_SECTION( FftBitRev, "ramfuncs");
#pragma CODE_SECTION( FftCmplxRx2, "ramfuncs");
#pragma CODE_SECTION( FftInvCmplxRx2, "ramfuncs");
#pragma CODE_SECTION( FftReal2Cmplx, "ramfuncs");
#pragma CODE_SECTION( FftInvCmplxRx2Scale, "ramfuncs");
#pragma CODE_SECTION( FftRect2Pol, "ramfuncs");
//#pragma CODE_SECTION( FftGetMag, "ramfuncs");
//#pragma CODE_SECTION( FftGetAng, "ramfuncs");
#endif

/*
 **********************************************************************************************************************************
 *                                                        GLOBAL FUNCTIONS                                                        *
 **********************************************************************************************************************************
 */

/*
 * Radix 2 forward FFT with complex input
 */
void FftCmplxRx2 (INT32U n, FP32 *x, FP32 *w, INT32U nMin)          
{                                                                 
    INT32U n2;
    INT32U ie;
    INT32U ia;
    INT32U i;
    INT32U j;
    INT32U k;
    INT32U m;
    FP32   rtemp;
    FP32   itemp;
    FP32   c;
    FP32   s;
                                                      
    n2 = n;
    ie = 1;
                                                      
    for(k = n; k > nMin; k >>= 1)
    {
        n2 >>= 1;
        ia = 0;
        for(j=0; j < ie; j++)
        {
            for(i=0; i < n2; i++)
            {
                c = w[2*i];
                s = w[2*i+1];
                m = ia + n2;
                rtemp     = x[2*ia]   - x[2*m];
                x[2*ia]   = x[2*ia]   + x[2*m];
                itemp     = x[2*ia+1] - x[2*m+1];
                x[2*ia+1] = x[2*ia+1] + x[2*m+1];
                x[2*m]    = c*rtemp   - s*itemp;
                x[2*m+1]  = c*itemp   + s*rtemp;
                ia++;
            }
            ia += n2;
        }
        ie <<= 1;
        w = w + k;
    }
}   /* FftCmplxRx2()    */

/*
 * Inverse complex radix-2 decimation in frequency FFT
 */
void FftInvCmplxRx2(INT32U n, FP32 * x, FP32 * w, INT32U nMin)
{
    INT32U n2;
    INT32U ie;
    INT32U ia;
    INT32U i;
    INT32U j;
    INT32U k;
    INT32U m;
    FP32   rtemp;
    FP32   itemp;
    FP32   c;
    FP32   s;

    n2 = n;
    ie = 1;

    for(k = n; k > nMin; k >>= 1)
    {
        n2 >>= 1;
        ia = 0;
        for(j=0; j < ie; j++)
        {
            for(i=0; i < n2; i++)
            {
                c         = w[2*i];
                s         = w[2*i+1];
                m         = ia        + n2;
                rtemp     = x[2*ia]   - x[2*m];
                x[2*ia]   = x[2*ia]   + x[2*m];
                itemp     = x[2*ia+1] - x[2*m+1];
                x[2*ia+1] = x[2*ia+1] + x[2*m+1];
                x[2*m]    = c*rtemp   + s*itemp;
                x[2*m+1]  = c*itemp   - s*rtemp;
                ia++;
            }
            ia += n2;
        }
        ie <<= 1;
        w = w + k;
    }
}   /* FftInvCmplxRx2() */

/*
 * Generate the FFT coefficient table
 */
void FftGenTwfRx2 (FP32 *w, INT32U n)
{
    INT32U  i;                                                          /* Common indices                                         */
    INT32U  j;
    FP32    e  = cfPi*2.0f/n;                                             /* Auxilliary value                                       */

    for(j=1; j < n; j <<= 1)                                            /* Calculate twiddle factors                              */
    {
        for(i=0; i < ( n>>1 ); i += j)
        {
            *w++   = ((FP32)  cos(i*e));
            *w++   = ((FP32) -sin(i*e));
        }
    }
}   /* FftGenTwfRx2()   */

/*
 * Bit-reverse the FFT output
 */
void FftBitRev (FP32 *x, INT32U n)
{
    INT32U i;                                                           /* Common indices                                         */
    INT32U j;
    INT32U k;
    FP32   rtemp;                                                       /* Real part scratch value                                */
    FP32   itemp;                                                       /* Imaginary part scratch value                           */

    j = 0;
    for(i=1; i < (n-1); i++)                                            /* Sort FFT output sequence in normal frequency order     */
    {
        k = n >> 1;
        while(k <= j)
        {
            j -= k;
            k >>= 1;
        }
        j += k;
        if(i < j)
        {
            rtemp    = x[j*2];
            x[j*2]   = x[i*2];
            x[i*2]   = rtemp;
            itemp    = x[j*2+1];
            x[j*2+1] = x[i*2+1];
            x[i*2+1] = itemp;
        }
    }
}   /* FftBitRev()  */

/*
 * The follwoing C code is used to perform the final scaling of the IFFT
 */
void FftInvCmplxRx2Scale (FP32 *x, INT32U n) 
{ 
    INT32U i; 
    FP32   inv;
    
    inv = 1.0f / n; 

    for(i = 0; i < n; i++) 
    { 
        x[2*i]   = inv * x[2*i]; 
        x[2*i+1] = inv * x[2*i+1]; 
    } 
}   /* FftInvCmplxRx2Scale()    */

/*
 * Covert real input sequences into complex sequences
 */
void FftReal2Cmplx (FP32 *r, FP32 *c, INT32U n)
{
    INT32U k;                                                           /* Common index                                           */

    for (k=0; k<n; k++)                                                 /* Conversion real into complex sequence                  */
    {
        c[2*k]     = r[k];                                              /* Takeover real part                                     */
        c[2*k + 1] = 0.0;                                               /* Zeroize imaginary part                                 */
    }
}   /* FftReal2Cmplx()  */

/*
 * Converter between rectangular and polar coordinates
 */
void FftRect2Pol (FP32 *x, INT32U n, INT32U n1)
{
    INT32U k;                                                           /* Common index                                           */
    FP32   mag;                                                         /* Auxilliary magnitude value                             */
    FP32   ang;                                                         /* Auxilliary angle value                                 */

    n1 = MTH_MIN(n1, n/2 - 1);                                          /* Limitation of frequency components to be converted     */

    mag = x[0]/((FP32) n);                                              /* DC - value                                             */
    ang = ((FP32) 0.0);

    x[0] = mag;                                                         /* Store polar coordinates                                */
    x[1] = ang;

    for (k=1; k<=n1; k++)                                               /* Higher harmonics                                       */
    {
        mag = 2.0f*((FP32) sqrt(x[2*k]*x[2*k] + x[2*k+1]*x[2*k+1]))     /* Calculate magnitude                                    */
            /((FP32) n);   
        ang = ((FP32) atan2(x[2*k+1], x[2*k]));                         /* Calculate angle in radiant                             */

        x[2*k]     = mag;                                               /* Store polar coordinates                                */
        x[2*k + 1] = ang;
    }
}   /* FftRect2Pol()    */

/*
 * Get magnitude of a given harmonic
 */
FP32 FftGetMag (FP32 *x, INT32U n, INT32U prd)
{
    return x[2*n*prd];
}   /* FftGetMag()  */

/*
 * Get phase of a given harmonic
 */
FP32 FftGetAng (FP32 *x, INT32U n, INT32U prd)
{
    return x[2*n*prd + 1];
}   /* FftGetAng()  */
