/**
 * @file    filt.c
 * @brief   Digital filter functions.
 *
 * @par     Copyright:
 *
 *          Thermo Electron (Karlsruhe) GmbH, Dieselstrasse 4, 76227 Karlsruhe, Germany
 *
 *          (c) Copyright 2010, Thermo Fisher Scientific
 *
 *          All rights reserved. Thermo Fisher Scientific source code is an unpublished work and the use of a copyright notice does
 *          not imply otherwise. This source code contains confidential, trade secret material of Thermo Fisher Scientific. Any
 *          attempt or participation in deciphering, decoding, reverse engineering or in any way altering the source code is
 *          strictly prohibited, unless the prior written consent of Thermo Fisher Scientific is obtained.
 *
 * @par     Creation Date:
 *
 *          15-Feb-2010
 *
 * @par     Last Modification:
 *
 *          06-Apr-2010
 *
 * @par     Project:
 *
 *          MARS III
 *
 * @par     Editor:
 *
 *          Microsoft Visual C++ 6.0
 *
 * @par     Compiler:
 *
 *          Microsoft Visual C++ 6.0
 *
 * @author  Uwe Bauer
 *
 * @par     Revision History:
 */

/*
 **********************************************************************************************************************************
 *                                                           INCLUDES                                                             *
 **********************************************************************************************************************************
 */
#include <math.h>
#include "Project.h"		// all global things as Type definitions, Hardware def., ...
#include "mth.h"
#include "filt.h"
#include "angle.h"
#include "Measmode.h"


/*
 **********************************************************************************************************************************
 *                                                        GLOBAL FUNCTIONS                                                        *
 **********************************************************************************************************************************
 */

/*!
\fn		void InitSigMavg(t_stcFiltMavg *p_stcSig)
\brief  calculate mavg filter value from Torque
\author	K�ster
\date	20.01.2016
\param	actual Torque
\return	void
\detail Changes:
*/
void InitSigMavg(t_stcFiltMavg *p_stcSig)
{
    // calculate PhiAbs Index for data array Mavg-Filter
    p_stcSig->unIdx = 0;
    p_stcSig->unIdxOld = 0;
    p_stcSig->unLen = SIG_MAVG_LENGTH;
    // data buffer not yet FULL
    p_stcSig->unState = FALSE;
    // allocate memory for the Mavg size
    p_stcSig->pfDatBuf = (float *)calloc(SIG_MAVG_LENGTH, sizeof(float));
    //p_stcSig->pfDatBuf = p_stcSig->afDatBuf;
    // set the Mavg-Filter object
    FiltMAvgInit(&p_stcSig->objSigMavg, p_stcSig->pfDatBuf, p_stcSig->unLen);
    // calculate PhiAbs Index for data array Mavg-Filter for the actual start value
    p_stcSig->unIdx = (Uint16) (floor((SIG_MAVG_LENGTH / cfPi_2)*(*PtrAngleAbs() - (floor(*PtrAngleAbs()/cfPi_2)*cfPi_2))) );
}

/*!
\fn		void InitCondSigMavg(t_stcFiltMavg *p_stcSig)
\brief  preset filter after change in measu. mode to avoid spikes in signal
\author	K�ster
\date	20.01.2016
\param	actual Torque
\return	void
\detail Changes:
*/
void InitCondSigMavg(t_stcFiltMavg *p_stcSig)
{
    // calculate PhiAbs Index for data array Mavg-Filter
    p_stcSig->unIdx = 0;
    p_stcSig->unIdxOld = 0;
    p_stcSig->unLen = SIG_MAVG_LENGTH;
    // data buffer not yet FULL
    p_stcSig->unState = FALSE;
    p_stcSig->objSigMavg.fFull = 0;
    p_stcSig->objSigMavg.uK =0;
    p_stcSig->objSigMavg.fSum = 0.0;
    p_stcSig->objSigMavg.pfData = p_stcSig->pfDatBuf;
    // calculate PhiAbs Index for data array Mavg-Filter for the actual start value
    p_stcSig->unIdx = (Uint16) (floor((SIG_MAVG_LENGTH / cfPi_2)*(*PtrAngleAbs() - (floor(*PtrAngleAbs()/cfPi_2)*cfPi_2))) );
}



void ResetSigMavg(t_stcFiltMavg *p_stcSig)
{
    // calculate PhiAbs Index for data array Mavg-Filter
    p_stcSig->unIdx = 0;
    // data buffer not yet FULL
    p_stcSig->unState = FALSE;
    // free memory for the Mavg size
    free(p_stcSig->pfDatBuf);
}


/*!
\fn		void SignalCalcMavg(t_stcTrqMavg *p_stcSig, float32 p_fActTrq, float32 p_fPhiAbs)
\brief  calculate mavg filter value from an input signal for 1/4 turn (= 2PI/4)
\author	K�ster
\date	20.01.2016
\param	
\return	void
\detail Changes:
*/
float32 SignalCalcMavg(t_stcFiltMavg *p_stcSig, float32 p_fActSig, float32 p_fPhiAbs)
{
    float32 fOut;
    // if omega is more then 300 rpm, take every value, because speed is to fast
    if( *PtrMeasSetvalue() > 31.41593 )
    {
        // calculate Filter
        fOut = FilterMAvg(&p_stcSig->objSigMavg, p_fActSig);
    }
    else
    {
        // calculate PhiAbs Index for data array Mavg-Filter
        p_stcSig->unIdx = (Uint16) (floor((SIG_MAVG_LENGTH / cfPi_2)*(p_fPhiAbs - (floor(p_fPhiAbs/cfPi_2)*cfPi_2))) );
        // filter only when new index
        if(p_stcSig->unIdxOld != p_stcSig->unIdx)
        {
            // calculate Filter
            fOut = FilterMAvg(&p_stcSig->objSigMavg, p_fActSig);
            p_stcSig->unIdxOld = p_stcSig->unIdx;
        }
        else
        {
            // no new calcul., then old value
            fOut = p_stcSig->objSigMavg.fY;
        }
    }
    // Buffer full, then state = 1
    p_stcSig->unState = p_stcSig->objSigMavg.fFull;
    return fOut;
}


/*!
\fn	float FilterMAvg (objMavg *p, const float32 p_fData)
\brief Moving-Average-Filter with up to 2^6 values.
       In this algorythm every call the sum is calculated over the
       complete array
			
\author	R. Koester
\date	    26.01.2016
\param[in]
\return	void

\details Changes:

*/  
float32 FilterMAvg (objMavg *p, const float32 p_fData)
{
    int16 i;
    // check if memory is complete and mark as FULL
    if (p->uK >= p->uN)
    {
        p->uK    = 0;
        p->fFull = FILTER_FULL;
    }
    // store actual value to next position
    *(p->pfData + p->uK) = p_fData;
    // index to next position
    p->uK++;
    
    // delete old sum value
    p->fSum = 0.0;

    if (p->fFull == FILTER_FULL)
    {
        // calculate sum over the complete memory
        for(i = p->uN -1; 0 <= i; i--)
        {
            p->fSum += *(p->pfData + i);
        }
        p->fY = p->fSum/p->uN;
    }
    else
    {
        // calculate sum over the actual memory size
        for(i = p->uK -1; 0 <= i; i--)
        {
            p->fSum += *(p->pfData + i);
        }
        p->fY = p->fSum/p->uK;
    }

    return p->fY;
}   /* FilterMAvg()   */



/*!
\fn	float FiltMAvg (objMavg *p, const float32 p_fData)
\brief Moving-Average-Filter mit bis zu 2^16 Werten
			
\author	U. Bauer
\date	    30.03.2015
\param[in]
\return	void

\details Changes:
            13.01.15 names to HAAKE style-guide
*/  
float32 FiltMAvg (objMavg *p, const float32 p_fData)
{
    if (p->uK >= p->uN)
    {
        p->uK    = 0;
        p->fFull = ((Uint16) 1);
    }

    p->fSum -= *(p->pfData + p->uK);
    p->fSum += p_fData;
    *(p->pfData + p->uK) = p_fData;
    p->uK++;

    if (p->fFull)
    {
        p->fY = p->fSum/p->uN;
    }
    else
    {
        p->fY = p->fSum/p->uK;
    }
   
    return p->fY;
}   /* FiltMAVG()   */





/*!
\fn	void FiltMAvgInit (objMavg *p, float32 *data, const Uint32 n)
\brief Init function for Moving-Average-Filter
			
\author	U. Bauer
\date	    30.03.2015
\param[in]
\return	void

\details Changes:
            13.01.15 names to HAAKE style-guide
*/  

void FiltMAvgInit (objMavg *p, float32 *data, const Uint16 n)
{
    // set Buffer pointer to data array
    p->pfData = data;
    // store number of values (size of dataarray)
    p->uN    = n;
    // reset data array to 0.0
    for (p->uK = 0; p->uK < p->uN; p->uK++)
    {
        *(p->pfData + p->uK) = ((float32) 0.0);
    }
    // delete actual counter, sum and Full semaphore
    p->uK    = ((Uint32) 0);
    p->fSum  = ((float32) 0.0);
    p->fFull = ((Uint16) 0);
}




/*!
\fn	float FilterLinReg (objMavg *p, const float32 p_fData)
\brief Linear Regression with up to 2^16 values.
       In this algorythm every call the sum is calculated over the
       complete array
			
\author	R. Koester
\date	    27.01.2016
\param[in]
\return	void

\details Changes:

*/

     
float32 FilterLinReg (objMavg *p, float32 p_fDataX, float32 p_fDataY)
{
    volatile Uint16 i, j, k;
    volatile float32 fSumX, fSumY, fSumXx, fSumXy;
    fSumX = fSumY = fSumXy = fSumXx = 0.0;
    // check if memory is complete and mark as FULL
    if (p->uK >= p->uN)
    {
        p->uK    = 0;
        p->fFull = FILTER_FULL;
    }
    // store actual value to next position
    *(p->pfData + p->uK) = p_fDataY;
    // index to next position
    p->uK++;


    // regression length at start is shorter
    if (p->fFull == FILTER_FULL)
    {
        j = p->uN;
        // filter full, start at uK value
        k = p->uK;
    }
    else
    {
        j = p->uK;
        // filter not full, start at 0 value
        k = 0;
    }

    // calculate REGRESSION sums over the actual memory
    if( j>1)
    {
        for(i = 0; i <= (j-1); i++)
        {
            if(k >= p->uN)
                k=0;
            // sum of all y values
            // 23.02.16 bug ??? fSumY += *(p->pfData + i);
            fSumY += *(p->pfData + k);
            // sum of all x values
            fSumX += (i) * p_fDataX;
            // sum of all y * x values
            fSumXy += *(p->pfData + k) * ((i) * p_fDataX);
            // sum of all x� values
            fSumXx += ((i) * p_fDataX) * ((i) * p_fDataX);
            k++;
        }
        // Berechnen der Geradengleichung y = a + bx, zun�chst wird nur b ben�tigt
        p->fY = (fSumXy-((fSumX*fSumY)/(float)j)) / (fSumXx-((fSumX*fSumX)/(float)j));
    }
    else
    {
        p->fY = 0.0;
    }
    return p->fY;
    //return fSumX;
}



/*!
\fn	void FiltLinRegInit(objMavg *p, float32 *data, const Uint16 n)
\brief Initilisation of Linear Regression with up to 2^16 values.
       			
\author	R. Koester
\date	    27.01.2016
\param[in]  *p - pointer to filter object
            data - adress of data buffer
            n - length of data buffer
\return	void

\details Changes:

*/ 
void FiltLinRegInit(objMavg *p, float32 *data, const Uint16 n)
{
    // set Buffer pointer to data array
    p->pfData = data;
    // store number of values (size of dataarray)
    p->uN    = n;
    // reset data array to 0.0
    for (p->uK = 0; p->uK < p->uN; p->uK++)
    {
        *(p->pfData + p->uK) = ((float32) 0.0);
    }
    // delete actual counter, sum and Full semaphore
    p->uK    = ((Uint32) 0);
    //p->fSum  = ((float32) 0.0);
    p->fFull = ((Uint16) 0);
}
