/**
 * @file    fosc.c
 * @brief   Fast Oscillation module
 *
 * @par     Copyright:
 *
 *          Thermo Electron (Karlsruhe) GmbH, Dieselstrasse 4, 76227 Karlsruhe, Germany
 *
 *          (c) Copyright 2009 - 2010, Thermo Fisher Scientific
 *
 *          All rights reserved. Thermo Fisher Scientific source code is an unpublished work and the use of a copyright notice does
 *          not imply otherwise. This source code contains confidential, trade secret material of Thermo Fisher Scientific. Any
 *          attempt or participation in deciphering, decoding, reverse engineering or in any way altering the source code is
 *          strictly prohibited, unless the prior written consent of Thermo Fisher Scientific is obtained.
 *
 * @par     Creation Date:
 *
 *          05-Dec-2009
 *
 * @par     Last Modification:
 *
 *          01-Apr-2011
 *
 * @par     Project:
 *
 *          MARS III
 *
 * @par     Editor:
 *
 *          Microsoft Visual C++ 6.0
 *
 * @par     Compiler:
 *
 *          Microsoft Visual C++ 6.0
 *
 * @author  Uwe Bauer
 *
 * @par     Revision History:
 *
 *          Automatic angular drift compensation based on simple P-controller algorithm, FFT DC offset information and rough
 *          viscosity estimation implemented.
 *
 *	 @par   Changes:
 *          09.05.14   FoscSetParam(), init. of OptFastPropInit() always, although during f-sweeps
 *			23.05.14   min. torque amplitude from 5e-06 to 20e-06 Nm
 *          19.01.16   reset gain in FoscDevObj, erste Schwingung war noch mit alter Amplitude
 *          16.02.16   in Sweep-Modes init OptFastProp only at the first Step FoscSetParam()
 *          26.04.16   VTiQ Version mit TMC, VTiQ-Air war ohne
 *          06.07.16   VTiQ Version, Achtung bei 50 kHz min. Abtastzeit min. 300�s!!!
 */

/*
 **********************************************************************************************************************************
 *                                                           INCLUDES                                                             *
 **********************************************************************************************************************************
 */

#include <math.h>
#define FOSC_MODULE
#include "Project.h"		// all global things as Type definitions, Hardware def., ...
#include "Uc1dsp.h"
#include "fosc.h"
#include "mth.h"
#include "CsFastOsc.h"
#include "Device.h"
#include "correction.h"
#include "angle.h"
#include "Pmsm.h"
#include "msc.h"
#include "cbuf.h"
#include "dft.h"
#include "sig.h"

FOSC_EXT FOSC_DEV  	Fosc_DevObj;                                 /**< Fast oscillation device object.                              */

float32      *pafStAmpFiltTbl;	    /**< First harmonic strain amplitude filter data table.           */
float32      *pafDeltaFiltTbl;	    /**< First harmonic delta angle filter data table.                */
float32      *pafTrqAmpFiltTbl;	    /**< First harmonic torque amplitude filter data table.           */

#ifdef FLASH
#pragma CODE_SECTION(FoscCyclic, "ramfuncs");
#pragma CODE_SECTION(FoscCyclicCalc, "ramfuncs");
#endif


/*
 **********************************************************************************************************************************
 *                                                   LOCAL FUNCTION PROTOTYPES                                                    *
 **********************************************************************************************************************************
 */


/*
 **********************************************************************************************************************************
 *                                                   GLOBAL INTERFACE FUNCTIONS                                                   *
 **********************************************************************************************************************************
 */

void PtrFoscTrqAmp(FP32 val)
{
    Src_SigGenSetAmpFP32(&Fosc_DevObj.sSigGen, val);
}

void PtrFoscStrainAmp(FP32 val)
{
    Src_SigGenSetAmpFP32(&Fosc_DevObj.sSigGen, val);
}

// Speicher freigeben
void FoscStop(void)
{
    /* Begin UwBa   */
    CBuf_DestroyFP32(&Fosc_DevObj.sTrqCBuf  );
    CBuf_DestroyFP32(&Fosc_DevObj.sAngleCBuf);
    /* End UwBa     */

    free(pafStAmpFiltTbl);
    free(pafDeltaFiltTbl);
    free(pafTrqAmpFiltTbl);
}

/*
 * Interface function for configuration of the fast oscillation device object.
 */
void FoscCfg (void)
{
	if( *PtrFoscSweepMode() == 0)
	{
        /* Begin UwBa   */
        CBuf_CfgFP32(&Fosc_DevObj.sTrqCBuf  , FOSC_MAX_NBR_SMP);
        CBuf_CfgFP32(&Fosc_DevObj.sAngleCBuf, FOSC_MAX_NBR_SMP);

        DFT_GoertzelSngPntSetupFP32(&Fosc_DevObj.sTrqDFT, &Fosc_DevObj.sTrqCBuf  );
        DFT_GoertzelSngPntSetupFP32(&Fosc_DevObj.sAngleDFT, &Fosc_DevObj.sAngleCBuf);
        /* End UwBa     */

        // 19.01.16 reset gain
        Fosc_DevObj.gain = 1.0;

		pafStAmpFiltTbl  = (float32 *)calloc(FOSC_MAX_NBR_SMP, sizeof(float32));
		pafDeltaFiltTbl  = (float32 *)calloc(FOSC_MAX_NBR_SMP, sizeof(float32));
		pafTrqAmpFiltTbl = (float32 *)calloc(FOSC_MAX_NBR_SMP, sizeof(float32));
	}

    /*
     * Define minimum torque amplitude of the used rheometer device.
     */
    Fosc_DevObj.minTrqAmp = FOSC_DEV_MIN_TRQ_AMP;
}

/*
 * Interface function to set the parameters for the measurement and overtone analysis.
 */
Uint16 FoscSet (float32 frq, Uint32 unHar, float32 fAmpl, Uint16 usMeasMode, float32 fSetStrain, float32 maxTrq)
{
    return FoscSetParam(&Fosc_DevObj, frq, unHar, fAmpl, usMeasMode, fSetStrain, maxTrq);
}

/*
 * Fast oscillation cyclic function.
 */
float32 FoscCyclic (float32 strain, float32 *pTrqTbl, float32 *pStTbl, float32 *pDeltaTbl)
{
    return FoscCyclicCalc(&Fosc_DevObj, strain, pTrqTbl, pStTbl, pDeltaTbl);
}

/*
 * Read the ISR cycle time.
 */
float32 FoscGetIsrCycleTime (void)
{
    return Fosc_DevObj.isrCycleTime;
}   /* FoscGetIsrCycleTime()    */

/*
 * Read the FFT table length.
 */
Uint32 FoscGetTblLen (void)
{
    return DFT_GOERTZEL_GET_NBR_SMP(&Fosc_DevObj.sTrqDFT);
}   /* FoscGetTblLen()  */

/*
 * The function returns the status of the overtone analysis.
 */
Uint16 FoscIsRdy (void)
{
    return (CBufIsFull(&Fosc_DevObj.sTrqCBuf) && CBufIsFull(&Fosc_DevObj.sAngleCBuf));
    // return 1;
}   /* FoscIsRdy()  */

/*
 **********************************************************************************************************************************
 *                                                        GLOBAL FUNCTIONS                                                        *
 **********************************************************************************************************************************
 */
//unsigned long ulTestCnt2 = 0;
//unsigned long ulTestCnt = 0;
//float32 afTest[900];

/*
 * Set the parameters for the torque signal generator and overtone analysis.
 */
Uint16 FoscSetParam (FOSC_DEV *pDev, float32 frq, Uint32 unHar, float32 fAmpl, Uint16 usMeasMode, float32 fSetStrain, float32 maxTrq)
{
    // frequency 0.1...100Hz
	// number of harmonics 1....
	// Measmode CSOSC (1) or CDOSC (2)
	// 
    if( (FOSC_MIN_OSC_FRQ > frq) || (FOSC_MAX_OSC_FRQ < frq) || (((Uint32) 0) >= unHar) || (((Uint16) 2) < usMeasMode) 
    	|| ((((Uint16) 2) == usMeasMode) && (((float32) 0.0) >= fSetStrain)) )
    {
        return ((Uint16) 1);
    }
    else                                                            /* Takeover parameter set:                                    */
    {
        pDev->frq        = frq;                                     /* Oscillation frequency.                                     */

        /* @todo:   Limit overtone analysis to 1st harmonic.    */

        pDev->nHarmonics = unHar;                                   /* Number of harmonics for overtone analysis.                 */

        /* Begin UwBa   */
        DFT_GoertzelSngPntCfgFP32(&pDev->sTrqDFT, 
                                   frq, 
                                   FOSC_MIN_SMP_TM,
                                   FOSC_MAX_NBR_SMP);
        
        DFT_GoertzelSngPntCfgFP32(&pDev->sAngleDFT, 
                                   frq, 
                                   FOSC_MIN_SMP_TM,
                                   FOSC_MAX_NBR_SMP);

        pDev->isrCycleTime = 
            DFT_GOERTZEL_GET_ISR_CYCLE_TM(&pDev->sTrqDFT);          /* Take over ISR cycle time from DFT configuration.           */

        /* @todo:   Initialization tsg module with ISR cycle time as time increment */

        pDev->PassCtr  = ((INT32U) 1);                              /* Initialize counter so that first value is recorded.        */
        pDev->PassCnts =                                            /* Take over decimation factor.                               */ 
            DFT_GOERTZEL_GET_DEC_FACTOR(&pDev->sTrqDFT);

        pDev->fsoPassCtr = 
            DFT_GOERTZEL_GET_DEC_FACTOR(&pDev->sTrqDFT) *           /* Initialize counter so that the first optimization starts...*/
            DFT_GOERTZEL_GET_NBR_SMP   (&pDev->sTrqDFT);            /* after the first full oscillation wave.                     */

        pDev->fsoPassCnts = (INT32U)(pDev->fsoPassCtr * (1.0/FOSC_OPT_UPD_CY));      /* 4 optimizations per oscillation period.                    */

        CBuf_InitFP32(&pDev->sTrqCBuf  , 0.0f);                     /* Zeroize torque buffer.                                     */
        CBuf_InitFP32(&pDev->sAngleCBuf, 0.0f);                     /* Zeroize deformation angle buffer.                          */

                                                                    /* Configurate signal generator depending on operating mode.  */
        if (CSOSC == usMeasMode)                                    /* CS-OSC mode.                                               */
        {
            Src_SigGenParamFP32(&pDev->sSigGen,
                                SRC_WF_COSINE,
                                fAmpl,
                                pDev->frq,
                                SRC_UNITS_HZ,
                                pDev->isrCycleTime);
        }
        else                                                        /* CD-OSC mode.                                               */
        {
            Src_SigGenParamFP32(&pDev->sSigGen,
                                SRC_WF_COSINE,
                                pDev->minTrqAmp,
                                pDev->frq,
                                SRC_UNITS_HZ,
                                pDev->isrCycleTime);
        }

        Src_SigGenResetFP32(&pDev->sSigGen);

        pDev->trq = ((float32) 0.0);                                /* Reset torque output.                                       */

		// ########## Configure fast strain propagation: ##########
		
		// CS or CD mode? Give the information to the Object
		pDev->enFso = usMeasMode;

		// Take over set deformation
        pDev->fsoSetVal = fSetStrain;

        // 16.02.16 TESTWEISE nur 1x
//        if( *PtrFoscSweepMode() == 0)
//	    {    
		    /* Initialize fast strain propagation with start values */
	        OptFastPropInit(&pDev->optObj);
		    /* Set maximum torque for fast strain propagation.            */
            OptFastPropSetMaxWeight(&pDev->optObj,maxTrq/FOSC_DEV_MIN_TRQ_AMP);
//        }
        pDev->trqDftCorr = ((float32) 0.0);

		// ########## Initialize moving average filters for first harmonics: ##########

		// Strain amplitude filter
        FiltMAvgInit(&pDev->sFiltStAmp, pafStAmpFiltTbl, pDev->sTrqDFT.NbrSmp);
		// Delta angle filter
        FiltMAvgInit(&pDev->sFiltDelta, pafDeltaFiltTbl, pDev->sTrqDFT.NbrSmp);
		// Torque amplitude filter
        FiltMAvgInit(&pDev->sFiltTrqAmp, pafTrqAmpFiltTbl, pDev->sTrqDFT.NbrSmp);

        return ((Uint16) 0);
    }
}   /* FoscSetParam()   */

/*
 * Fast oscillation cyclic function.
 */
float32 FoscCyclicCalc (FOSC_DEV *p, float32 strain, float32 *pTrqTbl, float32 *pStTbl, float32 *pDeltaTbl)
{
    /* Begin UwBa   */
    volatile FP32   delta;                                              /* Auxillary value for delta phase angle calculation.     */
    volatile float32 fHelp;

	/*
	 *******************************************************************************************************************************
	 * Save torque and deformation angle time course values and calculate new torque
     *******************************************************************************************************************************
	 */

    // 14.11.16, ab hier (1) bis (3) 1 Hz 250 Punkte 190�s
    // 14.11.16, ab hier (1) bis (2) 1 Hz 250 Punkte 1�s (8�s wenn von FOC unterbrochen)
    p->PassCtr--;

    if (0 == p->PassCtr)                                                /* Decimation of torque and deformation angle time course.*/
    {
        // 320 ns
        CBuf_WrtFP32(&p->sTrqCBuf  , p->trq);                           /* Buffer current torque.                                 */
        CBuf_WrtFP32(&p->sAngleCBuf, strain);                           /* Buffer current angle.                                  */
        p->PassCtr = p->PassCnts;                                       /* Re-load pass counter with time decimation factor.      */
    }
    p->trq = Src_SigGenFP32(&p->sSigGen);                                  /* Calculate new torque with highest time resolution.     */
	/*
	 *******************************************************************************************************************************
	 * CD amplitude control
     *******************************************************************************************************************************
	 */
    if (p->enFso == CDOSC)
    {
        p->fsoPassCtr--;

        if (0 == p->fsoPassCtr)
        {
            p->gain = OptFastProp(&p->optObj, p->fsoSetVal, DFT_GOERTZEL_GET_MAG(&p->sAngleDFT));
            p->fsoPassCtr = p->fsoPassCnts;
        }
        p->trq *= p->gain;
    }

	fHelp = p->trq;

  // 14.11.16, (2) Zeitmessung bis hier

	// active Tmc-Correction
	if( *PtrMscCorrMode() == 1 )
	{
		fHelp = fHelp + MscCorr(&tMscData, *PtrAngleAbs(), fHelp);
	}

	// ***************** Output to motor control ********************
	*PtrFocSetCurrent() = fHelp *  ( *PtrCal1Motor() );

	/*
	 *******************************************************************************************************************************
	 * Calculate discrete Fourier Transform
     *******************************************************************************************************************************
	 */

    // 250 Punkte 88 �s, 200 Punkte 68 �s
    GpioDataRegs.GPASET.bit.GPIO31 = 1;
    DFT_GoertzelSngPntFP32(&p->sTrqDFT);
    GpioDataRegs.GPACLEAR.bit.GPIO31 = 1;
    // 250 Punkte 88�s, 200 Punkte 68 �s
    DFT_GoertzelSngPntFP32(&p->sAngleDFT);                              /* Calculate angle DFT.                                   */

    // 14.11.16, ab hier bis Ende 2,4�s (10�s wenn von FOC unterbrochen)


    fHelp = DFT_GOERTZEL_GET_MAG  (&p->sTrqDFT  );
    *(pTrqTbl    ) = FiltMAvg(&p->sFiltTrqAmp,fHelp);
    //*(pTrqTbl    ) = DFT_GOERTZEL_GET_MAG  (&p->sTrqDFT  );             /* Save current torque magnitude.                         */

    *(pTrqTbl + 1) = DFT_GOERTZEL_GET_ANGLE(&p->sTrqDFT  );             /* Save current torque phase angle [rad].                 */

    fHelp = DFT_GOERTZEL_GET_MAG  (&p->sAngleDFT);
    *(pStTbl     ) = FiltMAvg(&p->sFiltStAmp,fHelp);
    //*(pStTbl     ) = DFT_GOERTZEL_GET_MAG  (&p->sAngleDFT);             /* Save current deformation magnitude.                    */
    *(pStTbl  + 1) = DFT_GOERTZEL_GET_ANGLE(&p->sAngleDFT);             /* Save current deformation phase angle [rad].            */

    delta = DFT_GOERTZEL_GET_ANGLE(&p->sTrqDFT  ) -                     /* Calculate delta phase angle [rad].                     */
            DFT_GOERTZEL_GET_ANGLE(&p->sAngleDFT);
    if (0.0f > delta)                                                   /* Phase jump detected?                                   */
    {
        delta += MTH_2PI;                                               /* Unwrap delta phase angle [rad].                        */
    }
    delta = FiltMAvg(&p->sFiltDelta, delta);

    *pDeltaTbl = delta;                                                 /* Save current delta phase angle [rad].                  */
    // 
	  // 14.11.16, (3) Zeitmessung bis hier

    return p->trq;                                                      /* Output current torque.                                 */
    /* End UwBa     */
}   /* FoscCyclicCalc() */

/*
 **********************************************************************************************************************************
 *                                                        LOCAL FUNCTIONS                                                         *
 **********************************************************************************************************************************
 */
