/*
************************************************************************************************************************************
*                                              READY-TO-USE EMBEDDED BUILDING BLOCKS
*
*                             (c) Copyright 2015-2016; Thermo Electron (Karlsruhe) GmbH; Karlsruhe-Germany
*
*       All rights reserved. Protected by international copyright laws. 
*
*       TMO source code is an unpublished work and the use of a copyright notice does not imply otherwise. This source code
*       contains confidential, trade secret material of TMO. Any attempt or participation in deciphering, decoding, reverse
*       engineering or in any way altering the source code is strictly prohibited, unless the prior written consent of TMO
*       is obtained.
*
************************************************************************************************************************************
*/

/**
************************************************************************************************************************************
*
*                                                         FILE DESCRIPTION
*
* @file         list.c
* @brief        Abstract Data Type Linked List
* @details      The module provides basic abstract data types and functions for the implementation of linked lists. Linked lists
*               consist of a series of elements which are arranged in a specific order and linked together.  They are suitable for
*               managing big data volumes and realization of stacks, queues and error/exception handlers. The linked lists in this
*               module have very efficient insert and delete operations. Because of the fact that the application and associated
*               specific data size is not known the methods in this basic module uses dynamic memory allocation features.
* @version      V1.00.00
* @author       UwBa
*/

/*
************************************************************************************************************************************
*                                                          INCLUDE FILES
************************************************************************************************************************************
*/

#include "project.h"
#include "list.h"
#include "err.h"                                                    /* Because of error numbers.                                  */

/*
************************************************************************************************************************************
*                                                   EXTERNAL C LANGUAGE LINKAGE
*
* Note(s) : (1) C++ compilers must 'extern'ally declare all C function prototypes & variable/object declarations for correct C
*               language linkage.
************************************************************************************************************************************
*/

#ifdef __cplusplus
extern "C" {                                                        /* See Note #1.                                               */
#endif

/*
************************************************************************************************************************************
*                                                          LOCAL DEFINES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                         LOCAL CONSTANTS
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                         LOCAL DATA TYPES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                           LOCAL TABLES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                         LOCAL VARIABLES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                     LOCAL FUNCTION PROTOTYPES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
************************************************************************************************************************************
*                                                         GLOBAL FUNCTIONS
************************************************************************************************************************************
************************************************************************************************************************************
*/

/*
 *  Initialization of a linked list
 */
void List_Init (LIST *p_list, void (*fnct)(void *data))
{
    p_list->Size    = 0;
    p_list->Destroy = fnct;
    p_list->HeadPtr = NULL;
    p_list->TailPtr = NULL;
}   /* List_Init()  */

/*
 *  Memory deallocation of a linked list
 */
void List_Destroy (LIST *p_list)
{
    void *p_data;

    while (0 < LIST_SIZE(p_list))                                       /* Delete all elements in the linked list.                */
    {
        if ((0    == List_RemNext(p_list, NULL, (void **)&p_data)) && 
            (NULL != p_list->Destroy))
        {
            p_list->Destroy(p_data);                                    /* Call destroy function for data memory deallocation.    */
        }
    }

    memset(p_list, 0, sizeof(LIST));                                    /* Delete the complete content of the linked list.        */
}   /* List_Destroy()   */

/*
 *  Insert an application-specific data element in a linked list
 */
INT32U List_InsNext (LIST *p_list, LIST_ELMT *p_elmt, const void *p_data)
{
    LIST_ELMT   *new_elmt;

    if (NULL == (new_elmt = (LIST_ELMT *)malloc(sizeof(LIST_ELMT))))    /* Allocate memory for one new element.                   */
    {
        return ERR_ENOMEM;                                              /* Dynamic memory allocation failed.                      */
    }
                                                                        /* Insert new element in the list:                        */
    new_elmt->DataPtr = (void *)p_data;

    if (NULL == p_elmt)
    {                                                                   /* Insert at the beginning (head) of the list.            */
        if (0 == LIST_SIZE(p_list))
        {
            p_list->TailPtr = new_elmt;
        }

        new_elmt->NextElmtPtr = p_list->HeadPtr;
        p_list->HeadPtr       = new_elmt;
    }
    else
    {                                                                   /* Inserting at a predetermined position in the list.     */
        if (NULL == p_elmt->NextElmtPtr)
        {
            p_list->TailPtr = new_elmt;
        }

        new_elmt->NextElmtPtr = p_elmt->NextElmtPtr;
        p_elmt->NextElmtPtr   = new_elmt;
    }

    p_list->Size++;                                                     /* Correct size of the list to the just inserted element. */

    return ERR_SUCCESS;
}   /* List_InsNext()   */

/*
 *  Remove an application-specific data element in a linked list
 */
INT32U List_RemNext (LIST *p_list, LIST_ELMT *p_elmt, void **p_data)
{
    LIST_ELMT   *old_elmt;

    if (0 == LIST_SIZE(p_list))                                         /* Prevent deletion of an empty list.                     */
    {
        return ERR_FAILURE;
    }
                                                                        /* Remove element from list:                              */
    if (NULL == p_elmt)
    {
       *p_data          = p_list->HeadPtr->DataPtr;                     /* Remove at beginning (head) of list.                    */
        old_elmt        = p_list->HeadPtr;
        p_list->HeadPtr = p_list->HeadPtr->NextElmtPtr;

        if (1 == LIST_SIZE(p_list))
        {
            p_list->TailPtr = NULL;
        }
    }
    else
    {                                                                   /* Remove at a predetermined position in the list.        */
        if (NULL == p_elmt->NextElmtPtr)
        {
            return ERR_FAILURE;
        }

       *p_data              = p_elmt->NextElmtPtr->DataPtr;
        old_elmt            = p_elmt->NextElmtPtr;
        p_elmt->NextElmtPtr = p_elmt->NextElmtPtr->NextElmtPtr;

        if (NULL == p_elmt->NextElmtPtr)
        {
            p_list->TailPtr = p_elmt;
        }
    }

    free(old_elmt);                                                     /* Free allocated memory only for the abstract data type. */

    p_list->Size--;                                                     /* Reduce list size to the just removed element.          */

    return ERR_SUCCESS;
}   /* List_RemNext()   */

/*
************************************************************************************************************************************
************************************************************************************************************************************
*                                                         LOCAL FUNCTIONS
************************************************************************************************************************************
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                 EXTERNAL C LANGUAGE LINKAGE END
************************************************************************************************************************************
*/

#ifdef __cplusplus
}                                                                                   /* End of 'extern'al C language linkage.      */
#endif
