/**
 * @file    msc.c
 * @brief   Micro stress control
 *
 *          The module contains all functionalities regarding the micro stress
 *          control beginning from determination of the MSC-table, diagnosis and
 *          correction.
 *
 * @par     Copyright:
 *
 *          Thermo Electron (Karlsruhe) GmbH, Dieselstrasse 4, 
 *          76227 Karlsruhe, Germany
 *
 *          (c) Copyright 2011 - 2012, Thermo Fisher Scientific
 *
 *          All rights reserved. Thermo Fisher Scientific source code is an
 *          unpublished work and the use of a copyright notice does not imply
 *          otherwise. This source code contains confidential, trade secret
 *          material of Thermo Fisher Scientific. Any attempt or participation
 *          in deciphering, decoding, reverse engineering or in any way altering
 *          the source code is strictly prohibited, unless the prior written
 *          consent of Thermo Fisher Scientific is obtained.
 *
 * @par     Creation Date:
 *
 *          17-Jul-2012
 *
 * @par     Last Modification:
 *
 *          24-Jul-2012
 *
 * @par     Project:
 *
 *          Visco Vision K1
 *
 * @par     Editor:
 *
 *          Microsoft Visual C++ 8.0
 *
 * @par     Compiler:
 *
 *          Microsoft Visual C++ 8.0
 *
 * @author  Uwe Bauer
 *
 * @par     Revision History:
 */

/*
 *******************************************************************************
 *                                  INCLUDES                                   *
 *******************************************************************************
 */
#include <math.h>
#include "Project.h"		// all global things as Type definitions, Hardware def., ...
#include "msc.h"
#include "mth.h"
#include "fft.h"


#ifdef FLASH
#pragma CODE_SECTION( MscCorr, "ramfuncs");
#endif

MSC_OBJ tMscData;

/*
 *******************************************************************************
 *                          LOCAL FUNCTIONS PROTOTYPES                         *
 *******************************************************************************
 */

/*
 * The function generates linearly spaced vectors.
 */
FP32 * Msc_LinSpace(const FP32 x1, const FP32 x2, INT32U n);

/*
 * Calculate power spectral density.
 */
void Msc_Psd(FP32 *x, FP32 *pwr, INT32U n);

/*
 * The function sorts the elements of a vector in abscending order.
 */
void Msc_Sort(FP32 *x, INT32U *idx, INT32U n);

/*
 * Real part of complex numbers.
 */
void Msc_Real(FP32 *x, FP32 *r, INT32U n);

/*
 *******************************************************************************
 *                              GLOBAL FUNCTIONS                               *
 *******************************************************************************
 */

/*
 * Configurate micro stress control.
 */
void MscCfg (MSC_OBJ *p, INT32U nTblLen, FP32 *pafTrq, FP32 *pafAbsAng,
             INT32U nNoFrq)
{
    if ((2 > nTblLen) || (0 == pafTrq) || (0 == pafAbsAng) || (0 == nNoFrq))
    {
        p->nSta = ((INT32U) 1);
    }
    else
    {
        p->nTblLen    = nTblLen;
        p->pafTrq     = pafTrq;
        p->pafAbsAng  = pafAbsAng;
		p->nNoFrq     = 2.0f*nNoFrq;

		// todo 20.09.13
		//p->fIdxFactor = ((FP32) (nTblLen)) / (4*cf2Pi);
		p->fIdxFactor = ((FP32) (nTblLen)) / (cf2Pi);
        p->nSta       = ((INT32U) 0);
    }
	p->fTrqDC = 0.0;
}   /* MscCfg() */

/*
 * Initialize micro stress control data aquisition.
 */
//unsigned int unIdxOff;
void MscDatAqInit (MSC_OBJ *p)
{
    INT32U k;

    // fill data table Phiabs with -1.0
    for (k = 0; k < p->nTblLen; k++)
    {
        *(p->pafAbsAng + k) = ((FP32) -1.0);
    }

    p->nAqCnt  = ((INT32U) 0);
    p->nRecRdy = ((INT32U) 0);
	// todo 20.09.13
	//unIdxOff = 0;

}   /* MscDatAqInit()   */


/*
 * Micro stress control data aquisition.
 */
INT32U MscDatAq (MSC_OBJ *p, FP32 fAbsAng, FP32 fTrq)
{
    INT32U nIdx;

    nIdx = ((INT32U) (fAbsAng * p->fIdxFactor));

    if (-1.0f == *(p->pafAbsAng + nIdx))
    {
        *(p->pafAbsAng + nIdx) = fAbsAng;
        *(p->pafTrq    + nIdx) = fTrq;
        p->nRecRdy = ((INT32U) 0);
        p->nAqCnt++;
        return p->nRecRdy;
    }
    else
    {
        if (p->nAqCnt >= p->nTblLen)
        {
            p->nRecRdy = ((INT32U) 1);
            return p->nRecRdy;
        }
        else
        {
            p->nRecRdy = ((INT32U) 0);
            return p->nRecRdy;
        }   
    }
}   /* MscDatAq()   */

/*
 * Calculate MSC tables.
 */
void MscTblCalc (MSC_OBJ *p)
{
    INT32U  i;              /* Common index.                                  */
    INT32U  j;              /* Common index.                                  */           
    INT32U  nFftLen;        /* FFT length.                                    */
    FP32    *pafAbsAngIp;   /* Linearly spaced absolute angle vector.         */
    FP32    *pafTrqIp;      /* Linearly spaced torque vector.                 */
    FP32    *pafTrqFft;     /* FFT torque vector.                             */
    FP32    *pafFftFac;     /* FFT twiddle factors.                           */
    FP32    *pafPwr;        /* Power vector.                                  */
    INT32U  *pafIdx;        /* Sorted index vector.                           */
    FP32    *pafTrqAppFft;  /* IFFT approximated torque.                      */
    FP32    normOrg;        /* Euclidean length of origin signal.             */
    FP32    normApp;        /* Euclidean length of approximated signal.       */

    /*
     * For the FFT calculation, the values must be equidistantly distributed.
     * The measured and recorded values are not exactly equidistantly
     * distributed. The equidistant distribution for the FFT calculation is
     * done by using linear table interpolation. The table interpolation has
     * no exapolation therefore the initial and finale values are aligned to
     * the theoretical limits.
     */
    *(p->pafAbsAng)                  = ((FP32) 0.0);
    
    *(p->pafAbsAng + p->nTblLen - 1) = (cf2Pi);
    //*(p->pafAbsAng + p->nTblLen - 1) = ((FP32) (2.0*MTH_PI));
    *(p->pafTrq) = (*p->pafTrq + *(p->pafTrq + p->nTblLen - 1))*0.5f;
    
    /*
     * Set the length (maximum possible power of 2) of the FFT automatically as
     * a function of the recorded number of data points.
     */
    // 6�s
    nFftLen = (INT32U) pow(2.0, floor(log(p->nTblLen)/log(2.0)));
	
    /*
     * Memory allocation for the auxiliary values.
     */
	// 75�s
    pafAbsAngIp  = Msc_LinSpace(0.0f, cf2Pi, nFftLen);


    //pafAbsAngIp  = Msc_LinSpace(0.0f, ((FP32) (2.0*MTH_PI)), nFftLen);

    // 250�s
    pafTrqIp     = (FP32 *)calloc(nFftLen, sizeof(FP32));
    pafTrqFft    = (FP32 *)calloc(2*nFftLen, sizeof(FP32));
    pafFftFac    = (FP32 *)calloc(2*nFftLen, sizeof(FP32));
    pafPwr       = (FP32 *)calloc(nFftLen, sizeof(FP32));
    pafIdx       = (INT32U *)calloc(nFftLen, sizeof(INT32U));
    pafTrqAppFft = (FP32 *)calloc(2*nFftLen, sizeof(FP32));
	
    /*
     * Generate linearly spaced torque vector. For the calculation of the torque
     * vector linear table interpolation and the measured values are used.
     */

    // 750�s
    for (i = 0; i < nFftLen; i++)
    {
        *(pafTrqIp + i) =
            MthInterp1D(p->pafAbsAng, p->pafTrq, *(pafAbsAngIp + i), p->nTblLen);
    }
    /*
     * Calculate FFT of the linearly spaced torque.
     */

    // 40�s
    FftReal2Cmplx(pafTrqIp, pafTrqFft, nFftLen);
	
    // 640�s
    FftGenTwfRx2(pafFftFac, nFftLen);
    FftCmplxRx2(nFftLen, pafTrqFft, pafFftFac, 1);
	

    // 85�s
    FftBitRev(pafTrqFft, nFftLen);

    /*
     * Calculate DC-offset torque which is caused bay the bearings
     */
    p->fTrqDC = *(pafTrqFft)/nFftLen;

    /*
     * For the calculation of the MSC table the DC free spectrum is used.
     */
    *(pafTrqFft) = 0.0f;

    /*
     * Calculate the power spectral density. For the determination of the
     * frequencies which contains the largest power in the signal the elements
     * of the power density vector is sorted in descending order. For the
     * inverse FFT only the bins which contain most of the power will be copied.
     */

	// 8640�s
    Msc_Psd(pafTrqFft, pafPwr, nFftLen);
    Msc_Sort(pafPwr, pafIdx, nFftLen);
    normOrg = MthVecSum(pafPwr, nFftLen);
    normApp = ((FP32) 0.0);
    for (i = 0; i < p->nNoFrq; i++)
    {
        j = *(pafIdx + i);
        *(pafTrqAppFft + 2*j)     = *(pafTrqFft + 2*j);
        *(pafTrqAppFft + 2*j + 1) = *(pafTrqFft + 2*j + 1);
        normApp += *(pafPwr + j);
    }
    p->fPerc = ((FP32) 100.0) * normApp/normOrg;

    /*
     * Approximation of the original signal with inverser FFT
     */
    // 480�s
    FftInvCmplxRx2(nFftLen, pafTrqAppFft, pafFftFac, 1);
    
    // 90�s
    FftBitRev(pafTrqAppFft, nFftLen);

    // 55�s
    FftInvCmplxRx2Scale(pafTrqAppFft, nFftLen);

    /*
     * Construct normalized MSC tables.
     */

    // 35�s
    Msc_Real(pafTrqAppFft, p->pafTrq, nFftLen);
    for (i = 0; i < nFftLen; i++)
    {
        // absolute Korrektur
        *(p->pafAbsAng + i) = *(pafAbsAngIp + i);
		// normierte Korrektur
		//*(p->pafTrq    + i) = *(p->pafTrq   + i)/p->fTrqDC;   
    }

    /*
     * Free allocated memors.
     */
    free(pafAbsAngIp);
    free(pafTrqIp);
    free(pafTrqFft);
    free(pafFftFac);
    free(pafPwr);
    free(pafIdx);
    free(pafTrqAppFft);
}   /* MscTblCalc() */

/*
 * Micro stress control correction.
 */
FP32 MscCorr (MSC_OBJ *p, FP32 fAbsAng, FP32 fTrq)
{
  // 24.06.16, only if table with resonable values (normal = 128)
  if( tMscData.nTblLen < 4 )
  {
     return 0.0;
  }
  else
  {
     // normierte Korrektur
     //return (fTrq*MthInterp1D(p->pafAbsAng, p->pafTrq, fAbsAng, p->nTblLen));
     // absolute Korrektur
	   return (MthInterp1D(p->pafAbsAng, p->pafTrq, fAbsAng, p->nTblLen));
  }
}   /* MscCorr()    */

/*
 *******************************************************************************
 *                               LOCAL FUNCTIONS                               *
 *******************************************************************************
 */

/*
 * The function generates linearly spaced vectors.
 *
 * @param   a   Start value
 * @param   b   End value
 * @param   n   Number of points
 * @returns     the linear spaced vector.
 */
FP32 * Msc_LinSpace (const FP32 a, const FP32 b, INT32U n)
{
   FP32   delta;        /* Linear distance between two data points.           */
   INT32U i;            /* Common index.                                      */
   FP32   *x;           /* Linearly spaced vector.                            */

   x = (FP32 *)calloc(n, sizeof(FP32));
   
   /*
    * Calculate linear distance between two data points.
    */
   delta = (b - a)/(n -1);

   /*
    * Generate linearly spaced vector.
    */
   for (i = 0; i < n; i++)
   {
      x[i] = i*delta;
   }

   return x;
}   /* Msc_LinSpace()   */

/*
 * Calculate power spectral density.
 *
 * @param   x   Pointer to complex FFT vector
 * @param   pwr Pointer to real power sepctral density vector
 * @param   n   Vector length
 */
void Msc_Psd (FP32 *x, FP32 *pwr, INT32U n)
{
    INT32U i;

    for (i = 0; i < n; i++)
    {
        pwr[i] = (FP32) ((pow(x[2*i], 2.0) + pow(x[2*i + 1], 2.0))/n);
    }
}  /* Msc_Psd()   */

/*
 * The function sorts the elements of a vector in ascending order.
 *
 * @param   x   Pointer to vector
 * @param   idx Pointer to index vector.
 * @param   n   Vectro length
 */
void Msc_Sort (FP32 *x, INT32U *idx, INT32U n)
{
   INT32U i, j, pos;            /* Common index.                              */
   FP32   val;                  /* Auxillary value.                           */

   for (i = 0; i < n; i++)      /* Construct original index vector.           */
   {
      *(idx + i) = i;
   }

   /*
    * Sort vector elements in ascending order.
    */
   for (j = 0; j < (n-1); j++)
   {
      for (i = 0; i < (n-1); i++)
      {
         if (*(x + i) < *(x + i + 1))
         {
            val = *(x + i);
            *(x + i) = *(x + i + 1);
            *(x + i + 1) = val;

            pos = *(idx + i);
            *(idx + i) = *(idx + i + 1);
            *(idx + i + 1) = pos;
         }
      }
   }
}   /* Msc_Sort()   */

/*
 * Real part of complex numbers.
 *
 * The function return the real elements of a complex array.
 *
 * @param   x   Complex array.
 * @param   r   Real array.
 * @param   n   Array length.
 */
void Msc_Real (FP32 *x, FP32 *r, INT32U n)
{
    INT32U i;               /* Common index.                                  */

   for (i = 0; i < n; i++)  /* Real part.                                     */
   {
      r[i] = x[2*i];
   }
}   /* Msc_Real()   */




float32 TblSearchMinMax(MSC_OBJ *p)
{
	float32 fMin, fMax;
	int16 i;
	// reset values min max
	fMin = fMax = *(p->pafTrq);
	for( i=0; i<p->nTblLen; i++)
	{
		if( *(p->pafTrq +i) < fMin )
		{
			fMin = *(p->pafTrq +i);
		}

		if( *(p->pafTrq +i) > fMax )
		{
			fMax = *(p->pafTrq +i);
		}
	}
	return fabs(fMin) + fabs(fMax);
}

#ifdef RAUS

INT32U MscDatAq (MSC_OBJ *p, FP32 fAbsAng, FP32 fTrq)
{
    INT32U nIdx;

	// todo 20.09.13
    if (p->nAqCnt == p->nTblLen / 4)
		unIdxOff = p->nTblLen / 4;
    else if (p->nAqCnt == p->nTblLen / 2)
		unIdxOff = p->nTblLen / 2;
    else if (p->nAqCnt == ( (p->nTblLen / 2) + (p->nTblLen / 4) ) )
		unIdxOff = ( (p->nTblLen / 2) + (p->nTblLen / 4) );
	else
		//NOP for Breakpoint
		asm(" nop");
    
    nIdx = ((INT32U) (fAbsAng * p->fIdxFactor)) + unIdxOff;

    if (-1.0f == *(p->pafAbsAng + nIdx ))
    {
        *(p->pafAbsAng + nIdx ) = fAbsAng;
        *(p->pafTrq    + nIdx ) = fTrq;
        p->nRecRdy = ((INT32U) 0);
        p->nAqCnt++;
        return p->nRecRdy;
    }
    else
    {
        if (p->nAqCnt >= p->nTblLen)
        {
            p->nRecRdy = ((INT32U) 1);
            return p->nRecRdy;
        }
        else
        {
            p->nRecRdy = ((INT32U) 0);
            return p->nRecRdy;
        }   
    }
}   /* MscDatAq()   */


#endif
