/**
 * @file    mth.c
 *
 *          Add-on mathematical functions
 *
 * @par     Copyright:
 *
 *          Thermo Electron (Karlsruhe) GmbH, Dieselstrasse 4,
 *          76227 Karlsrue, Germany
 *
 *          (c) Copyright 2009 - 2012, Thermo Electron
 *
 *          All rights reserved. Thermo Electron source code is an unpublished
 *          work and the use of a copyright notice does not imply otherwise.
 *          This source code contains confidential, trade secret material of
 *          Thermo Electron. Any attempt or participation in deciphering,
 *          decoding, reverse engineering or in any way altering the source
 *          code is strictly prohibited, unless the prior written consent of
 *          Thermo Electron is obtained.
 *
 * @par     Creation Date:
 *
 *          05-Dec-2009
 *
 * @par     Last Modification:
 *
 *          25-Jul-2012
 *
 * @par     Project:
 *
 *          Visco Vision K1
 *
 * @par     Editor:
 *
 *          Microsoft Visual C++ 8.0
 *
 * @par     Compiler:
 *
 *          Microsoft Visual C++ 8.0
 *
 * @author  Uwe Bauer
 *
 * @par     Revision History:
 */

/*
 *******************************************************************************
 *                                  INCLUDES                                   *
 *******************************************************************************
 */

#include <math.h>
#include "project.h"		// all global things as Type definitions, Hardware def., ...
#include "mth.h"
#include "err.h"

#ifdef FLASH
#pragma CODE_SECTION( MthInterp1D, "ramfuncs");
#pragma CODE_SECTION( Mth_Locate, "ramfuncs");


#endif



/*
 *******************************************************************************
 *                              LOCAL DATA TYPES                               *
 *******************************************************************************
 */


/**
 * Table index look-up result data object.
 */
typedef struct LOCATE_OBJ {
    INT32U  idx;    /**< Table index of the left interval limit.              */
    BOOLEAN lim;    /**< Indication for reaching one of the table boundarys.  */
} LOCATE_OBJ;


/*
 *******************************************************************************
 *                              GLOBAL VARIABLES                               *
 *******************************************************************************
 */

// Pi as const float in Flash Memory
const float cfPi_2 = 1.570796327;
const float cfPi   = 3.141592654;
const float cf2Pi  = 6.283185307;
//const float cf180_Pi = 57.295779513082320876798154814105;


/*
 *******************************************************************************
 *                          LOCAL FUNCTION PROTOTYPES                          *
 *******************************************************************************
 */

/*
 * Table index look-up
 */
static LOCATE_OBJ Mth_Locate(const FP32 X[], const FP32 xi, const INT32U n); 

/*
 *******************************************************************************
 *                              GLOBAL FUNCTIONS                               *
 *******************************************************************************
 */

/*
 * Logarithm-Dualis
 */
FP64 MthLog2 (FP64 x)
{
    return log(x)/log(2.0);
}   /* MthLog2()    */

/*
 * Signum function
 */
FP64 MthSign (const FP64 x)
{
   return ((0.0 < x) ? 1.0 : ((0.0 > x) ? -1.0 : 0.0));
}  /* MthSign()   */

/*
 * 1-D linear table interpolation.
 */
FP32 MthInterp1D (const FP32 X[], const FP32 Y[], const FP32 xi, const INT32U n)
{
    FP32       yL;      /* Left function value.                               */
    FP32       yR;      /* Right function value.                              */
    FP32       t;       /* Auxiliary variable for the linear interpolation.   */
    INT32U     idxL;    /* Table index of the left interval limit.            */
    INT32U     idxR;    /* Table index of the right interval limit.           */
    LOCATE_OBJ res;     /* Result for table index look-up.                    */

    /*
    * Determine the left table index.
    */
    res  = Mth_Locate(X, xi, n);
    idxL = res.idx;

    /* 
    * Does the argument lie outside of the table?
    */
    if (MTH_TRUE == res.lim)
    {
        idxR = idxL;    /* return the last tabulated value (no exapolation)   */
        t    = 0.0;
    }
    else
    {
        /*
         * Calculate the auxiliary value for the following linear interpolation.
         */
        idxR = idxL + 1u;
        t    = (xi - X[idxL])/(X[idxR] - X[idxL]);
    }

    /*
     * Linear interpolation between the two nodes and return the found
     * approximation.
     */
    yL = *(Y + idxL);
    yR = *(Y + idxR);
    return (1.0f - t)*yL + (t*yR);
}  /* MthInterp1D()  */

/*
 * Sum of vector elements.
 */
FP32 MthVecSum (FP32 *pVec, INT32U n)
{
    INT32U i;
    FP32   sum;

    sum = ((FP64) 0.0);

    for (i = 0; i < n; i++)
    {
        sum += *(pVec + i);
    }

    return sum;
}   /* MthVecSum()  */


/*
 *******************************************************************************
 *                              LOCAL FUNCTIONS                                *
 *******************************************************************************
 */

/**
 * Table index look-up.
 *
 * From a given array X[] with the length n and a value xi the function
 * determines an index k in such a way that xi lies between X[k] and X[k+1].
 * The values in the given array must either monotonically rise or fall.
 *
 * @param   X  The vector contains the arguments of the tabulated function
 *             values.
 * @param   xi Not tabulated argument xi for which the associated function
 *             value must be determined.
 * @param   n  Table length.
 * @returns    the table index which corresponds with the left interval end and
 *             the limitation status. Status 0 indicates interpolation and 1
 *             exapolation.
 */
static LOCATE_OBJ Mth_Locate (const FP32 X[], const FP32 xi, const INT32U n)
{
    INT32U     ku;      /* Upper index limit.                                 */
    INT32U     kl;      /* Lower index limit.                                 */
    INT32U     km;      /* Interval median.                                   */
    INT08U     ascnd;   /* Monotony behaviour (0:= dec or 1:= inc).           */
    LOCATE_OBJ res;     /* Result for table index look-up.                    */

    /*
     * Initialisation of the lower and upper index limits
     */
    kl = 0;
    ku = n - 1u;

    /*
     * Determine the monotony bahaviour.
     */
    ascnd = (X[n - 1u] >= X[0]);

    /*
     * Determine the interpolation intervall.
     */
    while ((ku - kl) > 1u)      /* Wasn't the smallest interval reached yet?  */
    {
        km = (ku + kl) >> 1;    /* Calculate interval median                  */

        if ((xi >= X[km]) == ascnd)
        {
            kl = km;            /* Found a new lower limit                    */
        }
        else
        {
            ku = km;            /* Found a new upper limit                    */
        }
    }

    /*
     * Checking of the interval limits.
     */
    if (((MTH_TRUE == ascnd) && (xi <= X[0])) || 
        ((MTH_FALSE == ascnd) && (xi >= X[0])))
    {
        res.lim = MTH_TRUE;
        res.idx = 0;
        return res;
    }
    else
    {
        if (((MTH_TRUE == ascnd) && (xi >= X[n - 1u])) || 
           ((MTH_FALSE == ascnd) && (xi <= X[n - 1u])))
        {
            res.lim = MTH_TRUE;
            res.idx = n - 1u;
            return res;
        }
        else
        {
            res.lim = MTH_FALSE;
            res.idx = kl;
            return res;
        }
    }
}  /* Mth_Locate() */




/****************************************************************************/
/*  MODF() - break into integer and fractional parts                        */
/*                                                                          */
/*  if x < -2 ^ 23 or x > 2 ^ 23, modf will return 0, and x -> memory       */
/*                                                                          */
/*  else result = x - int(x), int(x) -> memory location                     */
/*                                                                          */
/****************************************************************************/
 
#define TWO16	  65536	     /* 2 ^ BITS                            */ 
Uint16 Modulof(float value, Uint16 *iptr)
{
    Uint16 mid; /* the integral part of x */
 
    /**********************************************************************/
    /* for values >2^(bits in mantissa) or values <-2^(bits in mantissa)  */
    /**********************************************************************/
 
    if (value > TWO16 || value < - TWO16 || value == 0.0)
    {
        *iptr = (Uint16)value;
 
        /******************************************************************/
        /* fraction = 0                                                   */
        /******************************************************************/
 
        return 0;
    }
    else
    {
        /******************************************************************/
	/* This will be incorrect for values larger than 32-bits.         */
        /******************************************************************/
        mid = (Uint16) value;     /* since (long) -1.5 = -1.0 */
 
        /******************************************************************/
        /* save the integer part of value where iptr points               */
        /******************************************************************/
 
        *iptr = mid;
 
        /******************************************************************/
        /* return the fractional part of value                            */
        /******************************************************************/
 
        return (Uint16)(value - mid);
    }
}

/*!
\fn	float LimitFloat(float fVal, float fMin, float fMax)
\brief Limitierung eine float Wertes auf Grenzen oben und unten
			
\author	K�ster
\date	    30.03.2015
\param[in]
\return	void

\details Changes:
*/  
float LimitFloat(float fVal, float fMin, float fMax)
{
	if ( fVal > fMax )
	{
		return (fMax);
	}
	else if ( fVal < fMin )
	{
		return (fMin);
	}
	else
	{
		return (fVal);
	}	
}


/*
************************************************************************************************************************************
*                                                    ELEMENTARY MATH FUNCTIONS
************************************************************************************************************************************
*/

/*
 *  Complex base-b logarithm 
 */
FP64 Mth_LogBase (const FP64 a, const FP64 b)
{
    return log(a) / log(b);
}   /* Mth_LogBase()    */

/*
 *  Natural logarithm log(1 + x)
 */
FP64 Mth_Log1Plus (const FP64 x)
{
  volatile FP64 y;

  y = (1.0 + x);

  return (log(y) - (((y - 1.0) - x) / y)) ;                                         /* Cancels errors with IEEE arithmetic.       */
}   /* Mth_Log1Plus()   */

/*
 *  Magnitude of a complex number
 */
FP64 Mth_CmplxAbs (const FP64 x, const FP64 y)
{
    volatile FP64 x_abs;
    volatile FP64 y_abs;
    volatile FP64 min;
    volatile FP64 max;
    volatile FP64 rat;
    volatile FP64 root;

    if ((0.0 == x) && (0.0 == y))
    {
        return 0.0;
    }
    else
    {
        x_abs = fabs(x);
        y_abs = fabs(y);
        min   = MTH_MIN(x_abs, y_abs);
        max   = MTH_MAX(x_abs, y_abs);
        rat   = min/max;
        root  = sqrt(1.0 + rat*rat);

        if (max < MTH_FP64_MAX/root)
        {
            return (max*root);
        }
        else
        {
            return (0);//ERR_HDLR_VAL("Mth_Abs(): Overflow error", ERR_EOVRFLW, MTH_FP64_MAX); 
        }
    }
}   /* Mth_CmplxAbs()   */


#ifdef RAUS
/*
************************************************************************************************************************************
*                                                       HYPERBOLIC FUNCTIONS
************************************************************************************************************************************
*/

/*
 *  Arccosine hyperbolic
 */
FP64 Mth_Acosh (const FP64 x)
{
    volatile FP64 t;

    if ((1.0 / MTH_FP64_EPS_SQRT) < x)
    {
        return (log(x) + MTH_LN2);
    }
    else if (2.0 < x)
    {
        return log((2.0 * x) - (1.0 / (sqrt((x * x) - 1.0) + x)));
    }
    else if (1.0 < x)
    {
        t = (x - 1.0);
        return Mth_Log1Plus(t + sqrt((2.0 * t) + (t * t)));
    }
    else if (1.0 == x)
    {
        return 0.0;
    }
    else
    {
        //ERR_HDLR_VAL("Mth_Acosh(): Not a number", ERR_ENAN, ERR_NAN);
    }
}   /* Mth_Acosh()  */

/*
 *  Arcsine hyperbolic
 */
FP64 Mth_Asinh (const FP64 x)
{
    volatile FP64 x_abs;
    volatile FP64 x_abs_sqr;
    volatile FP64 s;

    x_abs = fabs(x);
    s     = MTH_SIGN(x);

    if ((1.0 / MTH_FP64_EPS_SQRT) < x_abs)
    {
        return (s * (log(x_abs) + MTH_LN2));
    }
    else if (2.0 < x_abs)
    {
        return (s * log((2.0 * x_abs) + (1.0 / (x_abs + sqrt(x_abs * x_abs + 1.0)))));
    }
    else if (MTH_FP64_EPS_SQRT < x_abs)
    {
        x_abs_sqr = x_abs * x_abs;
        return (s * Mth_Log1Plus(x_abs + (x_abs_sqr / (1.0 + sqrt(1.0 + x_abs_sqr)))));
    }
    else
    {
        return x;
    }
}   /* Mth_Asinh()  */

/*
 *  Arctangent hyperbolic
 */
FP64 Mth_Atanh (const FP64 x)
{
    volatile FP64 x_abs;
    volatile FP64 s;

    x_abs = fabs(x);
    s     = MTH_SIGN(x);

    if (1.0 < x_abs)
    {
        //ERR_HDLR_VAL("Mth_Atanh(): Not a number", ERR_ENAN, ERR_NAN);
    }
    else if (1.0 == x_abs)
    {
        /** @todo   INF exception handling. */
        return (0.0 > x) ? ERR_NEG_INF : ERR_POS_INF;
    }
    else if (0.5 <= x_abs)
    {
        return (s * 0.5 * Mth_Log1Plus((2.0 * x_abs) / (1.0 - x_abs)));
    }
    else if (MTH_FP64_EPS < x_abs)
    {
        return (s * 0.5 * Mth_Log1Plus((2.0 * x_abs) + (2.0 * x_abs * x_abs) / (1.0 - x_abs)));
    }
    else
    {
        return x;
    }
}   /* Mth_Atanh()  */




// Spline , direct from Numerical recipes
void spline(float32 x[], float32 y[], Uint16 n, float32 yp1, float32 ypn, float32 y2[])
{
	Uint16 i,k;
	float32 p,qn,sig,un,*u;
	u = (FP32 *)calloc(n, sizeof(FP32));
	//u=vector(1,n-1);
	//The lower boundary condition is set either to be "natural" 
	if (yp1 > 0.99e30)
	{
		y2[1]=u[1]=0.0; 
	}
	else
	{ 
		y2[1] = -0.5;
		u[1]=(3.0/(x[2]-x[1]))*((y[2]-y[1])/(x[2]-x[1])-yp1);
	}
	for (i=2;i<=n-1;i++)
	{ 
		sig=(x[i]-x[i-1])/(x[i+1]-x[i-1]);
		p=sig*y2[i-1]+2.0;
		y2[i]=(sig-1.0)/p;
		u[i]=(y[i+1]-y[i])/(x[i+1]-x[i]) - (y[i]-y[i-1])/(x[i]-x[i-1]);
		u[i]=(6.0*u[i]/(x[i+1]-x[i-1])-sig*u[i-1])/p;
	}
	if (ypn > 0.99e30)
	{
		qn=un=0.0;
	}
	else
	{
		qn=0.5;
		un=(3.0/(x[n]-x[n-1]))*(ypn-(y[n]-y[n-1])/(x[n]-x[n-1]));
	}
		y2[n]=(un-qn*u[n-1])/(qn*y2[n-1]+1.0);
		for (k=n-1;k>=1;k--)
		{
			y2[k]=y2[k]*y2[k+1]+u[k];
		}
		free(u);
		//free_vector(u,1,n-1);
}


// Spline Interpolation, direct from Numerical recipes
float32 splint(float xa[], float ya[], float y2a[], int n, float x)
{
	//void nrerror(char error_text[]);
	int klo,khi,k;
	float h,b,a;
	float d, e, f, g, fri;

	klo=1;
	khi=n;
	while (khi-klo > 1)
	{
		k=(khi+klo) >> 1;
		if( xa[k] > x )
			khi=k;
		else
			klo=k;
	}
	h=xa[khi]-xa[klo];
	
	// todo division trough zero
	if (h == 0.0)
	{
		// nrerror("Bad xa input to routine splint");
		ESTOP0;
	}
	a=(xa[khi]-x)/h;
	b=(x-xa[klo])/h;
	e= a*ya[klo]+b*ya[khi];
	f= (a*a*a-a)*y2a[klo];
	g= (b*b*b-b)*y2a[khi];
	d= (f+g)*h*h/6.0;
	fri = e + d;
	return(fri);
	//return (a*ya[klo]+b*ya[khi]+((a*a*a-a)*y2a[klo]+(b*b*b-b)*y2a[khi])*(h*h)/6.0);
	//*y=a*ya[klo]+b*ya[khi]+((a*a*a-a)*y2a[klo]+(b*b*b-b)*y2a[khi])*(h*h)/6.0;
}
#endif
