/**
 * @file    opt.c
 * @brief   Optimization module
 *
 * @par     Copyright:
 *
 *          Thermo Electron (Karlsruhe) GmbH, Dieselstrasse 4, 76227 Karlsruhe, Germany
 *
 *          (c) Copyright 2010, Thermo Fisher Scientific
 *
 *          All rights reserved. Thermo Fisher Scientific source code is an unpublished work and the use of a copyright notice does
 *          not imply otherwise. This source code contains confidential, trade secret material of Thermo Fisher Scientific. Any
 *          attempt or participation in deciphering, decoding, reverse engineering or in any way altering the source code is
 *          strictly prohibited, unless the prior written consent of Thermo Fisher Scientific is obtained.
 *
 * @par     Creation Date:
 *
 *          11-Feb-2010
 *
 * @par     Last Modification:
 *
 *          24-Mar-2011		die Beschleunigung des Startverhaltens des Optimierungsalgorithmus eine Berechnung
 *							gnstiger Startwerte implementiert. Die Berechnung der Startwerte basiert auf der Auswertung
 *							der ersten Vollschwingung und einer linearen Exapolation der erforderlichen Drehmomente.
 *							Die Vorbelegung mit gnstigen Startwerten erfolgt immer nach einem Neustart
 *							der Oszillationsmessung und nach dem Vorliegen der ersten Vollschwingung.
 *							Neben der Exapolation fr die Berechnung der Startwerte wurden zustzlich eine Zustandsvariable
 *							und die Zustnde Idle und Run definiert. Bei der Initialisierung der Oszillation wird der Zustand
 *							Idle voreingestellt. Beim Start der eigentlichen Messung werden einmalig die Startwerte vorbelegt
 *							und in den Zustand Run gewechselt.
 *
 *			17-Jan-2015		Initialisierung in OptFastProp testweise raus wg. CDOSC-Mode
 *			22-Oct-2015		testweise wieder rein, Promlem bei CDOSC Ampl.-Sweep
 *			28-Oct-2015		testweise wieder raus, wg. CDOSC-Mode
 *
 * @par     Project:
 *
 *          MARS III
 *
 * @par     Editor:
 *
 *          Microsoft Visual C++ 6.0
 *
 * @par     Compiler:
 *
 *          Microsoft Visual C++ 6.0
 *
 * @author  Uwe Bauer
 *
 * @par     Revision History:
 *          08.03.16, in Sweep-Mode, use old values for W (weight)
 */

/*
 **********************************************************************************************************************************
 *                                                           INCLUDES                                                             *
 **********************************************************************************************************************************
 */
#include "Project.h"		// all global things as Type definitions, Hardware def., ...
#include "opt.h"
#include "mth.h"
#include "CsFastOsc.h"


/*
 **********************************************************************************************************************************
 *                                                        GLOBAL FUNCTIONS                                                        *
 **********************************************************************************************************************************
 */

 /*
 * Improved nonlinear fast propagation optimization procedure.
 */
FP32 OptFastProp (OPT_FPROP_DEV *p,const FP32 tgtVal,const FP32 actVal)
{
    #ifdef RAUS // 04.04.16 testweise
    if (OPT_IDLE == p->state)
    {
        if( *PtrFoscSweepMode() == 0)
        {    
            p->Wk_1     = MTH_MIN(tgtVal/actVal*0.5f, OPT_DELTA_MAX);
        }
        else
        {
            p->Wk_1 = p->Wk;
        }
        p->state    = OPT_RUN;
    }
    #endif
 
    p->Tk = tgtVal;
    p->Yk = actVal;
    p->Ek = (p->Tk - p->Yk) / p->Tk;

    
    p->Delta = MTH_MIN(p->Wk_1 * OPT_LRATE_PLUS * fabs(p->Ek), p->DeltaMax);
    p->dWk   = ((FP32) MthSign(p->Ek) * p->Delta);
    p->Wk    = p->Wk_1 + p->dWk;
    p->Wk    = MTH_MIN(MTH_MAX(p->Wk, p->Wmin), p->Wmax);
    // 
    p->Wk_1     = p->Wk;
 
    return p->Wk;
}

 
/*
 * Initialize the nonlinear fast propagation optimization procedure.
 */
void OptFastPropInit (OPT_FPROP_DEV *p)
{
    if( *PtrFoscSweepMode() == 0)
    {    

        p->Wk_1     = 1.0f;
        p->dEdWk_1  = 0.0f;
        p->Wmin     = 1.0f;
        p->Wmax     = 0.1f / 0.000005f;
        p->DeltaMin = ((FP32)1e-3);
        p->DeltaMax = 0.5f * p->Wmax;
    }
    p->state = OPT_IDLE;
}   /* OptFastPropInit()    */
 
/*
 * Set the maximum weight of the nonlinear fast propagation optimization.
 */
Uint16 OptFastPropSetMaxWeight (OPT_FPROP_DEV *p, const FP32 val)
{
    if (1.0f < val)
    {
        p->Wmax = val;
        return 0;
    }
    else
    {
        return 1;
    }
}   /* OptFastPropSetMaxWeightFP32()    */


// ***************** Old version **********************************************
#ifdef RAUS
/*
 * Nonlinear fast propagation optimization procedure.
 */
float32 OptFastProp (OPT_FPROP_DEV *p, float32 tgtVal, float32 actVal)
{
#ifdef VTIQ_AIR	// 27.01.15 testweise raus, 29.10.15 bei VTiQ-Air drin lassen
    if (OPT_IDLE == p->state)
    {
        p->delta[0] = MTH_MIN(tgtVal/actVal*0.03f, OPT_DELTA_MAX);
        p->delta[1] = p->delta[0];
        p->dW[0]    = p->delta[0];
        p->dW[1]    = p->delta[0];
        // 08.03.16, in Sweep-Mode, use old values for W (weight)
        if( *PtrFoscSweepMode() == 0)
        {    
            p->W[1]     = MTH_MIN(tgtVal/actVal*0.75f, OPT_DELTA_MAX);
        }
        else
        {
            p->W[1] = p->W[0];
        }
        p->state    = OPT_RUN;
    }
#endif

    p->E[0]    = tgtVal - actVal;       /* Calculate error.                                       */

    p->dEdW[0] = p->E[0]/p->dW[0];      /* Calculate partial derivative.                          */

    if (0.0f < p->dEdW[1]*p->dEdW[0])   /* Update step size depending on partial derivative signs.*/
    {
        p->delta[0] = 
            MTH_MIN(p->delta[1]*OPT_LRATE_PLUS, OPT_DELTA_MAX);
    }
    else
    {
        if (0.0f > p->dEdW[1]*p->dEdW[0])
        {
            p->delta[0] = 
                MTH_MAX(p->delta[1]*OPT_LRATE_MINUS, OPT_DELTA_MIN);
        }
    }
    p->dW[0] = ((float32) MthSign(p->E[0])*p->delta[0]);            /* Weight update depending on the sign of the error.      */
    p->W[0]  = p->W[1] + p->dW[0];

    // 24.02.14 hier Variation OPT_DELTA_MIN
    //p->W[0] = MTH_MIN(MTH_MAX(p->W[0], OPT_DELTA_MIN), p->maxWeight);
    p->W[0] = MTH_MIN(MTH_MAX(p->W[0], 1.0f), p->maxWeight);
    p->E[1]     = p->E[0];                                          /* Store actual values for recursive calculation.         */
    p->dEdW[1]  = p->dEdW[0];
    p->delta[1] = p->delta[0];
    p->dW[1]    = p->dW[0];
    p->W[1]     = p->W[0];

    return p->W[0];                                                 /* Output actual weight.                                  */
}   /* OptFastProp()   */

/*
 * Initialize the nonlinear fast propagation optimization procedure.
 */
void OptFastPropInit (OPT_FPROP_DEV *p)
{
    // 08.03.16, do not reset in Sweep-Mode
    if( *PtrFoscSweepMode() == 0)
    {    
        p->delta[0]  = OPT_DELTA_INIT;      /* Initialize the step size with a start value.           */
        p->delta[1]  = OPT_DELTA_INIT;

        p->dW[0]     = OPT_DELTA_INIT;      /* Initialize the weight update with a start value.       */
        p->dW[1]     = OPT_DELTA_INIT;
        p->W[0]      = OPT_WEIGHT_INIT;     /* Initialize the weight with a start value.              */
        p->W[1]      = OPT_WEIGHT_INIT;
        p->maxWeight = OPT_WEIGHT_MAX;      /* Initialize maximum weight with default value.          */

        p->E[0]      = 0.0f;                /* Reset actual and previous error value to zero.         */
        p->E[1]      = 0.0f;

        p->dEdW[0]   = 0.0f;                /* Reset actual and previous partial derivatives to zero. */
        p->dEdW[1]   = 0.0f;
    }
#ifdef VTIQ_AIR	// 27.01.15 testweise raus
    p->state     = OPT_IDLE;                /* Initialize state variable                              */
#endif
}   /* OptFastPropInit()    */

/*
 * Set the maximum weight of the nonlinear fast propagation optimization.
 */
Uint16 OptFastPropSetMaxWeight (OPT_FPROP_DEV *p, const float32 val)
{
    if (OPT_DELTA_MIN < val)
    {
        p->maxWeight = val;

        return ((Uint16) 0);
    }
    else
    {
        return ((Uint16) 1);
    }
}   /* OptFastPropSetMaxWeight()    */
#endif
