/*
************************************************************************************************************************************
*                                              READY-TO-USE EMBEDDED BUILDING BLOCKS
*
*                             (c) Copyright 2015-2016; Thermo Electron (Karlsruhe) GmbH; Karlsruhe-Germany
*
*       All rights reserved. Protected by international copyright laws. 
*
*       TMO source code is an unpublished work and the use of a copyright notice does not imply otherwise. This source code
*       contains confidential, trade secret material of TMO. Any attempt or participation in deciphering, decoding, reverse
*       engineering or in any way altering the source code is strictly prohibited, unless the prior written consent of TMO
*       is obtained.
*
************************************************************************************************************************************
*/

/**
************************************************************************************************************************************
*
*                                                         FILE DESCRIPTION
*
* @file         sig.c
* @brief        Signal Generator 
* @details      The signal generator can produce one of five different waveforms: 
*
*               1. Sine wave
*               2. Cosine wave
*               3. square wave
*               4. Sawtooth wave
*               5. Random wave
*
*               Signal parameters are expressed in Hertz.
*
* @version      V1.00.00
* @author       UwBa
*
************************************************************************************************************************************
* Notes         (1) The note(s) section should be omitted if no notes are given.
************************************************************************************************************************************
*/

/*
************************************************************************************************************************************
*                                                          INCLUDE FILES
************************************************************************************************************************************
*/

#include <math.h>                                                   /* Because of sine, cosine.                                   */

#include "sig.h"
#include "mth.h"                                                    /* Because of math constants and signum function.             */
#include "err.h"                                                    /* Because of error handling and error numbers.               */

/*
************************************************************************************************************************************
*                                                   EXTERNAL C LANGUAGE LINKAGE
*
* Note(s) : (1) C++ compilers must 'extern'ally declare all C function prototypes & variable/object declarations for correct C
*               language linkage.
************************************************************************************************************************************
*/

#ifdef __cplusplus
extern "C" {                                                                        /* See Note #1.                               */
#endif


#ifdef FLASH
#pragma CODE_SECTION(Sig_GenFP32, "ramfuncs");
#endif


/*
************************************************************************************************************************************
*                                                          LOCAL DEFINES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                         LOCAL CONSTANTS
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                         LOCAL DATA TYPES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                           LOCAL TABLES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                         LOCAL VARIABLES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                     LOCAL FUNCTION PROTOTYPES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
************************************************************************************************************************************
*                                                         GLOBAL FUNCTIONS
************************************************************************************************************************************
************************************************************************************************************************************
*/

/*
 *  Configure 32-bit precision signal generator
 */
INT32U Sig_GenCfgFP32 (SIG_GEN_FP32_T *p, INT32U form, FP32 amp, FP32 frq, FP32 tm)
{
    if ((SIG_SINE     != form) &&
        (SIG_COSINE   != form) &&
        (SIG_SQUARE   != form) &&
        (SIG_SAWTOOTH != form) &&
        (SIG_RANDOM   != form))
    {
        //ERR_HDLR("Invalid wave form!", ERR_EINVAL);
    }

    if (0.0 >= amp)
    {
        //ERR_HDLR("Invalid signal amplitude!", ERR_ERANGE);
    }

    if (0.0 >= frq)
    {
        //ERR_HDLR("Invalid signal frequency!", ERR_ERANGE);
    }

    if (0.0 >= tm)
    {
        //ERR_HDLR("Invalid sample time!", ERR_ERANGE);
    }
                                                                    /* Take over parameters:                                      */
    p->WaveForm = form;                                             /* Signal wave form.                                          */
    p->Amp      = amp ;                                             /* Signal amplitude.                                          */
    p->Frq      = frq ;                                             /* Signal frequency.                                          */
    p->SmpTm    = tm  ;                                             /* Sample time.                                               */

    p->Perd = ((FP32) 1.0) / p->Frq;                                /* Calculate signal period [s].                               */

    p->Tm  = ((FP32) 0.0);                                          /* Rest runtime.                                              */
    p->Out = ((FP32) 0.0);                                          /* Reset signal generator output.                             */

    switch (p->WaveForm)
    {
        case SIG_SINE  :
        case SIG_COSINE:
        case SIG_SQUARE:
             p->Omega = (FP32)(MTH_2PI * p->Frq);                   /* Calculate angular frequency [rad/s].                       */
             break;

        case SIG_SAWTOOTH:
             p->Slope  = (2.0f * p->Amp) / p->Perd;                 /* Calculate signal slope.                                    */
             p->Offset = p->Amp;                                    /* Calculate signal offset.                                   */
             break;
        
        default:
             ;
    }

    return ERR_SUCCESS;
}   /* Sig_GenCfgFP32() */

/*
 *  32-bit precision signal generator
 */
FP32 Sig_GenFP32 (SIG_GEN_FP32_T *p)
{
    switch (p->WaveForm)
    {
        case SIG_SINE:
             p->Out = (FP32)(p->Amp * sin(p->Omega * p->Tm));
             break;
        
        case SIG_COSINE:
             p->Out = (FP32)(p->Amp * cos(p->Omega * p->Tm));
             break;

        case SIG_SQUARE:
             p->Out = (FP32)(p->Amp * MTH_SIGN(sin(p->Omega * p->Tm)));
             break;

        case SIG_SAWTOOTH:
             p->Out = p->Slope * p->Tm + p->Offset;
             break;

        case SIG_RANDOM:
             /* @todo:  Generate pseudo random signal.  */
             p->Out = ((FP32) 0.0);
             break;

        default:
             p->Out = ((FP32) 0.0);
    }

    p->Tm = p->Tm + p->SmpTm;

    if (p->Tm >= p->Perd)
    {
        p->Tm = ((FP32) 0.0);
    }

    return p->Out;
}   /* Sig_GenFP32()    */

void Sig_GenResetFP32 (SIG_GEN_FP32_T *p)
{
    p->Tm  = ((FP32) 0.0);
    p->Out = ((FP32) 0.0);
}   /* Sig_GenResetFP32()   */

/*
 *  Set signal amplitude 32-bit precision signal generator
 */
INT32U Sig_GenSetAmpFP32 (SIG_GEN_FP32_T *p, FP32 amp)
{
    if (0.0 >= amp)
    {
        //ERR_HDLR("Invalid signal amplitude!", ERR_ERANGE);
    }

    p->Amp      = amp ;                                             /* Take over signal amplitude.                                */

    if (SIG_SAWTOOTH == p->WaveForm)
    {
        p->Slope  = (2.0f * p->Amp) / p->Perd;                      /* Calculate signal slope.                                    */
        p->Offset = p->Amp;                                         /* Calculate signal offset.                                   */
    }

    return ERR_SUCCESS;
}   /* Sig_GenSetAmpFP32() */


/*
************************************************************************************************************************************
************************************************************************************************************************************
*                                                         LOCAL FUNCTIONS
************************************************************************************************************************************
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                 EXTERNAL C LANGUAGE LINKAGE END
************************************************************************************************************************************
*/

#ifdef __cplusplus
}                                                                                   /* End of 'extern'al C language linkage.      */
#endif
