/*
************************************************************************************************************************************
*                                              READY-TO-USE EMBEDDED BUILDING BLOCKS
*
*                             (c) Copyright 2015-2016; Thermo Electron (Karlsruhe) GmbH; Karlsruhe-Germany
*
*       All rights reserved. Protected by international copyright laws. 
*
*       TMO source code is an unpublished work and the use of a copyright notice does not imply otherwise. This source code
*       contains confidential, trade secret material of TMO. Any attempt or participation in deciphering, decoding, reverse
*       engineering or in any way altering the source code is strictly prohibited, unless the prior written consent of TMO
*       is obtained.
*
************************************************************************************************************************************
*/

/**
************************************************************************************************************************************
*
*                                                         FILE DESCRIPTION
*
* @file         src.c
* @brief        Generation of Signals 
* @details      The signal generator can produce one of five different waveforms: 
*
*               1. Sine wave
*               2. Cosine wave
*               3. square wave
*               4. Sawtooth wave
*               5. Random wave
*
*               Signal parameters are expressed in Hertz.
*
* @version      V1.00.00
* @author       UwBa
*
************************************************************************************************************************************
* Notes         (1) The note(s) section should be omitted if no notes are given.
************************************************************************************************************************************
*/

/*
************************************************************************************************************************************
*                                                          INCLUDE FILES
************************************************************************************************************************************
*/

#include <math.h>                                                   /* Because of sine, cosine.                                   */

#include "src.h"
#include "mth.h"                                                    /* Because of math constants and signum function.             */
#include "err.h"                                                    /* Because of error handling and error numbers.               */

/*
************************************************************************************************************************************
*                                                   EXTERNAL C LANGUAGE LINKAGE
*
* Note(s) : (1) C++ compilers must 'extern'ally declare all C function prototypes & variable/object declarations for correct C
*               language linkage.
************************************************************************************************************************************
*/

#ifdef __cplusplus
extern "C" {                                                        /* See Note #1.                                               */
#endif

#ifdef FLASH
#pragma CODE_SECTION(Src_SigGenFP32, "ramfuncs");
#endif



/*
************************************************************************************************************************************
*                                                          LOCAL DEFINES
************************************************************************************************************************************
*/

#if !defined(SRC_LFSR_MIN_LEN)
#define SRC_LFSR_MIN_LEN    ((INT16U)2)                             /**< Minimum length of the linear feedback shift register.    */
#endif

#if !defined(SRC_LFSR_MAX_LEN)
#define SRC_LFSR_MAX_LEN    ((INT16U)16)                            /**< Maximum length of the linear feedback shift register.    */
#endif

/*
************************************************************************************************************************************
*                                                         LOCAL CONSTANTS
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                         LOCAL DATA TYPES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                           LOCAL TABLES
************************************************************************************************************************************
*/

/** Linear feedback shift register configuration table
 *
 *  The configuration table is constructed column-oriented and includes the following information:
 *
 *      Column      Information
 *      1           Linear feedback shift register length
 *      2           Number of feedback stages
 *      3 - 6       Taps
 */
static const INT16U SRC_LFSR_CFG_TBL [15][6] = {                /*  n   Polynom                             Taps                  */
    { 2,  2,  0,  1,  0,  0},                                   /*  2   1 + x^ 1 + x^ 2                     0 and 1               */
    { 3,  2,  0,  1,  0,  0},                                   /*  3   1 + x^ 2 + x^ 3                     0 and 1               */
    { 4,  2,  0,  1,  0,  0},                                   /*  4   1 + x^ 3 + x^ 4                     0 and 1               */
    { 5,  2,  0,  2,  0,  0},                                   /*  5   1 + x^ 3 + x^ 5                     0 and 2               */
    { 6,  2,  0,  1,  0,  0},                                   /*  6   1 + x^ 5 + x^ 6                     0 and 1               */
    { 7,  2,  0,  1,  0,  0},                                   /*  7   1 + x^ 6 + x^ 7                     0 and 1               */
    { 8,  4,  0,  2,  3,  4},                                   /*  8   1 + x^ 4 + x^ 5 + x^ 6 + x^ 8       0, 2, 3 and 4         */
    { 9,  2,  0,  4,  0,  0},                                   /*  9   1 + x^ 5 + x^ 9                     0 and 4               */
    {10,  2,  0,  3,  0,  0},                                   /* 10   1 + x^ 7 + x^10                     0 and 3               */
    {11,  2,  0,  2,  0,  0},                                   /* 11   1 + x^ 9 + x^11                     0 and 2               */
    {12,  4,  0,  6,  8, 11},                                   /* 12   1 + x^ 1 + x^ 4 + x^ 6 + x^12       0, 6, 8 and 11        */
    {13,  4,  0,  9, 10, 12},                                   /* 13   1 + x^ 1 + x^ 3 + x^ 4 + x^13       0, 9, 10 and 12       */
    {14,  4,  0,  9, 11, 13},                                   /* 14   1 + x^ 1 + x^ 3 + x^ 5 + x^14       0, 9, 11 and 13       */
    {15,  2,  0,  1,  0,  0},                                   /* 15   1 + x^14 + x^15                     0 and 1               */
    {16,  4,  0,  1,  3, 12}                                    /* 16   1 + x^ 4 + x^13 + x^15 + x^16       0, 1, 3 and 12        */
};

/*
************************************************************************************************************************************
*                                                         LOCAL VARIABLES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                     LOCAL FUNCTION PROTOTYPES
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
************************************************************************************************************************************
*                                                         GLOBAL FUNCTIONS
************************************************************************************************************************************
************************************************************************************************************************************
*/

/*
 *  Parameterization 32-bit precision signal generator
 */
INT16U Src_SigGenParamFP32 (SRC_SIG_GEN_FP32_T *p, const INT16U wave_form, const FP32 amp, const FP32 frq, const INT16U units,
                            const FP32 smp_tm)
{
    INT16U col;                                                         /* Column index for random number configuration.          */

    if ((SRC_WF_SINE     != wave_form) &&                               /* Check for valid wave form?                             */
        (SRC_WF_COSINE   != wave_form) &&
        (SRC_WF_SQUARE   != wave_form) &&
        (SRC_WF_SAWTOOTH != wave_form) &&
        (SRC_WF_RANDOM   != wave_form))
    {
        //ERR_HDLR("Invalid wave form!", ERR_EINVAL);
    }

    if (0.0 >= frq)                                                     /* Check for valid frequency range?                       */
    {
        //ERR_HDLR("Invalid signal frequency!", ERR_ERANGE);
    }

    if (0.0 >= smp_tm)                                                  /* Check for valid sample time range?                     */
    {
        //ERR_HDLR("Invalid sample time!", ERR_ERANGE);
    }

    if ((SRC_UNITS_HZ      != units) &&                                 /* Check for valid units?                                 */
        (SRC_UNITS_RADPSEC != units))
    {
        //ERR_HDLR("Invalid signal units!", ERR_EINVAL);
    }

    if (SRC_MIN_NBR_SMP > ((INT32U)(1.0f / (smp_tm * frq))))            /* Check for Shannon sampling theorem?                    */
    {
        //ERR_HDLR("Not complied with sampling theorem!", ERR_EINVAL);
    }

                                                                        /* Take over parameters:                                  */
    p->WaveForm = wave_form;                                            /* Signal wave form.                                      */
    p->Amp      = amp      ;                                            /* Signal amplitude.                                      */
    p->SmpTm    = smp_tm   ;                                            /* Sample time.                                           */
    p->Units    = units    ;                                            /* Signal units.                                          */

    if (SRC_UNITS_RADPSEC == p->Units)
    {
        p->Frq = (FP32)(frq / MTH_2PI);                                 /* Convert frequency from rad/s into Hertz.               */
    }
    else
    {
        p->Frq = frq;                                                   /* Take over signal frequency.                            */
    }

    p->Perd = (((FP32)1.0) / p->Frq) - p->SmpTm;                        /* Calculate period length [s].                           */

    p->Tm  = ((FP32)0.0);                                               /* Reset runtime.                                         */
    p->Out = ((FP32)0.0);                                               /* Zeroize signal generator output.                       */

    switch (p->WaveForm)
    {
        case SRC_WF_SINE  :
        case SRC_WF_COSINE:
        case SRC_WF_SQUARE:
             p->Omega = (FP32)(MTH_2PI * p->Frq);                       /* Calculate angular frequency [rad/s].                   */
             break;

        case SRC_WF_SAWTOOTH:
             p->Slope  = -(2.0f * p->Amp) / p->Perd;                    /* Calculate signal slope.                                */
             p->Offset = p->Amp;                                        /* Calculate signal offset.                               */
             break;

        case SRC_WF_RANDOM:
             p->Len = (INT16U)Mth_LogBase(p->Perd / p->SmpTm, 2.0);     /* Calculate register length based on period N.           */
             p->Len = MTH_MAX(MTH_MIN(p->Len, SRC_LFSR_MAX_LEN),        /* Balancing register length to the allowed limits.       */
                              SRC_LFSR_MIN_LEN);

             col = p->Len - SRC_LFSR_MIN_LEN;                           /* Calculate configuration table column index based on ...*/
                                                                        /* register and minimum length.                           */
             p->NbrFdbkStages = SRC_LFSR_CFG_TBL[col][1];               /* Determine number of feedback stages and taps based ... */
             p->TapTbl[0]     = SRC_LFSR_CFG_TBL[col][2];               /* on defined configuration table and required ...        */
             p->TapTbl[1]     = SRC_LFSR_CFG_TBL[col][3];               /* regsiter length.                                       */
             p->TapTbl[2]     = SRC_LFSR_CFG_TBL[col][4];
             p->TapTbl[3]     = SRC_LFSR_CFG_TBL[col][5];

             p->LFSR = 0x0001;                                          /* Any nonzero start value will work.                     */

             p->Amp    = 2.0f * amp;                                    /* Bipolar signals.                                       */
             p->Offset = 0.5f;
             break;
        
        default:                                                        /* Should never happen!                                   */
             ;
    }

    return ERR_SUCCESS;                                                 /* Confirm successful parameterization.                   */
}   /* Src_SigGenParamFP32()    */

/*
 *  32-bit precision signal generator
 */
FP32 Src_SigGenFP32 (SRC_SIG_GEN_FP32_T *p)
{                                                                           /* Local variables for random wave form generation:...*/
    INT16U  bit;                                                            /* Linear feedback shift register input bit.          */
    INT16U  k;                                                              /* Common run index.                                  */


    switch (p->WaveForm)
    {
        case SRC_WF_SINE:                                                   /* Sine wave form generation.                         */
             p->Out = (FP32)(p->Amp * sin(p->Omega * p->Tm));
             break;
        
        case SRC_WF_COSINE:                                                 /* Cosine wave form generation.                       */
             p->Out = (FP32)(p->Amp * cos(p->Omega * p->Tm));
             break;

        case SRC_WF_SQUARE:                                                 /* Square wave form generation.                       */
             p->Out = (FP32)(p->Amp * MTH_SIGN(sin(p->Omega * p->Tm)));
             break;

        case SRC_WF_SAWTOOTH:                                               /* Sawtooth wave form gneration.                      */
             p->Out = p->Slope * p->Tm + p->Offset;
             break;

        case SRC_WF_RANDOM:                                                 /* Random wave form generation.                       */

             /*
              **********************************************************************************************************************
              * Caclulate the linear feedback shift register input bit based on:
              *
              *     1. Current content of the lienar feedback shift register.
              *     2. Given characteristic Fibonacci feedback polynomial.
              *     3. Exclusive OR function.
              *
              **********************************************************************************************************************
              */

             bit = (p->LFSR >> p->TapTbl[0]);

             for (k = 1; k < p->NbrFdbkStages; k++)
             {
                 bit ^= (p->LFSR >> p->TapTbl[k]);
             }

             bit &= 0x0001;

             /*
              **********************************************************************************************************************
              * Update linear feedback shift register.
              *
              *     1. Shift complete register content by one position to the right.
              *     2. Insert the current input bit at upper left register end.
              *
              **********************************************************************************************************************
              */

             p->LFSR = ((p->LFSR >> 1) | (bit << (p->Len - 1)));

             /*
              **********************************************************************************************************************
              * Generate discrete binary noise signal:
              *
              *     1. Reading bit content at right register end.
              *     2. Calculate uni- or bipolar output based on the given offset and with required amplitude.
              *
              **********************************************************************************************************************
              */

             p->Out = p->Amp * (((FP32)(p->LFSR & 0x0001)) - p->Offset);
             break;

        default:                                                            /* Zeroize output for invalid wave form.              */
             p->Out = ((FP32) 0.0);
    }

    p->Tm = p->Tm + p->SmpTm;                                               /* Update ellapsed run time.                          */

    if (p->Tm >= p->Perd)
    {
        p->Tm = ((FP32)0.0);                                                /* Reset runtime if period length is exceeded.        */
    }

    return p->Out;
}   /* Src_SigGenFP32() */

/*
 *  Reset 32-bit precision signal generator
 */
void Src_SigGenResetFP32 (SRC_SIG_GEN_FP32_T *p)
{
    p->Tm  = ((FP32)0.0);                                           /* Reset runtime.                                             */
    p->Out = ((FP32)0.0);                                           /* Zeroize signal generator output.                           */
}   /* Src_SigGenResetFP32()    */

/*
 *  Set signal amplitude of 32-bit precision signal generator
 */
void Src_SigGenSetAmpFP32 (SRC_SIG_GEN_FP32_T *p, const FP32 amp)
{
    switch (p->WaveForm)
    {
        case SRC_WF_SINE:
        case SRC_WF_COSINE:
        case SRC_WF_SQUARE:
             p->Amp = amp;                                          /* Take over new signal amplitude.                            */
             break;

        case SRC_WF_SAWTOOTH:
             p->Amp    = amp;                                       /* Take over new signal amplitude.                            */
             p->Slope  = (2.0f * p->Amp) / p->Perd;                 /* Re-calculate signal slope.                                 */
             p->Offset = p->Amp;                                    /* Re-calculate signal offset.                                */
             break;

        case SRC_WF_RANDOM:
             p->Amp    = 2.0f * amp;                                /* Double signal amplitude to generate bipolar signals.       */
             p->Offset = 0.5f;
             break;

        default:
             ;
    }
}   /* Src_SigGenSetAmpFP32()   */

/*
************************************************************************************************************************************
************************************************************************************************************************************
*                                                         LOCAL FUNCTIONS
************************************************************************************************************************************
************************************************************************************************************************************
*/


/*
************************************************************************************************************************************
*                                                 EXTERNAL C LANGUAGE LINKAGE END
************************************************************************************************************************************
*/

#ifdef __cplusplus
}                                                                                   /* End of 'extern'al C language linkage.      */
#endif
