/**
 * @file    tsg.c
 * @brief   Torque signal generator
 *
 * @par     Copyright:
 *
 *          Thermo Electron (Karlsruhe) GmbH, Dieselstrasse 4, 76227 Karlsruhe, Germany
 *
 *          (c) Copyright 2010, Thermo Fisher Scientific
 *
 *          All rights reserved. Thermo Fisher Scientific source code is an unpublished work and the use of a copyright notice does
 *          not imply otherwise. This source code contains confidential, trade secret material of Thermo Fisher Scientific. Any
 *          attempt or participation in deciphering, decoding, reverse engineering or in any way altering the source code is
 *          strictly prohibited, unless the prior written consent of Thermo Fisher Scientific is obtained.
 *
 * @par     Creation Date:
 *
 *          22-Jan-2010
 *
 * @par     Last Modification:
 *
 *          07-Apr-2010
 *
 * @par     Project:
 *
 *          MARS III
 *
 * @par     Editor:
 *
 *          Microsoft Visual C++ 6.0
 *
 * @par     Compiler:
 *
 *          Microsoft Visual C++ 6.0
 *
 * @author  Uwe Bauer
 *
 * @par     Revision History:
 */

/*
 **********************************************************************************************************************************
 *                                                           INCLUDES                                                             *
 **********************************************************************************************************************************
 */
#include "Project.h"		// all global things as Type definitions, Hardware def., ...
#include <math.h>
#include "tsg.h"
#include "mth.h"
//#include "C28x_FPU_FastRTS.h"

/*
 **********************************************************************************************************************************
 *                                                        GLOBAL FUNCTIONS                                                        *
 **********************************************************************************************************************************
 */

/*
 * Configuration of the torque signal generator.
 */
Uint16 TsgCfgSineTbl (TSG_DEV *pDev, float32 *pTrqSineTbl, Uint32 n)
{
    Uint32 k;                                                   /* Common index                                                   */
    float32   d  = 2.0f*cfPi/n;                                      /* Increment                                                      */

    if ((0 != pTrqSineTbl) && (0 == (n % 2)) && (4 < n))
    {
        pDev->pTrqSineTbl = pTrqSineTbl;
        pDev->n           = n;

        for (k=0; k<n; k++)                                     /* Calculate tabulated sine values                                */
        {
            *(pTrqSineTbl + k) = ((float32) cos(k*d));
        }

        return ((Uint16) 0);
    }
    else
    {
        return ((Uint16) 1);
    }
}   /* TsgCfgSineTbl()  */

/*
 * Set the increment to output the tabulated torque sine values.
 */
Uint16 TsgSetSineTblInc (TSG_DEV *pDev, Uint32 inc)
{
    if (((1 == inc) || ((1 < inc) && (0 == (inc % 2)))) && (inc < pDev->n))
    {
        pDev->inc = inc;

        return ((Uint16) 0);
    }
    else
    {
        return ((Uint16) 1);
    }
}   /* TsgSetSineTblInc()   */

/*
 * The function set the timer resolution.
 */
Uint16 TsgSetTmrRes (TSG_DEV *pDev, float32 res)
{
    if (0 < res)
    {
        pDev->dt = res;
        return ((Uint16) 0);
    }
    else
    {
        return ((Uint16) 1);
    }
}   /* TsgSetTmrRes()   */

/*
 * Reset of the torque signal generator.
 */
void TsgReset (TSG_DEV *pDev)
{
    pDev->mode = TSG_STOP;
    pDev->k    = ((Uint32) 0);
    pDev->t    = ((float32) 0);
}   /* TsgReset()   */

#ifdef RAUS
/*
 * Configure the amplitude modulator for a torque signal generator.
 */
void TsgCfgAmpMod (TSG_DEV *pDev, float32 (*fun)(float32 *, float32), float32 *arg)
{
    //pDev->ampModFun = fun;
    //pDev->ampModArg = arg;
	
}   /* TsgCfgAmpMod()   */


/*
 * Set the operating mode for a torque signal generator.
 */
Uint16 TsgStartMod (TSG_DEV *pDev, Uint16 mode)
{
    if ((TSG_STOP == mode) || (TSG_START == mode))
    {
        pDev->mode = mode;

        return ((Uint16) 0);
    }
    else
    {
        return ((Uint16) 1);
    }
}   /* TsgStartMod()    */
#endif

/*
 * Calculate sine torque signal.
 */
float32 TsgCalcTrq (TSG_DEV *pDev)
{
    if (pDev->k < pDev->n)                                          /* Sine generator                                             */
    {
        pDev->trq = *(pDev->pTrqSineTbl + pDev->k);
    }
    else
    {
        pDev->trq = *(pDev->pTrqSineTbl);

        pDev->k = ((Uint32) 0);
    }

    pDev->k += pDev->inc;

	#ifdef RAUS // 26.03.14 no more Ampl.-Mode control
    if ((0 != pDev->ampModFun) && (0 != pDev->ampModArg))           /* Amplitude modulator                                        */
    {
        pDev->amp = pDev->ampModFun(pDev->ampModArg, pDev->t);
		// todo 22.11.13
		//pDev->amp = pDev->amp;	
    }
	#endif

    pDev->trq *= pDev->amp;

    switch (pDev->mode)                                             /* Timer                                                      */
    {
        case TSG_STOP:

             pDev->t = 0.0;
             break;

        case TSG_START:

             pDev->t += pDev->dt;
             break;

        default:

             pDev->t = 0.0;
    }

    return pDev->trq;                                               /* Output actual torque                                       */
}   /* TsgCalcTrq() */

/*
 * Read actual torque amplitude.
 */
float32 TsgGetTrqAmp (TSG_DEV *pDev)
{
    return pDev->amp;
}   /* TsgGetTrqAmp()   */

//Set actual torque amplitude

void TsgSetTrqAmp (TSG_DEV *pDev, float32 fAmp)
{
    pDev->amp = fAmp;
}


/*
 * Heaviside amplitude modulation function
 */
float32 TsgHeavisideFun (float32 *pCoefTbl, float32 t)
{
    return *(pCoefTbl);
}   /* TsgHeavisideFun()    */



